#include <assert.h>

#include <aterm2.h>
#include <deprecated.h>
#include "SDFME.h"

/*{{{  conversion functions */

ATerm SDF_stringToChars(const char *str)
{
  int len = strlen(str);
  int i;
  ATermList result = ATempty;

  for (i = len - 1; i >= 0; i--) {
    result = ATinsert(result, (ATerm) ATmakeInt(str[i]));
  }

  return (ATerm) result;
}

ATerm SDF_byteToChar(char ch)
{
    return (ATerm) ATmakeInt(ch);
}

char *SDF_charsToString(ATerm arg)
{
  ATermList list = (ATermList) arg;
  int len = ATgetLength(list);
  int i;
  char *str;

  str = (char *) malloc(len+1);
  if (str == NULL) {
      return NULL;
  }

  for (i = 0; !ATisEmpty(list); list = ATgetNext(list), i++) {
    str[i] = (char) ATgetInt((ATermInt) ATgetFirst(list));
  }
  str[i] = '\0';

  return str;
}

char SDF_charToByte(ATerm arg)
{
    return (char) ATgetInt((ATermInt) arg);
}


/*}}}  */

/*{{{  typedefs */

typedef struct ATerm _SDF_OptLayout;
typedef struct ATerm _SDF_Layout;
typedef struct ATerm _SDF_LexLayoutList;
typedef struct ATerm _SDF_AFun;
typedef struct ATerm _SDF_ATerm;
typedef struct ATerm _SDF_Annotation;
typedef struct ATerm _SDF_Start;
typedef struct ATerm _SDF_ATermArgs;
typedef struct ATerm _SDF_ATermElems;
typedef struct ATerm _SDF_ATermAnnos;
typedef struct ATerm _SDF_IntCon;
typedef struct ATerm _SDF_OptExp;
typedef struct ATerm _SDF_RealCon;
typedef struct ATerm _SDF_Grammar;
typedef struct ATerm _SDF_Alias;
typedef struct ATerm _SDF_Aliases;
typedef struct ATerm _SDF_AliasList;
typedef struct ATerm _SDF_Symbol;
typedef struct ATerm _SDF_CharRange;
typedef struct ATerm _SDF_CharRanges;
typedef struct ATerm _SDF_OptCharRanges;
typedef struct ATerm _SDF_CharClass;
typedef struct ATerm _SDF_LexNumChar;
typedef struct ATerm _SDF_NumChar;
typedef struct ATerm _SDF_LexShortChar;
typedef struct ATerm _SDF_ShortChar;
typedef struct ATerm _SDF_Character;
typedef struct ATerm _SDF_ATermAttribute;
typedef struct ATerm _SDF_Attribute;
typedef struct ATerm _SDF_Attributes;
typedef struct ATerm _SDF_Production;
typedef struct ATerm _SDF_Productions;
typedef struct ATerm _SDF_AttributeList;
typedef struct ATerm _SDF_ProductionList;
typedef struct ATerm _SDF_Label;
typedef struct ATerm _SDF_SymbolArguments;
typedef struct ATerm _SDF_LexModuleWord;
typedef struct ATerm _SDF_ModuleWord;
typedef struct ATerm _SDF_LexModuleId;
typedef struct ATerm _SDF_ModuleId;
typedef struct ATerm _SDF_Definition;
typedef struct ATerm _SDF_Module;
typedef struct ATerm _SDF_Section;
typedef struct ATerm _SDF_Sections;
typedef struct ATerm _SDF_ModuleName;
typedef struct ATerm _SDF_ImpSection;
typedef struct ATerm _SDF_Imports;
typedef struct ATerm _SDF_Import;
typedef struct ATerm _SDF_ModuleList;
typedef struct ATerm _SDF_ImpSectionList;
typedef struct ATerm _SDF_SectionList;
typedef struct ATerm _SDF_ImportList;
typedef struct ATerm _SDF_Associativity;
typedef struct ATerm _SDF_ArgumentIndicator;
typedef struct ATerm _SDF_Group;
typedef struct ATerm _SDF_Priority;
typedef struct ATerm _SDF_Priorities;
typedef struct ATerm _SDF_NatConArguments;
typedef struct ATerm _SDF_GroupList;
typedef struct ATerm _SDF_PriorityList;
typedef struct ATerm _SDF_SymbolTail;
typedef struct ATerm _SDF_SymbolRest;
typedef struct ATerm _SDF_Renamings;
typedef struct ATerm _SDF_Renaming;
typedef struct ATerm _SDF_RenamingList;
typedef struct ATerm _SDF_Lookahead;
typedef struct ATerm _SDF_Lookaheads;
typedef struct ATerm _SDF_Restriction;
typedef struct ATerm _SDF_Restrictions;
typedef struct ATerm _SDF_LookaheadList;
typedef struct ATerm _SDF_RestrictionList;
typedef struct ATerm _SDF_SDF;
typedef struct ATerm _SDF_Sort;
typedef struct ATerm _SDF_LexSort;
typedef struct ATerm _SDF_SymbolParameters;
typedef struct ATerm _SDF_Symbols;
typedef struct ATerm _SDF_SymbolList;
typedef struct ATerm _SDF_LexLayout;
typedef struct ATerm _SDF_LexStrChar;
typedef struct ATerm _SDF_StrChar;
typedef struct ATerm _SDF_LexStrCon;
typedef struct ATerm _SDF_StrCon;
typedef struct ATerm _SDF_LexStrCharChars;
typedef struct ATerm _SDF_LexNatCon;
typedef struct ATerm _SDF_NatCon;
typedef struct ATerm _SDF_LexIdCon;
typedef struct ATerm _SDF_IdCon;

/*}}}  */

/*{{{  void SDF_initSDFMEApi(void) */

void SDF_initSDFMEApi(void)
{
  init_SDFME_dict();
}

/*}}}  */

/*{{{  protect functions */

void SDF_protectOptLayout(SDF_OptLayout *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLayout(SDF_Layout *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLexLayoutList(SDF_LexLayoutList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectAFun(SDF_AFun *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectATerm(SDF_ATerm *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectAnnotation(SDF_Annotation *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectStart(SDF_Start *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectATermArgs(SDF_ATermArgs *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectATermElems(SDF_ATermElems *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectATermAnnos(SDF_ATermAnnos *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectIntCon(SDF_IntCon *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectOptExp(SDF_OptExp *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectRealCon(SDF_RealCon *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectGrammar(SDF_Grammar *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectAlias(SDF_Alias *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectAliases(SDF_Aliases *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectAliasList(SDF_AliasList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectSymbol(SDF_Symbol *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectCharRange(SDF_CharRange *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectCharRanges(SDF_CharRanges *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectOptCharRanges(SDF_OptCharRanges *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectCharClass(SDF_CharClass *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLexNumChar(SDF_LexNumChar *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectNumChar(SDF_NumChar *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLexShortChar(SDF_LexShortChar *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectShortChar(SDF_ShortChar *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectCharacter(SDF_Character *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectATermAttribute(SDF_ATermAttribute *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectAttribute(SDF_Attribute *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectAttributes(SDF_Attributes *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectProduction(SDF_Production *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectProductions(SDF_Productions *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectAttributeList(SDF_AttributeList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectProductionList(SDF_ProductionList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLabel(SDF_Label *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectSymbolArguments(SDF_SymbolArguments *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLexModuleWord(SDF_LexModuleWord *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectModuleWord(SDF_ModuleWord *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLexModuleId(SDF_LexModuleId *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectModuleId(SDF_ModuleId *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectDefinition(SDF_Definition *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectModule(SDF_Module *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectSection(SDF_Section *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectSections(SDF_Sections *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectModuleName(SDF_ModuleName *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectImpSection(SDF_ImpSection *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectImports(SDF_Imports *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectImport(SDF_Import *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectModuleList(SDF_ModuleList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectImpSectionList(SDF_ImpSectionList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectSectionList(SDF_SectionList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectImportList(SDF_ImportList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectAssociativity(SDF_Associativity *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectArgumentIndicator(SDF_ArgumentIndicator *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectGroup(SDF_Group *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectPriority(SDF_Priority *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectPriorities(SDF_Priorities *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectNatConArguments(SDF_NatConArguments *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectGroupList(SDF_GroupList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectPriorityList(SDF_PriorityList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectSymbolTail(SDF_SymbolTail *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectSymbolRest(SDF_SymbolRest *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectRenamings(SDF_Renamings *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectRenaming(SDF_Renaming *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectRenamingList(SDF_RenamingList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLookahead(SDF_Lookahead *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLookaheads(SDF_Lookaheads *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectRestriction(SDF_Restriction *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectRestrictions(SDF_Restrictions *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLookaheadList(SDF_LookaheadList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectRestrictionList(SDF_RestrictionList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectSDF(SDF_SDF *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectSort(SDF_Sort *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLexSort(SDF_LexSort *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectSymbolParameters(SDF_SymbolParameters *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectSymbols(SDF_Symbols *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectSymbolList(SDF_SymbolList *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLexLayout(SDF_LexLayout *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLexStrChar(SDF_LexStrChar *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectStrChar(SDF_StrChar *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLexStrCon(SDF_LexStrCon *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectStrCon(SDF_StrCon *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLexStrCharChars(SDF_LexStrCharChars *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLexNatCon(SDF_LexNatCon *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectNatCon(SDF_NatCon *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectLexIdCon(SDF_LexIdCon *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void SDF_protectIdCon(SDF_IdCon *arg)
{
  ATprotect((ATerm*)((void*) arg));
}


/*}}}  */
/*{{{  term conversion functions */

/*{{{  SDF_OptLayout SDF_OptLayoutFromTerm(ATerm t) */

SDF_OptLayout SDF_OptLayoutFromTerm(ATerm t)
{
  return (SDF_OptLayout)t;
}

/*}}}  */
/*{{{  ATerm SDF_OptLayoutToTerm(SDF_OptLayout arg) */

ATerm SDF_OptLayoutToTerm(SDF_OptLayout arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Layout SDF_LayoutFromTerm(ATerm t) */

SDF_Layout SDF_LayoutFromTerm(ATerm t)
{
  return (SDF_Layout)t;
}

/*}}}  */
/*{{{  ATerm SDF_LayoutToTerm(SDF_Layout arg) */

ATerm SDF_LayoutToTerm(SDF_Layout arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_LexLayoutList SDF_LexLayoutListFromTerm(ATerm t) */

SDF_LexLayoutList SDF_LexLayoutListFromTerm(ATerm t)
{
  return (SDF_LexLayoutList)t;
}

/*}}}  */
/*{{{  ATerm SDF_LexLayoutListToTerm(SDF_LexLayoutList arg) */

ATerm SDF_LexLayoutListToTerm(SDF_LexLayoutList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_AFun SDF_AFunFromTerm(ATerm t) */

SDF_AFun SDF_AFunFromTerm(ATerm t)
{
  return (SDF_AFun)t;
}

/*}}}  */
/*{{{  ATerm SDF_AFunToTerm(SDF_AFun arg) */

ATerm SDF_AFunToTerm(SDF_AFun arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_ATermFromTerm(ATerm t) */

SDF_ATerm SDF_ATermFromTerm(ATerm t)
{
  return (SDF_ATerm)t;
}

/*}}}  */
/*{{{  ATerm SDF_ATermToTerm(SDF_ATerm arg) */

ATerm SDF_ATermToTerm(SDF_ATerm arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Annotation SDF_AnnotationFromTerm(ATerm t) */

SDF_Annotation SDF_AnnotationFromTerm(ATerm t)
{
  return (SDF_Annotation)t;
}

/*}}}  */
/*{{{  ATerm SDF_AnnotationToTerm(SDF_Annotation arg) */

ATerm SDF_AnnotationToTerm(SDF_Annotation arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Start SDF_StartFromTerm(ATerm t) */

SDF_Start SDF_StartFromTerm(ATerm t)
{
  return (SDF_Start)t;
}

/*}}}  */
/*{{{  ATerm SDF_StartToTerm(SDF_Start arg) */

ATerm SDF_StartToTerm(SDF_Start arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_ATermArgs SDF_ATermArgsFromTerm(ATerm t) */

SDF_ATermArgs SDF_ATermArgsFromTerm(ATerm t)
{
  return (SDF_ATermArgs)t;
}

/*}}}  */
/*{{{  ATerm SDF_ATermArgsToTerm(SDF_ATermArgs arg) */

ATerm SDF_ATermArgsToTerm(SDF_ATermArgs arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_ATermElems SDF_ATermElemsFromTerm(ATerm t) */

SDF_ATermElems SDF_ATermElemsFromTerm(ATerm t)
{
  return (SDF_ATermElems)t;
}

/*}}}  */
/*{{{  ATerm SDF_ATermElemsToTerm(SDF_ATermElems arg) */

ATerm SDF_ATermElemsToTerm(SDF_ATermElems arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_ATermAnnos SDF_ATermAnnosFromTerm(ATerm t) */

SDF_ATermAnnos SDF_ATermAnnosFromTerm(ATerm t)
{
  return (SDF_ATermAnnos)t;
}

/*}}}  */
/*{{{  ATerm SDF_ATermAnnosToTerm(SDF_ATermAnnos arg) */

ATerm SDF_ATermAnnosToTerm(SDF_ATermAnnos arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_IntCon SDF_IntConFromTerm(ATerm t) */

SDF_IntCon SDF_IntConFromTerm(ATerm t)
{
  return (SDF_IntCon)t;
}

/*}}}  */
/*{{{  ATerm SDF_IntConToTerm(SDF_IntCon arg) */

ATerm SDF_IntConToTerm(SDF_IntCon arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_OptExp SDF_OptExpFromTerm(ATerm t) */

SDF_OptExp SDF_OptExpFromTerm(ATerm t)
{
  return (SDF_OptExp)t;
}

/*}}}  */
/*{{{  ATerm SDF_OptExpToTerm(SDF_OptExp arg) */

ATerm SDF_OptExpToTerm(SDF_OptExp arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_RealCon SDF_RealConFromTerm(ATerm t) */

SDF_RealCon SDF_RealConFromTerm(ATerm t)
{
  return (SDF_RealCon)t;
}

/*}}}  */
/*{{{  ATerm SDF_RealConToTerm(SDF_RealCon arg) */

ATerm SDF_RealConToTerm(SDF_RealCon arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_GrammarFromTerm(ATerm t) */

SDF_Grammar SDF_GrammarFromTerm(ATerm t)
{
  return (SDF_Grammar)t;
}

/*}}}  */
/*{{{  ATerm SDF_GrammarToTerm(SDF_Grammar arg) */

ATerm SDF_GrammarToTerm(SDF_Grammar arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Alias SDF_AliasFromTerm(ATerm t) */

SDF_Alias SDF_AliasFromTerm(ATerm t)
{
  return (SDF_Alias)t;
}

/*}}}  */
/*{{{  ATerm SDF_AliasToTerm(SDF_Alias arg) */

ATerm SDF_AliasToTerm(SDF_Alias arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Aliases SDF_AliasesFromTerm(ATerm t) */

SDF_Aliases SDF_AliasesFromTerm(ATerm t)
{
  return (SDF_Aliases)t;
}

/*}}}  */
/*{{{  ATerm SDF_AliasesToTerm(SDF_Aliases arg) */

ATerm SDF_AliasesToTerm(SDF_Aliases arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_AliasList SDF_AliasListFromTerm(ATerm t) */

SDF_AliasList SDF_AliasListFromTerm(ATerm t)
{
  return (SDF_AliasList)t;
}

/*}}}  */
/*{{{  ATerm SDF_AliasListToTerm(SDF_AliasList arg) */

ATerm SDF_AliasListToTerm(SDF_AliasList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_SymbolFromTerm(ATerm t) */

SDF_Symbol SDF_SymbolFromTerm(ATerm t)
{
  return (SDF_Symbol)t;
}

/*}}}  */
/*{{{  ATerm SDF_SymbolToTerm(SDF_Symbol arg) */

ATerm SDF_SymbolToTerm(SDF_Symbol arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_CharRange SDF_CharRangeFromTerm(ATerm t) */

SDF_CharRange SDF_CharRangeFromTerm(ATerm t)
{
  return (SDF_CharRange)t;
}

/*}}}  */
/*{{{  ATerm SDF_CharRangeToTerm(SDF_CharRange arg) */

ATerm SDF_CharRangeToTerm(SDF_CharRange arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_CharRangesFromTerm(ATerm t) */

SDF_CharRanges SDF_CharRangesFromTerm(ATerm t)
{
  return (SDF_CharRanges)t;
}

/*}}}  */
/*{{{  ATerm SDF_CharRangesToTerm(SDF_CharRanges arg) */

ATerm SDF_CharRangesToTerm(SDF_CharRanges arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_OptCharRanges SDF_OptCharRangesFromTerm(ATerm t) */

SDF_OptCharRanges SDF_OptCharRangesFromTerm(ATerm t)
{
  return (SDF_OptCharRanges)t;
}

/*}}}  */
/*{{{  ATerm SDF_OptCharRangesToTerm(SDF_OptCharRanges arg) */

ATerm SDF_OptCharRangesToTerm(SDF_OptCharRanges arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_CharClass SDF_CharClassFromTerm(ATerm t) */

SDF_CharClass SDF_CharClassFromTerm(ATerm t)
{
  return (SDF_CharClass)t;
}

/*}}}  */
/*{{{  ATerm SDF_CharClassToTerm(SDF_CharClass arg) */

ATerm SDF_CharClassToTerm(SDF_CharClass arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_LexNumChar SDF_LexNumCharFromTerm(ATerm t) */

SDF_LexNumChar SDF_LexNumCharFromTerm(ATerm t)
{
  return (SDF_LexNumChar)t;
}

/*}}}  */
/*{{{  ATerm SDF_LexNumCharToTerm(SDF_LexNumChar arg) */

ATerm SDF_LexNumCharToTerm(SDF_LexNumChar arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_NumChar SDF_NumCharFromTerm(ATerm t) */

SDF_NumChar SDF_NumCharFromTerm(ATerm t)
{
  return (SDF_NumChar)t;
}

/*}}}  */
/*{{{  ATerm SDF_NumCharToTerm(SDF_NumChar arg) */

ATerm SDF_NumCharToTerm(SDF_NumChar arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_LexShortChar SDF_LexShortCharFromTerm(ATerm t) */

SDF_LexShortChar SDF_LexShortCharFromTerm(ATerm t)
{
  return (SDF_LexShortChar)t;
}

/*}}}  */
/*{{{  ATerm SDF_LexShortCharToTerm(SDF_LexShortChar arg) */

ATerm SDF_LexShortCharToTerm(SDF_LexShortChar arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_ShortChar SDF_ShortCharFromTerm(ATerm t) */

SDF_ShortChar SDF_ShortCharFromTerm(ATerm t)
{
  return (SDF_ShortChar)t;
}

/*}}}  */
/*{{{  ATerm SDF_ShortCharToTerm(SDF_ShortChar arg) */

ATerm SDF_ShortCharToTerm(SDF_ShortChar arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Character SDF_CharacterFromTerm(ATerm t) */

SDF_Character SDF_CharacterFromTerm(ATerm t)
{
  return (SDF_Character)t;
}

/*}}}  */
/*{{{  ATerm SDF_CharacterToTerm(SDF_Character arg) */

ATerm SDF_CharacterToTerm(SDF_Character arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_ATermAttribute SDF_ATermAttributeFromTerm(ATerm t) */

SDF_ATermAttribute SDF_ATermAttributeFromTerm(ATerm t)
{
  return (SDF_ATermAttribute)t;
}

/*}}}  */
/*{{{  ATerm SDF_ATermAttributeToTerm(SDF_ATermAttribute arg) */

ATerm SDF_ATermAttributeToTerm(SDF_ATermAttribute arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Attribute SDF_AttributeFromTerm(ATerm t) */

SDF_Attribute SDF_AttributeFromTerm(ATerm t)
{
  return (SDF_Attribute)t;
}

/*}}}  */
/*{{{  ATerm SDF_AttributeToTerm(SDF_Attribute arg) */

ATerm SDF_AttributeToTerm(SDF_Attribute arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Attributes SDF_AttributesFromTerm(ATerm t) */

SDF_Attributes SDF_AttributesFromTerm(ATerm t)
{
  return (SDF_Attributes)t;
}

/*}}}  */
/*{{{  ATerm SDF_AttributesToTerm(SDF_Attributes arg) */

ATerm SDF_AttributesToTerm(SDF_Attributes arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Production SDF_ProductionFromTerm(ATerm t) */

SDF_Production SDF_ProductionFromTerm(ATerm t)
{
  return (SDF_Production)t;
}

/*}}}  */
/*{{{  ATerm SDF_ProductionToTerm(SDF_Production arg) */

ATerm SDF_ProductionToTerm(SDF_Production arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Productions SDF_ProductionsFromTerm(ATerm t) */

SDF_Productions SDF_ProductionsFromTerm(ATerm t)
{
  return (SDF_Productions)t;
}

/*}}}  */
/*{{{  ATerm SDF_ProductionsToTerm(SDF_Productions arg) */

ATerm SDF_ProductionsToTerm(SDF_Productions arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_AttributeList SDF_AttributeListFromTerm(ATerm t) */

SDF_AttributeList SDF_AttributeListFromTerm(ATerm t)
{
  return (SDF_AttributeList)t;
}

/*}}}  */
/*{{{  ATerm SDF_AttributeListToTerm(SDF_AttributeList arg) */

ATerm SDF_AttributeListToTerm(SDF_AttributeList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_ProductionList SDF_ProductionListFromTerm(ATerm t) */

SDF_ProductionList SDF_ProductionListFromTerm(ATerm t)
{
  return (SDF_ProductionList)t;
}

/*}}}  */
/*{{{  ATerm SDF_ProductionListToTerm(SDF_ProductionList arg) */

ATerm SDF_ProductionListToTerm(SDF_ProductionList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Label SDF_LabelFromTerm(ATerm t) */

SDF_Label SDF_LabelFromTerm(ATerm t)
{
  return (SDF_Label)t;
}

/*}}}  */
/*{{{  ATerm SDF_LabelToTerm(SDF_Label arg) */

ATerm SDF_LabelToTerm(SDF_Label arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_SymbolArguments SDF_SymbolArgumentsFromTerm(ATerm t) */

SDF_SymbolArguments SDF_SymbolArgumentsFromTerm(ATerm t)
{
  return (SDF_SymbolArguments)t;
}

/*}}}  */
/*{{{  ATerm SDF_SymbolArgumentsToTerm(SDF_SymbolArguments arg) */

ATerm SDF_SymbolArgumentsToTerm(SDF_SymbolArguments arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_LexModuleWord SDF_LexModuleWordFromTerm(ATerm t) */

SDF_LexModuleWord SDF_LexModuleWordFromTerm(ATerm t)
{
  return (SDF_LexModuleWord)t;
}

/*}}}  */
/*{{{  ATerm SDF_LexModuleWordToTerm(SDF_LexModuleWord arg) */

ATerm SDF_LexModuleWordToTerm(SDF_LexModuleWord arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_ModuleWord SDF_ModuleWordFromTerm(ATerm t) */

SDF_ModuleWord SDF_ModuleWordFromTerm(ATerm t)
{
  return (SDF_ModuleWord)t;
}

/*}}}  */
/*{{{  ATerm SDF_ModuleWordToTerm(SDF_ModuleWord arg) */

ATerm SDF_ModuleWordToTerm(SDF_ModuleWord arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_LexModuleId SDF_LexModuleIdFromTerm(ATerm t) */

SDF_LexModuleId SDF_LexModuleIdFromTerm(ATerm t)
{
  return (SDF_LexModuleId)t;
}

/*}}}  */
/*{{{  ATerm SDF_LexModuleIdToTerm(SDF_LexModuleId arg) */

ATerm SDF_LexModuleIdToTerm(SDF_LexModuleId arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_ModuleId SDF_ModuleIdFromTerm(ATerm t) */

SDF_ModuleId SDF_ModuleIdFromTerm(ATerm t)
{
  return (SDF_ModuleId)t;
}

/*}}}  */
/*{{{  ATerm SDF_ModuleIdToTerm(SDF_ModuleId arg) */

ATerm SDF_ModuleIdToTerm(SDF_ModuleId arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Definition SDF_DefinitionFromTerm(ATerm t) */

SDF_Definition SDF_DefinitionFromTerm(ATerm t)
{
  return (SDF_Definition)t;
}

/*}}}  */
/*{{{  ATerm SDF_DefinitionToTerm(SDF_Definition arg) */

ATerm SDF_DefinitionToTerm(SDF_Definition arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Module SDF_ModuleFromTerm(ATerm t) */

SDF_Module SDF_ModuleFromTerm(ATerm t)
{
  return (SDF_Module)t;
}

/*}}}  */
/*{{{  ATerm SDF_ModuleToTerm(SDF_Module arg) */

ATerm SDF_ModuleToTerm(SDF_Module arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Section SDF_SectionFromTerm(ATerm t) */

SDF_Section SDF_SectionFromTerm(ATerm t)
{
  return (SDF_Section)t;
}

/*}}}  */
/*{{{  ATerm SDF_SectionToTerm(SDF_Section arg) */

ATerm SDF_SectionToTerm(SDF_Section arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Sections SDF_SectionsFromTerm(ATerm t) */

SDF_Sections SDF_SectionsFromTerm(ATerm t)
{
  return (SDF_Sections)t;
}

/*}}}  */
/*{{{  ATerm SDF_SectionsToTerm(SDF_Sections arg) */

ATerm SDF_SectionsToTerm(SDF_Sections arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_ModuleName SDF_ModuleNameFromTerm(ATerm t) */

SDF_ModuleName SDF_ModuleNameFromTerm(ATerm t)
{
  return (SDF_ModuleName)t;
}

/*}}}  */
/*{{{  ATerm SDF_ModuleNameToTerm(SDF_ModuleName arg) */

ATerm SDF_ModuleNameToTerm(SDF_ModuleName arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_ImpSection SDF_ImpSectionFromTerm(ATerm t) */

SDF_ImpSection SDF_ImpSectionFromTerm(ATerm t)
{
  return (SDF_ImpSection)t;
}

/*}}}  */
/*{{{  ATerm SDF_ImpSectionToTerm(SDF_ImpSection arg) */

ATerm SDF_ImpSectionToTerm(SDF_ImpSection arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Imports SDF_ImportsFromTerm(ATerm t) */

SDF_Imports SDF_ImportsFromTerm(ATerm t)
{
  return (SDF_Imports)t;
}

/*}}}  */
/*{{{  ATerm SDF_ImportsToTerm(SDF_Imports arg) */

ATerm SDF_ImportsToTerm(SDF_Imports arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Import SDF_ImportFromTerm(ATerm t) */

SDF_Import SDF_ImportFromTerm(ATerm t)
{
  return (SDF_Import)t;
}

/*}}}  */
/*{{{  ATerm SDF_ImportToTerm(SDF_Import arg) */

ATerm SDF_ImportToTerm(SDF_Import arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_ModuleList SDF_ModuleListFromTerm(ATerm t) */

SDF_ModuleList SDF_ModuleListFromTerm(ATerm t)
{
  return (SDF_ModuleList)t;
}

/*}}}  */
/*{{{  ATerm SDF_ModuleListToTerm(SDF_ModuleList arg) */

ATerm SDF_ModuleListToTerm(SDF_ModuleList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_ImpSectionList SDF_ImpSectionListFromTerm(ATerm t) */

SDF_ImpSectionList SDF_ImpSectionListFromTerm(ATerm t)
{
  return (SDF_ImpSectionList)t;
}

/*}}}  */
/*{{{  ATerm SDF_ImpSectionListToTerm(SDF_ImpSectionList arg) */

ATerm SDF_ImpSectionListToTerm(SDF_ImpSectionList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_SectionList SDF_SectionListFromTerm(ATerm t) */

SDF_SectionList SDF_SectionListFromTerm(ATerm t)
{
  return (SDF_SectionList)t;
}

/*}}}  */
/*{{{  ATerm SDF_SectionListToTerm(SDF_SectionList arg) */

ATerm SDF_SectionListToTerm(SDF_SectionList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_ImportList SDF_ImportListFromTerm(ATerm t) */

SDF_ImportList SDF_ImportListFromTerm(ATerm t)
{
  return (SDF_ImportList)t;
}

/*}}}  */
/*{{{  ATerm SDF_ImportListToTerm(SDF_ImportList arg) */

ATerm SDF_ImportListToTerm(SDF_ImportList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Associativity SDF_AssociativityFromTerm(ATerm t) */

SDF_Associativity SDF_AssociativityFromTerm(ATerm t)
{
  return (SDF_Associativity)t;
}

/*}}}  */
/*{{{  ATerm SDF_AssociativityToTerm(SDF_Associativity arg) */

ATerm SDF_AssociativityToTerm(SDF_Associativity arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_ArgumentIndicator SDF_ArgumentIndicatorFromTerm(ATerm t) */

SDF_ArgumentIndicator SDF_ArgumentIndicatorFromTerm(ATerm t)
{
  return (SDF_ArgumentIndicator)t;
}

/*}}}  */
/*{{{  ATerm SDF_ArgumentIndicatorToTerm(SDF_ArgumentIndicator arg) */

ATerm SDF_ArgumentIndicatorToTerm(SDF_ArgumentIndicator arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Group SDF_GroupFromTerm(ATerm t) */

SDF_Group SDF_GroupFromTerm(ATerm t)
{
  return (SDF_Group)t;
}

/*}}}  */
/*{{{  ATerm SDF_GroupToTerm(SDF_Group arg) */

ATerm SDF_GroupToTerm(SDF_Group arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Priority SDF_PriorityFromTerm(ATerm t) */

SDF_Priority SDF_PriorityFromTerm(ATerm t)
{
  return (SDF_Priority)t;
}

/*}}}  */
/*{{{  ATerm SDF_PriorityToTerm(SDF_Priority arg) */

ATerm SDF_PriorityToTerm(SDF_Priority arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Priorities SDF_PrioritiesFromTerm(ATerm t) */

SDF_Priorities SDF_PrioritiesFromTerm(ATerm t)
{
  return (SDF_Priorities)t;
}

/*}}}  */
/*{{{  ATerm SDF_PrioritiesToTerm(SDF_Priorities arg) */

ATerm SDF_PrioritiesToTerm(SDF_Priorities arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_NatConArguments SDF_NatConArgumentsFromTerm(ATerm t) */

SDF_NatConArguments SDF_NatConArgumentsFromTerm(ATerm t)
{
  return (SDF_NatConArguments)t;
}

/*}}}  */
/*{{{  ATerm SDF_NatConArgumentsToTerm(SDF_NatConArguments arg) */

ATerm SDF_NatConArgumentsToTerm(SDF_NatConArguments arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_GroupList SDF_GroupListFromTerm(ATerm t) */

SDF_GroupList SDF_GroupListFromTerm(ATerm t)
{
  return (SDF_GroupList)t;
}

/*}}}  */
/*{{{  ATerm SDF_GroupListToTerm(SDF_GroupList arg) */

ATerm SDF_GroupListToTerm(SDF_GroupList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_PriorityList SDF_PriorityListFromTerm(ATerm t) */

SDF_PriorityList SDF_PriorityListFromTerm(ATerm t)
{
  return (SDF_PriorityList)t;
}

/*}}}  */
/*{{{  ATerm SDF_PriorityListToTerm(SDF_PriorityList arg) */

ATerm SDF_PriorityListToTerm(SDF_PriorityList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_SymbolTail SDF_SymbolTailFromTerm(ATerm t) */

SDF_SymbolTail SDF_SymbolTailFromTerm(ATerm t)
{
  return (SDF_SymbolTail)t;
}

/*}}}  */
/*{{{  ATerm SDF_SymbolTailToTerm(SDF_SymbolTail arg) */

ATerm SDF_SymbolTailToTerm(SDF_SymbolTail arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_SymbolRest SDF_SymbolRestFromTerm(ATerm t) */

SDF_SymbolRest SDF_SymbolRestFromTerm(ATerm t)
{
  return (SDF_SymbolRest)t;
}

/*}}}  */
/*{{{  ATerm SDF_SymbolRestToTerm(SDF_SymbolRest arg) */

ATerm SDF_SymbolRestToTerm(SDF_SymbolRest arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Renamings SDF_RenamingsFromTerm(ATerm t) */

SDF_Renamings SDF_RenamingsFromTerm(ATerm t)
{
  return (SDF_Renamings)t;
}

/*}}}  */
/*{{{  ATerm SDF_RenamingsToTerm(SDF_Renamings arg) */

ATerm SDF_RenamingsToTerm(SDF_Renamings arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Renaming SDF_RenamingFromTerm(ATerm t) */

SDF_Renaming SDF_RenamingFromTerm(ATerm t)
{
  return (SDF_Renaming)t;
}

/*}}}  */
/*{{{  ATerm SDF_RenamingToTerm(SDF_Renaming arg) */

ATerm SDF_RenamingToTerm(SDF_Renaming arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_RenamingList SDF_RenamingListFromTerm(ATerm t) */

SDF_RenamingList SDF_RenamingListFromTerm(ATerm t)
{
  return (SDF_RenamingList)t;
}

/*}}}  */
/*{{{  ATerm SDF_RenamingListToTerm(SDF_RenamingList arg) */

ATerm SDF_RenamingListToTerm(SDF_RenamingList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Lookahead SDF_LookaheadFromTerm(ATerm t) */

SDF_Lookahead SDF_LookaheadFromTerm(ATerm t)
{
  return (SDF_Lookahead)t;
}

/*}}}  */
/*{{{  ATerm SDF_LookaheadToTerm(SDF_Lookahead arg) */

ATerm SDF_LookaheadToTerm(SDF_Lookahead arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_LookaheadsFromTerm(ATerm t) */

SDF_Lookaheads SDF_LookaheadsFromTerm(ATerm t)
{
  return (SDF_Lookaheads)t;
}

/*}}}  */
/*{{{  ATerm SDF_LookaheadsToTerm(SDF_Lookaheads arg) */

ATerm SDF_LookaheadsToTerm(SDF_Lookaheads arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Restriction SDF_RestrictionFromTerm(ATerm t) */

SDF_Restriction SDF_RestrictionFromTerm(ATerm t)
{
  return (SDF_Restriction)t;
}

/*}}}  */
/*{{{  ATerm SDF_RestrictionToTerm(SDF_Restriction arg) */

ATerm SDF_RestrictionToTerm(SDF_Restriction arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Restrictions SDF_RestrictionsFromTerm(ATerm t) */

SDF_Restrictions SDF_RestrictionsFromTerm(ATerm t)
{
  return (SDF_Restrictions)t;
}

/*}}}  */
/*{{{  ATerm SDF_RestrictionsToTerm(SDF_Restrictions arg) */

ATerm SDF_RestrictionsToTerm(SDF_Restrictions arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_LookaheadList SDF_LookaheadListFromTerm(ATerm t) */

SDF_LookaheadList SDF_LookaheadListFromTerm(ATerm t)
{
  return (SDF_LookaheadList)t;
}

/*}}}  */
/*{{{  ATerm SDF_LookaheadListToTerm(SDF_LookaheadList arg) */

ATerm SDF_LookaheadListToTerm(SDF_LookaheadList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_RestrictionList SDF_RestrictionListFromTerm(ATerm t) */

SDF_RestrictionList SDF_RestrictionListFromTerm(ATerm t)
{
  return (SDF_RestrictionList)t;
}

/*}}}  */
/*{{{  ATerm SDF_RestrictionListToTerm(SDF_RestrictionList arg) */

ATerm SDF_RestrictionListToTerm(SDF_RestrictionList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_SDF SDF_SDFFromTerm(ATerm t) */

SDF_SDF SDF_SDFFromTerm(ATerm t)
{
  return (SDF_SDF)t;
}

/*}}}  */
/*{{{  ATerm SDF_SDFToTerm(SDF_SDF arg) */

ATerm SDF_SDFToTerm(SDF_SDF arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Sort SDF_SortFromTerm(ATerm t) */

SDF_Sort SDF_SortFromTerm(ATerm t)
{
  return (SDF_Sort)t;
}

/*}}}  */
/*{{{  ATerm SDF_SortToTerm(SDF_Sort arg) */

ATerm SDF_SortToTerm(SDF_Sort arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_LexSort SDF_LexSortFromTerm(ATerm t) */

SDF_LexSort SDF_LexSortFromTerm(ATerm t)
{
  return (SDF_LexSort)t;
}

/*}}}  */
/*{{{  ATerm SDF_LexSortToTerm(SDF_LexSort arg) */

ATerm SDF_LexSortToTerm(SDF_LexSort arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_SymbolParameters SDF_SymbolParametersFromTerm(ATerm t) */

SDF_SymbolParameters SDF_SymbolParametersFromTerm(ATerm t)
{
  return (SDF_SymbolParameters)t;
}

/*}}}  */
/*{{{  ATerm SDF_SymbolParametersToTerm(SDF_SymbolParameters arg) */

ATerm SDF_SymbolParametersToTerm(SDF_SymbolParameters arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_Symbols SDF_SymbolsFromTerm(ATerm t) */

SDF_Symbols SDF_SymbolsFromTerm(ATerm t)
{
  return (SDF_Symbols)t;
}

/*}}}  */
/*{{{  ATerm SDF_SymbolsToTerm(SDF_Symbols arg) */

ATerm SDF_SymbolsToTerm(SDF_Symbols arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_SymbolList SDF_SymbolListFromTerm(ATerm t) */

SDF_SymbolList SDF_SymbolListFromTerm(ATerm t)
{
  return (SDF_SymbolList)t;
}

/*}}}  */
/*{{{  ATerm SDF_SymbolListToTerm(SDF_SymbolList arg) */

ATerm SDF_SymbolListToTerm(SDF_SymbolList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_LexLayout SDF_LexLayoutFromTerm(ATerm t) */

SDF_LexLayout SDF_LexLayoutFromTerm(ATerm t)
{
  return (SDF_LexLayout)t;
}

/*}}}  */
/*{{{  ATerm SDF_LexLayoutToTerm(SDF_LexLayout arg) */

ATerm SDF_LexLayoutToTerm(SDF_LexLayout arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_LexStrChar SDF_LexStrCharFromTerm(ATerm t) */

SDF_LexStrChar SDF_LexStrCharFromTerm(ATerm t)
{
  return (SDF_LexStrChar)t;
}

/*}}}  */
/*{{{  ATerm SDF_LexStrCharToTerm(SDF_LexStrChar arg) */

ATerm SDF_LexStrCharToTerm(SDF_LexStrChar arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_StrChar SDF_StrCharFromTerm(ATerm t) */

SDF_StrChar SDF_StrCharFromTerm(ATerm t)
{
  return (SDF_StrChar)t;
}

/*}}}  */
/*{{{  ATerm SDF_StrCharToTerm(SDF_StrChar arg) */

ATerm SDF_StrCharToTerm(SDF_StrChar arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_LexStrCon SDF_LexStrConFromTerm(ATerm t) */

SDF_LexStrCon SDF_LexStrConFromTerm(ATerm t)
{
  return (SDF_LexStrCon)t;
}

/*}}}  */
/*{{{  ATerm SDF_LexStrConToTerm(SDF_LexStrCon arg) */

ATerm SDF_LexStrConToTerm(SDF_LexStrCon arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_StrCon SDF_StrConFromTerm(ATerm t) */

SDF_StrCon SDF_StrConFromTerm(ATerm t)
{
  return (SDF_StrCon)t;
}

/*}}}  */
/*{{{  ATerm SDF_StrConToTerm(SDF_StrCon arg) */

ATerm SDF_StrConToTerm(SDF_StrCon arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_LexStrCharChars SDF_LexStrCharCharsFromTerm(ATerm t) */

SDF_LexStrCharChars SDF_LexStrCharCharsFromTerm(ATerm t)
{
  return (SDF_LexStrCharChars)t;
}

/*}}}  */
/*{{{  ATerm SDF_LexStrCharCharsToTerm(SDF_LexStrCharChars arg) */

ATerm SDF_LexStrCharCharsToTerm(SDF_LexStrCharChars arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_LexNatCon SDF_LexNatConFromTerm(ATerm t) */

SDF_LexNatCon SDF_LexNatConFromTerm(ATerm t)
{
  return (SDF_LexNatCon)t;
}

/*}}}  */
/*{{{  ATerm SDF_LexNatConToTerm(SDF_LexNatCon arg) */

ATerm SDF_LexNatConToTerm(SDF_LexNatCon arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_NatCon SDF_NatConFromTerm(ATerm t) */

SDF_NatCon SDF_NatConFromTerm(ATerm t)
{
  return (SDF_NatCon)t;
}

/*}}}  */
/*{{{  ATerm SDF_NatConToTerm(SDF_NatCon arg) */

ATerm SDF_NatConToTerm(SDF_NatCon arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_LexIdCon SDF_LexIdConFromTerm(ATerm t) */

SDF_LexIdCon SDF_LexIdConFromTerm(ATerm t)
{
  return (SDF_LexIdCon)t;
}

/*}}}  */
/*{{{  ATerm SDF_LexIdConToTerm(SDF_LexIdCon arg) */

ATerm SDF_LexIdConToTerm(SDF_LexIdCon arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  SDF_IdCon SDF_IdConFromTerm(ATerm t) */

SDF_IdCon SDF_IdConFromTerm(ATerm t)
{
  return (SDF_IdCon)t;
}

/*}}}  */
/*{{{  ATerm SDF_IdConToTerm(SDF_IdCon arg) */

ATerm SDF_IdConToTerm(SDF_IdCon arg)
{
  return (ATerm)arg;
}

/*}}}  */

/*}}}  */
/*{{{  list functions */

int SDF_getLexLayoutListLength (SDF_LexLayoutList arg) {
  return ATgetLength((ATermList) arg);
}
SDF_LexLayoutList SDF_reverseLexLayoutList(SDF_LexLayoutList arg) {
  return (SDF_LexLayoutList) ATreverse((ATermList) arg);
}
SDF_LexLayoutList SDF_appendLexLayoutList(SDF_LexLayoutList arg, SDF_LexLayout elem) {
  return (SDF_LexLayoutList) ATappend((ATermList) arg, (ATerm) ((ATerm) elem));
}
SDF_LexLayoutList SDF_concatLexLayoutList(SDF_LexLayoutList arg0, SDF_LexLayoutList arg1) {
  return (SDF_LexLayoutList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_LexLayoutList SDF_sliceLexLayoutList(SDF_LexLayoutList arg, int start, int end) {
  return (SDF_LexLayoutList) ATgetSlice((ATermList) arg, start, end);
}
SDF_LexLayout SDF_getLexLayoutListLexLayoutAt(SDF_LexLayoutList arg, int index) {
 return (SDF_LexLayout)ATelementAt((ATermList) arg,index);
}
SDF_LexLayoutList SDF_replaceLexLayoutListLexLayoutAt(SDF_LexLayoutList arg, SDF_LexLayout elem, int index) {
 return (SDF_LexLayoutList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index);
}
SDF_LexLayoutList SDF_makeLexLayoutList2(SDF_LexLayout elem1, SDF_LexLayout elem2) {
  return (SDF_LexLayoutList) ATmakeList2((ATerm) ((ATerm) elem2), (ATerm) ((ATerm) elem2));
}
SDF_LexLayoutList SDF_makeLexLayoutList3(SDF_LexLayout elem1, SDF_LexLayout elem2, SDF_LexLayout elem3) {
  return (SDF_LexLayoutList) ATmakeList3((ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3));
}
SDF_LexLayoutList SDF_makeLexLayoutList4(SDF_LexLayout elem1, SDF_LexLayout elem2, SDF_LexLayout elem3, SDF_LexLayout elem4) {
  return (SDF_LexLayoutList) ATmakeList4((ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4));
}
SDF_LexLayoutList SDF_makeLexLayoutList5(SDF_LexLayout elem1, SDF_LexLayout elem2, SDF_LexLayout elem3, SDF_LexLayout elem4, SDF_LexLayout elem5) {
  return (SDF_LexLayoutList) ATmakeList5((ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5));
}
SDF_LexLayoutList SDF_makeLexLayoutList6(SDF_LexLayout elem1, SDF_LexLayout elem2, SDF_LexLayout elem3, SDF_LexLayout elem4, SDF_LexLayout elem5, SDF_LexLayout elem6) {
  return (SDF_LexLayoutList) ATmakeList6((ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6));
}
int SDF_getATermArgsLength (SDF_ATermArgs arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 4) + 1;
}
SDF_ATermArgs SDF_reverseATermArgs(SDF_ATermArgs arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATerm sep2;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep2);
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_ATermArgs) result;
}
SDF_ATermArgs SDF_appendATermArgs(SDF_ATermArgs arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm arg1) {
  return SDF_concatATermArgs(arg0, wsAfterHead, wsAfterSep, SDF_makeATermArgsSingle(arg1));
}
SDF_ATermArgs SDF_concatATermArgs(SDF_ATermArgs arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATermArgs arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeATermArgsMany((SDF_ATerm)ATgetFirst((ATermList) arg0), wsAfterHead, wsAfterSep,  arg1);
  arg1 = (SDF_ATermArgs) ATgetNext((ATermList) arg1);
  return (SDF_ATermArgs) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_ATermArgs SDF_sliceATermArgs(SDF_ATermArgs arg, int start, int end) {
  return (SDF_ATermArgs) ATgetSlice((ATermList) arg, start * 4, end * 4);
}
SDF_ATerm SDF_getATermArgsATermAt(SDF_ATermArgs arg, int index) {
 return (SDF_ATerm)ATelementAt((ATermList) arg,index * 4);
}
SDF_ATermArgs SDF_replaceATermArgsATermAt(SDF_ATermArgs arg, SDF_ATerm elem, int index) {
 return (SDF_ATermArgs) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 4);
}
SDF_ATermArgs SDF_makeATermArgs2(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm elem1, SDF_ATerm elem2) {
  return SDF_makeATermArgsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeATermArgsSingle(elem2));
}
SDF_ATermArgs SDF_makeATermArgs3(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm elem1, SDF_ATerm elem2, SDF_ATerm elem3) {
  return SDF_makeATermArgsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeATermArgs2(wsAfterHead, wsAfterSep, elem2, elem3));
}
SDF_ATermArgs SDF_makeATermArgs4(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm elem1, SDF_ATerm elem2, SDF_ATerm elem3, SDF_ATerm elem4) {
  return SDF_makeATermArgsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeATermArgs3(wsAfterHead, wsAfterSep, elem2, elem3, elem4));
}
SDF_ATermArgs SDF_makeATermArgs5(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm elem1, SDF_ATerm elem2, SDF_ATerm elem3, SDF_ATerm elem4, SDF_ATerm elem5) {
  return SDF_makeATermArgsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeATermArgs4(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5));
}
SDF_ATermArgs SDF_makeATermArgs6(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm elem1, SDF_ATerm elem2, SDF_ATerm elem3, SDF_ATerm elem4, SDF_ATerm elem5, SDF_ATerm elem6) {
  return SDF_makeATermArgsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeATermArgs5(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getATermElemsLength (SDF_ATermElems arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 4) + 1;
}
SDF_ATermElems SDF_reverseATermElems(SDF_ATermElems arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATerm sep2;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep2);
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_ATermElems) result;
}
SDF_ATermElems SDF_appendATermElems(SDF_ATermElems arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm arg1) {
  return SDF_concatATermElems(arg0, wsAfterHead, wsAfterSep, SDF_makeATermElemsSingle(arg1));
}
SDF_ATermElems SDF_concatATermElems(SDF_ATermElems arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATermElems arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeATermElemsMany((SDF_ATerm)ATgetFirst((ATermList) arg0), wsAfterHead, wsAfterSep,  arg1);
  arg1 = (SDF_ATermElems) ATgetNext((ATermList) arg1);
  return (SDF_ATermElems) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_ATermElems SDF_sliceATermElems(SDF_ATermElems arg, int start, int end) {
  return (SDF_ATermElems) ATgetSlice((ATermList) arg, start * 4, end * 4);
}
SDF_ATerm SDF_getATermElemsATermAt(SDF_ATermElems arg, int index) {
 return (SDF_ATerm)ATelementAt((ATermList) arg,index * 4);
}
SDF_ATermElems SDF_replaceATermElemsATermAt(SDF_ATermElems arg, SDF_ATerm elem, int index) {
 return (SDF_ATermElems) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 4);
}
SDF_ATermElems SDF_makeATermElems2(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm elem1, SDF_ATerm elem2) {
  return SDF_makeATermElemsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeATermElemsSingle(elem2));
}
SDF_ATermElems SDF_makeATermElems3(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm elem1, SDF_ATerm elem2, SDF_ATerm elem3) {
  return SDF_makeATermElemsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeATermElems2(wsAfterHead, wsAfterSep, elem2, elem3));
}
SDF_ATermElems SDF_makeATermElems4(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm elem1, SDF_ATerm elem2, SDF_ATerm elem3, SDF_ATerm elem4) {
  return SDF_makeATermElemsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeATermElems3(wsAfterHead, wsAfterSep, elem2, elem3, elem4));
}
SDF_ATermElems SDF_makeATermElems5(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm elem1, SDF_ATerm elem2, SDF_ATerm elem3, SDF_ATerm elem4, SDF_ATerm elem5) {
  return SDF_makeATermElemsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeATermElems4(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5));
}
SDF_ATermElems SDF_makeATermElems6(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm elem1, SDF_ATerm elem2, SDF_ATerm elem3, SDF_ATerm elem4, SDF_ATerm elem5, SDF_ATerm elem6) {
  return SDF_makeATermElemsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeATermElems5(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getATermAnnosLength (SDF_ATermAnnos arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 4) + 1;
}
SDF_ATermAnnos SDF_reverseATermAnnos(SDF_ATermAnnos arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATerm sep2;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep2);
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_ATermAnnos) result;
}
SDF_ATermAnnos SDF_appendATermAnnos(SDF_ATermAnnos arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm arg1) {
  return SDF_concatATermAnnos(arg0, wsAfterHead, wsAfterSep, SDF_makeATermAnnosSingle(arg1));
}
SDF_ATermAnnos SDF_concatATermAnnos(SDF_ATermAnnos arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATermAnnos arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeATermAnnosMany((SDF_ATerm)ATgetFirst((ATermList) arg0), wsAfterHead, wsAfterSep,  arg1);
  arg1 = (SDF_ATermAnnos) ATgetNext((ATermList) arg1);
  return (SDF_ATermAnnos) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_ATermAnnos SDF_sliceATermAnnos(SDF_ATermAnnos arg, int start, int end) {
  return (SDF_ATermAnnos) ATgetSlice((ATermList) arg, start * 4, end * 4);
}
SDF_ATerm SDF_getATermAnnosATermAt(SDF_ATermAnnos arg, int index) {
 return (SDF_ATerm)ATelementAt((ATermList) arg,index * 4);
}
SDF_ATermAnnos SDF_replaceATermAnnosATermAt(SDF_ATermAnnos arg, SDF_ATerm elem, int index) {
 return (SDF_ATermAnnos) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 4);
}
SDF_ATermAnnos SDF_makeATermAnnos2(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm elem1, SDF_ATerm elem2) {
  return SDF_makeATermAnnosMany(elem1, wsAfterHead, wsAfterSep, SDF_makeATermAnnosSingle(elem2));
}
SDF_ATermAnnos SDF_makeATermAnnos3(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm elem1, SDF_ATerm elem2, SDF_ATerm elem3) {
  return SDF_makeATermAnnosMany(elem1, wsAfterHead, wsAfterSep, SDF_makeATermAnnos2(wsAfterHead, wsAfterSep, elem2, elem3));
}
SDF_ATermAnnos SDF_makeATermAnnos4(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm elem1, SDF_ATerm elem2, SDF_ATerm elem3, SDF_ATerm elem4) {
  return SDF_makeATermAnnosMany(elem1, wsAfterHead, wsAfterSep, SDF_makeATermAnnos3(wsAfterHead, wsAfterSep, elem2, elem3, elem4));
}
SDF_ATermAnnos SDF_makeATermAnnos5(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm elem1, SDF_ATerm elem2, SDF_ATerm elem3, SDF_ATerm elem4, SDF_ATerm elem5) {
  return SDF_makeATermAnnosMany(elem1, wsAfterHead, wsAfterSep, SDF_makeATermAnnos4(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5));
}
SDF_ATermAnnos SDF_makeATermAnnos6(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATerm elem1, SDF_ATerm elem2, SDF_ATerm elem3, SDF_ATerm elem4, SDF_ATerm elem5, SDF_ATerm elem6) {
  return SDF_makeATermAnnosMany(elem1, wsAfterHead, wsAfterSep, SDF_makeATermAnnos5(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getAliasListLength (SDF_AliasList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 2) + 1;
}
SDF_AliasList SDF_reverseAliasList(SDF_AliasList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_AliasList) result;
}
SDF_AliasList SDF_appendAliasList(SDF_AliasList arg0, SDF_OptLayout wsAfterHead, SDF_Alias arg1) {
  return SDF_concatAliasList(arg0, wsAfterHead, SDF_makeAliasListSingle(arg1));
}
SDF_AliasList SDF_concatAliasList(SDF_AliasList arg0, SDF_OptLayout wsAfterHead, SDF_AliasList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeAliasListMany((SDF_Alias)ATgetFirst((ATermList) arg0), wsAfterHead,  arg1);
  arg1 = (SDF_AliasList) ATgetNext((ATermList) arg1);
  return (SDF_AliasList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_AliasList SDF_sliceAliasList(SDF_AliasList arg, int start, int end) {
  return (SDF_AliasList) ATgetSlice((ATermList) arg, start * 2, end * 2);
}
SDF_Alias SDF_getAliasListAliasAt(SDF_AliasList arg, int index) {
 return (SDF_Alias)ATelementAt((ATermList) arg,index * 2);
}
SDF_AliasList SDF_replaceAliasListAliasAt(SDF_AliasList arg, SDF_Alias elem, int index) {
 return (SDF_AliasList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 2);
}
SDF_AliasList SDF_makeAliasList2(SDF_OptLayout wsAfterHead, SDF_Alias elem1, SDF_Alias elem2) {
  return SDF_makeAliasListMany(elem1, wsAfterHead, SDF_makeAliasListSingle(elem2));
}
SDF_AliasList SDF_makeAliasList3(SDF_OptLayout wsAfterHead, SDF_Alias elem1, SDF_Alias elem2, SDF_Alias elem3) {
  return SDF_makeAliasListMany(elem1, wsAfterHead, SDF_makeAliasList2(wsAfterHead, elem2, elem3));
}
SDF_AliasList SDF_makeAliasList4(SDF_OptLayout wsAfterHead, SDF_Alias elem1, SDF_Alias elem2, SDF_Alias elem3, SDF_Alias elem4) {
  return SDF_makeAliasListMany(elem1, wsAfterHead, SDF_makeAliasList3(wsAfterHead, elem2, elem3, elem4));
}
SDF_AliasList SDF_makeAliasList5(SDF_OptLayout wsAfterHead, SDF_Alias elem1, SDF_Alias elem2, SDF_Alias elem3, SDF_Alias elem4, SDF_Alias elem5) {
  return SDF_makeAliasListMany(elem1, wsAfterHead, SDF_makeAliasList4(wsAfterHead, elem2, elem3, elem4, elem5));
}
SDF_AliasList SDF_makeAliasList6(SDF_OptLayout wsAfterHead, SDF_Alias elem1, SDF_Alias elem2, SDF_Alias elem3, SDF_Alias elem4, SDF_Alias elem5, SDF_Alias elem6) {
  return SDF_makeAliasListMany(elem1, wsAfterHead, SDF_makeAliasList5(wsAfterHead, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getAttributeListLength (SDF_AttributeList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 4) + 1;
}
SDF_AttributeList SDF_reverseAttributeList(SDF_AttributeList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATerm sep2;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep2);
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_AttributeList) result;
}
SDF_AttributeList SDF_appendAttributeList(SDF_AttributeList arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Attribute arg1) {
  return SDF_concatAttributeList(arg0, wsAfterHead, wsAfterSep, SDF_makeAttributeListSingle(arg1));
}
SDF_AttributeList SDF_concatAttributeList(SDF_AttributeList arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_AttributeList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeAttributeListMany((SDF_Attribute)ATgetFirst((ATermList) arg0), wsAfterHead, wsAfterSep,  arg1);
  arg1 = (SDF_AttributeList) ATgetNext((ATermList) arg1);
  return (SDF_AttributeList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_AttributeList SDF_sliceAttributeList(SDF_AttributeList arg, int start, int end) {
  return (SDF_AttributeList) ATgetSlice((ATermList) arg, start * 4, end * 4);
}
SDF_Attribute SDF_getAttributeListAttributeAt(SDF_AttributeList arg, int index) {
 return (SDF_Attribute)ATelementAt((ATermList) arg,index * 4);
}
SDF_AttributeList SDF_replaceAttributeListAttributeAt(SDF_AttributeList arg, SDF_Attribute elem, int index) {
 return (SDF_AttributeList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 4);
}
SDF_AttributeList SDF_makeAttributeList2(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Attribute elem1, SDF_Attribute elem2) {
  return SDF_makeAttributeListMany(elem1, wsAfterHead, wsAfterSep, SDF_makeAttributeListSingle(elem2));
}
SDF_AttributeList SDF_makeAttributeList3(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Attribute elem1, SDF_Attribute elem2, SDF_Attribute elem3) {
  return SDF_makeAttributeListMany(elem1, wsAfterHead, wsAfterSep, SDF_makeAttributeList2(wsAfterHead, wsAfterSep, elem2, elem3));
}
SDF_AttributeList SDF_makeAttributeList4(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Attribute elem1, SDF_Attribute elem2, SDF_Attribute elem3, SDF_Attribute elem4) {
  return SDF_makeAttributeListMany(elem1, wsAfterHead, wsAfterSep, SDF_makeAttributeList3(wsAfterHead, wsAfterSep, elem2, elem3, elem4));
}
SDF_AttributeList SDF_makeAttributeList5(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Attribute elem1, SDF_Attribute elem2, SDF_Attribute elem3, SDF_Attribute elem4, SDF_Attribute elem5) {
  return SDF_makeAttributeListMany(elem1, wsAfterHead, wsAfterSep, SDF_makeAttributeList4(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5));
}
SDF_AttributeList SDF_makeAttributeList6(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Attribute elem1, SDF_Attribute elem2, SDF_Attribute elem3, SDF_Attribute elem4, SDF_Attribute elem5, SDF_Attribute elem6) {
  return SDF_makeAttributeListMany(elem1, wsAfterHead, wsAfterSep, SDF_makeAttributeList5(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getProductionListLength (SDF_ProductionList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 2) + 1;
}
SDF_ProductionList SDF_reverseProductionList(SDF_ProductionList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_ProductionList) result;
}
SDF_ProductionList SDF_appendProductionList(SDF_ProductionList arg0, SDF_OptLayout wsAfterHead, SDF_Production arg1) {
  return SDF_concatProductionList(arg0, wsAfterHead, SDF_makeProductionListSingle(arg1));
}
SDF_ProductionList SDF_concatProductionList(SDF_ProductionList arg0, SDF_OptLayout wsAfterHead, SDF_ProductionList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeProductionListMany((SDF_Production)ATgetFirst((ATermList) arg0), wsAfterHead,  arg1);
  arg1 = (SDF_ProductionList) ATgetNext((ATermList) arg1);
  return (SDF_ProductionList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_ProductionList SDF_sliceProductionList(SDF_ProductionList arg, int start, int end) {
  return (SDF_ProductionList) ATgetSlice((ATermList) arg, start * 2, end * 2);
}
SDF_Production SDF_getProductionListProductionAt(SDF_ProductionList arg, int index) {
 return (SDF_Production)ATelementAt((ATermList) arg,index * 2);
}
SDF_ProductionList SDF_replaceProductionListProductionAt(SDF_ProductionList arg, SDF_Production elem, int index) {
 return (SDF_ProductionList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 2);
}
SDF_ProductionList SDF_makeProductionList2(SDF_OptLayout wsAfterHead, SDF_Production elem1, SDF_Production elem2) {
  return SDF_makeProductionListMany(elem1, wsAfterHead, SDF_makeProductionListSingle(elem2));
}
SDF_ProductionList SDF_makeProductionList3(SDF_OptLayout wsAfterHead, SDF_Production elem1, SDF_Production elem2, SDF_Production elem3) {
  return SDF_makeProductionListMany(elem1, wsAfterHead, SDF_makeProductionList2(wsAfterHead, elem2, elem3));
}
SDF_ProductionList SDF_makeProductionList4(SDF_OptLayout wsAfterHead, SDF_Production elem1, SDF_Production elem2, SDF_Production elem3, SDF_Production elem4) {
  return SDF_makeProductionListMany(elem1, wsAfterHead, SDF_makeProductionList3(wsAfterHead, elem2, elem3, elem4));
}
SDF_ProductionList SDF_makeProductionList5(SDF_OptLayout wsAfterHead, SDF_Production elem1, SDF_Production elem2, SDF_Production elem3, SDF_Production elem4, SDF_Production elem5) {
  return SDF_makeProductionListMany(elem1, wsAfterHead, SDF_makeProductionList4(wsAfterHead, elem2, elem3, elem4, elem5));
}
SDF_ProductionList SDF_makeProductionList6(SDF_OptLayout wsAfterHead, SDF_Production elem1, SDF_Production elem2, SDF_Production elem3, SDF_Production elem4, SDF_Production elem5, SDF_Production elem6) {
  return SDF_makeProductionListMany(elem1, wsAfterHead, SDF_makeProductionList5(wsAfterHead, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getSymbolArgumentsLength (SDF_SymbolArguments arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 4) + 1;
}
SDF_SymbolArguments SDF_reverseSymbolArguments(SDF_SymbolArguments arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATerm sep2;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep2);
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_SymbolArguments) result;
}
SDF_SymbolArguments SDF_appendSymbolArguments(SDF_SymbolArguments arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol arg1) {
  return SDF_concatSymbolArguments(arg0, wsAfterHead, wsAfterSep, SDF_makeSymbolArgumentsSingle(arg1));
}
SDF_SymbolArguments SDF_concatSymbolArguments(SDF_SymbolArguments arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_SymbolArguments arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeSymbolArgumentsMany((SDF_Symbol)ATgetFirst((ATermList) arg0), wsAfterHead, wsAfterSep,  arg1);
  arg1 = (SDF_SymbolArguments) ATgetNext((ATermList) arg1);
  return (SDF_SymbolArguments) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_SymbolArguments SDF_sliceSymbolArguments(SDF_SymbolArguments arg, int start, int end) {
  return (SDF_SymbolArguments) ATgetSlice((ATermList) arg, start * 4, end * 4);
}
SDF_Symbol SDF_getSymbolArgumentsSymbolAt(SDF_SymbolArguments arg, int index) {
 return (SDF_Symbol)ATelementAt((ATermList) arg,index * 4);
}
SDF_SymbolArguments SDF_replaceSymbolArgumentsSymbolAt(SDF_SymbolArguments arg, SDF_Symbol elem, int index) {
 return (SDF_SymbolArguments) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 4);
}
SDF_SymbolArguments SDF_makeSymbolArguments2(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol elem1, SDF_Symbol elem2) {
  return SDF_makeSymbolArgumentsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeSymbolArgumentsSingle(elem2));
}
SDF_SymbolArguments SDF_makeSymbolArguments3(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3) {
  return SDF_makeSymbolArgumentsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeSymbolArguments2(wsAfterHead, wsAfterSep, elem2, elem3));
}
SDF_SymbolArguments SDF_makeSymbolArguments4(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3, SDF_Symbol elem4) {
  return SDF_makeSymbolArgumentsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeSymbolArguments3(wsAfterHead, wsAfterSep, elem2, elem3, elem4));
}
SDF_SymbolArguments SDF_makeSymbolArguments5(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3, SDF_Symbol elem4, SDF_Symbol elem5) {
  return SDF_makeSymbolArgumentsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeSymbolArguments4(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5));
}
SDF_SymbolArguments SDF_makeSymbolArguments6(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3, SDF_Symbol elem4, SDF_Symbol elem5, SDF_Symbol elem6) {
  return SDF_makeSymbolArgumentsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeSymbolArguments5(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getModuleListLength (SDF_ModuleList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 2) + 1;
}
SDF_ModuleList SDF_reverseModuleList(SDF_ModuleList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_ModuleList) result;
}
SDF_ModuleList SDF_appendModuleList(SDF_ModuleList arg0, SDF_OptLayout wsAfterHead, SDF_Module arg1) {
  return SDF_concatModuleList(arg0, wsAfterHead, SDF_makeModuleListSingle(arg1));
}
SDF_ModuleList SDF_concatModuleList(SDF_ModuleList arg0, SDF_OptLayout wsAfterHead, SDF_ModuleList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeModuleListMany((SDF_Module)ATgetFirst((ATermList) arg0), wsAfterHead,  arg1);
  arg1 = (SDF_ModuleList) ATgetNext((ATermList) arg1);
  return (SDF_ModuleList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_ModuleList SDF_sliceModuleList(SDF_ModuleList arg, int start, int end) {
  return (SDF_ModuleList) ATgetSlice((ATermList) arg, start * 2, end * 2);
}
SDF_Module SDF_getModuleListModuleAt(SDF_ModuleList arg, int index) {
 return (SDF_Module)ATelementAt((ATermList) arg,index * 2);
}
SDF_ModuleList SDF_replaceModuleListModuleAt(SDF_ModuleList arg, SDF_Module elem, int index) {
 return (SDF_ModuleList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 2);
}
SDF_ModuleList SDF_makeModuleList2(SDF_OptLayout wsAfterHead, SDF_Module elem1, SDF_Module elem2) {
  return SDF_makeModuleListMany(elem1, wsAfterHead, SDF_makeModuleListSingle(elem2));
}
SDF_ModuleList SDF_makeModuleList3(SDF_OptLayout wsAfterHead, SDF_Module elem1, SDF_Module elem2, SDF_Module elem3) {
  return SDF_makeModuleListMany(elem1, wsAfterHead, SDF_makeModuleList2(wsAfterHead, elem2, elem3));
}
SDF_ModuleList SDF_makeModuleList4(SDF_OptLayout wsAfterHead, SDF_Module elem1, SDF_Module elem2, SDF_Module elem3, SDF_Module elem4) {
  return SDF_makeModuleListMany(elem1, wsAfterHead, SDF_makeModuleList3(wsAfterHead, elem2, elem3, elem4));
}
SDF_ModuleList SDF_makeModuleList5(SDF_OptLayout wsAfterHead, SDF_Module elem1, SDF_Module elem2, SDF_Module elem3, SDF_Module elem4, SDF_Module elem5) {
  return SDF_makeModuleListMany(elem1, wsAfterHead, SDF_makeModuleList4(wsAfterHead, elem2, elem3, elem4, elem5));
}
SDF_ModuleList SDF_makeModuleList6(SDF_OptLayout wsAfterHead, SDF_Module elem1, SDF_Module elem2, SDF_Module elem3, SDF_Module elem4, SDF_Module elem5, SDF_Module elem6) {
  return SDF_makeModuleListMany(elem1, wsAfterHead, SDF_makeModuleList5(wsAfterHead, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getImpSectionListLength (SDF_ImpSectionList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 2) + 1;
}
SDF_ImpSectionList SDF_reverseImpSectionList(SDF_ImpSectionList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_ImpSectionList) result;
}
SDF_ImpSectionList SDF_appendImpSectionList(SDF_ImpSectionList arg0, SDF_OptLayout wsAfterHead, SDF_ImpSection arg1) {
  return SDF_concatImpSectionList(arg0, wsAfterHead, SDF_makeImpSectionListSingle(arg1));
}
SDF_ImpSectionList SDF_concatImpSectionList(SDF_ImpSectionList arg0, SDF_OptLayout wsAfterHead, SDF_ImpSectionList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeImpSectionListMany((SDF_ImpSection)ATgetFirst((ATermList) arg0), wsAfterHead,  arg1);
  arg1 = (SDF_ImpSectionList) ATgetNext((ATermList) arg1);
  return (SDF_ImpSectionList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_ImpSectionList SDF_sliceImpSectionList(SDF_ImpSectionList arg, int start, int end) {
  return (SDF_ImpSectionList) ATgetSlice((ATermList) arg, start * 2, end * 2);
}
SDF_ImpSection SDF_getImpSectionListImpSectionAt(SDF_ImpSectionList arg, int index) {
 return (SDF_ImpSection)ATelementAt((ATermList) arg,index * 2);
}
SDF_ImpSectionList SDF_replaceImpSectionListImpSectionAt(SDF_ImpSectionList arg, SDF_ImpSection elem, int index) {
 return (SDF_ImpSectionList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 2);
}
SDF_ImpSectionList SDF_makeImpSectionList2(SDF_OptLayout wsAfterHead, SDF_ImpSection elem1, SDF_ImpSection elem2) {
  return SDF_makeImpSectionListMany(elem1, wsAfterHead, SDF_makeImpSectionListSingle(elem2));
}
SDF_ImpSectionList SDF_makeImpSectionList3(SDF_OptLayout wsAfterHead, SDF_ImpSection elem1, SDF_ImpSection elem2, SDF_ImpSection elem3) {
  return SDF_makeImpSectionListMany(elem1, wsAfterHead, SDF_makeImpSectionList2(wsAfterHead, elem2, elem3));
}
SDF_ImpSectionList SDF_makeImpSectionList4(SDF_OptLayout wsAfterHead, SDF_ImpSection elem1, SDF_ImpSection elem2, SDF_ImpSection elem3, SDF_ImpSection elem4) {
  return SDF_makeImpSectionListMany(elem1, wsAfterHead, SDF_makeImpSectionList3(wsAfterHead, elem2, elem3, elem4));
}
SDF_ImpSectionList SDF_makeImpSectionList5(SDF_OptLayout wsAfterHead, SDF_ImpSection elem1, SDF_ImpSection elem2, SDF_ImpSection elem3, SDF_ImpSection elem4, SDF_ImpSection elem5) {
  return SDF_makeImpSectionListMany(elem1, wsAfterHead, SDF_makeImpSectionList4(wsAfterHead, elem2, elem3, elem4, elem5));
}
SDF_ImpSectionList SDF_makeImpSectionList6(SDF_OptLayout wsAfterHead, SDF_ImpSection elem1, SDF_ImpSection elem2, SDF_ImpSection elem3, SDF_ImpSection elem4, SDF_ImpSection elem5, SDF_ImpSection elem6) {
  return SDF_makeImpSectionListMany(elem1, wsAfterHead, SDF_makeImpSectionList5(wsAfterHead, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getSectionListLength (SDF_SectionList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 2) + 1;
}
SDF_SectionList SDF_reverseSectionList(SDF_SectionList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_SectionList) result;
}
SDF_SectionList SDF_appendSectionList(SDF_SectionList arg0, SDF_OptLayout wsAfterHead, SDF_Section arg1) {
  return SDF_concatSectionList(arg0, wsAfterHead, SDF_makeSectionListSingle(arg1));
}
SDF_SectionList SDF_concatSectionList(SDF_SectionList arg0, SDF_OptLayout wsAfterHead, SDF_SectionList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeSectionListMany((SDF_Section)ATgetFirst((ATermList) arg0), wsAfterHead,  arg1);
  arg1 = (SDF_SectionList) ATgetNext((ATermList) arg1);
  return (SDF_SectionList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_SectionList SDF_sliceSectionList(SDF_SectionList arg, int start, int end) {
  return (SDF_SectionList) ATgetSlice((ATermList) arg, start * 2, end * 2);
}
SDF_Section SDF_getSectionListSectionAt(SDF_SectionList arg, int index) {
 return (SDF_Section)ATelementAt((ATermList) arg,index * 2);
}
SDF_SectionList SDF_replaceSectionListSectionAt(SDF_SectionList arg, SDF_Section elem, int index) {
 return (SDF_SectionList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 2);
}
SDF_SectionList SDF_makeSectionList2(SDF_OptLayout wsAfterHead, SDF_Section elem1, SDF_Section elem2) {
  return SDF_makeSectionListMany(elem1, wsAfterHead, SDF_makeSectionListSingle(elem2));
}
SDF_SectionList SDF_makeSectionList3(SDF_OptLayout wsAfterHead, SDF_Section elem1, SDF_Section elem2, SDF_Section elem3) {
  return SDF_makeSectionListMany(elem1, wsAfterHead, SDF_makeSectionList2(wsAfterHead, elem2, elem3));
}
SDF_SectionList SDF_makeSectionList4(SDF_OptLayout wsAfterHead, SDF_Section elem1, SDF_Section elem2, SDF_Section elem3, SDF_Section elem4) {
  return SDF_makeSectionListMany(elem1, wsAfterHead, SDF_makeSectionList3(wsAfterHead, elem2, elem3, elem4));
}
SDF_SectionList SDF_makeSectionList5(SDF_OptLayout wsAfterHead, SDF_Section elem1, SDF_Section elem2, SDF_Section elem3, SDF_Section elem4, SDF_Section elem5) {
  return SDF_makeSectionListMany(elem1, wsAfterHead, SDF_makeSectionList4(wsAfterHead, elem2, elem3, elem4, elem5));
}
SDF_SectionList SDF_makeSectionList6(SDF_OptLayout wsAfterHead, SDF_Section elem1, SDF_Section elem2, SDF_Section elem3, SDF_Section elem4, SDF_Section elem5, SDF_Section elem6) {
  return SDF_makeSectionListMany(elem1, wsAfterHead, SDF_makeSectionList5(wsAfterHead, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getImportListLength (SDF_ImportList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 2) + 1;
}
SDF_ImportList SDF_reverseImportList(SDF_ImportList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_ImportList) result;
}
SDF_ImportList SDF_appendImportList(SDF_ImportList arg0, SDF_OptLayout wsAfterHead, SDF_Import arg1) {
  return SDF_concatImportList(arg0, wsAfterHead, SDF_makeImportListSingle(arg1));
}
SDF_ImportList SDF_concatImportList(SDF_ImportList arg0, SDF_OptLayout wsAfterHead, SDF_ImportList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeImportListMany((SDF_Import)ATgetFirst((ATermList) arg0), wsAfterHead,  arg1);
  arg1 = (SDF_ImportList) ATgetNext((ATermList) arg1);
  return (SDF_ImportList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_ImportList SDF_sliceImportList(SDF_ImportList arg, int start, int end) {
  return (SDF_ImportList) ATgetSlice((ATermList) arg, start * 2, end * 2);
}
SDF_Import SDF_getImportListImportAt(SDF_ImportList arg, int index) {
 return (SDF_Import)ATelementAt((ATermList) arg,index * 2);
}
SDF_ImportList SDF_replaceImportListImportAt(SDF_ImportList arg, SDF_Import elem, int index) {
 return (SDF_ImportList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 2);
}
SDF_ImportList SDF_makeImportList2(SDF_OptLayout wsAfterHead, SDF_Import elem1, SDF_Import elem2) {
  return SDF_makeImportListMany(elem1, wsAfterHead, SDF_makeImportListSingle(elem2));
}
SDF_ImportList SDF_makeImportList3(SDF_OptLayout wsAfterHead, SDF_Import elem1, SDF_Import elem2, SDF_Import elem3) {
  return SDF_makeImportListMany(elem1, wsAfterHead, SDF_makeImportList2(wsAfterHead, elem2, elem3));
}
SDF_ImportList SDF_makeImportList4(SDF_OptLayout wsAfterHead, SDF_Import elem1, SDF_Import elem2, SDF_Import elem3, SDF_Import elem4) {
  return SDF_makeImportListMany(elem1, wsAfterHead, SDF_makeImportList3(wsAfterHead, elem2, elem3, elem4));
}
SDF_ImportList SDF_makeImportList5(SDF_OptLayout wsAfterHead, SDF_Import elem1, SDF_Import elem2, SDF_Import elem3, SDF_Import elem4, SDF_Import elem5) {
  return SDF_makeImportListMany(elem1, wsAfterHead, SDF_makeImportList4(wsAfterHead, elem2, elem3, elem4, elem5));
}
SDF_ImportList SDF_makeImportList6(SDF_OptLayout wsAfterHead, SDF_Import elem1, SDF_Import elem2, SDF_Import elem3, SDF_Import elem4, SDF_Import elem5, SDF_Import elem6) {
  return SDF_makeImportListMany(elem1, wsAfterHead, SDF_makeImportList5(wsAfterHead, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getNatConArgumentsLength (SDF_NatConArguments arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 4) + 1;
}
SDF_NatConArguments SDF_reverseNatConArguments(SDF_NatConArguments arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATerm sep2;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep2);
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_NatConArguments) result;
}
SDF_NatConArguments SDF_appendNatConArguments(SDF_NatConArguments arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_NatCon arg1) {
  return SDF_concatNatConArguments(arg0, wsAfterHead, wsAfterSep, SDF_makeNatConArgumentsSingle(arg1));
}
SDF_NatConArguments SDF_concatNatConArguments(SDF_NatConArguments arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_NatConArguments arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeNatConArgumentsMany((SDF_NatCon)ATgetFirst((ATermList) arg0), wsAfterHead, wsAfterSep,  arg1);
  arg1 = (SDF_NatConArguments) ATgetNext((ATermList) arg1);
  return (SDF_NatConArguments) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_NatConArguments SDF_sliceNatConArguments(SDF_NatConArguments arg, int start, int end) {
  return (SDF_NatConArguments) ATgetSlice((ATermList) arg, start * 4, end * 4);
}
SDF_NatCon SDF_getNatConArgumentsNatConAt(SDF_NatConArguments arg, int index) {
 return (SDF_NatCon)ATelementAt((ATermList) arg,index * 4);
}
SDF_NatConArguments SDF_replaceNatConArgumentsNatConAt(SDF_NatConArguments arg, SDF_NatCon elem, int index) {
 return (SDF_NatConArguments) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 4);
}
SDF_NatConArguments SDF_makeNatConArguments2(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_NatCon elem1, SDF_NatCon elem2) {
  return SDF_makeNatConArgumentsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeNatConArgumentsSingle(elem2));
}
SDF_NatConArguments SDF_makeNatConArguments3(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_NatCon elem1, SDF_NatCon elem2, SDF_NatCon elem3) {
  return SDF_makeNatConArgumentsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeNatConArguments2(wsAfterHead, wsAfterSep, elem2, elem3));
}
SDF_NatConArguments SDF_makeNatConArguments4(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_NatCon elem1, SDF_NatCon elem2, SDF_NatCon elem3, SDF_NatCon elem4) {
  return SDF_makeNatConArgumentsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeNatConArguments3(wsAfterHead, wsAfterSep, elem2, elem3, elem4));
}
SDF_NatConArguments SDF_makeNatConArguments5(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_NatCon elem1, SDF_NatCon elem2, SDF_NatCon elem3, SDF_NatCon elem4, SDF_NatCon elem5) {
  return SDF_makeNatConArgumentsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeNatConArguments4(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5));
}
SDF_NatConArguments SDF_makeNatConArguments6(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_NatCon elem1, SDF_NatCon elem2, SDF_NatCon elem3, SDF_NatCon elem4, SDF_NatCon elem5, SDF_NatCon elem6) {
  return SDF_makeNatConArgumentsMany(elem1, wsAfterHead, wsAfterSep, SDF_makeNatConArguments5(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getGroupListLength (SDF_GroupList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 4) + 1;
}
SDF_GroupList SDF_reverseGroupList(SDF_GroupList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATerm sep2;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep2);
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_GroupList) result;
}
SDF_GroupList SDF_appendGroupList(SDF_GroupList arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Group arg1) {
  return SDF_concatGroupList(arg0, wsAfterHead, wsAfterSep, SDF_makeGroupListSingle(arg1));
}
SDF_GroupList SDF_concatGroupList(SDF_GroupList arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_GroupList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeGroupListMany((SDF_Group)ATgetFirst((ATermList) arg0), wsAfterHead, wsAfterSep,  arg1);
  arg1 = (SDF_GroupList) ATgetNext((ATermList) arg1);
  return (SDF_GroupList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_GroupList SDF_sliceGroupList(SDF_GroupList arg, int start, int end) {
  return (SDF_GroupList) ATgetSlice((ATermList) arg, start * 4, end * 4);
}
SDF_Group SDF_getGroupListGroupAt(SDF_GroupList arg, int index) {
 return (SDF_Group)ATelementAt((ATermList) arg,index * 4);
}
SDF_GroupList SDF_replaceGroupListGroupAt(SDF_GroupList arg, SDF_Group elem, int index) {
 return (SDF_GroupList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 4);
}
SDF_GroupList SDF_makeGroupList2(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Group elem1, SDF_Group elem2) {
  return SDF_makeGroupListMany(elem1, wsAfterHead, wsAfterSep, SDF_makeGroupListSingle(elem2));
}
SDF_GroupList SDF_makeGroupList3(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Group elem1, SDF_Group elem2, SDF_Group elem3) {
  return SDF_makeGroupListMany(elem1, wsAfterHead, wsAfterSep, SDF_makeGroupList2(wsAfterHead, wsAfterSep, elem2, elem3));
}
SDF_GroupList SDF_makeGroupList4(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Group elem1, SDF_Group elem2, SDF_Group elem3, SDF_Group elem4) {
  return SDF_makeGroupListMany(elem1, wsAfterHead, wsAfterSep, SDF_makeGroupList3(wsAfterHead, wsAfterSep, elem2, elem3, elem4));
}
SDF_GroupList SDF_makeGroupList5(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Group elem1, SDF_Group elem2, SDF_Group elem3, SDF_Group elem4, SDF_Group elem5) {
  return SDF_makeGroupListMany(elem1, wsAfterHead, wsAfterSep, SDF_makeGroupList4(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5));
}
SDF_GroupList SDF_makeGroupList6(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Group elem1, SDF_Group elem2, SDF_Group elem3, SDF_Group elem4, SDF_Group elem5, SDF_Group elem6) {
  return SDF_makeGroupListMany(elem1, wsAfterHead, wsAfterSep, SDF_makeGroupList5(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getPriorityListLength (SDF_PriorityList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 4) + 1;
}
SDF_PriorityList SDF_reversePriorityList(SDF_PriorityList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATerm sep2;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep2);
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_PriorityList) result;
}
SDF_PriorityList SDF_appendPriorityList(SDF_PriorityList arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Priority arg1) {
  return SDF_concatPriorityList(arg0, wsAfterHead, wsAfterSep, SDF_makePriorityListSingle(arg1));
}
SDF_PriorityList SDF_concatPriorityList(SDF_PriorityList arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_PriorityList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makePriorityListMany((SDF_Priority)ATgetFirst((ATermList) arg0), wsAfterHead, wsAfterSep,  arg1);
  arg1 = (SDF_PriorityList) ATgetNext((ATermList) arg1);
  return (SDF_PriorityList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_PriorityList SDF_slicePriorityList(SDF_PriorityList arg, int start, int end) {
  return (SDF_PriorityList) ATgetSlice((ATermList) arg, start * 4, end * 4);
}
SDF_Priority SDF_getPriorityListPriorityAt(SDF_PriorityList arg, int index) {
 return (SDF_Priority)ATelementAt((ATermList) arg,index * 4);
}
SDF_PriorityList SDF_replacePriorityListPriorityAt(SDF_PriorityList arg, SDF_Priority elem, int index) {
 return (SDF_PriorityList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 4);
}
SDF_PriorityList SDF_makePriorityList2(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Priority elem1, SDF_Priority elem2) {
  return SDF_makePriorityListMany(elem1, wsAfterHead, wsAfterSep, SDF_makePriorityListSingle(elem2));
}
SDF_PriorityList SDF_makePriorityList3(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Priority elem1, SDF_Priority elem2, SDF_Priority elem3) {
  return SDF_makePriorityListMany(elem1, wsAfterHead, wsAfterSep, SDF_makePriorityList2(wsAfterHead, wsAfterSep, elem2, elem3));
}
SDF_PriorityList SDF_makePriorityList4(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Priority elem1, SDF_Priority elem2, SDF_Priority elem3, SDF_Priority elem4) {
  return SDF_makePriorityListMany(elem1, wsAfterHead, wsAfterSep, SDF_makePriorityList3(wsAfterHead, wsAfterSep, elem2, elem3, elem4));
}
SDF_PriorityList SDF_makePriorityList5(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Priority elem1, SDF_Priority elem2, SDF_Priority elem3, SDF_Priority elem4, SDF_Priority elem5) {
  return SDF_makePriorityListMany(elem1, wsAfterHead, wsAfterSep, SDF_makePriorityList4(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5));
}
SDF_PriorityList SDF_makePriorityList6(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Priority elem1, SDF_Priority elem2, SDF_Priority elem3, SDF_Priority elem4, SDF_Priority elem5, SDF_Priority elem6) {
  return SDF_makePriorityListMany(elem1, wsAfterHead, wsAfterSep, SDF_makePriorityList5(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getSymbolTailLength (SDF_SymbolTail arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 2) + 1;
}
SDF_SymbolTail SDF_reverseSymbolTail(SDF_SymbolTail arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_SymbolTail) result;
}
SDF_SymbolTail SDF_appendSymbolTail(SDF_SymbolTail arg0, SDF_OptLayout wsAfterHead, SDF_Symbol arg1) {
  return SDF_concatSymbolTail(arg0, wsAfterHead, SDF_makeSymbolTailSingle(arg1));
}
SDF_SymbolTail SDF_concatSymbolTail(SDF_SymbolTail arg0, SDF_OptLayout wsAfterHead, SDF_SymbolTail arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeSymbolTailMany((SDF_Symbol)ATgetFirst((ATermList) arg0), wsAfterHead,  arg1);
  arg1 = (SDF_SymbolTail) ATgetNext((ATermList) arg1);
  return (SDF_SymbolTail) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_SymbolTail SDF_sliceSymbolTail(SDF_SymbolTail arg, int start, int end) {
  return (SDF_SymbolTail) ATgetSlice((ATermList) arg, start * 2, end * 2);
}
SDF_Symbol SDF_getSymbolTailSymbolAt(SDF_SymbolTail arg, int index) {
 return (SDF_Symbol)ATelementAt((ATermList) arg,index * 2);
}
SDF_SymbolTail SDF_replaceSymbolTailSymbolAt(SDF_SymbolTail arg, SDF_Symbol elem, int index) {
 return (SDF_SymbolTail) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 2);
}
SDF_SymbolTail SDF_makeSymbolTail2(SDF_OptLayout wsAfterHead, SDF_Symbol elem1, SDF_Symbol elem2) {
  return SDF_makeSymbolTailMany(elem1, wsAfterHead, SDF_makeSymbolTailSingle(elem2));
}
SDF_SymbolTail SDF_makeSymbolTail3(SDF_OptLayout wsAfterHead, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3) {
  return SDF_makeSymbolTailMany(elem1, wsAfterHead, SDF_makeSymbolTail2(wsAfterHead, elem2, elem3));
}
SDF_SymbolTail SDF_makeSymbolTail4(SDF_OptLayout wsAfterHead, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3, SDF_Symbol elem4) {
  return SDF_makeSymbolTailMany(elem1, wsAfterHead, SDF_makeSymbolTail3(wsAfterHead, elem2, elem3, elem4));
}
SDF_SymbolTail SDF_makeSymbolTail5(SDF_OptLayout wsAfterHead, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3, SDF_Symbol elem4, SDF_Symbol elem5) {
  return SDF_makeSymbolTailMany(elem1, wsAfterHead, SDF_makeSymbolTail4(wsAfterHead, elem2, elem3, elem4, elem5));
}
SDF_SymbolTail SDF_makeSymbolTail6(SDF_OptLayout wsAfterHead, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3, SDF_Symbol elem4, SDF_Symbol elem5, SDF_Symbol elem6) {
  return SDF_makeSymbolTailMany(elem1, wsAfterHead, SDF_makeSymbolTail5(wsAfterHead, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getSymbolRestLength (SDF_SymbolRest arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 4) + 1;
}
SDF_SymbolRest SDF_reverseSymbolRest(SDF_SymbolRest arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATerm sep2;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep2);
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_SymbolRest) result;
}
SDF_SymbolRest SDF_appendSymbolRest(SDF_SymbolRest arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol arg1) {
  return SDF_concatSymbolRest(arg0, wsAfterHead, wsAfterSep, SDF_makeSymbolRestSingle(arg1));
}
SDF_SymbolRest SDF_concatSymbolRest(SDF_SymbolRest arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_SymbolRest arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeSymbolRestMany((SDF_Symbol)ATgetFirst((ATermList) arg0), wsAfterHead, wsAfterSep,  arg1);
  arg1 = (SDF_SymbolRest) ATgetNext((ATermList) arg1);
  return (SDF_SymbolRest) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_SymbolRest SDF_sliceSymbolRest(SDF_SymbolRest arg, int start, int end) {
  return (SDF_SymbolRest) ATgetSlice((ATermList) arg, start * 4, end * 4);
}
SDF_Symbol SDF_getSymbolRestSymbolAt(SDF_SymbolRest arg, int index) {
 return (SDF_Symbol)ATelementAt((ATermList) arg,index * 4);
}
SDF_SymbolRest SDF_replaceSymbolRestSymbolAt(SDF_SymbolRest arg, SDF_Symbol elem, int index) {
 return (SDF_SymbolRest) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 4);
}
SDF_SymbolRest SDF_makeSymbolRest2(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol elem1, SDF_Symbol elem2) {
  return SDF_makeSymbolRestMany(elem1, wsAfterHead, wsAfterSep, SDF_makeSymbolRestSingle(elem2));
}
SDF_SymbolRest SDF_makeSymbolRest3(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3) {
  return SDF_makeSymbolRestMany(elem1, wsAfterHead, wsAfterSep, SDF_makeSymbolRest2(wsAfterHead, wsAfterSep, elem2, elem3));
}
SDF_SymbolRest SDF_makeSymbolRest4(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3, SDF_Symbol elem4) {
  return SDF_makeSymbolRestMany(elem1, wsAfterHead, wsAfterSep, SDF_makeSymbolRest3(wsAfterHead, wsAfterSep, elem2, elem3, elem4));
}
SDF_SymbolRest SDF_makeSymbolRest5(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3, SDF_Symbol elem4, SDF_Symbol elem5) {
  return SDF_makeSymbolRestMany(elem1, wsAfterHead, wsAfterSep, SDF_makeSymbolRest4(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5));
}
SDF_SymbolRest SDF_makeSymbolRest6(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3, SDF_Symbol elem4, SDF_Symbol elem5, SDF_Symbol elem6) {
  return SDF_makeSymbolRestMany(elem1, wsAfterHead, wsAfterSep, SDF_makeSymbolRest5(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getRenamingListLength (SDF_RenamingList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 2) + 1;
}
SDF_RenamingList SDF_reverseRenamingList(SDF_RenamingList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_RenamingList) result;
}
SDF_RenamingList SDF_appendRenamingList(SDF_RenamingList arg0, SDF_OptLayout wsAfterHead, SDF_Renaming arg1) {
  return SDF_concatRenamingList(arg0, wsAfterHead, SDF_makeRenamingListSingle(arg1));
}
SDF_RenamingList SDF_concatRenamingList(SDF_RenamingList arg0, SDF_OptLayout wsAfterHead, SDF_RenamingList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeRenamingListMany((SDF_Renaming)ATgetFirst((ATermList) arg0), wsAfterHead,  arg1);
  arg1 = (SDF_RenamingList) ATgetNext((ATermList) arg1);
  return (SDF_RenamingList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_RenamingList SDF_sliceRenamingList(SDF_RenamingList arg, int start, int end) {
  return (SDF_RenamingList) ATgetSlice((ATermList) arg, start * 2, end * 2);
}
SDF_Renaming SDF_getRenamingListRenamingAt(SDF_RenamingList arg, int index) {
 return (SDF_Renaming)ATelementAt((ATermList) arg,index * 2);
}
SDF_RenamingList SDF_replaceRenamingListRenamingAt(SDF_RenamingList arg, SDF_Renaming elem, int index) {
 return (SDF_RenamingList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 2);
}
SDF_RenamingList SDF_makeRenamingList2(SDF_OptLayout wsAfterHead, SDF_Renaming elem1, SDF_Renaming elem2) {
  return SDF_makeRenamingListMany(elem1, wsAfterHead, SDF_makeRenamingListSingle(elem2));
}
SDF_RenamingList SDF_makeRenamingList3(SDF_OptLayout wsAfterHead, SDF_Renaming elem1, SDF_Renaming elem2, SDF_Renaming elem3) {
  return SDF_makeRenamingListMany(elem1, wsAfterHead, SDF_makeRenamingList2(wsAfterHead, elem2, elem3));
}
SDF_RenamingList SDF_makeRenamingList4(SDF_OptLayout wsAfterHead, SDF_Renaming elem1, SDF_Renaming elem2, SDF_Renaming elem3, SDF_Renaming elem4) {
  return SDF_makeRenamingListMany(elem1, wsAfterHead, SDF_makeRenamingList3(wsAfterHead, elem2, elem3, elem4));
}
SDF_RenamingList SDF_makeRenamingList5(SDF_OptLayout wsAfterHead, SDF_Renaming elem1, SDF_Renaming elem2, SDF_Renaming elem3, SDF_Renaming elem4, SDF_Renaming elem5) {
  return SDF_makeRenamingListMany(elem1, wsAfterHead, SDF_makeRenamingList4(wsAfterHead, elem2, elem3, elem4, elem5));
}
SDF_RenamingList SDF_makeRenamingList6(SDF_OptLayout wsAfterHead, SDF_Renaming elem1, SDF_Renaming elem2, SDF_Renaming elem3, SDF_Renaming elem4, SDF_Renaming elem5, SDF_Renaming elem6) {
  return SDF_makeRenamingListMany(elem1, wsAfterHead, SDF_makeRenamingList5(wsAfterHead, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getLookaheadListLength (SDF_LookaheadList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 4) + 1;
}
SDF_LookaheadList SDF_reverseLookaheadList(SDF_LookaheadList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATerm sep2;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep2);
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_LookaheadList) result;
}
SDF_LookaheadList SDF_appendLookaheadList(SDF_LookaheadList arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Lookahead arg1) {
  return SDF_concatLookaheadList(arg0, wsAfterHead, wsAfterSep, SDF_makeLookaheadListSingle(arg1));
}
SDF_LookaheadList SDF_concatLookaheadList(SDF_LookaheadList arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_LookaheadList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeLookaheadListMany((SDF_Lookahead)ATgetFirst((ATermList) arg0), wsAfterHead, wsAfterSep,  arg1);
  arg1 = (SDF_LookaheadList) ATgetNext((ATermList) arg1);
  return (SDF_LookaheadList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_LookaheadList SDF_sliceLookaheadList(SDF_LookaheadList arg, int start, int end) {
  return (SDF_LookaheadList) ATgetSlice((ATermList) arg, start * 4, end * 4);
}
SDF_Lookahead SDF_getLookaheadListLookaheadAt(SDF_LookaheadList arg, int index) {
 return (SDF_Lookahead)ATelementAt((ATermList) arg,index * 4);
}
SDF_LookaheadList SDF_replaceLookaheadListLookaheadAt(SDF_LookaheadList arg, SDF_Lookahead elem, int index) {
 return (SDF_LookaheadList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 4);
}
SDF_LookaheadList SDF_makeLookaheadList2(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Lookahead elem1, SDF_Lookahead elem2) {
  return SDF_makeLookaheadListMany(elem1, wsAfterHead, wsAfterSep, SDF_makeLookaheadListSingle(elem2));
}
SDF_LookaheadList SDF_makeLookaheadList3(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Lookahead elem1, SDF_Lookahead elem2, SDF_Lookahead elem3) {
  return SDF_makeLookaheadListMany(elem1, wsAfterHead, wsAfterSep, SDF_makeLookaheadList2(wsAfterHead, wsAfterSep, elem2, elem3));
}
SDF_LookaheadList SDF_makeLookaheadList4(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Lookahead elem1, SDF_Lookahead elem2, SDF_Lookahead elem3, SDF_Lookahead elem4) {
  return SDF_makeLookaheadListMany(elem1, wsAfterHead, wsAfterSep, SDF_makeLookaheadList3(wsAfterHead, wsAfterSep, elem2, elem3, elem4));
}
SDF_LookaheadList SDF_makeLookaheadList5(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Lookahead elem1, SDF_Lookahead elem2, SDF_Lookahead elem3, SDF_Lookahead elem4, SDF_Lookahead elem5) {
  return SDF_makeLookaheadListMany(elem1, wsAfterHead, wsAfterSep, SDF_makeLookaheadList4(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5));
}
SDF_LookaheadList SDF_makeLookaheadList6(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Lookahead elem1, SDF_Lookahead elem2, SDF_Lookahead elem3, SDF_Lookahead elem4, SDF_Lookahead elem5, SDF_Lookahead elem6) {
  return SDF_makeLookaheadListMany(elem1, wsAfterHead, wsAfterSep, SDF_makeLookaheadList5(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getRestrictionListLength (SDF_RestrictionList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 2) + 1;
}
SDF_RestrictionList SDF_reverseRestrictionList(SDF_RestrictionList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_RestrictionList) result;
}
SDF_RestrictionList SDF_appendRestrictionList(SDF_RestrictionList arg0, SDF_OptLayout wsAfterHead, SDF_Restriction arg1) {
  return SDF_concatRestrictionList(arg0, wsAfterHead, SDF_makeRestrictionListSingle(arg1));
}
SDF_RestrictionList SDF_concatRestrictionList(SDF_RestrictionList arg0, SDF_OptLayout wsAfterHead, SDF_RestrictionList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeRestrictionListMany((SDF_Restriction)ATgetFirst((ATermList) arg0), wsAfterHead,  arg1);
  arg1 = (SDF_RestrictionList) ATgetNext((ATermList) arg1);
  return (SDF_RestrictionList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_RestrictionList SDF_sliceRestrictionList(SDF_RestrictionList arg, int start, int end) {
  return (SDF_RestrictionList) ATgetSlice((ATermList) arg, start * 2, end * 2);
}
SDF_Restriction SDF_getRestrictionListRestrictionAt(SDF_RestrictionList arg, int index) {
 return (SDF_Restriction)ATelementAt((ATermList) arg,index * 2);
}
SDF_RestrictionList SDF_replaceRestrictionListRestrictionAt(SDF_RestrictionList arg, SDF_Restriction elem, int index) {
 return (SDF_RestrictionList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 2);
}
SDF_RestrictionList SDF_makeRestrictionList2(SDF_OptLayout wsAfterHead, SDF_Restriction elem1, SDF_Restriction elem2) {
  return SDF_makeRestrictionListMany(elem1, wsAfterHead, SDF_makeRestrictionListSingle(elem2));
}
SDF_RestrictionList SDF_makeRestrictionList3(SDF_OptLayout wsAfterHead, SDF_Restriction elem1, SDF_Restriction elem2, SDF_Restriction elem3) {
  return SDF_makeRestrictionListMany(elem1, wsAfterHead, SDF_makeRestrictionList2(wsAfterHead, elem2, elem3));
}
SDF_RestrictionList SDF_makeRestrictionList4(SDF_OptLayout wsAfterHead, SDF_Restriction elem1, SDF_Restriction elem2, SDF_Restriction elem3, SDF_Restriction elem4) {
  return SDF_makeRestrictionListMany(elem1, wsAfterHead, SDF_makeRestrictionList3(wsAfterHead, elem2, elem3, elem4));
}
SDF_RestrictionList SDF_makeRestrictionList5(SDF_OptLayout wsAfterHead, SDF_Restriction elem1, SDF_Restriction elem2, SDF_Restriction elem3, SDF_Restriction elem4, SDF_Restriction elem5) {
  return SDF_makeRestrictionListMany(elem1, wsAfterHead, SDF_makeRestrictionList4(wsAfterHead, elem2, elem3, elem4, elem5));
}
SDF_RestrictionList SDF_makeRestrictionList6(SDF_OptLayout wsAfterHead, SDF_Restriction elem1, SDF_Restriction elem2, SDF_Restriction elem3, SDF_Restriction elem4, SDF_Restriction elem5, SDF_Restriction elem6) {
  return SDF_makeRestrictionListMany(elem1, wsAfterHead, SDF_makeRestrictionList5(wsAfterHead, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getSymbolParametersLength (SDF_SymbolParameters arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 4) + 1;
}
SDF_SymbolParameters SDF_reverseSymbolParameters(SDF_SymbolParameters arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATerm sep1;
  ATerm sep2;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep2);
    result = ATinsert(result, sep1);
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
  sep1 = ATgetFirst(list);
  list = ATgetNext(list);
  sep2 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_SymbolParameters) result;
}
SDF_SymbolParameters SDF_appendSymbolParameters(SDF_SymbolParameters arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol arg1) {
  return SDF_concatSymbolParameters(arg0, wsAfterHead, wsAfterSep, SDF_makeSymbolParametersSingle(arg1));
}
SDF_SymbolParameters SDF_concatSymbolParameters(SDF_SymbolParameters arg0, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_SymbolParameters arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeSymbolParametersMany((SDF_Symbol)ATgetFirst((ATermList) arg0), wsAfterHead, wsAfterSep,  arg1);
  arg1 = (SDF_SymbolParameters) ATgetNext((ATermList) arg1);
  return (SDF_SymbolParameters) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_SymbolParameters SDF_sliceSymbolParameters(SDF_SymbolParameters arg, int start, int end) {
  return (SDF_SymbolParameters) ATgetSlice((ATermList) arg, start * 4, end * 4);
}
SDF_Symbol SDF_getSymbolParametersSymbolAt(SDF_SymbolParameters arg, int index) {
 return (SDF_Symbol)ATelementAt((ATermList) arg,index * 4);
}
SDF_SymbolParameters SDF_replaceSymbolParametersSymbolAt(SDF_SymbolParameters arg, SDF_Symbol elem, int index) {
 return (SDF_SymbolParameters) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 4);
}
SDF_SymbolParameters SDF_makeSymbolParameters2(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol elem1, SDF_Symbol elem2) {
  return SDF_makeSymbolParametersMany(elem1, wsAfterHead, wsAfterSep, SDF_makeSymbolParametersSingle(elem2));
}
SDF_SymbolParameters SDF_makeSymbolParameters3(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3) {
  return SDF_makeSymbolParametersMany(elem1, wsAfterHead, wsAfterSep, SDF_makeSymbolParameters2(wsAfterHead, wsAfterSep, elem2, elem3));
}
SDF_SymbolParameters SDF_makeSymbolParameters4(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3, SDF_Symbol elem4) {
  return SDF_makeSymbolParametersMany(elem1, wsAfterHead, wsAfterSep, SDF_makeSymbolParameters3(wsAfterHead, wsAfterSep, elem2, elem3, elem4));
}
SDF_SymbolParameters SDF_makeSymbolParameters5(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3, SDF_Symbol elem4, SDF_Symbol elem5) {
  return SDF_makeSymbolParametersMany(elem1, wsAfterHead, wsAfterSep, SDF_makeSymbolParameters4(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5));
}
SDF_SymbolParameters SDF_makeSymbolParameters6(SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3, SDF_Symbol elem4, SDF_Symbol elem5, SDF_Symbol elem6) {
  return SDF_makeSymbolParametersMany(elem1, wsAfterHead, wsAfterSep, SDF_makeSymbolParameters5(wsAfterHead, wsAfterSep, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getSymbolListLength (SDF_SymbolList arg) {
  if (ATisEmpty((ATermList) arg)) {
    return 0;
  }
  return (ATgetLength((ATermList) arg) / 2) + 1;
}
SDF_SymbolList SDF_reverseSymbolList(SDF_SymbolList arg) {
  ATermList list = (ATermList) arg;
  ATerm head;
  ATerm sep0;
  ATermList result;

 if (ATisEmpty(list) || ATgetLength(list) == 1) {
    return arg;
  }

  result = ATmakeList1(ATgetFirst(list));
      list = ATgetNext(list);
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);

  while (!ATisEmpty(list)) {
    result = ATinsert(result, sep0);

   head = ATgetFirst(list);
   result = ATinsert(result, head);
    list = ATgetNext(list);

   if (!ATisEmpty(list)) {
  sep0 = ATgetFirst(list);
  list = ATgetNext(list);
   }
  }

  return (SDF_SymbolList) result;
}
SDF_SymbolList SDF_appendSymbolList(SDF_SymbolList arg0, SDF_OptLayout wsAfterHead, SDF_Symbol arg1) {
  return SDF_concatSymbolList(arg0, wsAfterHead, SDF_makeSymbolListSingle(arg1));
}
SDF_SymbolList SDF_concatSymbolList(SDF_SymbolList arg0, SDF_OptLayout wsAfterHead, SDF_SymbolList arg1) {
  if (ATisEmpty((ATermList) arg0)) {
    return arg1;
  }
  arg1 = SDF_makeSymbolListMany((SDF_Symbol)ATgetFirst((ATermList) arg0), wsAfterHead,  arg1);
  arg1 = (SDF_SymbolList) ATgetNext((ATermList) arg1);
  return (SDF_SymbolList) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_SymbolList SDF_sliceSymbolList(SDF_SymbolList arg, int start, int end) {
  return (SDF_SymbolList) ATgetSlice((ATermList) arg, start * 2, end * 2);
}
SDF_Symbol SDF_getSymbolListSymbolAt(SDF_SymbolList arg, int index) {
 return (SDF_Symbol)ATelementAt((ATermList) arg,index * 2);
}
SDF_SymbolList SDF_replaceSymbolListSymbolAt(SDF_SymbolList arg, SDF_Symbol elem, int index) {
 return (SDF_SymbolList) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index * 2);
}
SDF_SymbolList SDF_makeSymbolList2(SDF_OptLayout wsAfterHead, SDF_Symbol elem1, SDF_Symbol elem2) {
  return SDF_makeSymbolListMany(elem1, wsAfterHead, SDF_makeSymbolListSingle(elem2));
}
SDF_SymbolList SDF_makeSymbolList3(SDF_OptLayout wsAfterHead, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3) {
  return SDF_makeSymbolListMany(elem1, wsAfterHead, SDF_makeSymbolList2(wsAfterHead, elem2, elem3));
}
SDF_SymbolList SDF_makeSymbolList4(SDF_OptLayout wsAfterHead, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3, SDF_Symbol elem4) {
  return SDF_makeSymbolListMany(elem1, wsAfterHead, SDF_makeSymbolList3(wsAfterHead, elem2, elem3, elem4));
}
SDF_SymbolList SDF_makeSymbolList5(SDF_OptLayout wsAfterHead, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3, SDF_Symbol elem4, SDF_Symbol elem5) {
  return SDF_makeSymbolListMany(elem1, wsAfterHead, SDF_makeSymbolList4(wsAfterHead, elem2, elem3, elem4, elem5));
}
SDF_SymbolList SDF_makeSymbolList6(SDF_OptLayout wsAfterHead, SDF_Symbol elem1, SDF_Symbol elem2, SDF_Symbol elem3, SDF_Symbol elem4, SDF_Symbol elem5, SDF_Symbol elem6) {
  return SDF_makeSymbolListMany(elem1, wsAfterHead, SDF_makeSymbolList5(wsAfterHead, elem2, elem3, elem4, elem5, elem6));
}
int SDF_getLexStrCharCharsLength (SDF_LexStrCharChars arg) {
  return ATgetLength((ATermList) arg);
}
SDF_LexStrCharChars SDF_reverseLexStrCharChars(SDF_LexStrCharChars arg) {
  return (SDF_LexStrCharChars) ATreverse((ATermList) arg);
}
SDF_LexStrCharChars SDF_appendLexStrCharChars(SDF_LexStrCharChars arg, SDF_LexStrChar elem) {
  return (SDF_LexStrCharChars) ATappend((ATermList) arg, (ATerm) ((ATerm) elem));
}
SDF_LexStrCharChars SDF_concatLexStrCharChars(SDF_LexStrCharChars arg0, SDF_LexStrCharChars arg1) {
  return (SDF_LexStrCharChars) ATconcat((ATermList) arg0, (ATermList) arg1);
}
SDF_LexStrCharChars SDF_sliceLexStrCharChars(SDF_LexStrCharChars arg, int start, int end) {
  return (SDF_LexStrCharChars) ATgetSlice((ATermList) arg, start, end);
}
SDF_LexStrChar SDF_getLexStrCharCharsLexStrCharAt(SDF_LexStrCharChars arg, int index) {
 return (SDF_LexStrChar)ATelementAt((ATermList) arg,index);
}
SDF_LexStrCharChars SDF_replaceLexStrCharCharsLexStrCharAt(SDF_LexStrCharChars arg, SDF_LexStrChar elem, int index) {
 return (SDF_LexStrCharChars) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index);
}
SDF_LexStrCharChars SDF_makeLexStrCharChars2(SDF_LexStrChar elem1, SDF_LexStrChar elem2) {
  return (SDF_LexStrCharChars) ATmakeList2((ATerm) ((ATerm) elem2), (ATerm) ((ATerm) elem2));
}
SDF_LexStrCharChars SDF_makeLexStrCharChars3(SDF_LexStrChar elem1, SDF_LexStrChar elem2, SDF_LexStrChar elem3) {
  return (SDF_LexStrCharChars) ATmakeList3((ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3));
}
SDF_LexStrCharChars SDF_makeLexStrCharChars4(SDF_LexStrChar elem1, SDF_LexStrChar elem2, SDF_LexStrChar elem3, SDF_LexStrChar elem4) {
  return (SDF_LexStrCharChars) ATmakeList4((ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4));
}
SDF_LexStrCharChars SDF_makeLexStrCharChars5(SDF_LexStrChar elem1, SDF_LexStrChar elem2, SDF_LexStrChar elem3, SDF_LexStrChar elem4, SDF_LexStrChar elem5) {
  return (SDF_LexStrCharChars) ATmakeList5((ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5));
}
SDF_LexStrCharChars SDF_makeLexStrCharChars6(SDF_LexStrChar elem1, SDF_LexStrChar elem2, SDF_LexStrChar elem3, SDF_LexStrChar elem4, SDF_LexStrChar elem5, SDF_LexStrChar elem6) {
  return (SDF_LexStrCharChars) ATmakeList6((ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6));
}

/*}}}  */
/*{{{  constructors */

/*{{{  SDF_OptLayout SDF_makeOptLayoutAbsent(void) */

SDF_OptLayout SDF_makeOptLayoutAbsent(void)
{
  return (SDF_OptLayout)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATempty, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATempty);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_makeOptLayoutPresent(SDF_Layout layout) */

SDF_OptLayout SDF_makeOptLayoutPresent(SDF_Layout layout)
{
  return (SDF_OptLayout)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl0(SDF_afun4))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm) layout));
}

/*}}}  */
/*{{{  SDF_Layout SDF_makeLayoutLexToCf(SDF_LexLayoutList list) */

SDF_Layout SDF_makeLayoutLexToCf(SDF_LexLayoutList list)
{
  return (SDF_Layout)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun7, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl0(SDF_afun4)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun7, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm) list)));
}

/*}}}  */
/*{{{  SDF_LexLayoutList SDF_makeLexLayoutListEmpty(void) */

SDF_LexLayoutList SDF_makeLexLayoutListEmpty(void)
{
  return (SDF_LexLayoutList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_LexLayoutList SDF_makeLexLayoutListSingle(SDF_LexLayout head) */

SDF_LexLayoutList SDF_makeLexLayoutListSingle(SDF_LexLayout head)
{
  return (SDF_LexLayoutList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_LexLayoutList SDF_makeLexLayoutListMany(SDF_LexLayout head, SDF_LexLayoutList tail) */

SDF_LexLayoutList SDF_makeLexLayoutListMany(SDF_LexLayout head, SDF_LexLayoutList tail)
{
  return (SDF_LexLayoutList)(ATerm)ATinsert((ATermList)tail, (ATerm) head);
}

/*}}}  */
/*{{{  SDF_AFun SDF_makeAFunQuoted(SDF_StrCon StrCon) */

SDF_AFun SDF_makeAFunQuoted(SDF_StrCon StrCon)
{
  return (SDF_AFun)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun10)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun11))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun15)))))), (ATerm)ATmakeList1((ATerm) StrCon));
}

/*}}}  */
/*{{{  SDF_AFun SDF_makeAFunUnquoted(SDF_IdCon IdCon) */

SDF_AFun SDF_makeAFunUnquoted(SDF_IdCon IdCon)
{
  return (SDF_AFun)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun16)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun11))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun17)))))), (ATerm)ATmakeList1((ATerm) IdCon));
}

/*}}}  */
/*{{{  SDF_ATerm SDF_makeATermInt(SDF_IntCon IntCon) */

SDF_ATerm SDF_makeATermInt(SDF_IntCon IntCon)
{
  return (SDF_ATerm)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun18)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun20)))))), (ATerm)ATmakeList1((ATerm) IntCon));
}

/*}}}  */
/*{{{  SDF_ATerm SDF_makeATermReal(SDF_RealCon RealCon) */

SDF_ATerm SDF_makeATermReal(SDF_RealCon RealCon)
{
  return (SDF_ATerm)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun21)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun22)))))), (ATerm)ATmakeList1((ATerm) RealCon));
}

/*}}}  */
/*{{{  SDF_ATerm SDF_makeATermFun(SDF_AFun fun) */

SDF_ATerm SDF_makeATermFun(SDF_AFun fun)
{
  return (SDF_ATerm)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun11)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun23)))))), (ATerm)ATmakeList1((ATerm) fun));
}

/*}}}  */
/*{{{  SDF_ATerm SDF_makeATermAppl(SDF_AFun fun, SDF_OptLayout wsAfterFun, SDF_OptLayout wsAfterParenOpen, SDF_ATermArgs args, SDF_OptLayout wsAfterArgs) */

SDF_ATerm SDF_makeATermAppl(SDF_AFun fun, SDF_OptLayout wsAfterFun, SDF_OptLayout wsAfterParenOpen, SDF_ATermArgs args, SDF_OptLayout wsAfterArgs)
{
  return (SDF_ATerm)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun26, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun11)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun29)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm) wsAfterArgs), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun26, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm) args)), (ATerm) wsAfterParenOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm) wsAfterFun), (ATerm) fun));
}

/*}}}  */
/*{{{  SDF_ATerm SDF_makeATermPlaceholder(SDF_OptLayout wsAfterLessThan, SDF_ATerm type, SDF_OptLayout wsAfterType) */

SDF_ATerm SDF_makeATermPlaceholder(SDF_OptLayout wsAfterLessThan, SDF_ATerm type, SDF_OptLayout wsAfterType)
{
  return (SDF_ATerm)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun31))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun32))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun33)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun31)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm) wsAfterType), (ATerm) type), (ATerm) wsAfterLessThan), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(60)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun32)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(60)))));
}

/*}}}  */
/*{{{  SDF_ATerm SDF_makeATermList(SDF_OptLayout wsAfterBracketOpen, SDF_ATermElems elems, SDF_OptLayout wsAfterElems) */

SDF_ATerm SDF_makeATermList(SDF_OptLayout wsAfterBracketOpen, SDF_ATermElems elems, SDF_OptLayout wsAfterElems)
{
  return (SDF_ATerm)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun34))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun35, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun36))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun37)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(93)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun34)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(93)))), (ATerm) wsAfterElems), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun35, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm) elems)), (ATerm) wsAfterBracketOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(91)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun36)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(91)))));
}

/*}}}  */
/*{{{  SDF_ATerm SDF_makeATermAnnotated(SDF_ATerm trm, SDF_OptLayout wsAfterTrm, SDF_Annotation Annotation) */

SDF_ATerm SDF_makeATermAnnotated(SDF_ATerm trm, SDF_OptLayout wsAfterTrm, SDF_Annotation Annotation)
{
  return (SDF_ATerm)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun38)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun39)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) Annotation), (ATerm) wsAfterTrm), (ATerm) trm));
}

/*}}}  */
/*{{{  SDF_Annotation SDF_makeAnnotationDefault(SDF_OptLayout wsAfterBraceOpen, SDF_ATermAnnos annos, SDF_OptLayout wsAfterAnnos) */

SDF_Annotation SDF_makeAnnotationDefault(SDF_OptLayout wsAfterBraceOpen, SDF_ATermAnnos annos, SDF_OptLayout wsAfterAnnos)
{
  return (SDF_Annotation)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun40))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun26, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun41))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun38))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun42)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(125)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun40)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(125)))), (ATerm) wsAfterAnnos), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun26, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm) annos)), (ATerm) wsAfterBraceOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(123)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun41)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(123)))));
}

/*}}}  */
/*{{{  SDF_Start SDF_makeStartATerm(SDF_OptLayout wsBefore, SDF_ATerm topATerm, SDF_OptLayout wsAfter, int ambCnt) */

SDF_Start SDF_makeStartATerm(SDF_OptLayout wsBefore, SDF_ATerm topATerm, SDF_OptLayout wsAfter, int ambCnt)
{
  return (SDF_Start)(ATerm)ATmakeAppl2(SDF_afun43, (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun44)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) wsAfter), (ATerm) topATerm), (ATerm) wsBefore)), (ATerm) (ATerm) ATmakeInt(ambCnt));
}

/*}}}  */
/*{{{  SDF_Start SDF_makeStartIntCon(SDF_OptLayout wsBefore, SDF_IntCon topIntCon, SDF_OptLayout wsAfter, int ambCnt) */

SDF_Start SDF_makeStartIntCon(SDF_OptLayout wsBefore, SDF_IntCon topIntCon, SDF_OptLayout wsAfter, int ambCnt)
{
  return (SDF_Start)(ATerm)ATmakeAppl2(SDF_afun43, (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun18))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun44)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) wsAfter), (ATerm) topIntCon), (ATerm) wsBefore)), (ATerm) (ATerm) ATmakeInt(ambCnt));
}

/*}}}  */
/*{{{  SDF_Start SDF_makeStartRealCon(SDF_OptLayout wsBefore, SDF_RealCon topRealCon, SDF_OptLayout wsAfter, int ambCnt) */

SDF_Start SDF_makeStartRealCon(SDF_OptLayout wsBefore, SDF_RealCon topRealCon, SDF_OptLayout wsAfter, int ambCnt)
{
  return (SDF_Start)(ATerm)ATmakeAppl2(SDF_afun43, (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun21))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun44)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) wsAfter), (ATerm) topRealCon), (ATerm) wsBefore)), (ATerm) (ATerm) ATmakeInt(ambCnt));
}

/*}}}  */
/*{{{  SDF_Start SDF_makeStartModule(SDF_OptLayout wsBefore, SDF_Module topModule, SDF_OptLayout wsAfter, int ambCnt) */

SDF_Start SDF_makeStartModule(SDF_OptLayout wsBefore, SDF_Module topModule, SDF_OptLayout wsAfter, int ambCnt)
{
  return (SDF_Start)(ATerm)ATmakeAppl2(SDF_afun43, (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun45))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun44)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) wsAfter), (ATerm) topModule), (ATerm) wsBefore)), (ATerm) (ATerm) ATmakeInt(ambCnt));
}

/*}}}  */
/*{{{  SDF_Start SDF_makeStartSDF(SDF_OptLayout wsBefore, SDF_SDF topSDF, SDF_OptLayout wsAfter, int ambCnt) */

SDF_Start SDF_makeStartSDF(SDF_OptLayout wsBefore, SDF_SDF topSDF, SDF_OptLayout wsAfter, int ambCnt)
{
  return (SDF_Start)(ATerm)ATmakeAppl2(SDF_afun43, (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun46))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun44)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) wsAfter), (ATerm) topSDF), (ATerm) wsBefore)), (ATerm) (ATerm) ATmakeInt(ambCnt));
}

/*}}}  */
/*{{{  SDF_ATermArgs SDF_makeATermArgsEmpty(void) */

SDF_ATermArgs SDF_makeATermArgsEmpty(void)
{
  return (SDF_ATermArgs)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_ATermArgs SDF_makeATermArgsSingle(SDF_ATerm head) */

SDF_ATermArgs SDF_makeATermArgsSingle(SDF_ATerm head)
{
  return (SDF_ATermArgs)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_ATermArgs SDF_makeATermArgsMany(SDF_ATerm head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATermArgs tail) */

SDF_ATermArgs SDF_makeATermArgsMany(SDF_ATerm head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATermArgs tail)
{
  if (SDF_isATermArgsEmpty(tail)) {
    return SDF_makeATermArgsSingle(head);
  }
  return (SDF_ATermArgs)(ATerm)ATinsert(ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterSep), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_ATermElems SDF_makeATermElemsEmpty(void) */

SDF_ATermElems SDF_makeATermElemsEmpty(void)
{
  return (SDF_ATermElems)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_ATermElems SDF_makeATermElemsSingle(SDF_ATerm head) */

SDF_ATermElems SDF_makeATermElemsSingle(SDF_ATerm head)
{
  return (SDF_ATermElems)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_ATermElems SDF_makeATermElemsMany(SDF_ATerm head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATermElems tail) */

SDF_ATermElems SDF_makeATermElemsMany(SDF_ATerm head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATermElems tail)
{
  if (SDF_isATermElemsEmpty(tail)) {
    return SDF_makeATermElemsSingle(head);
  }
  return (SDF_ATermElems)(ATerm)ATinsert(ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterSep), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_ATermAnnos SDF_makeATermAnnosEmpty(void) */

SDF_ATermAnnos SDF_makeATermAnnosEmpty(void)
{
  return (SDF_ATermAnnos)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_ATermAnnos SDF_makeATermAnnosSingle(SDF_ATerm head) */

SDF_ATermAnnos SDF_makeATermAnnosSingle(SDF_ATerm head)
{
  return (SDF_ATermAnnos)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_ATermAnnos SDF_makeATermAnnosMany(SDF_ATerm head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATermAnnos tail) */

SDF_ATermAnnos SDF_makeATermAnnosMany(SDF_ATerm head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_ATermAnnos tail)
{
  if (SDF_isATermAnnosEmpty(tail)) {
    return SDF_makeATermAnnosSingle(head);
  }
  return (SDF_ATermAnnos)(ATerm)ATinsert(ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterSep), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_IntCon SDF_makeIntConNatural(SDF_NatCon NatCon) */

SDF_IntCon SDF_makeIntConNatural(SDF_NatCon NatCon)
{
  return (SDF_IntCon)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun47)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun18))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun48)))))), (ATerm)ATmakeList1((ATerm) NatCon));
}

/*}}}  */
/*{{{  SDF_IntCon SDF_makeIntConPositive(SDF_OptLayout wsAfterPos, SDF_NatCon NatCon) */

SDF_IntCon SDF_makeIntConPositive(SDF_OptLayout wsAfterPos, SDF_NatCon NatCon)
{
  return (SDF_IntCon)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun47)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun49))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun18))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun50)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) NatCon), (ATerm) wsAfterPos), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(43)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun49)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(43)))));
}

/*}}}  */
/*{{{  SDF_IntCon SDF_makeIntConNegative(SDF_OptLayout wsAfterNeg, SDF_NatCon NatCon) */

SDF_IntCon SDF_makeIntConNegative(SDF_OptLayout wsAfterNeg, SDF_NatCon NatCon)
{
  return (SDF_IntCon)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun47)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun51))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun18))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun52)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) NatCon), (ATerm) wsAfterNeg), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun51)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))));
}

/*}}}  */
/*{{{  SDF_OptExp SDF_makeOptExpPresent(SDF_OptLayout wsAfterE, SDF_IntCon IntCon) */

SDF_OptExp SDF_makeOptExpPresent(SDF_OptLayout wsAfterE, SDF_IntCon IntCon)
{
  return (SDF_OptExp)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun18)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun53))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun54))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun55)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) IntCon), (ATerm) wsAfterE), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun53)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))));
}

/*}}}  */
/*{{{  SDF_OptExp SDF_makeOptExpAbsent(void) */

SDF_OptExp SDF_makeOptExpAbsent(void)
{
  return (SDF_OptExp)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATempty, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun54))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun56)))))), (ATerm)ATempty);
}

/*}}}  */
/*{{{  SDF_RealCon SDF_makeRealConRealCon(SDF_IntCon base, SDF_OptLayout wsAfterBase, SDF_OptLayout wsAfterPeriod, SDF_NatCon decimal, SDF_OptLayout wsAfterDecimal, SDF_OptExp exp) */

SDF_RealCon SDF_makeRealConRealCon(SDF_IntCon base, SDF_OptLayout wsAfterBase, SDF_OptLayout wsAfterPeriod, SDF_NatCon decimal, SDF_OptLayout wsAfterDecimal, SDF_OptExp exp)
{
  return (SDF_RealCon)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun54)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun47)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun57))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun18)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun21))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun58)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) exp), (ATerm) wsAfterDecimal), (ATerm) decimal), (ATerm) wsAfterPeriod), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(46)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun57)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(46)))), (ATerm) wsAfterBase), (ATerm) base));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarAliases(SDF_OptLayout wsAfterAliases, SDF_Aliases Aliases) */

SDF_Grammar SDF_makeGrammarAliases(SDF_OptLayout wsAfterAliases, SDF_Aliases Aliases)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun59)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun60))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun60)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) Aliases), (ATerm) wsAfterAliases), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun60)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(108)), (ATerm)ATmakeInt(97)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarLexicalSyntax(SDF_OptLayout wsAfterLexical, SDF_OptLayout wsAfterSyntax, SDF_Productions Productions) */

SDF_Grammar SDF_makeGrammarLexicalSyntax(SDF_OptLayout wsAfterLexical, SDF_OptLayout wsAfterSyntax, SDF_Productions Productions)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun62)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun63))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun64))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun65)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) Productions), (ATerm) wsAfterSyntax), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(120)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(121)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun63)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(120)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(110)), (ATerm)ATmakeInt(121)), (ATerm)ATmakeInt(115)))), (ATerm) wsAfterLexical), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(120)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun64)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(108)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(99)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(120)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(108)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarContextFreeSyntax(SDF_OptLayout wsAfterContextFree, SDF_OptLayout wsAfterSyntax, SDF_Productions Productions) */

SDF_Grammar SDF_makeGrammarContextFreeSyntax(SDF_OptLayout wsAfterContextFree, SDF_OptLayout wsAfterSyntax, SDF_Productions Productions)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun62)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun63))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun66))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun67)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) Productions), (ATerm) wsAfterSyntax), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(120)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(121)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun63)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(120)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(110)), (ATerm)ATmakeInt(121)), (ATerm)ATmakeInt(115)))), (ATerm) wsAfterContextFree), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(102)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(120)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun66)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(102)), (ATerm)ATmakeInt(45)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(120)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(110)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(99)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarVariables(SDF_OptLayout wsAfterVariables, SDF_Productions Productions) */

SDF_Grammar SDF_makeGrammarVariables(SDF_OptLayout wsAfterVariables, SDF_Productions Productions)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun62)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun68))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun68)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) Productions), (ATerm) wsAfterVariables), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(98)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(118)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun68)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(108)), (ATerm)ATmakeInt(98)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(118)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarLexicalVariables(SDF_OptLayout wsAfterLexical, SDF_OptLayout wsAfterVariables, SDF_Productions Productions) */

SDF_Grammar SDF_makeGrammarLexicalVariables(SDF_OptLayout wsAfterLexical, SDF_OptLayout wsAfterVariables, SDF_Productions Productions)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun62)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun68))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun64))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun69)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) Productions), (ATerm) wsAfterVariables), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(98)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(118)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun68)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(108)), (ATerm)ATmakeInt(98)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(118)))), (ATerm) wsAfterLexical), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(120)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun64)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(108)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(99)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(120)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(108)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarEmptyGrammar(void) */

SDF_Grammar SDF_makeGrammarEmptyGrammar(void)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun70))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun71)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(47)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun70)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(41)), (ATerm)ATmakeInt(47)), (ATerm)ATmakeInt(40)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarConcGrammars(SDF_Grammar left, SDF_OptLayout wsAfterLeft, SDF_Grammar right) */

SDF_Grammar SDF_makeGrammarConcGrammars(SDF_Grammar left, SDF_OptLayout wsAfterLeft, SDF_Grammar right)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun72)))), (ATerm)ATmakeAppl1(SDF_afun73, (ATerm)ATmakeAppl0(SDF_afun74))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) right), (ATerm) wsAfterLeft), (ATerm) left));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarBracket(SDF_OptLayout wsAfterParenOpen, SDF_Grammar Grammar, SDF_OptLayout wsAfterGrammar) */

SDF_Grammar SDF_makeGrammarBracket(SDF_OptLayout wsAfterParenOpen, SDF_Grammar Grammar, SDF_OptLayout wsAfterGrammar)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl0(SDF_afun75)))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm) wsAfterGrammar), (ATerm) Grammar), (ATerm) wsAfterParenOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarSyntax(SDF_OptLayout wsAfterSyntax, SDF_Productions Productions) */

SDF_Grammar SDF_makeGrammarSyntax(SDF_OptLayout wsAfterSyntax, SDF_Productions Productions)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun62)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun63))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun63)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) Productions), (ATerm) wsAfterSyntax), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(120)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(121)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun63)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(120)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(110)), (ATerm)ATmakeInt(121)), (ATerm)ATmakeInt(115)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarImpSection(SDF_ImpSection ImpSection) */

SDF_Grammar SDF_makeGrammarImpSection(SDF_ImpSection ImpSection)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun76)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun77)))))), (ATerm)ATmakeList1((ATerm) ImpSection));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarPriorities(SDF_OptLayout wsAfterPriorities, SDF_Priorities Priorities) */

SDF_Grammar SDF_makeGrammarPriorities(SDF_OptLayout wsAfterPriorities, SDF_Priorities Priorities)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun78)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun79))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun79)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) Priorities), (ATerm) wsAfterPriorities), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(112)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun79)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(112)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarRestrictions(SDF_OptLayout wsAfterRestrictions, SDF_Restrictions Restrictions) */

SDF_Grammar SDF_makeGrammarRestrictions(SDF_OptLayout wsAfterRestrictions, SDF_Restrictions Restrictions)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun80)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun81))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun81)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) Restrictions), (ATerm) wsAfterRestrictions), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun81)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(110)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(99)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(114)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarLexicalPriorities(SDF_OptLayout wsAfterLexical, SDF_OptLayout wsAfterPriorities, SDF_Priorities Priorities) */

SDF_Grammar SDF_makeGrammarLexicalPriorities(SDF_OptLayout wsAfterLexical, SDF_OptLayout wsAfterPriorities, SDF_Priorities Priorities)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun78)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun79))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun64))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun82)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) Priorities), (ATerm) wsAfterPriorities), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(112)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun79)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(112)))), (ATerm) wsAfterLexical), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(120)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun64)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(108)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(99)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(120)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(108)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarContextFreePriorities(SDF_OptLayout wsAfterContextFree, SDF_OptLayout wsAfterPriorities, SDF_Priorities Priorities) */

SDF_Grammar SDF_makeGrammarContextFreePriorities(SDF_OptLayout wsAfterContextFree, SDF_OptLayout wsAfterPriorities, SDF_Priorities Priorities)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun78)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun79))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun66))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun83)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) Priorities), (ATerm) wsAfterPriorities), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(112)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun79)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(112)))), (ATerm) wsAfterContextFree), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(102)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(120)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun66)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(102)), (ATerm)ATmakeInt(45)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(120)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(110)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(99)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarLexicalRestrictions(SDF_OptLayout wsAfterLexical, SDF_OptLayout wsAfterRestrictions, SDF_Restrictions Restrictions) */

SDF_Grammar SDF_makeGrammarLexicalRestrictions(SDF_OptLayout wsAfterLexical, SDF_OptLayout wsAfterRestrictions, SDF_Restrictions Restrictions)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun80)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun81))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun64))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun84)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) Restrictions), (ATerm) wsAfterRestrictions), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun81)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(110)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(99)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(114)))), (ATerm) wsAfterLexical), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(120)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun64)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(108)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(99)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(120)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(108)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarContextFreeRestrictions(SDF_OptLayout wsAfterContextFree, SDF_OptLayout wsAfterRestrictions, SDF_Restrictions Restrictions) */

SDF_Grammar SDF_makeGrammarContextFreeRestrictions(SDF_OptLayout wsAfterContextFree, SDF_OptLayout wsAfterRestrictions, SDF_Restrictions Restrictions)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun80)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun81))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun66))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun85)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) Restrictions), (ATerm) wsAfterRestrictions), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun81)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(110)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(99)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(114)))), (ATerm) wsAfterContextFree), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(102)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(120)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun66)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(102)), (ATerm)ATmakeInt(45)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(120)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(110)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(99)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarSorts(SDF_OptLayout wsAfterSorts, SDF_Symbols Symbols) */

SDF_Grammar SDF_makeGrammarSorts(SDF_OptLayout wsAfterSorts, SDF_Symbols Symbols)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun86)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun87))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun87)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) Symbols), (ATerm) wsAfterSorts), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun87)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(115)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarKernelStartSymbols(SDF_OptLayout wsAfterStartSymbols, SDF_Symbols Symbols) */

SDF_Grammar SDF_makeGrammarKernelStartSymbols(SDF_OptLayout wsAfterStartSymbols, SDF_Symbols Symbols)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun86)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun88))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun89)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) Symbols), (ATerm) wsAfterStartSymbols), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(98)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(109)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(121)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun88)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(108)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(98)), (ATerm)ATmakeInt(109)), (ATerm)ATmakeInt(121)), (ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(45)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(115)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarLexicalStartSymbols(SDF_OptLayout wsAfterLexical, SDF_OptLayout wsAfterStartSymbols, SDF_Symbols Symbols) */

SDF_Grammar SDF_makeGrammarLexicalStartSymbols(SDF_OptLayout wsAfterLexical, SDF_OptLayout wsAfterStartSymbols, SDF_Symbols Symbols)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun86)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun88))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun64))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun90)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) Symbols), (ATerm) wsAfterStartSymbols), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(98)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(109)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(121)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun88)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(108)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(98)), (ATerm)ATmakeInt(109)), (ATerm)ATmakeInt(121)), (ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(45)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(115)))), (ATerm) wsAfterLexical), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(120)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun64)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(108)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(99)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(120)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(108)))));
}

/*}}}  */
/*{{{  SDF_Grammar SDF_makeGrammarContextFreeStartSymbols(SDF_OptLayout wsAfterContextFree, SDF_OptLayout wsAfterStartSymbols, SDF_Symbols Symbols) */

SDF_Grammar SDF_makeGrammarContextFreeStartSymbols(SDF_OptLayout wsAfterContextFree, SDF_OptLayout wsAfterStartSymbols, SDF_Symbols Symbols)
{
  return (SDF_Grammar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun86)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun88))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun66))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun91)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) Symbols), (ATerm) wsAfterStartSymbols), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(98)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(109)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(121)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun88)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(108)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(98)), (ATerm)ATmakeInt(109)), (ATerm)ATmakeInt(121)), (ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(45)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(115)))), (ATerm) wsAfterContextFree), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(102)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(120)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun66)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(102)), (ATerm)ATmakeInt(45)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(120)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(110)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(99)))));
}

/*}}}  */
/*{{{  SDF_Alias SDF_makeAliasAlias(SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol, SDF_OptLayout wsAfterArrow, SDF_Symbol alias) */

SDF_Alias SDF_makeAliasAlias(SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol, SDF_OptLayout wsAfterArrow, SDF_Symbol alias)
{
  return (SDF_Alias)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun93))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun94))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun95)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) alias), (ATerm) wsAfterArrow), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun93)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(62)), (ATerm)ATmakeInt(45)))), (ATerm) wsAfterSymbol), (ATerm) Symbol));
}

/*}}}  */
/*{{{  SDF_Aliases SDF_makeAliasesDefault(SDF_AliasList list) */

SDF_Aliases SDF_makeAliasesDefault(SDF_AliasList list)
{
  return (SDF_Aliases)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun94))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun59))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun94))))), (ATerm) list)));
}

/*}}}  */
/*{{{  SDF_AliasList SDF_makeAliasListEmpty(void) */

SDF_AliasList SDF_makeAliasListEmpty(void)
{
  return (SDF_AliasList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_AliasList SDF_makeAliasListSingle(SDF_Alias head) */

SDF_AliasList SDF_makeAliasListSingle(SDF_Alias head)
{
  return (SDF_AliasList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_AliasList SDF_makeAliasListMany(SDF_Alias head, SDF_OptLayout wsAfterHead, SDF_AliasList tail) */

SDF_AliasList SDF_makeAliasListMany(SDF_Alias head, SDF_OptLayout wsAfterHead, SDF_AliasList tail)
{
  if (SDF_isAliasListEmpty(tail)) {
    return SDF_makeAliasListSingle(head);
  }
  return (SDF_AliasList)(ATerm)ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolCf(SDF_OptLayout wsAfterLessThan, SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol, SDF_OptLayout wsAfterCF) */

SDF_Symbol SDF_makeSymbolCf(SDF_OptLayout wsAfterLessThan, SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol, SDF_OptLayout wsAfterCF)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun31))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun97))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun32))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun98)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun31)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm) wsAfterCF), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(70)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(67)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun97)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(70)), (ATerm)ATmakeInt(67)), (ATerm)ATmakeInt(45)))), (ATerm) wsAfterSymbol), (ATerm) Symbol), (ATerm) wsAfterLessThan), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(60)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun32)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(60)))));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolLex(SDF_OptLayout wsAfterLessThan, SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol, SDF_OptLayout wsAfterLEX) */

SDF_Symbol SDF_makeSymbolLex(SDF_OptLayout wsAfterLessThan, SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol, SDF_OptLayout wsAfterLEX)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun31))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun99))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun32))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun100)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun31)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm) wsAfterLEX), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(88)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(69)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(76)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun99)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(88)), (ATerm)ATmakeInt(69)), (ATerm)ATmakeInt(76)), (ATerm)ATmakeInt(45)))), (ATerm) wsAfterSymbol), (ATerm) Symbol), (ATerm) wsAfterLessThan), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(60)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun32)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(60)))));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolVarsym(SDF_OptLayout wsAfterLessThan, SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol, SDF_OptLayout wsAfterVAR) */

SDF_Symbol SDF_makeSymbolVarsym(SDF_OptLayout wsAfterLessThan, SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol, SDF_OptLayout wsAfterVAR)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun31))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun101))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun32))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun102)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun31)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm) wsAfterVAR), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(82)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(65)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(86)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun101)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(82)), (ATerm)ATmakeInt(65)), (ATerm)ATmakeInt(86)), (ATerm)ATmakeInt(45)))), (ATerm) wsAfterSymbol), (ATerm) Symbol), (ATerm) wsAfterLessThan), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(60)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun32)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(60)))));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolLayout(void) */

SDF_Symbol SDF_makeSymbolLayout(void)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun103))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun104)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(84)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(85)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(79)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(89)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(65)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(76)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun103)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(84)), (ATerm)ATmakeInt(85)), (ATerm)ATmakeInt(79)), (ATerm)ATmakeInt(89)), (ATerm)ATmakeInt(65)), (ATerm)ATmakeInt(76)))));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolCharClass(SDF_CharClass CharClass) */

SDF_Symbol SDF_makeSymbolCharClass(SDF_CharClass CharClass)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun106)))))), (ATerm)ATmakeList1((ATerm) CharClass));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolLabel(SDF_Label Label, SDF_OptLayout wsAfterLabel, SDF_OptLayout wsAfterColon, SDF_Symbol Symbol) */

SDF_Symbol SDF_makeSymbolLabel(SDF_Label Label, SDF_OptLayout wsAfterLabel, SDF_OptLayout wsAfterColon, SDF_Symbol Symbol)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun107))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun108)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun109)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) Symbol), (ATerm) wsAfterColon), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(58)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun107)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(58)))), (ATerm) wsAfterLabel), (ATerm) Label));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolLifting(SDF_OptLayout wsAfterLeftQuote, SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol) */

SDF_Symbol SDF_makeSymbolLifting(SDF_OptLayout wsAfterLeftQuote, SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun110))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun110))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun111)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(96)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun110)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(96)))), (ATerm) wsAfterSymbol), (ATerm) Symbol), (ATerm) wsAfterLeftQuote), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(96)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun110)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(96)))));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolLit(SDF_StrCon string) */

SDF_Symbol SDF_makeSymbolLit(SDF_StrCon string)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun10)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun112)))))), (ATerm)ATmakeList1((ATerm) string));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolEmpty(SDF_OptLayout wsAfterParenOpen) */

SDF_Symbol SDF_makeSymbolEmpty(SDF_OptLayout wsAfterParenOpen)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun113)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm) wsAfterParenOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolSeq(SDF_OptLayout wsAfterParenOpen, SDF_Symbol head, SDF_OptLayout wsAfterHead, SDF_SymbolTail tail, SDF_OptLayout wsAfterTail) */

SDF_Symbol SDF_makeSymbolSeq(SDF_OptLayout wsAfterParenOpen, SDF_Symbol head, SDF_OptLayout wsAfterHead, SDF_SymbolTail tail, SDF_OptLayout wsAfterTail)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun7, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun114)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm) wsAfterTail), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun7, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))))), (ATerm) tail)), (ATerm) wsAfterHead), (ATerm) head), (ATerm) wsAfterParenOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolOpt(SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol) */

SDF_Symbol SDF_makeSymbolOpt(SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun115))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun116)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(63)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun115)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(63)))), (ATerm) wsAfterSymbol), (ATerm) Symbol));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolIter(SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol) */

SDF_Symbol SDF_makeSymbolIter(SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun49))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun117)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(43)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun49)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(43)))), (ATerm) wsAfterSymbol), (ATerm) Symbol));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolIterStar(SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol) */

SDF_Symbol SDF_makeSymbolIterStar(SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun118))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun119)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(42)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun118)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(42)))), (ATerm) wsAfterSymbol), (ATerm) Symbol));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolIterSep(SDF_OptLayout wsAfterBraceOpen, SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol, SDF_Symbol sep, SDF_OptLayout wsAfterSep, SDF_OptLayout wsAfterBraceClose) */

SDF_Symbol SDF_makeSymbolIterSep(SDF_OptLayout wsAfterBraceOpen, SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol, SDF_Symbol sep, SDF_OptLayout wsAfterSep, SDF_OptLayout wsAfterBraceClose)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun49))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun40))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun41))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun120)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(43)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun49)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(43)))), (ATerm) wsAfterBraceClose), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(125)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun40)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(125)))), (ATerm) wsAfterSep), (ATerm) sep), (ATerm) wsAfterSymbol), (ATerm) Symbol), (ATerm) wsAfterBraceOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(123)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun41)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(123)))));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolIterStarSep(SDF_OptLayout wsAfterBraceOpen, SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol, SDF_Symbol sep, SDF_OptLayout wsAfterSep, SDF_OptLayout wsAfterBraceClose) */

SDF_Symbol SDF_makeSymbolIterStarSep(SDF_OptLayout wsAfterBraceOpen, SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol, SDF_Symbol sep, SDF_OptLayout wsAfterSep, SDF_OptLayout wsAfterBraceClose)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun118))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun40))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun41))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun121)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(42)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun118)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(42)))), (ATerm) wsAfterBraceClose), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(125)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun40)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(125)))), (ATerm) wsAfterSep), (ATerm) sep), (ATerm) wsAfterSymbol), (ATerm) Symbol), (ATerm) wsAfterBraceOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(123)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun41)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(123)))));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolTuple(SDF_OptLayout wsAfterLessThan, SDF_Symbol head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterComma, SDF_SymbolRest rest, SDF_OptLayout wsAfterRest) */

SDF_Symbol SDF_makeSymbolTuple(SDF_OptLayout wsAfterLessThan, SDF_Symbol head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterComma, SDF_SymbolRest rest, SDF_OptLayout wsAfterRest)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun31))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun26, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun32))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun122)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun31)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm) wsAfterRest), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun26, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm) rest)), (ATerm) wsAfterComma), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm) wsAfterHead), (ATerm) head), (ATerm) wsAfterLessThan), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(60)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun32)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(60)))));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolPair(SDF_Symbol left, SDF_OptLayout wsAfterLeft, SDF_OptLayout wsAfterHash, SDF_Symbol right) */

SDF_Symbol SDF_makeSymbolPair(SDF_Symbol left, SDF_OptLayout wsAfterLeft, SDF_OptLayout wsAfterHash, SDF_Symbol right)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun123))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun124)))), (ATerm)ATmakeAppl1(SDF_afun73, (ATerm)ATmakeAppl0(SDF_afun125))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) right), (ATerm) wsAfterHash), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(35)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun123)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(35)))), (ATerm) wsAfterLeft), (ATerm) left));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolFunc(SDF_OptLayout wsAfterParenOpen, SDF_Symbols arguments, SDF_OptLayout wsAfterArguments, SDF_OptLayout wsAfterEqualsGreaterThan, SDF_Symbol results, SDF_OptLayout wsAfterResults) */

SDF_Symbol SDF_makeSymbolFunc(SDF_OptLayout wsAfterParenOpen, SDF_Symbols arguments, SDF_OptLayout wsAfterArguments, SDF_OptLayout wsAfterEqualsGreaterThan, SDF_Symbol results, SDF_OptLayout wsAfterResults)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun126))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun86)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun127)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm) wsAfterResults), (ATerm) results), (ATerm) wsAfterEqualsGreaterThan), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(61)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun126)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(62)), (ATerm)ATmakeInt(61)))), (ATerm) wsAfterArguments), (ATerm) arguments), (ATerm) wsAfterParenOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolAlt(SDF_Symbol left, SDF_OptLayout wsAfterLeft, SDF_OptLayout wsAfterBar, SDF_Symbol right) */

SDF_Symbol SDF_makeSymbolAlt(SDF_Symbol left, SDF_OptLayout wsAfterLeft, SDF_OptLayout wsAfterBar, SDF_Symbol right)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun128))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun129)))), (ATerm)ATmakeAppl1(SDF_afun73, (ATerm)ATmakeAppl0(SDF_afun125))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) right), (ATerm) wsAfterBar), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(124)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun128)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(124)))), (ATerm) wsAfterLeft), (ATerm) left));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolStrategy(SDF_OptLayout wsAfterParenOpen, SDF_Symbol left, SDF_OptLayout wsAfterLeft, SDF_OptLayout wsAfterGreaterThan, SDF_Symbol right, SDF_OptLayout wsAfterRight) */

SDF_Symbol SDF_makeSymbolStrategy(SDF_OptLayout wsAfterParenOpen, SDF_Symbol left, SDF_OptLayout wsAfterLeft, SDF_OptLayout wsAfterGreaterThan, SDF_Symbol right, SDF_OptLayout wsAfterRight)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun93))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun130)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm) wsAfterRight), (ATerm) right), (ATerm) wsAfterGreaterThan), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun93)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(62)), (ATerm)ATmakeInt(45)))), (ATerm) wsAfterLeft), (ATerm) left), (ATerm) wsAfterParenOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolStart(void) */

SDF_Symbol SDF_makeSymbolStart(void)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun44))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun131)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(84)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(82)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(65)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(84)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(83)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(60)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun44)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(62)), (ATerm)ATmakeInt(84)), (ATerm)ATmakeInt(82)), (ATerm)ATmakeInt(65)), (ATerm)ATmakeInt(84)), (ATerm)ATmakeInt(83)), (ATerm)ATmakeInt(60)))));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolFileStart(void) */

SDF_Symbol SDF_makeSymbolFileStart(void)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun132))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun133)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(83)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(60)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun132)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(62)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(83)), (ATerm)ATmakeInt(60)))));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolSort(SDF_Sort Sort) */

SDF_Symbol SDF_makeSymbolSort(SDF_Sort Sort)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun134)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun135)))))), (ATerm)ATmakeList1((ATerm) Sort));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolParameterizedSort(SDF_Sort Sort, SDF_OptLayout wsAfterSort, SDF_OptLayout wsAfterBracketOpenBracketOpen, SDF_SymbolParameters parameters, SDF_OptLayout wsAfterParameters) */

SDF_Symbol SDF_makeSymbolParameterizedSort(SDF_Sort Sort, SDF_OptLayout wsAfterSort, SDF_OptLayout wsAfterBracketOpenBracketOpen, SDF_SymbolParameters parameters, SDF_OptLayout wsAfterParameters)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun136))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun26, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun137))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun134)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun138)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(93)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(93)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun136)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(93)), (ATerm)ATmakeInt(93)))), (ATerm) wsAfterParameters), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun26, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm) parameters)), (ATerm) wsAfterBracketOpenBracketOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(91)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(91)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun137)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(91)), (ATerm)ATmakeInt(91)))), (ATerm) wsAfterSort), (ATerm) Sort));
}

/*}}}  */
/*{{{  SDF_Symbol SDF_makeSymbolBracket(SDF_OptLayout wsAfterParenOpen, SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol) */

SDF_Symbol SDF_makeSymbolBracket(SDF_OptLayout wsAfterParenOpen, SDF_Symbol Symbol, SDF_OptLayout wsAfterSymbol)
{
  return (SDF_Symbol)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl0(SDF_afun75)))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm) wsAfterSymbol), (ATerm) Symbol), (ATerm) wsAfterParenOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))));
}

/*}}}  */
/*{{{  SDF_CharRange SDF_makeCharRangeDefault(SDF_Character Character) */

SDF_CharRange SDF_makeCharRangeDefault(SDF_Character Character)
{
  return (SDF_CharRange)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun139)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun140))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm) Character));
}

/*}}}  */
/*{{{  SDF_CharRange SDF_makeCharRangeRange(SDF_Character start, SDF_OptLayout wsAfterStart, SDF_OptLayout wsAfter, SDF_Character end) */

SDF_CharRange SDF_makeCharRangeRange(SDF_Character start, SDF_OptLayout wsAfterStart, SDF_OptLayout wsAfter, SDF_Character end)
{
  return (SDF_CharRange)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun139)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun51))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun139)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun140))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun141)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) end), (ATerm) wsAfter), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun51)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm) wsAfterStart), (ATerm) start));
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_makeCharRangesDefault(SDF_CharRange CharRange) */

SDF_CharRanges SDF_makeCharRangesDefault(SDF_CharRange CharRange)
{
  return (SDF_CharRanges)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun140)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun142))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm) CharRange));
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_makeCharRangesConc(SDF_CharRanges left, SDF_OptLayout wsAfterLeft, SDF_CharRanges right) */

SDF_CharRanges SDF_makeCharRangesConc(SDF_CharRanges left, SDF_OptLayout wsAfterLeft, SDF_CharRanges right)
{
  return (SDF_CharRanges)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun142)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun142)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun142))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl0(SDF_afun143))), (ATerm)ATmakeAppl1(SDF_afun73, (ATerm)ATmakeAppl0(SDF_afun125))), (ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun144)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) right), (ATerm) wsAfterLeft), (ATerm) left));
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_makeCharRangesBracket(SDF_OptLayout wsAfterParenOpen, SDF_CharRanges CharRanges, SDF_OptLayout wsAfterCharRanges) */

SDF_CharRanges SDF_makeCharRangesBracket(SDF_OptLayout wsAfterParenOpen, SDF_CharRanges CharRanges, SDF_OptLayout wsAfterCharRanges)
{
  return (SDF_CharRanges)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun142)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun142))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl0(SDF_afun75)))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm) wsAfterCharRanges), (ATerm) CharRanges), (ATerm) wsAfterParenOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))));
}

/*}}}  */
/*{{{  SDF_OptCharRanges SDF_makeOptCharRangesAbsent(void) */

SDF_OptCharRanges SDF_makeOptCharRangesAbsent(void)
{
  return (SDF_OptCharRanges)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATempty, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun145))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun56)))))), (ATerm)ATempty);
}

/*}}}  */
/*{{{  SDF_OptCharRanges SDF_makeOptCharRangesPresent(SDF_CharRanges CharRanges) */

SDF_OptCharRanges SDF_makeOptCharRangesPresent(SDF_CharRanges CharRanges)
{
  return (SDF_OptCharRanges)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun142)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun145))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun55)))))), (ATerm)ATmakeList1((ATerm) CharRanges));
}

/*}}}  */
/*{{{  SDF_CharClass SDF_makeCharClassSimpleCharclass(SDF_OptLayout wsAfterBracketOpen, SDF_OptCharRanges OptCharRanges, SDF_OptLayout wsAfterOptCharRanges) */

SDF_CharClass SDF_makeCharClassSimpleCharclass(SDF_OptLayout wsAfterBracketOpen, SDF_OptCharRanges OptCharRanges, SDF_OptLayout wsAfterOptCharRanges)
{
  return (SDF_CharClass)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun34))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun145)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun36))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun146)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(93)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun34)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(93)))), (ATerm) wsAfterOptCharRanges), (ATerm) OptCharRanges), (ATerm) wsAfterBracketOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(91)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun36)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(91)))));
}

/*}}}  */
/*{{{  SDF_CharClass SDF_makeCharClassComp(SDF_OptLayout wsAfterTilde, SDF_CharClass CharClass) */

SDF_CharClass SDF_makeCharClassComp(SDF_OptLayout wsAfterTilde, SDF_CharClass CharClass)
{
  return (SDF_CharClass)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun147))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun148)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) CharClass), (ATerm) wsAfterTilde), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(126)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun147)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(126)))));
}

/*}}}  */
/*{{{  SDF_CharClass SDF_makeCharClassDiff(SDF_CharClass left, SDF_OptLayout wsAfterLeft, SDF_OptLayout wsAfterSlash, SDF_CharClass right) */

SDF_CharClass SDF_makeCharClassDiff(SDF_CharClass left, SDF_OptLayout wsAfterLeft, SDF_OptLayout wsAfterSlash, SDF_CharClass right)
{
  return (SDF_CharClass)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun149))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl0(SDF_afun143))), (ATerm)ATmakeAppl1(SDF_afun73, (ATerm)ATmakeAppl0(SDF_afun150))), (ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun151)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) right), (ATerm) wsAfterSlash), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(47)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun149)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(47)))), (ATerm) wsAfterLeft), (ATerm) left));
}

/*}}}  */
/*{{{  SDF_CharClass SDF_makeCharClassIsect(SDF_CharClass left, SDF_OptLayout wsAfterLeft, SDF_OptLayout wsAfterSlashBackslash, SDF_CharClass right) */

SDF_CharClass SDF_makeCharClassIsect(SDF_CharClass left, SDF_OptLayout wsAfterLeft, SDF_OptLayout wsAfterSlashBackslash, SDF_CharClass right)
{
  return (SDF_CharClass)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun152))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl0(SDF_afun143))), (ATerm)ATmakeAppl1(SDF_afun73, (ATerm)ATmakeAppl0(SDF_afun150))), (ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun153)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) right), (ATerm) wsAfterSlashBackslash), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(92)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(47)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun152)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(92)), (ATerm)ATmakeInt(47)))), (ATerm) wsAfterLeft), (ATerm) left));
}

/*}}}  */
/*{{{  SDF_CharClass SDF_makeCharClassUnion(SDF_CharClass left, SDF_OptLayout wsAfterLeft, SDF_OptLayout wsAfterBackslashSlash, SDF_CharClass right) */

SDF_CharClass SDF_makeCharClassUnion(SDF_CharClass left, SDF_OptLayout wsAfterLeft, SDF_OptLayout wsAfterBackslashSlash, SDF_CharClass right)
{
  return (SDF_CharClass)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun154))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun73, (ATerm)ATmakeAppl0(SDF_afun150))), (ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun155)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) right), (ATerm) wsAfterBackslashSlash), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(47)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(92)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun154)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(47)), (ATerm)ATmakeInt(92)))), (ATerm) wsAfterLeft), (ATerm) left));
}

/*}}}  */
/*{{{  SDF_CharClass SDF_makeCharClassBracket(SDF_OptLayout wsAfterParenOpen, SDF_CharClass CharClass, SDF_OptLayout wsAfterCharClass) */

SDF_CharClass SDF_makeCharClassBracket(SDF_OptLayout wsAfterParenOpen, SDF_CharClass CharClass, SDF_OptLayout wsAfterCharClass)
{
  return (SDF_CharClass)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl0(SDF_afun156)), (ATerm)ATmakeAppl0(SDF_afun75)))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm) wsAfterCharClass), (ATerm) CharClass), (ATerm) wsAfterParenOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))));
}

/*}}}  */
/*{{{  SDF_LexNumChar SDF_makeLexNumCharDigits(const char* number) */

SDF_LexNumChar SDF_makeLexNumCharDigits(const char* number)
{
  return (SDF_LexNumChar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun7, (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(48), (ATerm)ATmakeInt(57))))))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(92)))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun158))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun159)))))), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun7, (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(48), (ATerm)ATmakeInt(57))))))), (ATerm) ((ATerm) SDF_stringToChars(number)))), (ATerm)ATmakeInt(92)));
}

/*}}}  */
/*{{{  SDF_NumChar SDF_makeNumCharLexToCf(SDF_LexNumChar NumChar) */

SDF_NumChar SDF_makeNumCharLexToCf(SDF_LexNumChar NumChar)
{
  return (SDF_NumChar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun158)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun158))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm) NumChar));
}

/*}}}  */
/*{{{  SDF_LexShortChar SDF_makeLexShortCharRegular(char character) */

SDF_LexShortChar SDF_makeLexShortCharRegular(char character)
{
  return (SDF_LexShortChar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(97), (ATerm)ATmakeInt(122))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(65), (ATerm)ATmakeInt(90))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(48), (ATerm)ATmakeInt(57))))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun160))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun161)))))), (ATerm)ATmakeList1((ATerm) ((ATerm) SDF_byteToChar(character))));
}

/*}}}  */
/*{{{  SDF_LexShortChar SDF_makeLexShortCharEscaped(char escape) */

SDF_LexShortChar SDF_makeLexShortCharEscaped(char escape)
{
  return (SDF_LexShortChar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(123), (ATerm)ATmakeInt(255))), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(110)), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(91), (ATerm)ATmakeInt(96))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(58), (ATerm)ATmakeInt(64))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(32), (ATerm)ATmakeInt(47))))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(92)))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun160))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun162)))))), (ATerm)ATinsert(ATmakeList1((ATerm) ((ATerm) SDF_byteToChar(escape))), (ATerm)ATmakeInt(92)));
}

/*}}}  */
/*{{{  SDF_ShortChar SDF_makeShortCharLexToCf(SDF_LexShortChar ShortChar) */

SDF_ShortChar SDF_makeShortCharLexToCf(SDF_LexShortChar ShortChar)
{
  return (SDF_ShortChar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun160)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun160))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm) ShortChar));
}

/*}}}  */
/*{{{  SDF_Character SDF_makeCharacterNumeric(SDF_NumChar NumChar) */

SDF_Character SDF_makeCharacterNumeric(SDF_NumChar NumChar)
{
  return (SDF_Character)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun158)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun139))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun163)))))), (ATerm)ATmakeList1((ATerm) NumChar));
}

/*}}}  */
/*{{{  SDF_Character SDF_makeCharacterShort(SDF_ShortChar ShortChar) */

SDF_Character SDF_makeCharacterShort(SDF_ShortChar ShortChar)
{
  return (SDF_Character)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun160)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun139))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun164)))))), (ATerm)ATmakeList1((ATerm) ShortChar));
}

/*}}}  */
/*{{{  SDF_Character SDF_makeCharacterTop(void) */

SDF_Character SDF_makeCharacterTop(void)
{
  return (SDF_Character)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun165))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun139))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun166)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(80)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(79)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(84)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(92)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun165)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(80)), (ATerm)ATmakeInt(79)), (ATerm)ATmakeInt(84)), (ATerm)ATmakeInt(92)))));
}

/*}}}  */
/*{{{  SDF_Character SDF_makeCharacterEof(void) */

SDF_Character SDF_makeCharacterEof(void)
{
  return (SDF_Character)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun167))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun139))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun168)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(70)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(79)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(69)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(92)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun167)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(70)), (ATerm)ATmakeInt(79)), (ATerm)ATmakeInt(69)), (ATerm)ATmakeInt(92)))));
}

/*}}}  */
/*{{{  SDF_Character SDF_makeCharacterBot(void) */

SDF_Character SDF_makeCharacterBot(void)
{
  return (SDF_Character)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun169))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun139))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun170)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(84)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(79)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(66)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(92)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun169)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(84)), (ATerm)ATmakeInt(79)), (ATerm)ATmakeInt(66)), (ATerm)ATmakeInt(92)))));
}

/*}}}  */
/*{{{  SDF_Character SDF_makeCharacterLabelUnderscoreStart(void) */

SDF_Character SDF_makeCharacterLabelUnderscoreStart(void)
{
  return (SDF_Character)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun171))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun139))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun172)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(84)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(82)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(65)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(84)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(83)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(95)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(76)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(69)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(66)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(65)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(76)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(92)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun171)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(84)), (ATerm)ATmakeInt(82)), (ATerm)ATmakeInt(65)), (ATerm)ATmakeInt(84)), (ATerm)ATmakeInt(83)), (ATerm)ATmakeInt(95)), (ATerm)ATmakeInt(76)), (ATerm)ATmakeInt(69)), (ATerm)ATmakeInt(66)), (ATerm)ATmakeInt(65)), (ATerm)ATmakeInt(76)), (ATerm)ATmakeInt(92)))));
}

/*}}}  */
/*{{{  SDF_ATermAttribute SDF_makeATermAttributeDefault(SDF_ATerm aterm) */

SDF_ATermAttribute SDF_makeATermAttributeDefault(SDF_ATerm aterm)
{
  return (SDF_ATermAttribute)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun19)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun173))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun42)))))), (ATerm)ATmakeList1((ATerm) aterm));
}

/*}}}  */
/*{{{  SDF_Attribute SDF_makeAttributeTerm(SDF_ATermAttribute ATermAttribute) */

SDF_Attribute SDF_makeAttributeTerm(SDF_ATermAttribute ATermAttribute)
{
  return (SDF_Attribute)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun173)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun174))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun175)))))), (ATerm)ATmakeList1((ATerm) ATermAttribute));
}

/*}}}  */
/*{{{  SDF_Attribute SDF_makeAttributeId(SDF_OptLayout wsAfterId, SDF_OptLayout wsAfterParenOpen, SDF_ModuleName ModuleName, SDF_OptLayout wsAfterModuleName) */

SDF_Attribute SDF_makeAttributeId(SDF_OptLayout wsAfterId, SDF_OptLayout wsAfterParenOpen, SDF_ModuleName ModuleName, SDF_OptLayout wsAfterModuleName)
{
  return (SDF_Attribute)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun176)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun177))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun174))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun177)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm) wsAfterModuleName), (ATerm) ModuleName), (ATerm) wsAfterParenOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm) wsAfterId), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(100)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun177)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(100)), (ATerm)ATmakeInt(105)))));
}

/*}}}  */
/*{{{  SDF_Attribute SDF_makeAttributeBracket(void) */

SDF_Attribute SDF_makeAttributeBracket(void)
{
  return (SDF_Attribute)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun178))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun174))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun178)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(107)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(98)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun178)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(107)), (ATerm)ATmakeInt(99)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(98)))));
}

/*}}}  */
/*{{{  SDF_Attribute SDF_makeAttributeAssoc(SDF_Associativity Associativity) */

SDF_Attribute SDF_makeAttributeAssoc(SDF_Associativity Associativity)
{
  return (SDF_Attribute)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun179)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun174))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun180)))))), (ATerm)ATmakeList1((ATerm) Associativity));
}

/*}}}  */
/*{{{  SDF_Attribute SDF_makeAttributeReject(void) */

SDF_Attribute SDF_makeAttributeReject(void)
{
  return (SDF_Attribute)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun181))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun174))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun181)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(106)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun181)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(99)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(106)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(114)))));
}

/*}}}  */
/*{{{  SDF_Attribute SDF_makeAttributePrefer(void) */

SDF_Attribute SDF_makeAttributePrefer(void)
{
  return (SDF_Attribute)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun182))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun174))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun182)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(102)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(112)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun182)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(102)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(112)))));
}

/*}}}  */
/*{{{  SDF_Attribute SDF_makeAttributeAvoid(void) */

SDF_Attribute SDF_makeAttributeAvoid(void)
{
  return (SDF_Attribute)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun183))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun174))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun183)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(100)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(118)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun183)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(100)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(118)), (ATerm)ATmakeInt(97)))));
}

/*}}}  */
/*{{{  SDF_Attributes SDF_makeAttributesAttrs(SDF_OptLayout wsAfterBraceOpen, SDF_AttributeList list, SDF_OptLayout wsAfterList) */

SDF_Attributes SDF_makeAttributesAttrs(SDF_OptLayout wsAfterBraceOpen, SDF_AttributeList list, SDF_OptLayout wsAfterList)
{
  return (SDF_Attributes)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun40))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun35, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun174)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun41))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun184))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun185)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(125)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun40)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(125)))), (ATerm) wsAfterList), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun35, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun174)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm) list)), (ATerm) wsAfterBraceOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(123)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun41)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(123)))));
}

/*}}}  */
/*{{{  SDF_Attributes SDF_makeAttributesNoAttrs(void) */

SDF_Attributes SDF_makeAttributesNoAttrs(void)
{
  return (SDF_Attributes)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATempty, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun184))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun186)))))), (ATerm)ATempty);
}

/*}}}  */
/*{{{  SDF_Production SDF_makeProductionProd(SDF_Symbols Symbols, SDF_OptLayout wsAfterSymbols, SDF_OptLayout wsAfterGreaterThan, SDF_Symbol result, SDF_OptLayout wsAfterResult, SDF_Attributes Attributes) */

SDF_Production SDF_makeProductionProd(SDF_Symbols Symbols, SDF_OptLayout wsAfterSymbols, SDF_OptLayout wsAfterGreaterThan, SDF_Symbol result, SDF_OptLayout wsAfterResult, SDF_Attributes Attributes)
{
  return (SDF_Production)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun184)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun93))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun86)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun187))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun188)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) Attributes), (ATerm) wsAfterResult), (ATerm) result), (ATerm) wsAfterGreaterThan), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun93)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(62)), (ATerm)ATmakeInt(45)))), (ATerm) wsAfterSymbols), (ATerm) Symbols));
}

/*}}}  */
/*{{{  SDF_Production SDF_makeProductionUnquotedFun(SDF_IdCon fun, SDF_OptLayout wsAfterFun, SDF_OptLayout wsAfterParenOpen, SDF_SymbolArguments arguments, SDF_OptLayout wsAfterArguments, SDF_OptLayout wsAfterParenClose, SDF_OptLayout wsAfterGreaterThan, SDF_Symbol result, SDF_OptLayout wsAfterResult, SDF_Attributes Attributes) */

SDF_Production SDF_makeProductionUnquotedFun(SDF_IdCon fun, SDF_OptLayout wsAfterFun, SDF_OptLayout wsAfterParenOpen, SDF_SymbolArguments arguments, SDF_OptLayout wsAfterArguments, SDF_OptLayout wsAfterParenClose, SDF_OptLayout wsAfterGreaterThan, SDF_Symbol result, SDF_OptLayout wsAfterResult, SDF_Attributes Attributes)
{
  return (SDF_Production)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun184)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun93))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun35, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun16)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun187))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun189)))), (ATerm)ATmakeAppl0(SDF_afun156)))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) Attributes), (ATerm) wsAfterResult), (ATerm) result), (ATerm) wsAfterGreaterThan), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun93)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(62)), (ATerm)ATmakeInt(45)))), (ATerm) wsAfterParenClose), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm) wsAfterArguments), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun35, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm) arguments)), (ATerm) wsAfterParenOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm) wsAfterFun), (ATerm) fun));
}

/*}}}  */
/*{{{  SDF_Production SDF_makeProductionQuotedFun(SDF_StrCon str, SDF_OptLayout wsAfterStr, SDF_OptLayout wsAfterParenOpen, SDF_SymbolArguments arguments, SDF_OptLayout wsAfterArguments, SDF_OptLayout wsAfterParenClose, SDF_OptLayout wsAfterGreaterThan, SDF_Symbol result, SDF_OptLayout wsAfterResult, SDF_Attributes Attributes) */

SDF_Production SDF_makeProductionQuotedFun(SDF_StrCon str, SDF_OptLayout wsAfterStr, SDF_OptLayout wsAfterParenOpen, SDF_SymbolArguments arguments, SDF_OptLayout wsAfterArguments, SDF_OptLayout wsAfterParenClose, SDF_OptLayout wsAfterGreaterThan, SDF_Symbol result, SDF_OptLayout wsAfterResult, SDF_Attributes Attributes)
{
  return (SDF_Production)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun184)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun93))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun35, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun10)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun187))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun190)))), (ATerm)ATmakeAppl0(SDF_afun156)))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) Attributes), (ATerm) wsAfterResult), (ATerm) result), (ATerm) wsAfterGreaterThan), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun93)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(62)), (ATerm)ATmakeInt(45)))), (ATerm) wsAfterParenClose), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm) wsAfterArguments), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun35, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm) arguments)), (ATerm) wsAfterParenOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm) wsAfterStr), (ATerm) str));
}

/*}}}  */
/*{{{  SDF_Productions SDF_makeProductionsDefault(SDF_ProductionList list) */

SDF_Productions SDF_makeProductionsDefault(SDF_ProductionList list)
{
  return (SDF_Productions)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun187))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun62))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun187))))), (ATerm) list)));
}

/*}}}  */
/*{{{  SDF_AttributeList SDF_makeAttributeListEmpty(void) */

SDF_AttributeList SDF_makeAttributeListEmpty(void)
{
  return (SDF_AttributeList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_AttributeList SDF_makeAttributeListSingle(SDF_Attribute head) */

SDF_AttributeList SDF_makeAttributeListSingle(SDF_Attribute head)
{
  return (SDF_AttributeList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_AttributeList SDF_makeAttributeListMany(SDF_Attribute head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_AttributeList tail) */

SDF_AttributeList SDF_makeAttributeListMany(SDF_Attribute head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_AttributeList tail)
{
  if (SDF_isAttributeListEmpty(tail)) {
    return SDF_makeAttributeListSingle(head);
  }
  return (SDF_AttributeList)(ATerm)ATinsert(ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterSep), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_ProductionList SDF_makeProductionListEmpty(void) */

SDF_ProductionList SDF_makeProductionListEmpty(void)
{
  return (SDF_ProductionList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_ProductionList SDF_makeProductionListSingle(SDF_Production head) */

SDF_ProductionList SDF_makeProductionListSingle(SDF_Production head)
{
  return (SDF_ProductionList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_ProductionList SDF_makeProductionListMany(SDF_Production head, SDF_OptLayout wsAfterHead, SDF_ProductionList tail) */

SDF_ProductionList SDF_makeProductionListMany(SDF_Production head, SDF_OptLayout wsAfterHead, SDF_ProductionList tail)
{
  if (SDF_isProductionListEmpty(tail)) {
    return SDF_makeProductionListSingle(head);
  }
  return (SDF_ProductionList)(ATerm)ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_Label SDF_makeLabelQuoted(SDF_StrCon StrCon) */

SDF_Label SDF_makeLabelQuoted(SDF_StrCon StrCon)
{
  return (SDF_Label)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun10)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun108))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun15)))))), (ATerm)ATmakeList1((ATerm) StrCon));
}

/*}}}  */
/*{{{  SDF_Label SDF_makeLabelUnquoted(SDF_IdCon IdCon) */

SDF_Label SDF_makeLabelUnquoted(SDF_IdCon IdCon)
{
  return (SDF_Label)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun16)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun108))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun17)))))), (ATerm)ATmakeList1((ATerm) IdCon));
}

/*}}}  */
/*{{{  SDF_SymbolArguments SDF_makeSymbolArgumentsEmpty(void) */

SDF_SymbolArguments SDF_makeSymbolArgumentsEmpty(void)
{
  return (SDF_SymbolArguments)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_SymbolArguments SDF_makeSymbolArgumentsSingle(SDF_Symbol head) */

SDF_SymbolArguments SDF_makeSymbolArgumentsSingle(SDF_Symbol head)
{
  return (SDF_SymbolArguments)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_SymbolArguments SDF_makeSymbolArgumentsMany(SDF_Symbol head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_SymbolArguments tail) */

SDF_SymbolArguments SDF_makeSymbolArgumentsMany(SDF_Symbol head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_SymbolArguments tail)
{
  if (SDF_isSymbolArgumentsEmpty(tail)) {
    return SDF_makeSymbolArgumentsSingle(head);
  }
  return (SDF_SymbolArguments)(ATerm)ATinsert(ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterSep), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_LexModuleWord SDF_makeLexModuleWordWord(const char* letters) */

SDF_LexModuleWord SDF_makeLexModuleWordWord(const char* letters)
{
  return (SDF_LexModuleWord)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun7, (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(97), (ATerm)ATmakeInt(122))), (ATerm)ATmakeInt(95)), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(65), (ATerm)ATmakeInt(90))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(48), (ATerm)ATmakeInt(57))), (ATerm)ATmakeInt(45)))))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun191))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun192)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun7, (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(97), (ATerm)ATmakeInt(122))), (ATerm)ATmakeInt(95)), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(65), (ATerm)ATmakeInt(90))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(48), (ATerm)ATmakeInt(57))), (ATerm)ATmakeInt(45)))))), (ATerm) ((ATerm) SDF_stringToChars(letters)))));
}

/*}}}  */
/*{{{  SDF_ModuleWord SDF_makeModuleWordLexToCf(SDF_LexModuleWord ModuleWord) */

SDF_ModuleWord SDF_makeModuleWordLexToCf(SDF_LexModuleWord ModuleWord)
{
  return (SDF_ModuleWord)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun191)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun191))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm) ModuleWord));
}

/*}}}  */
/*{{{  SDF_LexModuleId SDF_makeLexModuleIdLeaf(SDF_LexModuleWord ModuleWord) */

SDF_LexModuleId SDF_makeLexModuleIdLeaf(SDF_LexModuleWord ModuleWord)
{
  return (SDF_LexModuleId)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun191)))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun193))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun194)))))), (ATerm)ATmakeList1((ATerm) ModuleWord));
}

/*}}}  */
/*{{{  SDF_LexModuleId SDF_makeLexModuleIdRoot(SDF_LexModuleId basename) */

SDF_LexModuleId SDF_makeLexModuleIdRoot(SDF_LexModuleId basename)
{
  return (SDF_LexModuleId)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun193)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun149))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun193))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun195)))))), (ATerm)ATinsert(ATmakeList1((ATerm) basename), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(47)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun149)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(47)))));
}

/*}}}  */
/*{{{  SDF_LexModuleId SDF_makeLexModuleIdPath(SDF_LexModuleWord dirname, SDF_LexModuleId basename) */

SDF_LexModuleId SDF_makeLexModuleIdPath(SDF_LexModuleWord dirname, SDF_LexModuleId basename)
{
  return (SDF_LexModuleId)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun193)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun149))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun191)))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun193))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun196)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) basename), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(47)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun149)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(47)))), (ATerm) dirname));
}

/*}}}  */
/*{{{  SDF_ModuleId SDF_makeModuleIdLexToCf(SDF_LexModuleId ModuleId) */

SDF_ModuleId SDF_makeModuleIdLexToCf(SDF_LexModuleId ModuleId)
{
  return (SDF_ModuleId)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun193)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun193))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm) ModuleId));
}

/*}}}  */
/*{{{  SDF_Definition SDF_makeDefinitionDefault(SDF_ModuleList list) */

SDF_Definition SDF_makeDefinitionDefault(SDF_ModuleList list)
{
  return (SDF_Definition)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun45))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun197))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun45))))), (ATerm) list)));
}

/*}}}  */
/*{{{  SDF_Module SDF_makeModuleModule(SDF_OptLayout wsAfterModule, SDF_ModuleName ModuleName, SDF_OptLayout wsAfterModuleName, SDF_ImpSectionList list, SDF_OptLayout wsAfterList, SDF_Sections Sections) */

SDF_Module SDF_makeModuleModule(SDF_OptLayout wsAfterModule, SDF_ModuleName ModuleName, SDF_OptLayout wsAfterModuleName, SDF_ImpSectionList list, SDF_OptLayout wsAfterList, SDF_Sections Sections)
{
  return (SDF_Module)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun198)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun76))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun176)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun199))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun45))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun199)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) Sections), (ATerm) wsAfterList), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun76))))), (ATerm) list)), (ATerm) wsAfterModuleName), (ATerm) ModuleName), (ATerm) wsAfterModule), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(117)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(100)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(109)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun199)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(108)), (ATerm)ATmakeInt(117)), (ATerm)ATmakeInt(100)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(109)))));
}

/*}}}  */
/*{{{  SDF_Section SDF_makeSectionExports(SDF_OptLayout wsAfterExports, SDF_Grammar Grammar) */

SDF_Section SDF_makeSectionExports(SDF_OptLayout wsAfterExports, SDF_Grammar Grammar)
{
  return (SDF_Section)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun200))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun201))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun200)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) Grammar), (ATerm) wsAfterExports), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(112)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(120)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun200)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(112)), (ATerm)ATmakeInt(120)), (ATerm)ATmakeInt(101)))));
}

/*}}}  */
/*{{{  SDF_Section SDF_makeSectionHiddens(SDF_OptLayout wsAfterHiddens, SDF_Grammar Grammar) */

SDF_Section SDF_makeSectionHiddens(SDF_OptLayout wsAfterHiddens, SDF_Grammar Grammar)
{
  return (SDF_Section)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun61)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun202))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun201))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun202)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) Grammar), (ATerm) wsAfterHiddens), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(100)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(100)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(104)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun202)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(110)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(100)), (ATerm)ATmakeInt(100)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(104)))));
}

/*}}}  */
/*{{{  SDF_Sections SDF_makeSectionsDefault(SDF_SectionList list) */

SDF_Sections SDF_makeSectionsDefault(SDF_SectionList list)
{
  return (SDF_Sections)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun201))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun198))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun201))))), (ATerm) list)));
}

/*}}}  */
/*{{{  SDF_ModuleName SDF_makeModuleNameUnparameterized(SDF_ModuleId ModuleId) */

SDF_ModuleName SDF_makeModuleNameUnparameterized(SDF_ModuleId ModuleId)
{
  return (SDF_ModuleName)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun193)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun176))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun203)))))), (ATerm)ATmakeList1((ATerm) ModuleId));
}

/*}}}  */
/*{{{  SDF_ModuleName SDF_makeModuleNameParameterized(SDF_ModuleId ModuleId, SDF_OptLayout wsAfterModuleId, SDF_OptLayout wsAfterBracketOpen, SDF_Symbols params, SDF_OptLayout wsAfterParams) */

SDF_ModuleName SDF_makeModuleNameParameterized(SDF_ModuleId ModuleId, SDF_OptLayout wsAfterModuleId, SDF_OptLayout wsAfterBracketOpen, SDF_Symbols params, SDF_OptLayout wsAfterParams)
{
  return (SDF_ModuleName)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun34))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun86)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun36))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun193)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun176))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun204)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(93)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun34)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(93)))), (ATerm) wsAfterParams), (ATerm) params), (ATerm) wsAfterBracketOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(91)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun36)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(91)))), (ATerm) wsAfterModuleId), (ATerm) ModuleId));
}

/*}}}  */
/*{{{  SDF_ImpSection SDF_makeImpSectionImports(SDF_OptLayout wsAfterImports, SDF_Imports list) */

SDF_ImpSection SDF_makeImpSectionImports(SDF_OptLayout wsAfterImports, SDF_Imports list)
{
  return (SDF_ImpSection)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun205)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun206))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun76))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun206)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) list), (ATerm) wsAfterImports), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(112)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(109)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun206)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(114)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(112)), (ATerm)ATmakeInt(109)), (ATerm)ATmakeInt(105)))));
}

/*}}}  */
/*{{{  SDF_Imports SDF_makeImportsDefault(SDF_ImportList list) */

SDF_Imports SDF_makeImportsDefault(SDF_ImportList list)
{
  return (SDF_Imports)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun207))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun205))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun207))))), (ATerm) list)));
}

/*}}}  */
/*{{{  SDF_Import SDF_makeImportModule(SDF_ModuleName ModuleName) */

SDF_Import SDF_makeImportModule(SDF_ModuleName ModuleName)
{
  return (SDF_Import)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun176)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun207))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun199)))))), (ATerm)ATmakeList1((ATerm) ModuleName));
}

/*}}}  */
/*{{{  SDF_Import SDF_makeImportRenamedModule(SDF_ModuleName ModuleName, SDF_OptLayout wsAfterModuleName, SDF_Renamings Renamings) */

SDF_Import SDF_makeImportRenamedModule(SDF_ModuleName ModuleName, SDF_OptLayout wsAfterModuleName, SDF_Renamings Renamings)
{
  return (SDF_Import)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun208)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun176)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun207))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun209)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) Renamings), (ATerm) wsAfterModuleName), (ATerm) ModuleName));
}

/*}}}  */
/*{{{  SDF_Import SDF_makeImportBracket(SDF_OptLayout wsAfterParenOpen, SDF_Import Import, SDF_OptLayout wsAfterImport) */

SDF_Import SDF_makeImportBracket(SDF_OptLayout wsAfterParenOpen, SDF_Import Import, SDF_OptLayout wsAfterImport)
{
  return (SDF_Import)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun207)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun207))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl0(SDF_afun75)))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm) wsAfterImport), (ATerm) Import), (ATerm) wsAfterParenOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))));
}

/*}}}  */
/*{{{  SDF_ModuleList SDF_makeModuleListEmpty(void) */

SDF_ModuleList SDF_makeModuleListEmpty(void)
{
  return (SDF_ModuleList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_ModuleList SDF_makeModuleListSingle(SDF_Module head) */

SDF_ModuleList SDF_makeModuleListSingle(SDF_Module head)
{
  return (SDF_ModuleList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_ModuleList SDF_makeModuleListMany(SDF_Module head, SDF_OptLayout wsAfterHead, SDF_ModuleList tail) */

SDF_ModuleList SDF_makeModuleListMany(SDF_Module head, SDF_OptLayout wsAfterHead, SDF_ModuleList tail)
{
  if (SDF_isModuleListEmpty(tail)) {
    return SDF_makeModuleListSingle(head);
  }
  return (SDF_ModuleList)(ATerm)ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_ImpSectionList SDF_makeImpSectionListEmpty(void) */

SDF_ImpSectionList SDF_makeImpSectionListEmpty(void)
{
  return (SDF_ImpSectionList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_ImpSectionList SDF_makeImpSectionListSingle(SDF_ImpSection head) */

SDF_ImpSectionList SDF_makeImpSectionListSingle(SDF_ImpSection head)
{
  return (SDF_ImpSectionList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_ImpSectionList SDF_makeImpSectionListMany(SDF_ImpSection head, SDF_OptLayout wsAfterHead, SDF_ImpSectionList tail) */

SDF_ImpSectionList SDF_makeImpSectionListMany(SDF_ImpSection head, SDF_OptLayout wsAfterHead, SDF_ImpSectionList tail)
{
  if (SDF_isImpSectionListEmpty(tail)) {
    return SDF_makeImpSectionListSingle(head);
  }
  return (SDF_ImpSectionList)(ATerm)ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_SectionList SDF_makeSectionListEmpty(void) */

SDF_SectionList SDF_makeSectionListEmpty(void)
{
  return (SDF_SectionList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_SectionList SDF_makeSectionListSingle(SDF_Section head) */

SDF_SectionList SDF_makeSectionListSingle(SDF_Section head)
{
  return (SDF_SectionList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_SectionList SDF_makeSectionListMany(SDF_Section head, SDF_OptLayout wsAfterHead, SDF_SectionList tail) */

SDF_SectionList SDF_makeSectionListMany(SDF_Section head, SDF_OptLayout wsAfterHead, SDF_SectionList tail)
{
  if (SDF_isSectionListEmpty(tail)) {
    return SDF_makeSectionListSingle(head);
  }
  return (SDF_SectionList)(ATerm)ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_ImportList SDF_makeImportListEmpty(void) */

SDF_ImportList SDF_makeImportListEmpty(void)
{
  return (SDF_ImportList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_ImportList SDF_makeImportListSingle(SDF_Import head) */

SDF_ImportList SDF_makeImportListSingle(SDF_Import head)
{
  return (SDF_ImportList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_ImportList SDF_makeImportListMany(SDF_Import head, SDF_OptLayout wsAfterHead, SDF_ImportList tail) */

SDF_ImportList SDF_makeImportListMany(SDF_Import head, SDF_OptLayout wsAfterHead, SDF_ImportList tail)
{
  if (SDF_isImportListEmpty(tail)) {
    return SDF_makeImportListSingle(head);
  }
  return (SDF_ImportList)(ATerm)ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_Associativity SDF_makeAssociativityLeft(void) */

SDF_Associativity SDF_makeAssociativityLeft(void)
{
  return (SDF_Associativity)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun210))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun179))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun210)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(102)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(108)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun210)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(102)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(108)))));
}

/*}}}  */
/*{{{  SDF_Associativity SDF_makeAssociativityRight(void) */

SDF_Associativity SDF_makeAssociativityRight(void)
{
  return (SDF_Associativity)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun211))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun179))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun211)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(104)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(103)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(114)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun211)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(104)), (ATerm)ATmakeInt(103)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(114)))));
}

/*}}}  */
/*{{{  SDF_Associativity SDF_makeAssociativityNonAssoc(void) */

SDF_Associativity SDF_makeAssociativityNonAssoc(void)
{
  return (SDF_Associativity)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun212))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun179))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun212)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun212)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(99)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(97)), (ATerm)ATmakeInt(45)), (ATerm)ATmakeInt(110)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(110)))));
}

/*}}}  */
/*{{{  SDF_Associativity SDF_makeAssociativityAssoc(void) */

SDF_Associativity SDF_makeAssociativityAssoc(void)
{
  return (SDF_Associativity)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun180))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun179))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun180)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(99)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(115)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(97)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun180)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(99)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(115)), (ATerm)ATmakeInt(97)))));
}

/*}}}  */
/*{{{  SDF_ArgumentIndicator SDF_makeArgumentIndicatorDefault(SDF_OptLayout wsAfterLessThan, SDF_NatConArguments arguments, SDF_OptLayout wsAfterArguments) */

SDF_ArgumentIndicator SDF_makeArgumentIndicatorDefault(SDF_OptLayout wsAfterLessThan, SDF_NatConArguments arguments, SDF_OptLayout wsAfterArguments)
{
  return (SDF_ArgumentIndicator)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun31))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun26, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun47)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun32))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun213))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun42)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun31)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm) wsAfterArguments), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun26, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun47)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm) arguments)), (ATerm) wsAfterLessThan), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(60)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun32)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(60)))));
}

/*}}}  */
/*{{{  SDF_Group SDF_makeGroupWithArguments(SDF_Group Group, SDF_OptLayout wsAfterGroup, SDF_ArgumentIndicator ArgumentIndicator) */

SDF_Group SDF_makeGroupWithArguments(SDF_Group Group, SDF_OptLayout wsAfterGroup, SDF_ArgumentIndicator ArgumentIndicator)
{
  return (SDF_Group)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun213)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun214)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun214))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun215)))), (ATerm)ATmakeAppl1(SDF_afun73, (ATerm)ATmakeAppl0(SDF_afun216))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) ArgumentIndicator), (ATerm) wsAfterGroup), (ATerm) Group));
}

/*}}}  */
/*{{{  SDF_Group SDF_makeGroupSimpleGroup(SDF_Production Production) */

SDF_Group SDF_makeGroupSimpleGroup(SDF_Production Production)
{
  return (SDF_Group)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun187)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun214))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun217)))))), (ATerm)ATmakeList1((ATerm) Production));
}

/*}}}  */
/*{{{  SDF_Group SDF_makeGroupProdsGroup(SDF_OptLayout wsAfterBraceOpen, SDF_Productions Productions, SDF_OptLayout wsAfterProductions) */

SDF_Group SDF_makeGroupProdsGroup(SDF_OptLayout wsAfterBraceOpen, SDF_Productions Productions, SDF_OptLayout wsAfterProductions)
{
  return (SDF_Group)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun40))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun62)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun41))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun214))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun218)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(125)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun40)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(125)))), (ATerm) wsAfterProductions), (ATerm) Productions), (ATerm) wsAfterBraceOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(123)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun41)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(123)))));
}

/*}}}  */
/*{{{  SDF_Group SDF_makeGroupAssocGroup(SDF_OptLayout wsAfterBraceOpen, SDF_Associativity Associativity, SDF_OptLayout wsAfterAssociativity, SDF_OptLayout wsAfterColon, SDF_Productions Productions, SDF_OptLayout wsAfterProductions) */

SDF_Group SDF_makeGroupAssocGroup(SDF_OptLayout wsAfterBraceOpen, SDF_Associativity Associativity, SDF_OptLayout wsAfterAssociativity, SDF_OptLayout wsAfterColon, SDF_Productions Productions, SDF_OptLayout wsAfterProductions)
{
  return (SDF_Group)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun40))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun62)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun107))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun179)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun41))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun214))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun219)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(125)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun40)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(125)))), (ATerm) wsAfterProductions), (ATerm) Productions), (ATerm) wsAfterColon), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(58)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun107)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(58)))), (ATerm) wsAfterAssociativity), (ATerm) Associativity), (ATerm) wsAfterBraceOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(123)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun41)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(123)))));
}

/*}}}  */
/*{{{  SDF_Priority SDF_makePriorityChain(SDF_GroupList list) */

SDF_Priority SDF_makePriorityChain(SDF_GroupList list)
{
  return (SDF_Priority)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun26, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun214)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun31))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun220))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun221)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun26, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun214)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun31))))), (ATerm) list)));
}

/*}}}  */
/*{{{  SDF_Priority SDF_makePriorityAssoc(SDF_Group left, SDF_OptLayout wsAfterLeft, SDF_Associativity Associativity, SDF_OptLayout wsAfterAssociativity, SDF_Group right) */

SDF_Priority SDF_makePriorityAssoc(SDF_Group left, SDF_OptLayout wsAfterLeft, SDF_Associativity Associativity, SDF_OptLayout wsAfterAssociativity, SDF_Group right)
{
  return (SDF_Priority)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun214)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun179)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun214)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun220))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun180)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) right), (ATerm) wsAfterAssociativity), (ATerm) Associativity), (ATerm) wsAfterLeft), (ATerm) left));
}

/*}}}  */
/*{{{  SDF_Priorities SDF_makePrioritiesDefault(SDF_PriorityList list) */

SDF_Priorities SDF_makePrioritiesDefault(SDF_PriorityList list)
{
  return (SDF_Priorities)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun35, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun220)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun78))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun35, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun220)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm) list)));
}

/*}}}  */
/*{{{  SDF_NatConArguments SDF_makeNatConArgumentsEmpty(void) */

SDF_NatConArguments SDF_makeNatConArgumentsEmpty(void)
{
  return (SDF_NatConArguments)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_NatConArguments SDF_makeNatConArgumentsSingle(SDF_NatCon head) */

SDF_NatConArguments SDF_makeNatConArgumentsSingle(SDF_NatCon head)
{
  return (SDF_NatConArguments)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_NatConArguments SDF_makeNatConArgumentsMany(SDF_NatCon head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_NatConArguments tail) */

SDF_NatConArguments SDF_makeNatConArgumentsMany(SDF_NatCon head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_NatConArguments tail)
{
  if (SDF_isNatConArgumentsEmpty(tail)) {
    return SDF_makeNatConArgumentsSingle(head);
  }
  return (SDF_NatConArguments)(ATerm)ATinsert(ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterSep), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_GroupList SDF_makeGroupListEmpty(void) */

SDF_GroupList SDF_makeGroupListEmpty(void)
{
  return (SDF_GroupList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_GroupList SDF_makeGroupListSingle(SDF_Group head) */

SDF_GroupList SDF_makeGroupListSingle(SDF_Group head)
{
  return (SDF_GroupList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_GroupList SDF_makeGroupListMany(SDF_Group head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_GroupList tail) */

SDF_GroupList SDF_makeGroupListMany(SDF_Group head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_GroupList tail)
{
  if (SDF_isGroupListEmpty(tail)) {
    return SDF_makeGroupListSingle(head);
  }
  return (SDF_GroupList)(ATerm)ATinsert(ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterSep), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun31)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_PriorityList SDF_makePriorityListEmpty(void) */

SDF_PriorityList SDF_makePriorityListEmpty(void)
{
  return (SDF_PriorityList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_PriorityList SDF_makePriorityListSingle(SDF_Priority head) */

SDF_PriorityList SDF_makePriorityListSingle(SDF_Priority head)
{
  return (SDF_PriorityList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_PriorityList SDF_makePriorityListMany(SDF_Priority head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_PriorityList tail) */

SDF_PriorityList SDF_makePriorityListMany(SDF_Priority head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_PriorityList tail)
{
  if (SDF_isPriorityListEmpty(tail)) {
    return SDF_makePriorityListSingle(head);
  }
  return (SDF_PriorityList)(ATerm)ATinsert(ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterSep), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_SymbolTail SDF_makeSymbolTailEmpty(void) */

SDF_SymbolTail SDF_makeSymbolTailEmpty(void)
{
  return (SDF_SymbolTail)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_SymbolTail SDF_makeSymbolTailSingle(SDF_Symbol head) */

SDF_SymbolTail SDF_makeSymbolTailSingle(SDF_Symbol head)
{
  return (SDF_SymbolTail)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_SymbolTail SDF_makeSymbolTailMany(SDF_Symbol head, SDF_OptLayout wsAfterHead, SDF_SymbolTail tail) */

SDF_SymbolTail SDF_makeSymbolTailMany(SDF_Symbol head, SDF_OptLayout wsAfterHead, SDF_SymbolTail tail)
{
  if (SDF_isSymbolTailEmpty(tail)) {
    return SDF_makeSymbolTailSingle(head);
  }
  return (SDF_SymbolTail)(ATerm)ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_SymbolRest SDF_makeSymbolRestEmpty(void) */

SDF_SymbolRest SDF_makeSymbolRestEmpty(void)
{
  return (SDF_SymbolRest)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_SymbolRest SDF_makeSymbolRestSingle(SDF_Symbol head) */

SDF_SymbolRest SDF_makeSymbolRestSingle(SDF_Symbol head)
{
  return (SDF_SymbolRest)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_SymbolRest SDF_makeSymbolRestMany(SDF_Symbol head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_SymbolRest tail) */

SDF_SymbolRest SDF_makeSymbolRestMany(SDF_Symbol head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_SymbolRest tail)
{
  if (SDF_isSymbolRestEmpty(tail)) {
    return SDF_makeSymbolRestSingle(head);
  }
  return (SDF_SymbolRest)(ATerm)ATinsert(ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterSep), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_Renamings SDF_makeRenamingsRenamings(SDF_OptLayout wsAfterBracketOpen, SDF_RenamingList list, SDF_OptLayout wsAfterList) */

SDF_Renamings SDF_makeRenamingsRenamings(SDF_OptLayout wsAfterBracketOpen, SDF_RenamingList list, SDF_OptLayout wsAfterList)
{
  return (SDF_Renamings)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun34))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun222))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun36))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun208))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun223)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(93)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun34)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(93)))), (ATerm) wsAfterList), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun222))))), (ATerm) list)), (ATerm) wsAfterBracketOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(91)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun36)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(91)))));
}

/*}}}  */
/*{{{  SDF_Renaming SDF_makeRenamingSymbol(SDF_Symbol from, SDF_OptLayout wsAfterFrom, SDF_OptLayout wsAfterEqualsGreaterThan, SDF_Symbol to) */

SDF_Renaming SDF_makeRenamingSymbol(SDF_Symbol from, SDF_OptLayout wsAfterFrom, SDF_OptLayout wsAfterEqualsGreaterThan, SDF_Symbol to)
{
  return (SDF_Renaming)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun126))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun222))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun224)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) to), (ATerm) wsAfterEqualsGreaterThan), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(61)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun126)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(62)), (ATerm)ATmakeInt(61)))), (ATerm) wsAfterFrom), (ATerm) from));
}

/*}}}  */
/*{{{  SDF_Renaming SDF_makeRenamingProduction(SDF_Production fromProd, SDF_OptLayout wsAfterFromProd, SDF_OptLayout wsAfterEqualsGreaterThan, SDF_Production toProd) */

SDF_Renaming SDF_makeRenamingProduction(SDF_Production fromProd, SDF_OptLayout wsAfterFromProd, SDF_OptLayout wsAfterEqualsGreaterThan, SDF_Production toProd)
{
  return (SDF_Renaming)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun187)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun126))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun187)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun222))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun225)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) toProd), (ATerm) wsAfterEqualsGreaterThan), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(62)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(61)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun126)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(62)), (ATerm)ATmakeInt(61)))), (ATerm) wsAfterFromProd), (ATerm) fromProd));
}

/*}}}  */
/*{{{  SDF_RenamingList SDF_makeRenamingListEmpty(void) */

SDF_RenamingList SDF_makeRenamingListEmpty(void)
{
  return (SDF_RenamingList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_RenamingList SDF_makeRenamingListSingle(SDF_Renaming head) */

SDF_RenamingList SDF_makeRenamingListSingle(SDF_Renaming head)
{
  return (SDF_RenamingList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_RenamingList SDF_makeRenamingListMany(SDF_Renaming head, SDF_OptLayout wsAfterHead, SDF_RenamingList tail) */

SDF_RenamingList SDF_makeRenamingListMany(SDF_Renaming head, SDF_OptLayout wsAfterHead, SDF_RenamingList tail)
{
  if (SDF_isRenamingListEmpty(tail)) {
    return SDF_makeRenamingListSingle(head);
  }
  return (SDF_RenamingList)(ATerm)ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_Lookahead SDF_makeLookaheadCharClass(SDF_CharClass CharClass) */

SDF_Lookahead SDF_makeLookaheadCharClass(SDF_CharClass CharClass)
{
  return (SDF_Lookahead)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun226))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun106)))))), (ATerm)ATmakeList1((ATerm) CharClass));
}

/*}}}  */
/*{{{  SDF_Lookahead SDF_makeLookaheadSeq(SDF_CharClass head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterPeriod, SDF_Lookaheads tail) */

SDF_Lookahead SDF_makeLookaheadSeq(SDF_CharClass head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterPeriod, SDF_Lookaheads tail)
{
  return (SDF_Lookahead)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun227)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun57))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun105)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun226))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun114)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) tail), (ATerm) wsAfterPeriod), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(46)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun57)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(46)))), (ATerm) wsAfterHead), (ATerm) head));
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_makeLookaheadsSingle(SDF_Lookahead Lookahead) */

SDF_Lookaheads SDF_makeLookaheadsSingle(SDF_Lookahead Lookahead)
{
  return (SDF_Lookaheads)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun226)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun227))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun228)))))), (ATerm)ATmakeList1((ATerm) Lookahead));
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_makeLookaheadsAlt(SDF_Lookaheads left, SDF_OptLayout wsAfterLeft, SDF_OptLayout wsAfterBar, SDF_Lookaheads right) */

SDF_Lookaheads SDF_makeLookaheadsAlt(SDF_Lookaheads left, SDF_OptLayout wsAfterLeft, SDF_OptLayout wsAfterBar, SDF_Lookaheads right)
{
  return (SDF_Lookaheads)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun227)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun128))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun227)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun227))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun73, (ATerm)ATmakeAppl0(SDF_afun125))), (ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun129)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) right), (ATerm) wsAfterBar), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(124)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun128)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(124)))), (ATerm) wsAfterLeft), (ATerm) left));
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_makeLookaheadsBracket(SDF_OptLayout wsAfterParenOpen, SDF_Lookaheads Lookaheads, SDF_OptLayout wsAfterLookaheads) */

SDF_Lookaheads SDF_makeLookaheadsBracket(SDF_OptLayout wsAfterParenOpen, SDF_Lookaheads Lookaheads, SDF_OptLayout wsAfterLookaheads)
{
  return (SDF_Lookaheads)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun227)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun227))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl0(SDF_afun75)))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun25)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(41)))), (ATerm) wsAfterLookaheads), (ATerm) Lookaheads), (ATerm) wsAfterParenOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun28)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(40)))));
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_makeLookaheadsList(SDF_OptLayout wsAfterBracketOpenBracketOpen, SDF_LookaheadList list, SDF_OptLayout wsAfterList) */

SDF_Lookaheads SDF_makeLookaheadsList(SDF_OptLayout wsAfterBracketOpenBracketOpen, SDF_LookaheadList list, SDF_OptLayout wsAfterList)
{
  return (SDF_Lookaheads)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun136))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun35, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun226)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun137))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun227))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun37)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(93)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(93)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun136)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(93)), (ATerm)ATmakeInt(93)))), (ATerm) wsAfterList), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl2(SDF_afun35, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun226)), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27))))), (ATerm) list)), (ATerm) wsAfterBracketOpenBracketOpen), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(91)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(91)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun137)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(91)), (ATerm)ATmakeInt(91)))));
}

/*}}}  */
/*{{{  SDF_Restriction SDF_makeRestrictionFollow(SDF_Symbols Symbols, SDF_OptLayout wsAfterSymbols, SDF_OptLayout wsAfterSlash, SDF_Lookaheads Lookaheads) */

SDF_Restriction SDF_makeRestrictionFollow(SDF_Symbols Symbols, SDF_OptLayout wsAfterSymbols, SDF_OptLayout wsAfterSlash, SDF_Lookaheads Lookaheads)
{
  return (SDF_Restriction)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun227)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun229))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun86)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun230))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun231)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) Lookaheads), (ATerm) wsAfterSlash), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(47)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(45)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun229)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(45)), (ATerm)ATmakeInt(47)), (ATerm)ATmakeInt(45)))), (ATerm) wsAfterSymbols), (ATerm) Symbols));
}

/*}}}  */
/*{{{  SDF_Restrictions SDF_makeRestrictionsDefault(SDF_RestrictionList list) */

SDF_Restrictions SDF_makeRestrictionsDefault(SDF_RestrictionList list)
{
  return (SDF_Restrictions)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun230))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun80))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun230))))), (ATerm) list)));
}

/*}}}  */
/*{{{  SDF_LookaheadList SDF_makeLookaheadListEmpty(void) */

SDF_LookaheadList SDF_makeLookaheadListEmpty(void)
{
  return (SDF_LookaheadList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_LookaheadList SDF_makeLookaheadListSingle(SDF_Lookahead head) */

SDF_LookaheadList SDF_makeLookaheadListSingle(SDF_Lookahead head)
{
  return (SDF_LookaheadList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_LookaheadList SDF_makeLookaheadListMany(SDF_Lookahead head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_LookaheadList tail) */

SDF_LookaheadList SDF_makeLookaheadListMany(SDF_Lookahead head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_LookaheadList tail)
{
  if (SDF_isLookaheadListEmpty(tail)) {
    return SDF_makeLookaheadListSingle(head);
  }
  return (SDF_LookaheadList)(ATerm)ATinsert(ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterSep), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_RestrictionList SDF_makeRestrictionListEmpty(void) */

SDF_RestrictionList SDF_makeRestrictionListEmpty(void)
{
  return (SDF_RestrictionList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_RestrictionList SDF_makeRestrictionListSingle(SDF_Restriction head) */

SDF_RestrictionList SDF_makeRestrictionListSingle(SDF_Restriction head)
{
  return (SDF_RestrictionList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_RestrictionList SDF_makeRestrictionListMany(SDF_Restriction head, SDF_OptLayout wsAfterHead, SDF_RestrictionList tail) */

SDF_RestrictionList SDF_makeRestrictionListMany(SDF_Restriction head, SDF_OptLayout wsAfterHead, SDF_RestrictionList tail)
{
  if (SDF_isRestrictionListEmpty(tail)) {
    return SDF_makeRestrictionListSingle(head);
  }
  return (SDF_RestrictionList)(ATerm)ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_SDF SDF_makeSDFDefinition(SDF_OptLayout wsAfterDefinition, SDF_Definition Definition) */

SDF_SDF SDF_makeSDFDefinition(SDF_OptLayout wsAfterDefinition, SDF_Definition Definition)
{
  return (SDF_SDF)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun197)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun3, (ATerm)ATmakeAppl0(SDF_afun4)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun232))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun46))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun232)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) Definition), (ATerm) wsAfterDefinition), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(111)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(105)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(102)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(101)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(100)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun232)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(110)), (ATerm)ATmakeInt(111)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(110)), (ATerm)ATmakeInt(105)), (ATerm)ATmakeInt(102)), (ATerm)ATmakeInt(101)), (ATerm)ATmakeInt(100)))));
}

/*}}}  */
/*{{{  SDF_Sort SDF_makeSortLexToCf(SDF_LexSort Sort) */

SDF_Sort SDF_makeSortLexToCf(SDF_LexSort Sort)
{
  return (SDF_Sort)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun134)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun134))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm) Sort));
}

/*}}}  */
/*{{{  SDF_LexSort SDF_makeLexSortOneChar(char head) */

SDF_LexSort SDF_makeLexSortOneChar(char head)
{
  return (SDF_LexSort)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(65), (ATerm)ATmakeInt(90))))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun134))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun233)))))), (ATerm)ATmakeList1((ATerm) ((ATerm) SDF_byteToChar(head))));
}

/*}}}  */
/*{{{  SDF_LexSort SDF_makeLexSortMoreChars(char head, const char* middle, char last) */

SDF_LexSort SDF_makeLexSortMoreChars(char head, const char* middle, char last)
{
  return (SDF_LexSort)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(97), (ATerm)ATmakeInt(122))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(65), (ATerm)ATmakeInt(90))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(48), (ATerm)ATmakeInt(57))))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(97), (ATerm)ATmakeInt(122))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(65), (ATerm)ATmakeInt(90))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(48), (ATerm)ATmakeInt(57))), (ATerm)ATmakeInt(45)))))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(65), (ATerm)ATmakeInt(90))))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun134))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun234)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm) ((ATerm) SDF_byteToChar(last))), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(97), (ATerm)ATmakeInt(122))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(65), (ATerm)ATmakeInt(90))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(48), (ATerm)ATmakeInt(57))), (ATerm)ATmakeInt(45)))))), (ATerm) ((ATerm) SDF_stringToChars(middle)))), (ATerm) ((ATerm) SDF_byteToChar(head))));
}

/*}}}  */
/*{{{  SDF_SymbolParameters SDF_makeSymbolParametersEmpty(void) */

SDF_SymbolParameters SDF_makeSymbolParametersEmpty(void)
{
  return (SDF_SymbolParameters)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_SymbolParameters SDF_makeSymbolParametersSingle(SDF_Symbol head) */

SDF_SymbolParameters SDF_makeSymbolParametersSingle(SDF_Symbol head)
{
  return (SDF_SymbolParameters)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_SymbolParameters SDF_makeSymbolParametersMany(SDF_Symbol head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_SymbolParameters tail) */

SDF_SymbolParameters SDF_makeSymbolParametersMany(SDF_Symbol head, SDF_OptLayout wsAfterHead, SDF_OptLayout wsAfterSep, SDF_SymbolParameters tail)
{
  if (SDF_isSymbolParametersEmpty(tail)) {
    return SDF_makeSymbolParametersSingle(head);
  }
  return (SDF_SymbolParameters)(ATerm)ATinsert(ATinsert(ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterSep), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun27)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(44)))), (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_Symbols SDF_makeSymbolsDefault(SDF_SymbolList list) */

SDF_Symbols SDF_makeSymbolsDefault(SDF_SymbolList list)
{
  return (SDF_Symbols)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun86))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun92))))), (ATerm) list)));
}

/*}}}  */
/*{{{  SDF_SymbolList SDF_makeSymbolListEmpty(void) */

SDF_SymbolList SDF_makeSymbolListEmpty(void)
{
  return (SDF_SymbolList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_SymbolList SDF_makeSymbolListSingle(SDF_Symbol head) */

SDF_SymbolList SDF_makeSymbolListSingle(SDF_Symbol head)
{
  return (SDF_SymbolList)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_SymbolList SDF_makeSymbolListMany(SDF_Symbol head, SDF_OptLayout wsAfterHead, SDF_SymbolList tail) */

SDF_SymbolList SDF_makeSymbolListMany(SDF_Symbol head, SDF_OptLayout wsAfterHead, SDF_SymbolList tail)
{
  if (SDF_isSymbolListEmpty(tail)) {
    return SDF_makeSymbolListSingle(head);
  }
  return (SDF_SymbolList)(ATerm)ATinsert(ATinsert((ATermList)tail, (ATerm) wsAfterHead), (ATerm) head);
}

/*}}}  */
/*{{{  SDF_LexLayout SDF_makeLexLayoutWhitespace(char ch) */

SDF_LexLayout SDF_makeLexLayoutWhitespace(char ch)
{
  return (SDF_LexLayout)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(32)), (ATerm)ATmakeInt(13)), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(9), (ATerm)ATmakeInt(10))))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl0(SDF_afun4)), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun235)))))), (ATerm)ATmakeList1((ATerm) ((ATerm) SDF_byteToChar(ch))));
}

/*}}}  */
/*{{{  SDF_LexLayout SDF_makeLexLayoutLine(const char* line) */

SDF_LexLayout SDF_makeLexLayoutLine(const char* line)
{
  return (SDF_LexLayout)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun236))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(11), (ATerm)ATmakeInt(255))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(0), (ATerm)ATmakeInt(9))))))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun237))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl0(SDF_afun4)), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun238)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(10)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun236)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(10)))), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(11), (ATerm)ATmakeInt(255))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(0), (ATerm)ATmakeInt(9))))))), (ATerm) ((ATerm) SDF_stringToChars(line)))), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(37)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(37)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun237)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(37)), (ATerm)ATmakeInt(37)))));
}

/*}}}  */
/*{{{  SDF_LexLayout SDF_makeLexLayoutNested(const char* content) */

SDF_LexLayout SDF_makeLexLayoutNested(const char* content)
{
  return (SDF_LexLayout)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun239))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun7, (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(38), (ATerm)ATmakeInt(255))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(11), (ATerm)ATmakeInt(36))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(0), (ATerm)ATmakeInt(9))))))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun239))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl0(SDF_afun4)), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun240)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(37)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun239)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(37)))), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun7, (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(38), (ATerm)ATmakeInt(255))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(11), (ATerm)ATmakeInt(36))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(0), (ATerm)ATmakeInt(9))))))), (ATerm) ((ATerm) SDF_stringToChars(content)))), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(37)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun239)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(37)))));
}

/*}}}  */
/*{{{  SDF_LexStrChar SDF_makeLexStrCharNewline(void) */

SDF_LexStrChar SDF_makeLexStrCharNewline(void)
{
  return (SDF_LexStrChar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun241))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun242))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun243)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(110)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(92)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun241)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(110)), (ATerm)ATmakeInt(92)))));
}

/*}}}  */
/*{{{  SDF_LexStrChar SDF_makeLexStrCharTab(void) */

SDF_LexStrChar SDF_makeLexStrCharTab(void)
{
  return (SDF_LexStrChar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun244))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun242))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun245)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(116)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(92)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun244)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(116)), (ATerm)ATmakeInt(92)))));
}

/*}}}  */
/*{{{  SDF_LexStrChar SDF_makeLexStrCharQuote(void) */

SDF_LexStrChar SDF_makeLexStrCharQuote(void)
{
  return (SDF_LexStrChar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun246))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun242))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun247)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(34)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(92)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun246)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(34)), (ATerm)ATmakeInt(92)))));
}

/*}}}  */
/*{{{  SDF_LexStrChar SDF_makeLexStrCharBackslash(void) */

SDF_LexStrChar SDF_makeLexStrCharBackslash(void)
{
  return (SDF_LexStrChar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun248))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun242))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun249)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(92)))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(92)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun248)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeInt(92)), (ATerm)ATmakeInt(92)))));
}

/*}}}  */
/*{{{  SDF_LexStrChar SDF_makeLexStrCharDecimal(char a, char b, char c) */

SDF_LexStrChar SDF_makeLexStrCharDecimal(char a, char b, char c)
{
  return (SDF_LexStrChar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(48), (ATerm)ATmakeInt(57))))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(48), (ATerm)ATmakeInt(57))))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(48), (ATerm)ATmakeInt(57))))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun250))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun242))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun251)))))), (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm) ((ATerm) SDF_byteToChar(c))), (ATerm) ((ATerm) SDF_byteToChar(b))), (ATerm) ((ATerm) SDF_byteToChar(a))), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(92)))), (ATerm)ATmakeAppl1(SDF_afun24, (ATerm)ATmakeAppl0(SDF_afun250)), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm)ATmakeInt(92)))));
}

/*}}}  */
/*{{{  SDF_LexStrChar SDF_makeLexStrCharNormal(char ch) */

SDF_LexStrChar SDF_makeLexStrCharNormal(char ch)
{
  return (SDF_LexStrChar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(93), (ATerm)ATmakeInt(255))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(35), (ATerm)ATmakeInt(91))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(32), (ATerm)ATmakeInt(33))))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun242))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun252)))))), (ATerm)ATmakeList1((ATerm) ((ATerm) SDF_byteToChar(ch))));
}

/*}}}  */
/*{{{  SDF_StrChar SDF_makeStrCharLexToCf(SDF_LexStrChar StrChar) */

SDF_StrChar SDF_makeStrCharLexToCf(SDF_LexStrChar StrChar)
{
  return (SDF_StrChar)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun242)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun242))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm) StrChar));
}

/*}}}  */
/*{{{  SDF_LexStrCon SDF_makeLexStrConDefault(SDF_LexStrCharChars chars) */

SDF_LexStrCon SDF_makeLexStrConDefault(SDF_LexStrCharChars chars)
{
  return (SDF_LexStrCon)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(34)))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun242))))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeInt(34)))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun10))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun42)))))), (ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeInt(34)), (ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun242))))), (ATerm) chars)), (ATerm)ATmakeInt(34)));
}

/*}}}  */
/*{{{  SDF_StrCon SDF_makeStrConLexToCf(SDF_LexStrCon StrCon) */

SDF_StrCon SDF_makeStrConLexToCf(SDF_LexStrCon StrCon)
{
  return (SDF_StrCon)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun10)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun10))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm) StrCon));
}

/*}}}  */
/*{{{  SDF_LexStrCharChars SDF_makeLexStrCharCharsEmpty(void) */

SDF_LexStrCharChars SDF_makeLexStrCharCharsEmpty(void)
{
  return (SDF_LexStrCharChars)(ATerm)ATempty;
}

/*}}}  */
/*{{{  SDF_LexStrCharChars SDF_makeLexStrCharCharsSingle(SDF_LexStrChar head) */

SDF_LexStrCharChars SDF_makeLexStrCharCharsSingle(SDF_LexStrChar head)
{
  return (SDF_LexStrCharChars)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  SDF_LexStrCharChars SDF_makeLexStrCharCharsMany(SDF_LexStrChar head, SDF_LexStrCharChars tail) */

SDF_LexStrCharChars SDF_makeLexStrCharCharsMany(SDF_LexStrChar head, SDF_LexStrCharChars tail)
{
  return (SDF_LexStrCharChars)(ATerm)ATinsert((ATermList)tail, (ATerm) head);
}

/*}}}  */
/*{{{  SDF_LexNatCon SDF_makeLexNatConDigits(const char* list) */

SDF_LexNatCon SDF_makeLexNatConDigits(const char* list)
{
  return (SDF_LexNatCon)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun7, (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(48), (ATerm)ATmakeInt(57))))))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun47))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun159)))))), (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun7, (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(48), (ATerm)ATmakeInt(57))))))), (ATerm) ((ATerm) SDF_stringToChars(list)))));
}

/*}}}  */
/*{{{  SDF_NatCon SDF_makeNatConLexToCf(SDF_LexNatCon NatCon) */

SDF_NatCon SDF_makeNatConLexToCf(SDF_LexNatCon NatCon)
{
  return (SDF_NatCon)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun47)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun47))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm) NatCon));
}

/*}}}  */
/*{{{  SDF_LexIdCon SDF_makeLexIdConDefault(char head, const char* tail) */

SDF_LexIdCon SDF_makeLexIdConDefault(char head, const char* tail)
{
  return (SDF_LexIdCon)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(97), (ATerm)ATmakeInt(122))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(65), (ATerm)ATmakeInt(90))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(48), (ATerm)ATmakeInt(57))), (ATerm)ATmakeInt(45)))))), (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(97), (ATerm)ATmakeInt(122))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(65), (ATerm)ATmakeInt(90))))), (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun16))), (ATerm)ATmakeAppl1(SDF_afun12, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun13, (ATerm)ATmakeAppl1(SDF_afun14, (ATerm)ATmakeAppl0(SDF_afun42)))))), (ATerm)ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl1(SDF_afun8, (ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun96, (ATerm)ATmakeAppl1(SDF_afun30, (ATerm)ATinsert(ATinsert(ATinsert(ATmakeList1((ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(97), (ATerm)ATmakeInt(122))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(65), (ATerm)ATmakeInt(90))), (ATerm)ATmakeAppl2(SDF_afun157, (ATerm)ATmakeInt(48), (ATerm)ATmakeInt(57))), (ATerm)ATmakeInt(45)))))), (ATerm) ((ATerm) SDF_stringToChars(tail)))), (ATerm) ((ATerm) SDF_byteToChar(head))));
}

/*}}}  */
/*{{{  SDF_IdCon SDF_makeIdConLexToCf(SDF_LexIdCon IdCon) */

SDF_IdCon SDF_makeIdConLexToCf(SDF_LexIdCon IdCon)
{
  return (SDF_IdCon)(ATerm)ATmakeAppl2(SDF_afun0, (ATerm)ATmakeAppl3(SDF_afun1, (ATerm)ATmakeList1((ATerm)ATmakeAppl1(SDF_afun6, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun16)))), (ATerm)ATmakeAppl1(SDF_afun2, (ATerm)ATmakeAppl1(SDF_afun9, (ATerm)ATmakeAppl0(SDF_afun16))), (ATerm)ATmakeAppl0(SDF_afun5)), (ATerm)ATmakeList1((ATerm) IdCon));
}

/*}}}  */

/*}}}  */
/*{{{  equality functions */

ATbool SDF_isEqualOptLayout(SDF_OptLayout arg0, SDF_OptLayout arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLayout(SDF_Layout arg0, SDF_Layout arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLexLayoutList(SDF_LexLayoutList arg0, SDF_LexLayoutList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualAFun(SDF_AFun arg0, SDF_AFun arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualATerm(SDF_ATerm arg0, SDF_ATerm arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualAnnotation(SDF_Annotation arg0, SDF_Annotation arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualStart(SDF_Start arg0, SDF_Start arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualATermArgs(SDF_ATermArgs arg0, SDF_ATermArgs arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualATermElems(SDF_ATermElems arg0, SDF_ATermElems arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualATermAnnos(SDF_ATermAnnos arg0, SDF_ATermAnnos arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualIntCon(SDF_IntCon arg0, SDF_IntCon arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualOptExp(SDF_OptExp arg0, SDF_OptExp arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualRealCon(SDF_RealCon arg0, SDF_RealCon arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualGrammar(SDF_Grammar arg0, SDF_Grammar arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualAlias(SDF_Alias arg0, SDF_Alias arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualAliases(SDF_Aliases arg0, SDF_Aliases arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualAliasList(SDF_AliasList arg0, SDF_AliasList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualSymbol(SDF_Symbol arg0, SDF_Symbol arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualCharRange(SDF_CharRange arg0, SDF_CharRange arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualCharRanges(SDF_CharRanges arg0, SDF_CharRanges arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualOptCharRanges(SDF_OptCharRanges arg0, SDF_OptCharRanges arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualCharClass(SDF_CharClass arg0, SDF_CharClass arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLexNumChar(SDF_LexNumChar arg0, SDF_LexNumChar arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualNumChar(SDF_NumChar arg0, SDF_NumChar arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLexShortChar(SDF_LexShortChar arg0, SDF_LexShortChar arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualShortChar(SDF_ShortChar arg0, SDF_ShortChar arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualCharacter(SDF_Character arg0, SDF_Character arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualATermAttribute(SDF_ATermAttribute arg0, SDF_ATermAttribute arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualAttribute(SDF_Attribute arg0, SDF_Attribute arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualAttributes(SDF_Attributes arg0, SDF_Attributes arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualProduction(SDF_Production arg0, SDF_Production arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualProductions(SDF_Productions arg0, SDF_Productions arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualAttributeList(SDF_AttributeList arg0, SDF_AttributeList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualProductionList(SDF_ProductionList arg0, SDF_ProductionList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLabel(SDF_Label arg0, SDF_Label arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualSymbolArguments(SDF_SymbolArguments arg0, SDF_SymbolArguments arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLexModuleWord(SDF_LexModuleWord arg0, SDF_LexModuleWord arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualModuleWord(SDF_ModuleWord arg0, SDF_ModuleWord arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLexModuleId(SDF_LexModuleId arg0, SDF_LexModuleId arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualModuleId(SDF_ModuleId arg0, SDF_ModuleId arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualDefinition(SDF_Definition arg0, SDF_Definition arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualModule(SDF_Module arg0, SDF_Module arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualSection(SDF_Section arg0, SDF_Section arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualSections(SDF_Sections arg0, SDF_Sections arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualModuleName(SDF_ModuleName arg0, SDF_ModuleName arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualImpSection(SDF_ImpSection arg0, SDF_ImpSection arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualImports(SDF_Imports arg0, SDF_Imports arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualImport(SDF_Import arg0, SDF_Import arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualModuleList(SDF_ModuleList arg0, SDF_ModuleList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualImpSectionList(SDF_ImpSectionList arg0, SDF_ImpSectionList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualSectionList(SDF_SectionList arg0, SDF_SectionList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualImportList(SDF_ImportList arg0, SDF_ImportList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualAssociativity(SDF_Associativity arg0, SDF_Associativity arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualArgumentIndicator(SDF_ArgumentIndicator arg0, SDF_ArgumentIndicator arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualGroup(SDF_Group arg0, SDF_Group arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualPriority(SDF_Priority arg0, SDF_Priority arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualPriorities(SDF_Priorities arg0, SDF_Priorities arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualNatConArguments(SDF_NatConArguments arg0, SDF_NatConArguments arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualGroupList(SDF_GroupList arg0, SDF_GroupList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualPriorityList(SDF_PriorityList arg0, SDF_PriorityList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualSymbolTail(SDF_SymbolTail arg0, SDF_SymbolTail arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualSymbolRest(SDF_SymbolRest arg0, SDF_SymbolRest arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualRenamings(SDF_Renamings arg0, SDF_Renamings arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualRenaming(SDF_Renaming arg0, SDF_Renaming arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualRenamingList(SDF_RenamingList arg0, SDF_RenamingList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLookahead(SDF_Lookahead arg0, SDF_Lookahead arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLookaheads(SDF_Lookaheads arg0, SDF_Lookaheads arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualRestriction(SDF_Restriction arg0, SDF_Restriction arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualRestrictions(SDF_Restrictions arg0, SDF_Restrictions arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLookaheadList(SDF_LookaheadList arg0, SDF_LookaheadList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualRestrictionList(SDF_RestrictionList arg0, SDF_RestrictionList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualSDF(SDF_SDF arg0, SDF_SDF arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualSort(SDF_Sort arg0, SDF_Sort arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLexSort(SDF_LexSort arg0, SDF_LexSort arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualSymbolParameters(SDF_SymbolParameters arg0, SDF_SymbolParameters arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualSymbols(SDF_Symbols arg0, SDF_Symbols arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualSymbolList(SDF_SymbolList arg0, SDF_SymbolList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLexLayout(SDF_LexLayout arg0, SDF_LexLayout arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLexStrChar(SDF_LexStrChar arg0, SDF_LexStrChar arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualStrChar(SDF_StrChar arg0, SDF_StrChar arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLexStrCon(SDF_LexStrCon arg0, SDF_LexStrCon arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualStrCon(SDF_StrCon arg0, SDF_StrCon arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLexStrCharChars(SDF_LexStrCharChars arg0, SDF_LexStrCharChars arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLexNatCon(SDF_LexNatCon arg0, SDF_LexNatCon arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualNatCon(SDF_NatCon arg0, SDF_NatCon arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualLexIdCon(SDF_LexIdCon arg0, SDF_LexIdCon arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool SDF_isEqualIdCon(SDF_IdCon arg0, SDF_IdCon arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

/*}}}  */
/*{{{  SDF_OptLayout accessors */

/*{{{  ATbool SDF_isValidOptLayout(SDF_OptLayout arg) */

ATbool SDF_isValidOptLayout(SDF_OptLayout arg)
{
  if (SDF_isOptLayoutAbsent(arg)) {
    return ATtrue;
  }
  else if (SDF_isOptLayoutPresent(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isOptLayoutAbsent(SDF_OptLayout arg) */

inline ATbool SDF_isOptLayoutAbsent(SDF_OptLayout arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternOptLayoutAbsent);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isOptLayoutPresent(SDF_OptLayout arg) */

inline ATbool SDF_isOptLayoutPresent(SDF_OptLayout arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternOptLayoutPresent, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasOptLayoutLayout(SDF_OptLayout arg) */

ATbool SDF_hasOptLayoutLayout(SDF_OptLayout arg)
{
  if (SDF_isOptLayoutPresent(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_Layout SDF_getOptLayoutLayout(SDF_OptLayout arg) */

SDF_Layout SDF_getOptLayoutLayout(SDF_OptLayout arg)
{
  
    return (SDF_Layout)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_setOptLayoutLayout(SDF_OptLayout arg, SDF_Layout layout) */

SDF_OptLayout SDF_setOptLayoutLayout(SDF_OptLayout arg, SDF_Layout layout)
{
  if (SDF_isOptLayoutPresent(arg)) {
    return (SDF_OptLayout)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) layout), 0), 1);
  }

  ATabort("OptLayout has no Layout: %t\n", arg);
  return (SDF_OptLayout)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Layout accessors */

/*{{{  ATbool SDF_isValidLayout(SDF_Layout arg) */

ATbool SDF_isValidLayout(SDF_Layout arg)
{
  if (SDF_isLayoutLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLayoutLexToCf(SDF_Layout arg) */

inline ATbool SDF_isLayoutLexToCf(SDF_Layout arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternLayoutLexToCf, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasLayoutList(SDF_Layout arg) */

ATbool SDF_hasLayoutList(SDF_Layout arg)
{
  if (SDF_isLayoutLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_LexLayoutList SDF_getLayoutList(SDF_Layout arg) */

SDF_LexLayoutList SDF_getLayoutList(SDF_Layout arg)
{
  
    return (SDF_LexLayoutList)ATgetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), 1);
}

/*}}}  */
/*{{{  SDF_Layout SDF_setLayoutList(SDF_Layout arg, SDF_LexLayoutList list) */

SDF_Layout SDF_setLayoutList(SDF_Layout arg, SDF_LexLayoutList list)
{
  if (SDF_isLayoutLexToCf(arg)) {
    return (SDF_Layout)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), (ATerm)((ATerm) list), 1), 0), 1);
  }

  ATabort("Layout has no List: %t\n", arg);
  return (SDF_Layout)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_LexLayoutList accessors */

/*{{{  ATbool SDF_isValidLexLayoutList(SDF_LexLayoutList arg) */

ATbool SDF_isValidLexLayoutList(SDF_LexLayoutList arg)
{
  if (SDF_isLexLayoutListEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexLayoutListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexLayoutListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexLayoutListEmpty(SDF_LexLayoutList arg) */

inline ATbool SDF_isLexLayoutListEmpty(SDF_LexLayoutList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternLexLayoutListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexLayoutListSingle(SDF_LexLayoutList arg) */

inline ATbool SDF_isLexLayoutListSingle(SDF_LexLayoutList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexLayoutListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexLayoutListMany(SDF_LexLayoutList arg) */

inline ATbool SDF_isLexLayoutListMany(SDF_LexLayoutList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexLayoutListMany, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasLexLayoutListHead(SDF_LexLayoutList arg) */

ATbool SDF_hasLexLayoutListHead(SDF_LexLayoutList arg)
{
  if (SDF_isLexLayoutListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexLayoutListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexLayoutListTail(SDF_LexLayoutList arg) */

ATbool SDF_hasLexLayoutListTail(SDF_LexLayoutList arg)
{
  if (SDF_isLexLayoutListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_LexLayout SDF_getLexLayoutListHead(SDF_LexLayoutList arg) */

SDF_LexLayout SDF_getLexLayoutListHead(SDF_LexLayoutList arg)
{
  if (SDF_isLexLayoutListSingle(arg)) {
    return (SDF_LexLayout)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_LexLayout)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_LexLayoutList SDF_getLexLayoutListTail(SDF_LexLayoutList arg) */

SDF_LexLayoutList SDF_getLexLayoutListTail(SDF_LexLayoutList arg)
{
  
    return (SDF_LexLayoutList)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_LexLayoutList SDF_setLexLayoutListHead(SDF_LexLayoutList arg, SDF_LexLayout head) */

SDF_LexLayoutList SDF_setLexLayoutListHead(SDF_LexLayoutList arg, SDF_LexLayout head)
{
  if (SDF_isLexLayoutListSingle(arg)) {
    return (SDF_LexLayoutList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isLexLayoutListMany(arg)) {
    return (SDF_LexLayoutList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("LexLayoutList has no Head: %t\n", arg);
  return (SDF_LexLayoutList)NULL;
}

/*}}}  */
/*{{{  SDF_LexLayoutList SDF_setLexLayoutListTail(SDF_LexLayoutList arg, SDF_LexLayoutList tail) */

SDF_LexLayoutList SDF_setLexLayoutListTail(SDF_LexLayoutList arg, SDF_LexLayoutList tail)
{
  if (SDF_isLexLayoutListMany(arg)) {
    return (SDF_LexLayoutList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 1);
  }

  ATabort("LexLayoutList has no Tail: %t\n", arg);
  return (SDF_LexLayoutList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_AFun accessors */

/*{{{  ATbool SDF_isValidAFun(SDF_AFun arg) */

ATbool SDF_isValidAFun(SDF_AFun arg)
{
  if (SDF_isAFunQuoted(arg)) {
    return ATtrue;
  }
  else if (SDF_isAFunUnquoted(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isAFunQuoted(SDF_AFun arg) */

inline ATbool SDF_isAFunQuoted(SDF_AFun arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAFunQuoted, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isAFunUnquoted(SDF_AFun arg) */

inline ATbool SDF_isAFunUnquoted(SDF_AFun arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAFunUnquoted, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasAFunStrCon(SDF_AFun arg) */

ATbool SDF_hasAFunStrCon(SDF_AFun arg)
{
  if (SDF_isAFunQuoted(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAFunIdCon(SDF_AFun arg) */

ATbool SDF_hasAFunIdCon(SDF_AFun arg)
{
  if (SDF_isAFunUnquoted(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_StrCon SDF_getAFunStrCon(SDF_AFun arg) */

SDF_StrCon SDF_getAFunStrCon(SDF_AFun arg)
{
  
    return (SDF_StrCon)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_IdCon SDF_getAFunIdCon(SDF_AFun arg) */

SDF_IdCon SDF_getAFunIdCon(SDF_AFun arg)
{
  
    return (SDF_IdCon)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_AFun SDF_setAFunStrCon(SDF_AFun arg, SDF_StrCon StrCon) */

SDF_AFun SDF_setAFunStrCon(SDF_AFun arg, SDF_StrCon StrCon)
{
  if (SDF_isAFunQuoted(arg)) {
    return (SDF_AFun)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) StrCon), 0), 1);
  }

  ATabort("AFun has no StrCon: %t\n", arg);
  return (SDF_AFun)NULL;
}

/*}}}  */
/*{{{  SDF_AFun SDF_setAFunIdCon(SDF_AFun arg, SDF_IdCon IdCon) */

SDF_AFun SDF_setAFunIdCon(SDF_AFun arg, SDF_IdCon IdCon)
{
  if (SDF_isAFunUnquoted(arg)) {
    return (SDF_AFun)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) IdCon), 0), 1);
  }

  ATabort("AFun has no IdCon: %t\n", arg);
  return (SDF_AFun)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_ATerm accessors */

/*{{{  ATbool SDF_isValidATerm(SDF_ATerm arg) */

ATbool SDF_isValidATerm(SDF_ATerm arg)
{
  if (SDF_isATermInt(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermReal(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermFun(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermAppl(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermPlaceholder(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermList(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermAnnotated(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermInt(SDF_ATerm arg) */

inline ATbool SDF_isATermInt(SDF_ATerm arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternATermInt, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermReal(SDF_ATerm arg) */

inline ATbool SDF_isATermReal(SDF_ATerm arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternATermReal, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermFun(SDF_ATerm arg) */

inline ATbool SDF_isATermFun(SDF_ATerm arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternATermFun, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermAppl(SDF_ATerm arg) */

inline ATbool SDF_isATermAppl(SDF_ATerm arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternATermAppl, NULL, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermPlaceholder(SDF_ATerm arg) */

inline ATbool SDF_isATermPlaceholder(SDF_ATerm arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternATermPlaceholder, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermList(SDF_ATerm arg) */

inline ATbool SDF_isATermList(SDF_ATerm arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternATermList, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermAnnotated(SDF_ATerm arg) */

inline ATbool SDF_isATermAnnotated(SDF_ATerm arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternATermAnnotated, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasATermIntCon(SDF_ATerm arg) */

ATbool SDF_hasATermIntCon(SDF_ATerm arg)
{
  if (SDF_isATermInt(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermRealCon(SDF_ATerm arg) */

ATbool SDF_hasATermRealCon(SDF_ATerm arg)
{
  if (SDF_isATermReal(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermFun(SDF_ATerm arg) */

ATbool SDF_hasATermFun(SDF_ATerm arg)
{
  if (SDF_isATermFun(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermAppl(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermWsAfterFun(SDF_ATerm arg) */

ATbool SDF_hasATermWsAfterFun(SDF_ATerm arg)
{
  if (SDF_isATermAppl(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermWsAfterParenOpen(SDF_ATerm arg) */

ATbool SDF_hasATermWsAfterParenOpen(SDF_ATerm arg)
{
  if (SDF_isATermAppl(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermArgs(SDF_ATerm arg) */

ATbool SDF_hasATermArgs(SDF_ATerm arg)
{
  if (SDF_isATermAppl(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermWsAfterArgs(SDF_ATerm arg) */

ATbool SDF_hasATermWsAfterArgs(SDF_ATerm arg)
{
  if (SDF_isATermAppl(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermWsAfterLessThan(SDF_ATerm arg) */

ATbool SDF_hasATermWsAfterLessThan(SDF_ATerm arg)
{
  if (SDF_isATermPlaceholder(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermType(SDF_ATerm arg) */

ATbool SDF_hasATermType(SDF_ATerm arg)
{
  if (SDF_isATermPlaceholder(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermWsAfterType(SDF_ATerm arg) */

ATbool SDF_hasATermWsAfterType(SDF_ATerm arg)
{
  if (SDF_isATermPlaceholder(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermWsAfterBracketOpen(SDF_ATerm arg) */

ATbool SDF_hasATermWsAfterBracketOpen(SDF_ATerm arg)
{
  if (SDF_isATermList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermElems(SDF_ATerm arg) */

ATbool SDF_hasATermElems(SDF_ATerm arg)
{
  if (SDF_isATermList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermWsAfterElems(SDF_ATerm arg) */

ATbool SDF_hasATermWsAfterElems(SDF_ATerm arg)
{
  if (SDF_isATermList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermTrm(SDF_ATerm arg) */

ATbool SDF_hasATermTrm(SDF_ATerm arg)
{
  if (SDF_isATermAnnotated(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermWsAfterTrm(SDF_ATerm arg) */

ATbool SDF_hasATermWsAfterTrm(SDF_ATerm arg)
{
  if (SDF_isATermAnnotated(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermAnnotation(SDF_ATerm arg) */

ATbool SDF_hasATermAnnotation(SDF_ATerm arg)
{
  if (SDF_isATermAnnotated(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_IntCon SDF_getATermIntCon(SDF_ATerm arg) */

SDF_IntCon SDF_getATermIntCon(SDF_ATerm arg)
{
  
    return (SDF_IntCon)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_RealCon SDF_getATermRealCon(SDF_ATerm arg) */

SDF_RealCon SDF_getATermRealCon(SDF_ATerm arg)
{
  
    return (SDF_RealCon)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_AFun SDF_getATermFun(SDF_ATerm arg) */

SDF_AFun SDF_getATermFun(SDF_ATerm arg)
{
  if (SDF_isATermFun(arg)) {
    return (SDF_AFun)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
  }
  else 
    return (SDF_AFun)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getATermWsAfterFun(SDF_ATerm arg) */

SDF_OptLayout SDF_getATermWsAfterFun(SDF_ATerm arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getATermWsAfterParenOpen(SDF_ATerm arg) */

SDF_OptLayout SDF_getATermWsAfterParenOpen(SDF_ATerm arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_ATermArgs SDF_getATermArgs(SDF_ATerm arg) */

SDF_ATermArgs SDF_getATermArgs(SDF_ATerm arg)
{
  
    return (SDF_ATermArgs)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getATermWsAfterArgs(SDF_ATerm arg) */

SDF_OptLayout SDF_getATermWsAfterArgs(SDF_ATerm arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getATermWsAfterLessThan(SDF_ATerm arg) */

SDF_OptLayout SDF_getATermWsAfterLessThan(SDF_ATerm arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_ATerm SDF_getATermType(SDF_ATerm arg) */

SDF_ATerm SDF_getATermType(SDF_ATerm arg)
{
  
    return (SDF_ATerm)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getATermWsAfterType(SDF_ATerm arg) */

SDF_OptLayout SDF_getATermWsAfterType(SDF_ATerm arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getATermWsAfterBracketOpen(SDF_ATerm arg) */

SDF_OptLayout SDF_getATermWsAfterBracketOpen(SDF_ATerm arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_ATermElems SDF_getATermElems(SDF_ATerm arg) */

SDF_ATermElems SDF_getATermElems(SDF_ATerm arg)
{
  
    return (SDF_ATermElems)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getATermWsAfterElems(SDF_ATerm arg) */

SDF_OptLayout SDF_getATermWsAfterElems(SDF_ATerm arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_ATerm SDF_getATermTrm(SDF_ATerm arg) */

SDF_ATerm SDF_getATermTrm(SDF_ATerm arg)
{
  
    return (SDF_ATerm)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getATermWsAfterTrm(SDF_ATerm arg) */

SDF_OptLayout SDF_getATermWsAfterTrm(SDF_ATerm arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Annotation SDF_getATermAnnotation(SDF_ATerm arg) */

SDF_Annotation SDF_getATermAnnotation(SDF_ATerm arg)
{
  
    return (SDF_Annotation)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermIntCon(SDF_ATerm arg, SDF_IntCon IntCon) */

SDF_ATerm SDF_setATermIntCon(SDF_ATerm arg, SDF_IntCon IntCon)
{
  if (SDF_isATermInt(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) IntCon), 0), 1);
  }

  ATabort("ATerm has no IntCon: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermRealCon(SDF_ATerm arg, SDF_RealCon RealCon) */

SDF_ATerm SDF_setATermRealCon(SDF_ATerm arg, SDF_RealCon RealCon)
{
  if (SDF_isATermReal(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) RealCon), 0), 1);
  }

  ATabort("ATerm has no RealCon: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermFun(SDF_ATerm arg, SDF_AFun fun) */

SDF_ATerm SDF_setATermFun(SDF_ATerm arg, SDF_AFun fun)
{
  if (SDF_isATermFun(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) fun), 0), 1);
  }
  else if (SDF_isATermAppl(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) fun), 0), 1);
  }

  ATabort("ATerm has no Fun: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermWsAfterFun(SDF_ATerm arg, SDF_OptLayout wsAfterFun) */

SDF_ATerm SDF_setATermWsAfterFun(SDF_ATerm arg, SDF_OptLayout wsAfterFun)
{
  if (SDF_isATermAppl(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterFun), 1), 1);
  }

  ATabort("ATerm has no WsAfterFun: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermWsAfterParenOpen(SDF_ATerm arg, SDF_OptLayout wsAfterParenOpen) */

SDF_ATerm SDF_setATermWsAfterParenOpen(SDF_ATerm arg, SDF_OptLayout wsAfterParenOpen)
{
  if (SDF_isATermAppl(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenOpen), 3), 1);
  }

  ATabort("ATerm has no WsAfterParenOpen: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermArgs(SDF_ATerm arg, SDF_ATermArgs args) */

SDF_ATerm SDF_setATermArgs(SDF_ATerm arg, SDF_ATermArgs args)
{
  if (SDF_isATermAppl(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4), (ATerm)((ATerm) args), 1), 4), 1);
  }

  ATabort("ATerm has no Args: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermWsAfterArgs(SDF_ATerm arg, SDF_OptLayout wsAfterArgs) */

SDF_ATerm SDF_setATermWsAfterArgs(SDF_ATerm arg, SDF_OptLayout wsAfterArgs)
{
  if (SDF_isATermAppl(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterArgs), 5), 1);
  }

  ATabort("ATerm has no WsAfterArgs: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermWsAfterLessThan(SDF_ATerm arg, SDF_OptLayout wsAfterLessThan) */

SDF_ATerm SDF_setATermWsAfterLessThan(SDF_ATerm arg, SDF_OptLayout wsAfterLessThan)
{
  if (SDF_isATermPlaceholder(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLessThan), 1), 1);
  }

  ATabort("ATerm has no WsAfterLessThan: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermType(SDF_ATerm arg, SDF_ATerm type) */

SDF_ATerm SDF_setATermType(SDF_ATerm arg, SDF_ATerm type)
{
  if (SDF_isATermPlaceholder(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) type), 2), 1);
  }

  ATabort("ATerm has no Type: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermWsAfterType(SDF_ATerm arg, SDF_OptLayout wsAfterType) */

SDF_ATerm SDF_setATermWsAfterType(SDF_ATerm arg, SDF_OptLayout wsAfterType)
{
  if (SDF_isATermPlaceholder(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterType), 3), 1);
  }

  ATabort("ATerm has no WsAfterType: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermWsAfterBracketOpen(SDF_ATerm arg, SDF_OptLayout wsAfterBracketOpen) */

SDF_ATerm SDF_setATermWsAfterBracketOpen(SDF_ATerm arg, SDF_OptLayout wsAfterBracketOpen)
{
  if (SDF_isATermList(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBracketOpen), 1), 1);
  }

  ATabort("ATerm has no WsAfterBracketOpen: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermElems(SDF_ATerm arg, SDF_ATermElems elems) */

SDF_ATerm SDF_setATermElems(SDF_ATerm arg, SDF_ATermElems elems)
{
  if (SDF_isATermList(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), (ATerm)((ATerm) elems), 1), 2), 1);
  }

  ATabort("ATerm has no Elems: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermWsAfterElems(SDF_ATerm arg, SDF_OptLayout wsAfterElems) */

SDF_ATerm SDF_setATermWsAfterElems(SDF_ATerm arg, SDF_OptLayout wsAfterElems)
{
  if (SDF_isATermList(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterElems), 3), 1);
  }

  ATabort("ATerm has no WsAfterElems: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermTrm(SDF_ATerm arg, SDF_ATerm trm) */

SDF_ATerm SDF_setATermTrm(SDF_ATerm arg, SDF_ATerm trm)
{
  if (SDF_isATermAnnotated(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) trm), 0), 1);
  }

  ATabort("ATerm has no Trm: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermWsAfterTrm(SDF_ATerm arg, SDF_OptLayout wsAfterTrm) */

SDF_ATerm SDF_setATermWsAfterTrm(SDF_ATerm arg, SDF_OptLayout wsAfterTrm)
{
  if (SDF_isATermAnnotated(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterTrm), 1), 1);
  }

  ATabort("ATerm has no WsAfterTrm: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_setATermAnnotation(SDF_ATerm arg, SDF_Annotation Annotation) */

SDF_ATerm SDF_setATermAnnotation(SDF_ATerm arg, SDF_Annotation Annotation)
{
  if (SDF_isATermAnnotated(arg)) {
    return (SDF_ATerm)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Annotation), 2), 1);
  }

  ATabort("ATerm has no Annotation: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Annotation accessors */

/*{{{  ATbool SDF_isValidAnnotation(SDF_Annotation arg) */

ATbool SDF_isValidAnnotation(SDF_Annotation arg)
{
  if (SDF_isAnnotationDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isAnnotationDefault(SDF_Annotation arg) */

inline ATbool SDF_isAnnotationDefault(SDF_Annotation arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternAnnotationDefault, NULL, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasAnnotationWsAfterBraceOpen(SDF_Annotation arg) */

ATbool SDF_hasAnnotationWsAfterBraceOpen(SDF_Annotation arg)
{
  if (SDF_isAnnotationDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAnnotationAnnos(SDF_Annotation arg) */

ATbool SDF_hasAnnotationAnnos(SDF_Annotation arg)
{
  if (SDF_isAnnotationDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAnnotationWsAfterAnnos(SDF_Annotation arg) */

ATbool SDF_hasAnnotationWsAfterAnnos(SDF_Annotation arg)
{
  if (SDF_isAnnotationDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getAnnotationWsAfterBraceOpen(SDF_Annotation arg) */

SDF_OptLayout SDF_getAnnotationWsAfterBraceOpen(SDF_Annotation arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_ATermAnnos SDF_getAnnotationAnnos(SDF_Annotation arg) */

SDF_ATermAnnos SDF_getAnnotationAnnos(SDF_Annotation arg)
{
  
    return (SDF_ATermAnnos)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getAnnotationWsAfterAnnos(SDF_Annotation arg) */

SDF_OptLayout SDF_getAnnotationWsAfterAnnos(SDF_Annotation arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Annotation SDF_setAnnotationWsAfterBraceOpen(SDF_Annotation arg, SDF_OptLayout wsAfterBraceOpen) */

SDF_Annotation SDF_setAnnotationWsAfterBraceOpen(SDF_Annotation arg, SDF_OptLayout wsAfterBraceOpen)
{
  if (SDF_isAnnotationDefault(arg)) {
    return (SDF_Annotation)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBraceOpen), 1), 1);
  }

  ATabort("Annotation has no WsAfterBraceOpen: %t\n", arg);
  return (SDF_Annotation)NULL;
}

/*}}}  */
/*{{{  SDF_Annotation SDF_setAnnotationAnnos(SDF_Annotation arg, SDF_ATermAnnos annos) */

SDF_Annotation SDF_setAnnotationAnnos(SDF_Annotation arg, SDF_ATermAnnos annos)
{
  if (SDF_isAnnotationDefault(arg)) {
    return (SDF_Annotation)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), (ATerm)((ATerm) annos), 1), 2), 1);
  }

  ATabort("Annotation has no Annos: %t\n", arg);
  return (SDF_Annotation)NULL;
}

/*}}}  */
/*{{{  SDF_Annotation SDF_setAnnotationWsAfterAnnos(SDF_Annotation arg, SDF_OptLayout wsAfterAnnos) */

SDF_Annotation SDF_setAnnotationWsAfterAnnos(SDF_Annotation arg, SDF_OptLayout wsAfterAnnos)
{
  if (SDF_isAnnotationDefault(arg)) {
    return (SDF_Annotation)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterAnnos), 3), 1);
  }

  ATabort("Annotation has no WsAfterAnnos: %t\n", arg);
  return (SDF_Annotation)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Start accessors */

/*{{{  ATbool SDF_isValidStart(SDF_Start arg) */

ATbool SDF_isValidStart(SDF_Start arg)
{
  if (SDF_isStartATerm(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartIntCon(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartRealCon(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartModule(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartSDF(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isStartATerm(SDF_Start arg) */

inline ATbool SDF_isStartATerm(SDF_Start arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternStartATerm, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isStartIntCon(SDF_Start arg) */

inline ATbool SDF_isStartIntCon(SDF_Start arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternStartIntCon, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isStartRealCon(SDF_Start arg) */

inline ATbool SDF_isStartRealCon(SDF_Start arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternStartRealCon, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isStartModule(SDF_Start arg) */

inline ATbool SDF_isStartModule(SDF_Start arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternStartModule, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isStartSDF(SDF_Start arg) */

inline ATbool SDF_isStartSDF(SDF_Start arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternStartSDF, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasStartWsBefore(SDF_Start arg) */

ATbool SDF_hasStartWsBefore(SDF_Start arg)
{
  if (SDF_isStartATerm(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartIntCon(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartRealCon(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartModule(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartSDF(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasStartTopATerm(SDF_Start arg) */

ATbool SDF_hasStartTopATerm(SDF_Start arg)
{
  if (SDF_isStartATerm(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasStartWsAfter(SDF_Start arg) */

ATbool SDF_hasStartWsAfter(SDF_Start arg)
{
  if (SDF_isStartATerm(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartIntCon(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartRealCon(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartModule(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartSDF(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasStartAmbCnt(SDF_Start arg) */

ATbool SDF_hasStartAmbCnt(SDF_Start arg)
{
  if (SDF_isStartATerm(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartIntCon(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartRealCon(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartModule(arg)) {
    return ATtrue;
  }
  else if (SDF_isStartSDF(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasStartTopIntCon(SDF_Start arg) */

ATbool SDF_hasStartTopIntCon(SDF_Start arg)
{
  if (SDF_isStartIntCon(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasStartTopRealCon(SDF_Start arg) */

ATbool SDF_hasStartTopRealCon(SDF_Start arg)
{
  if (SDF_isStartRealCon(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasStartTopModule(SDF_Start arg) */

ATbool SDF_hasStartTopModule(SDF_Start arg)
{
  if (SDF_isStartModule(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasStartTopSDF(SDF_Start arg) */

ATbool SDF_hasStartTopSDF(SDF_Start arg)
{
  if (SDF_isStartSDF(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getStartWsBefore(SDF_Start arg) */

SDF_OptLayout SDF_getStartWsBefore(SDF_Start arg)
{
  if (SDF_isStartATerm(arg)) {
    return (SDF_OptLayout)ATgetFirst((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1));
  }
  else if (SDF_isStartIntCon(arg)) {
    return (SDF_OptLayout)ATgetFirst((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1));
  }
  else if (SDF_isStartRealCon(arg)) {
    return (SDF_OptLayout)ATgetFirst((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1));
  }
  else if (SDF_isStartModule(arg)) {
    return (SDF_OptLayout)ATgetFirst((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1));
  }
  else 
    return (SDF_OptLayout)ATgetFirst((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1));
}

/*}}}  */
/*{{{  SDF_ATerm SDF_getStartTopATerm(SDF_Start arg) */

SDF_ATerm SDF_getStartTopATerm(SDF_Start arg)
{
  
    return (SDF_ATerm)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getStartWsAfter(SDF_Start arg) */

SDF_OptLayout SDF_getStartWsAfter(SDF_Start arg)
{
  if (SDF_isStartATerm(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 2);
  }
  else if (SDF_isStartIntCon(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 2);
  }
  else if (SDF_isStartRealCon(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 2);
  }
  else if (SDF_isStartModule(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 2);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 2);
}

/*}}}  */
/*{{{  int SDF_getStartAmbCnt(SDF_Start arg) */

int SDF_getStartAmbCnt(SDF_Start arg)
{
  if (SDF_isStartATerm(arg)) {
    return (int)ATgetInt((ATermInt) ATgetArgument((ATermAppl)arg, 1));
  }
  else if (SDF_isStartIntCon(arg)) {
    return (int)ATgetInt((ATermInt) ATgetArgument((ATermAppl)arg, 1));
  }
  else if (SDF_isStartRealCon(arg)) {
    return (int)ATgetInt((ATermInt) ATgetArgument((ATermAppl)arg, 1));
  }
  else if (SDF_isStartModule(arg)) {
    return (int)ATgetInt((ATermInt) ATgetArgument((ATermAppl)arg, 1));
  }
  else 
    return (int)ATgetInt((ATermInt) ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_IntCon SDF_getStartTopIntCon(SDF_Start arg) */

SDF_IntCon SDF_getStartTopIntCon(SDF_Start arg)
{
  
    return (SDF_IntCon)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 1);
}

/*}}}  */
/*{{{  SDF_RealCon SDF_getStartTopRealCon(SDF_Start arg) */

SDF_RealCon SDF_getStartTopRealCon(SDF_Start arg)
{
  
    return (SDF_RealCon)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 1);
}

/*}}}  */
/*{{{  SDF_Module SDF_getStartTopModule(SDF_Start arg) */

SDF_Module SDF_getStartTopModule(SDF_Start arg)
{
  
    return (SDF_Module)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 1);
}

/*}}}  */
/*{{{  SDF_SDF SDF_getStartTopSDF(SDF_Start arg) */

SDF_SDF SDF_getStartTopSDF(SDF_Start arg)
{
  
    return (SDF_SDF)ATelementAt((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), 1);
}

/*}}}  */
/*{{{  SDF_Start SDF_setStartWsBefore(SDF_Start arg, SDF_OptLayout wsBefore) */

SDF_Start SDF_setStartWsBefore(SDF_Start arg, SDF_OptLayout wsBefore)
{
  if (SDF_isStartATerm(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsBefore), 0), 1), 0);
  }
  else if (SDF_isStartIntCon(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsBefore), 0), 1), 0);
  }
  else if (SDF_isStartRealCon(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsBefore), 0), 1), 0);
  }
  else if (SDF_isStartModule(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsBefore), 0), 1), 0);
  }
  else if (SDF_isStartSDF(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsBefore), 0), 1), 0);
  }

  ATabort("Start has no WsBefore: %t\n", arg);
  return (SDF_Start)NULL;
}

/*}}}  */
/*{{{  SDF_Start SDF_setStartTopATerm(SDF_Start arg, SDF_ATerm topATerm) */

SDF_Start SDF_setStartTopATerm(SDF_Start arg, SDF_ATerm topATerm)
{
  if (SDF_isStartATerm(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) topATerm), 1), 1), 0);
  }

  ATabort("Start has no TopATerm: %t\n", arg);
  return (SDF_Start)NULL;
}

/*}}}  */
/*{{{  SDF_Start SDF_setStartWsAfter(SDF_Start arg, SDF_OptLayout wsAfter) */

SDF_Start SDF_setStartWsAfter(SDF_Start arg, SDF_OptLayout wsAfter)
{
  if (SDF_isStartATerm(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsAfter), 2), 1), 0);
  }
  else if (SDF_isStartIntCon(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsAfter), 2), 1), 0);
  }
  else if (SDF_isStartRealCon(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsAfter), 2), 1), 0);
  }
  else if (SDF_isStartModule(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsAfter), 2), 1), 0);
  }
  else if (SDF_isStartSDF(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) wsAfter), 2), 1), 0);
  }

  ATabort("Start has no WsAfter: %t\n", arg);
  return (SDF_Start)NULL;
}

/*}}}  */
/*{{{  SDF_Start SDF_setStartAmbCnt(SDF_Start arg, int ambCnt) */

SDF_Start SDF_setStartAmbCnt(SDF_Start arg, int ambCnt)
{
  if (SDF_isStartATerm(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeInt(ambCnt)), 1);
  }
  else if (SDF_isStartIntCon(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeInt(ambCnt)), 1);
  }
  else if (SDF_isStartRealCon(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeInt(ambCnt)), 1);
  }
  else if (SDF_isStartModule(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeInt(ambCnt)), 1);
  }
  else if (SDF_isStartSDF(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeInt(ambCnt)), 1);
  }

  ATabort("Start has no AmbCnt: %t\n", arg);
  return (SDF_Start)NULL;
}

/*}}}  */
/*{{{  SDF_Start SDF_setStartTopIntCon(SDF_Start arg, SDF_IntCon topIntCon) */

SDF_Start SDF_setStartTopIntCon(SDF_Start arg, SDF_IntCon topIntCon)
{
  if (SDF_isStartIntCon(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) topIntCon), 1), 1), 0);
  }

  ATabort("Start has no TopIntCon: %t\n", arg);
  return (SDF_Start)NULL;
}

/*}}}  */
/*{{{  SDF_Start SDF_setStartTopRealCon(SDF_Start arg, SDF_RealCon topRealCon) */

SDF_Start SDF_setStartTopRealCon(SDF_Start arg, SDF_RealCon topRealCon)
{
  if (SDF_isStartRealCon(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) topRealCon), 1), 1), 0);
  }

  ATabort("Start has no TopRealCon: %t\n", arg);
  return (SDF_Start)NULL;
}

/*}}}  */
/*{{{  SDF_Start SDF_setStartTopModule(SDF_Start arg, SDF_Module topModule) */

SDF_Start SDF_setStartTopModule(SDF_Start arg, SDF_Module topModule)
{
  if (SDF_isStartModule(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) topModule), 1), 1), 0);
  }

  ATabort("Start has no TopModule: %t\n", arg);
  return (SDF_Start)NULL;
}

/*}}}  */
/*{{{  SDF_Start SDF_setStartTopSDF(SDF_Start arg, SDF_SDF topSDF) */

SDF_Start SDF_setStartTopSDF(SDF_Start arg, SDF_SDF topSDF)
{
  if (SDF_isStartSDF(arg)) {
    return (SDF_Start)ATsetArgument((ATermAppl)arg, (ATerm)ATsetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)ATgetArgument((ATermAppl)arg, 0), 1), (ATerm)((ATerm) topSDF), 1), 1), 0);
  }

  ATabort("Start has no TopSDF: %t\n", arg);
  return (SDF_Start)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_ATermArgs accessors */

/*{{{  ATbool SDF_isValidATermArgs(SDF_ATermArgs arg) */

ATbool SDF_isValidATermArgs(SDF_ATermArgs arg)
{
  if (SDF_isATermArgsEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermArgsSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermArgsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermArgsEmpty(SDF_ATermArgs arg) */

inline ATbool SDF_isATermArgsEmpty(SDF_ATermArgs arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternATermArgsEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermArgsSingle(SDF_ATermArgs arg) */

inline ATbool SDF_isATermArgsSingle(SDF_ATermArgs arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternATermArgsSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermArgsMany(SDF_ATermArgs arg) */

inline ATbool SDF_isATermArgsMany(SDF_ATermArgs arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternATermArgsMany, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasATermArgsHead(SDF_ATermArgs arg) */

ATbool SDF_hasATermArgsHead(SDF_ATermArgs arg)
{
  if (SDF_isATermArgsSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermArgsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermArgsWsAfterHead(SDF_ATermArgs arg) */

ATbool SDF_hasATermArgsWsAfterHead(SDF_ATermArgs arg)
{
  if (SDF_isATermArgsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermArgsWsAfterSep(SDF_ATermArgs arg) */

ATbool SDF_hasATermArgsWsAfterSep(SDF_ATermArgs arg)
{
  if (SDF_isATermArgsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermArgsTail(SDF_ATermArgs arg) */

ATbool SDF_hasATermArgsTail(SDF_ATermArgs arg)
{
  if (SDF_isATermArgsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_ATermArgs SDF_getATermArgsTail(SDF_ATermArgs arg) */

SDF_ATermArgs SDF_getATermArgsTail(SDF_ATermArgs arg)
{
  assert(!SDF_isATermArgsEmpty(arg) && "getTail on an empty list");
  if (SDF_isATermArgsSingle(arg)) {
    return (SDF_ATermArgs) SDF_makeATermArgsEmpty();
  }
  else {
  
    return (SDF_ATermArgs)ATgetTail((ATermList)arg, 4);
  }
}

/*}}}  */
/*{{{  SDF_ATerm SDF_getATermArgsHead(SDF_ATermArgs arg) */

SDF_ATerm SDF_getATermArgsHead(SDF_ATermArgs arg)
{
  if (SDF_isATermArgsSingle(arg)) {
    return (SDF_ATerm)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_ATerm)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getATermArgsWsAfterHead(SDF_ATermArgs arg) */

SDF_OptLayout SDF_getATermArgsWsAfterHead(SDF_ATermArgs arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getATermArgsWsAfterSep(SDF_ATermArgs arg) */

SDF_OptLayout SDF_getATermArgsWsAfterSep(SDF_ATermArgs arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 3);
}

/*}}}  */
/*{{{  SDF_ATermArgs SDF_setATermArgsHead(SDF_ATermArgs arg, SDF_ATerm head) */

SDF_ATermArgs SDF_setATermArgsHead(SDF_ATermArgs arg, SDF_ATerm head)
{
  if (SDF_isATermArgsSingle(arg)) {
    return (SDF_ATermArgs)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isATermArgsMany(arg)) {
    return (SDF_ATermArgs)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("ATermArgs has no Head: %t\n", arg);
  return (SDF_ATermArgs)NULL;
}

/*}}}  */
/*{{{  SDF_ATermArgs SDF_setATermArgsWsAfterHead(SDF_ATermArgs arg, SDF_OptLayout wsAfterHead) */

SDF_ATermArgs SDF_setATermArgsWsAfterHead(SDF_ATermArgs arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isATermArgsMany(arg)) {
    return (SDF_ATermArgs)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("ATermArgs has no WsAfterHead: %t\n", arg);
  return (SDF_ATermArgs)NULL;
}

/*}}}  */
/*{{{  SDF_ATermArgs SDF_setATermArgsWsAfterSep(SDF_ATermArgs arg, SDF_OptLayout wsAfterSep) */

SDF_ATermArgs SDF_setATermArgsWsAfterSep(SDF_ATermArgs arg, SDF_OptLayout wsAfterSep)
{
  if (SDF_isATermArgsMany(arg)) {
    return (SDF_ATermArgs)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterSep), 3);
  }

  ATabort("ATermArgs has no WsAfterSep: %t\n", arg);
  return (SDF_ATermArgs)NULL;
}

/*}}}  */
/*{{{  SDF_ATermArgs SDF_setATermArgsTail(SDF_ATermArgs arg, SDF_ATermArgs tail) */

SDF_ATermArgs SDF_setATermArgsTail(SDF_ATermArgs arg, SDF_ATermArgs tail)
{
  if (SDF_isATermArgsMany(arg)) {
    return (SDF_ATermArgs)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 4);
  }

  ATabort("ATermArgs has no Tail: %t\n", arg);
  return (SDF_ATermArgs)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_ATermElems accessors */

/*{{{  ATbool SDF_isValidATermElems(SDF_ATermElems arg) */

ATbool SDF_isValidATermElems(SDF_ATermElems arg)
{
  if (SDF_isATermElemsEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermElemsSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermElemsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermElemsEmpty(SDF_ATermElems arg) */

inline ATbool SDF_isATermElemsEmpty(SDF_ATermElems arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternATermElemsEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermElemsSingle(SDF_ATermElems arg) */

inline ATbool SDF_isATermElemsSingle(SDF_ATermElems arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternATermElemsSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermElemsMany(SDF_ATermElems arg) */

inline ATbool SDF_isATermElemsMany(SDF_ATermElems arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternATermElemsMany, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasATermElemsHead(SDF_ATermElems arg) */

ATbool SDF_hasATermElemsHead(SDF_ATermElems arg)
{
  if (SDF_isATermElemsSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermElemsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermElemsWsAfterHead(SDF_ATermElems arg) */

ATbool SDF_hasATermElemsWsAfterHead(SDF_ATermElems arg)
{
  if (SDF_isATermElemsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermElemsWsAfterSep(SDF_ATermElems arg) */

ATbool SDF_hasATermElemsWsAfterSep(SDF_ATermElems arg)
{
  if (SDF_isATermElemsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermElemsTail(SDF_ATermElems arg) */

ATbool SDF_hasATermElemsTail(SDF_ATermElems arg)
{
  if (SDF_isATermElemsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_ATermElems SDF_getATermElemsTail(SDF_ATermElems arg) */

SDF_ATermElems SDF_getATermElemsTail(SDF_ATermElems arg)
{
  assert(!SDF_isATermElemsEmpty(arg) && "getTail on an empty list");
  if (SDF_isATermElemsSingle(arg)) {
    return (SDF_ATermElems) SDF_makeATermElemsEmpty();
  }
  else {
  
    return (SDF_ATermElems)ATgetTail((ATermList)arg, 4);
  }
}

/*}}}  */
/*{{{  SDF_ATerm SDF_getATermElemsHead(SDF_ATermElems arg) */

SDF_ATerm SDF_getATermElemsHead(SDF_ATermElems arg)
{
  if (SDF_isATermElemsSingle(arg)) {
    return (SDF_ATerm)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_ATerm)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getATermElemsWsAfterHead(SDF_ATermElems arg) */

SDF_OptLayout SDF_getATermElemsWsAfterHead(SDF_ATermElems arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getATermElemsWsAfterSep(SDF_ATermElems arg) */

SDF_OptLayout SDF_getATermElemsWsAfterSep(SDF_ATermElems arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 3);
}

/*}}}  */
/*{{{  SDF_ATermElems SDF_setATermElemsHead(SDF_ATermElems arg, SDF_ATerm head) */

SDF_ATermElems SDF_setATermElemsHead(SDF_ATermElems arg, SDF_ATerm head)
{
  if (SDF_isATermElemsSingle(arg)) {
    return (SDF_ATermElems)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isATermElemsMany(arg)) {
    return (SDF_ATermElems)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("ATermElems has no Head: %t\n", arg);
  return (SDF_ATermElems)NULL;
}

/*}}}  */
/*{{{  SDF_ATermElems SDF_setATermElemsWsAfterHead(SDF_ATermElems arg, SDF_OptLayout wsAfterHead) */

SDF_ATermElems SDF_setATermElemsWsAfterHead(SDF_ATermElems arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isATermElemsMany(arg)) {
    return (SDF_ATermElems)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("ATermElems has no WsAfterHead: %t\n", arg);
  return (SDF_ATermElems)NULL;
}

/*}}}  */
/*{{{  SDF_ATermElems SDF_setATermElemsWsAfterSep(SDF_ATermElems arg, SDF_OptLayout wsAfterSep) */

SDF_ATermElems SDF_setATermElemsWsAfterSep(SDF_ATermElems arg, SDF_OptLayout wsAfterSep)
{
  if (SDF_isATermElemsMany(arg)) {
    return (SDF_ATermElems)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterSep), 3);
  }

  ATabort("ATermElems has no WsAfterSep: %t\n", arg);
  return (SDF_ATermElems)NULL;
}

/*}}}  */
/*{{{  SDF_ATermElems SDF_setATermElemsTail(SDF_ATermElems arg, SDF_ATermElems tail) */

SDF_ATermElems SDF_setATermElemsTail(SDF_ATermElems arg, SDF_ATermElems tail)
{
  if (SDF_isATermElemsMany(arg)) {
    return (SDF_ATermElems)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 4);
  }

  ATabort("ATermElems has no Tail: %t\n", arg);
  return (SDF_ATermElems)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_ATermAnnos accessors */

/*{{{  ATbool SDF_isValidATermAnnos(SDF_ATermAnnos arg) */

ATbool SDF_isValidATermAnnos(SDF_ATermAnnos arg)
{
  if (SDF_isATermAnnosEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermAnnosSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermAnnosMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermAnnosEmpty(SDF_ATermAnnos arg) */

inline ATbool SDF_isATermAnnosEmpty(SDF_ATermAnnos arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternATermAnnosEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermAnnosSingle(SDF_ATermAnnos arg) */

inline ATbool SDF_isATermAnnosSingle(SDF_ATermAnnos arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternATermAnnosSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermAnnosMany(SDF_ATermAnnos arg) */

inline ATbool SDF_isATermAnnosMany(SDF_ATermAnnos arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternATermAnnosMany, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasATermAnnosHead(SDF_ATermAnnos arg) */

ATbool SDF_hasATermAnnosHead(SDF_ATermAnnos arg)
{
  if (SDF_isATermAnnosSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isATermAnnosMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermAnnosWsAfterHead(SDF_ATermAnnos arg) */

ATbool SDF_hasATermAnnosWsAfterHead(SDF_ATermAnnos arg)
{
  if (SDF_isATermAnnosMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermAnnosWsAfterSep(SDF_ATermAnnos arg) */

ATbool SDF_hasATermAnnosWsAfterSep(SDF_ATermAnnos arg)
{
  if (SDF_isATermAnnosMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermAnnosTail(SDF_ATermAnnos arg) */

ATbool SDF_hasATermAnnosTail(SDF_ATermAnnos arg)
{
  if (SDF_isATermAnnosMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_ATermAnnos SDF_getATermAnnosTail(SDF_ATermAnnos arg) */

SDF_ATermAnnos SDF_getATermAnnosTail(SDF_ATermAnnos arg)
{
  assert(!SDF_isATermAnnosEmpty(arg) && "getTail on an empty list");
  if (SDF_isATermAnnosSingle(arg)) {
    return (SDF_ATermAnnos) SDF_makeATermAnnosEmpty();
  }
  else {
  
    return (SDF_ATermAnnos)ATgetTail((ATermList)arg, 4);
  }
}

/*}}}  */
/*{{{  SDF_ATerm SDF_getATermAnnosHead(SDF_ATermAnnos arg) */

SDF_ATerm SDF_getATermAnnosHead(SDF_ATermAnnos arg)
{
  if (SDF_isATermAnnosSingle(arg)) {
    return (SDF_ATerm)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_ATerm)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getATermAnnosWsAfterHead(SDF_ATermAnnos arg) */

SDF_OptLayout SDF_getATermAnnosWsAfterHead(SDF_ATermAnnos arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getATermAnnosWsAfterSep(SDF_ATermAnnos arg) */

SDF_OptLayout SDF_getATermAnnosWsAfterSep(SDF_ATermAnnos arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 3);
}

/*}}}  */
/*{{{  SDF_ATermAnnos SDF_setATermAnnosHead(SDF_ATermAnnos arg, SDF_ATerm head) */

SDF_ATermAnnos SDF_setATermAnnosHead(SDF_ATermAnnos arg, SDF_ATerm head)
{
  if (SDF_isATermAnnosSingle(arg)) {
    return (SDF_ATermAnnos)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isATermAnnosMany(arg)) {
    return (SDF_ATermAnnos)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("ATermAnnos has no Head: %t\n", arg);
  return (SDF_ATermAnnos)NULL;
}

/*}}}  */
/*{{{  SDF_ATermAnnos SDF_setATermAnnosWsAfterHead(SDF_ATermAnnos arg, SDF_OptLayout wsAfterHead) */

SDF_ATermAnnos SDF_setATermAnnosWsAfterHead(SDF_ATermAnnos arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isATermAnnosMany(arg)) {
    return (SDF_ATermAnnos)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("ATermAnnos has no WsAfterHead: %t\n", arg);
  return (SDF_ATermAnnos)NULL;
}

/*}}}  */
/*{{{  SDF_ATermAnnos SDF_setATermAnnosWsAfterSep(SDF_ATermAnnos arg, SDF_OptLayout wsAfterSep) */

SDF_ATermAnnos SDF_setATermAnnosWsAfterSep(SDF_ATermAnnos arg, SDF_OptLayout wsAfterSep)
{
  if (SDF_isATermAnnosMany(arg)) {
    return (SDF_ATermAnnos)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterSep), 3);
  }

  ATabort("ATermAnnos has no WsAfterSep: %t\n", arg);
  return (SDF_ATermAnnos)NULL;
}

/*}}}  */
/*{{{  SDF_ATermAnnos SDF_setATermAnnosTail(SDF_ATermAnnos arg, SDF_ATermAnnos tail) */

SDF_ATermAnnos SDF_setATermAnnosTail(SDF_ATermAnnos arg, SDF_ATermAnnos tail)
{
  if (SDF_isATermAnnosMany(arg)) {
    return (SDF_ATermAnnos)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 4);
  }

  ATabort("ATermAnnos has no Tail: %t\n", arg);
  return (SDF_ATermAnnos)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_IntCon accessors */

/*{{{  ATbool SDF_isValidIntCon(SDF_IntCon arg) */

ATbool SDF_isValidIntCon(SDF_IntCon arg)
{
  if (SDF_isIntConNatural(arg)) {
    return ATtrue;
  }
  else if (SDF_isIntConPositive(arg)) {
    return ATtrue;
  }
  else if (SDF_isIntConNegative(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isIntConNatural(SDF_IntCon arg) */

inline ATbool SDF_isIntConNatural(SDF_IntCon arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternIntConNatural, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isIntConPositive(SDF_IntCon arg) */

inline ATbool SDF_isIntConPositive(SDF_IntCon arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternIntConPositive, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isIntConNegative(SDF_IntCon arg) */

inline ATbool SDF_isIntConNegative(SDF_IntCon arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternIntConNegative, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasIntConNatCon(SDF_IntCon arg) */

ATbool SDF_hasIntConNatCon(SDF_IntCon arg)
{
  if (SDF_isIntConNatural(arg)) {
    return ATtrue;
  }
  else if (SDF_isIntConPositive(arg)) {
    return ATtrue;
  }
  else if (SDF_isIntConNegative(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasIntConWsAfterPos(SDF_IntCon arg) */

ATbool SDF_hasIntConWsAfterPos(SDF_IntCon arg)
{
  if (SDF_isIntConPositive(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasIntConWsAfterNeg(SDF_IntCon arg) */

ATbool SDF_hasIntConWsAfterNeg(SDF_IntCon arg)
{
  if (SDF_isIntConNegative(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_NatCon SDF_getIntConNatCon(SDF_IntCon arg) */

SDF_NatCon SDF_getIntConNatCon(SDF_IntCon arg)
{
  if (SDF_isIntConNatural(arg)) {
    return (SDF_NatCon)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
  }
  else if (SDF_isIntConPositive(arg)) {
    return (SDF_NatCon)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else 
    return (SDF_NatCon)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getIntConWsAfterPos(SDF_IntCon arg) */

SDF_OptLayout SDF_getIntConWsAfterPos(SDF_IntCon arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getIntConWsAfterNeg(SDF_IntCon arg) */

SDF_OptLayout SDF_getIntConWsAfterNeg(SDF_IntCon arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_IntCon SDF_setIntConNatCon(SDF_IntCon arg, SDF_NatCon NatCon) */

SDF_IntCon SDF_setIntConNatCon(SDF_IntCon arg, SDF_NatCon NatCon)
{
  if (SDF_isIntConNatural(arg)) {
    return (SDF_IntCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) NatCon), 0), 1);
  }
  else if (SDF_isIntConPositive(arg)) {
    return (SDF_IntCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) NatCon), 2), 1);
  }
  else if (SDF_isIntConNegative(arg)) {
    return (SDF_IntCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) NatCon), 2), 1);
  }

  ATabort("IntCon has no NatCon: %t\n", arg);
  return (SDF_IntCon)NULL;
}

/*}}}  */
/*{{{  SDF_IntCon SDF_setIntConWsAfterPos(SDF_IntCon arg, SDF_OptLayout wsAfterPos) */

SDF_IntCon SDF_setIntConWsAfterPos(SDF_IntCon arg, SDF_OptLayout wsAfterPos)
{
  if (SDF_isIntConPositive(arg)) {
    return (SDF_IntCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterPos), 1), 1);
  }

  ATabort("IntCon has no WsAfterPos: %t\n", arg);
  return (SDF_IntCon)NULL;
}

/*}}}  */
/*{{{  SDF_IntCon SDF_setIntConWsAfterNeg(SDF_IntCon arg, SDF_OptLayout wsAfterNeg) */

SDF_IntCon SDF_setIntConWsAfterNeg(SDF_IntCon arg, SDF_OptLayout wsAfterNeg)
{
  if (SDF_isIntConNegative(arg)) {
    return (SDF_IntCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterNeg), 1), 1);
  }

  ATabort("IntCon has no WsAfterNeg: %t\n", arg);
  return (SDF_IntCon)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_OptExp accessors */

/*{{{  ATbool SDF_isValidOptExp(SDF_OptExp arg) */

ATbool SDF_isValidOptExp(SDF_OptExp arg)
{
  if (SDF_isOptExpPresent(arg)) {
    return ATtrue;
  }
  else if (SDF_isOptExpAbsent(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isOptExpPresent(SDF_OptExp arg) */

inline ATbool SDF_isOptExpPresent(SDF_OptExp arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternOptExpPresent, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isOptExpAbsent(SDF_OptExp arg) */

inline ATbool SDF_isOptExpAbsent(SDF_OptExp arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternOptExpAbsent);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasOptExpWsAfterE(SDF_OptExp arg) */

ATbool SDF_hasOptExpWsAfterE(SDF_OptExp arg)
{
  if (SDF_isOptExpPresent(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasOptExpIntCon(SDF_OptExp arg) */

ATbool SDF_hasOptExpIntCon(SDF_OptExp arg)
{
  if (SDF_isOptExpPresent(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getOptExpWsAfterE(SDF_OptExp arg) */

SDF_OptLayout SDF_getOptExpWsAfterE(SDF_OptExp arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_IntCon SDF_getOptExpIntCon(SDF_OptExp arg) */

SDF_IntCon SDF_getOptExpIntCon(SDF_OptExp arg)
{
  
    return (SDF_IntCon)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptExp SDF_setOptExpWsAfterE(SDF_OptExp arg, SDF_OptLayout wsAfterE) */

SDF_OptExp SDF_setOptExpWsAfterE(SDF_OptExp arg, SDF_OptLayout wsAfterE)
{
  if (SDF_isOptExpPresent(arg)) {
    return (SDF_OptExp)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterE), 1), 1);
  }

  ATabort("OptExp has no WsAfterE: %t\n", arg);
  return (SDF_OptExp)NULL;
}

/*}}}  */
/*{{{  SDF_OptExp SDF_setOptExpIntCon(SDF_OptExp arg, SDF_IntCon IntCon) */

SDF_OptExp SDF_setOptExpIntCon(SDF_OptExp arg, SDF_IntCon IntCon)
{
  if (SDF_isOptExpPresent(arg)) {
    return (SDF_OptExp)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) IntCon), 2), 1);
  }

  ATabort("OptExp has no IntCon: %t\n", arg);
  return (SDF_OptExp)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_RealCon accessors */

/*{{{  ATbool SDF_isValidRealCon(SDF_RealCon arg) */

ATbool SDF_isValidRealCon(SDF_RealCon arg)
{
  if (SDF_isRealConRealCon(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isRealConRealCon(SDF_RealCon arg) */

inline ATbool SDF_isRealConRealCon(SDF_RealCon arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternRealConRealCon, NULL, NULL, NULL, NULL, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasRealConBase(SDF_RealCon arg) */

ATbool SDF_hasRealConBase(SDF_RealCon arg)
{
  if (SDF_isRealConRealCon(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRealConWsAfterBase(SDF_RealCon arg) */

ATbool SDF_hasRealConWsAfterBase(SDF_RealCon arg)
{
  if (SDF_isRealConRealCon(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRealConWsAfterPeriod(SDF_RealCon arg) */

ATbool SDF_hasRealConWsAfterPeriod(SDF_RealCon arg)
{
  if (SDF_isRealConRealCon(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRealConDecimal(SDF_RealCon arg) */

ATbool SDF_hasRealConDecimal(SDF_RealCon arg)
{
  if (SDF_isRealConRealCon(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRealConWsAfterDecimal(SDF_RealCon arg) */

ATbool SDF_hasRealConWsAfterDecimal(SDF_RealCon arg)
{
  if (SDF_isRealConRealCon(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRealConExp(SDF_RealCon arg) */

ATbool SDF_hasRealConExp(SDF_RealCon arg)
{
  if (SDF_isRealConRealCon(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_IntCon SDF_getRealConBase(SDF_RealCon arg) */

SDF_IntCon SDF_getRealConBase(SDF_RealCon arg)
{
  
    return (SDF_IntCon)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getRealConWsAfterBase(SDF_RealCon arg) */

SDF_OptLayout SDF_getRealConWsAfterBase(SDF_RealCon arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getRealConWsAfterPeriod(SDF_RealCon arg) */

SDF_OptLayout SDF_getRealConWsAfterPeriod(SDF_RealCon arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_NatCon SDF_getRealConDecimal(SDF_RealCon arg) */

SDF_NatCon SDF_getRealConDecimal(SDF_RealCon arg)
{
  
    return (SDF_NatCon)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getRealConWsAfterDecimal(SDF_RealCon arg) */

SDF_OptLayout SDF_getRealConWsAfterDecimal(SDF_RealCon arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_OptExp SDF_getRealConExp(SDF_RealCon arg) */

SDF_OptExp SDF_getRealConExp(SDF_RealCon arg)
{
  
    return (SDF_OptExp)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 6);
}

/*}}}  */
/*{{{  SDF_RealCon SDF_setRealConBase(SDF_RealCon arg, SDF_IntCon base) */

SDF_RealCon SDF_setRealConBase(SDF_RealCon arg, SDF_IntCon base)
{
  if (SDF_isRealConRealCon(arg)) {
    return (SDF_RealCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) base), 0), 1);
  }

  ATabort("RealCon has no Base: %t\n", arg);
  return (SDF_RealCon)NULL;
}

/*}}}  */
/*{{{  SDF_RealCon SDF_setRealConWsAfterBase(SDF_RealCon arg, SDF_OptLayout wsAfterBase) */

SDF_RealCon SDF_setRealConWsAfterBase(SDF_RealCon arg, SDF_OptLayout wsAfterBase)
{
  if (SDF_isRealConRealCon(arg)) {
    return (SDF_RealCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBase), 1), 1);
  }

  ATabort("RealCon has no WsAfterBase: %t\n", arg);
  return (SDF_RealCon)NULL;
}

/*}}}  */
/*{{{  SDF_RealCon SDF_setRealConWsAfterPeriod(SDF_RealCon arg, SDF_OptLayout wsAfterPeriod) */

SDF_RealCon SDF_setRealConWsAfterPeriod(SDF_RealCon arg, SDF_OptLayout wsAfterPeriod)
{
  if (SDF_isRealConRealCon(arg)) {
    return (SDF_RealCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterPeriod), 3), 1);
  }

  ATabort("RealCon has no WsAfterPeriod: %t\n", arg);
  return (SDF_RealCon)NULL;
}

/*}}}  */
/*{{{  SDF_RealCon SDF_setRealConDecimal(SDF_RealCon arg, SDF_NatCon decimal) */

SDF_RealCon SDF_setRealConDecimal(SDF_RealCon arg, SDF_NatCon decimal)
{
  if (SDF_isRealConRealCon(arg)) {
    return (SDF_RealCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) decimal), 4), 1);
  }

  ATabort("RealCon has no Decimal: %t\n", arg);
  return (SDF_RealCon)NULL;
}

/*}}}  */
/*{{{  SDF_RealCon SDF_setRealConWsAfterDecimal(SDF_RealCon arg, SDF_OptLayout wsAfterDecimal) */

SDF_RealCon SDF_setRealConWsAfterDecimal(SDF_RealCon arg, SDF_OptLayout wsAfterDecimal)
{
  if (SDF_isRealConRealCon(arg)) {
    return (SDF_RealCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterDecimal), 5), 1);
  }

  ATabort("RealCon has no WsAfterDecimal: %t\n", arg);
  return (SDF_RealCon)NULL;
}

/*}}}  */
/*{{{  SDF_RealCon SDF_setRealConExp(SDF_RealCon arg, SDF_OptExp exp) */

SDF_RealCon SDF_setRealConExp(SDF_RealCon arg, SDF_OptExp exp)
{
  if (SDF_isRealConRealCon(arg)) {
    return (SDF_RealCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) exp), 6), 1);
  }

  ATabort("RealCon has no Exp: %t\n", arg);
  return (SDF_RealCon)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Grammar accessors */

/*{{{  ATbool SDF_isValidGrammar(SDF_Grammar arg) */

ATbool SDF_isValidGrammar(SDF_Grammar arg)
{
  if (SDF_isGrammarAliases(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalSyntax(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarContextFreeSyntax(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarVariables(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalVariables(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarEmptyGrammar(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarConcGrammars(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarBracket(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarSyntax(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarImpSection(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarPriorities(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarRestrictions(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalPriorities(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarContextFreePriorities(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalRestrictions(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarContextFreeRestrictions(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarSorts(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarKernelStartSymbols(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalStartSymbols(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarContextFreeStartSymbols(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarAliases(SDF_Grammar arg) */

inline ATbool SDF_isGrammarAliases(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarAliases, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarLexicalSyntax(SDF_Grammar arg) */

inline ATbool SDF_isGrammarLexicalSyntax(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarLexicalSyntax, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarContextFreeSyntax(SDF_Grammar arg) */

inline ATbool SDF_isGrammarContextFreeSyntax(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarContextFreeSyntax, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarVariables(SDF_Grammar arg) */

inline ATbool SDF_isGrammarVariables(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarVariables, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarLexicalVariables(SDF_Grammar arg) */

inline ATbool SDF_isGrammarLexicalVariables(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarLexicalVariables, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarEmptyGrammar(SDF_Grammar arg) */

inline ATbool SDF_isGrammarEmptyGrammar(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarEmptyGrammar);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarConcGrammars(SDF_Grammar arg) */

inline ATbool SDF_isGrammarConcGrammars(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarConcGrammars, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarBracket(SDF_Grammar arg) */

inline ATbool SDF_isGrammarBracket(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarBracket, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarSyntax(SDF_Grammar arg) */

inline ATbool SDF_isGrammarSyntax(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarSyntax, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarImpSection(SDF_Grammar arg) */

inline ATbool SDF_isGrammarImpSection(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarImpSection, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarPriorities(SDF_Grammar arg) */

inline ATbool SDF_isGrammarPriorities(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarPriorities, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarRestrictions(SDF_Grammar arg) */

inline ATbool SDF_isGrammarRestrictions(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarRestrictions, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarLexicalPriorities(SDF_Grammar arg) */

inline ATbool SDF_isGrammarLexicalPriorities(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarLexicalPriorities, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarContextFreePriorities(SDF_Grammar arg) */

inline ATbool SDF_isGrammarContextFreePriorities(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarContextFreePriorities, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarLexicalRestrictions(SDF_Grammar arg) */

inline ATbool SDF_isGrammarLexicalRestrictions(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarLexicalRestrictions, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarContextFreeRestrictions(SDF_Grammar arg) */

inline ATbool SDF_isGrammarContextFreeRestrictions(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarContextFreeRestrictions, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarSorts(SDF_Grammar arg) */

inline ATbool SDF_isGrammarSorts(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarSorts, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarKernelStartSymbols(SDF_Grammar arg) */

inline ATbool SDF_isGrammarKernelStartSymbols(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarKernelStartSymbols, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarLexicalStartSymbols(SDF_Grammar arg) */

inline ATbool SDF_isGrammarLexicalStartSymbols(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarLexicalStartSymbols, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGrammarContextFreeStartSymbols(SDF_Grammar arg) */

inline ATbool SDF_isGrammarContextFreeStartSymbols(SDF_Grammar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGrammarContextFreeStartSymbols, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarWsAfterAliases(SDF_Grammar arg) */

ATbool SDF_hasGrammarWsAfterAliases(SDF_Grammar arg)
{
  if (SDF_isGrammarAliases(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarAliases(SDF_Grammar arg) */

ATbool SDF_hasGrammarAliases(SDF_Grammar arg)
{
  if (SDF_isGrammarAliases(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarWsAfterLexical(SDF_Grammar arg) */

ATbool SDF_hasGrammarWsAfterLexical(SDF_Grammar arg)
{
  if (SDF_isGrammarLexicalSyntax(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalVariables(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalPriorities(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalRestrictions(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalStartSymbols(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarWsAfterSyntax(SDF_Grammar arg) */

ATbool SDF_hasGrammarWsAfterSyntax(SDF_Grammar arg)
{
  if (SDF_isGrammarLexicalSyntax(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarContextFreeSyntax(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarSyntax(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarProductions(SDF_Grammar arg) */

ATbool SDF_hasGrammarProductions(SDF_Grammar arg)
{
  if (SDF_isGrammarLexicalSyntax(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarContextFreeSyntax(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarVariables(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalVariables(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarSyntax(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarWsAfterContextFree(SDF_Grammar arg) */

ATbool SDF_hasGrammarWsAfterContextFree(SDF_Grammar arg)
{
  if (SDF_isGrammarContextFreeSyntax(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarContextFreePriorities(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarContextFreeRestrictions(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarContextFreeStartSymbols(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarWsAfterVariables(SDF_Grammar arg) */

ATbool SDF_hasGrammarWsAfterVariables(SDF_Grammar arg)
{
  if (SDF_isGrammarVariables(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalVariables(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarLeft(SDF_Grammar arg) */

ATbool SDF_hasGrammarLeft(SDF_Grammar arg)
{
  if (SDF_isGrammarConcGrammars(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarWsAfterLeft(SDF_Grammar arg) */

ATbool SDF_hasGrammarWsAfterLeft(SDF_Grammar arg)
{
  if (SDF_isGrammarConcGrammars(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarRight(SDF_Grammar arg) */

ATbool SDF_hasGrammarRight(SDF_Grammar arg)
{
  if (SDF_isGrammarConcGrammars(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarWsAfterParenOpen(SDF_Grammar arg) */

ATbool SDF_hasGrammarWsAfterParenOpen(SDF_Grammar arg)
{
  if (SDF_isGrammarBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarGrammar(SDF_Grammar arg) */

ATbool SDF_hasGrammarGrammar(SDF_Grammar arg)
{
  if (SDF_isGrammarBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarWsAfterGrammar(SDF_Grammar arg) */

ATbool SDF_hasGrammarWsAfterGrammar(SDF_Grammar arg)
{
  if (SDF_isGrammarBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarImpSection(SDF_Grammar arg) */

ATbool SDF_hasGrammarImpSection(SDF_Grammar arg)
{
  if (SDF_isGrammarImpSection(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarWsAfterPriorities(SDF_Grammar arg) */

ATbool SDF_hasGrammarWsAfterPriorities(SDF_Grammar arg)
{
  if (SDF_isGrammarPriorities(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalPriorities(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarContextFreePriorities(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarPriorities(SDF_Grammar arg) */

ATbool SDF_hasGrammarPriorities(SDF_Grammar arg)
{
  if (SDF_isGrammarPriorities(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalPriorities(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarContextFreePriorities(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarWsAfterRestrictions(SDF_Grammar arg) */

ATbool SDF_hasGrammarWsAfterRestrictions(SDF_Grammar arg)
{
  if (SDF_isGrammarRestrictions(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalRestrictions(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarContextFreeRestrictions(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarRestrictions(SDF_Grammar arg) */

ATbool SDF_hasGrammarRestrictions(SDF_Grammar arg)
{
  if (SDF_isGrammarRestrictions(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalRestrictions(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarContextFreeRestrictions(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarWsAfterSorts(SDF_Grammar arg) */

ATbool SDF_hasGrammarWsAfterSorts(SDF_Grammar arg)
{
  if (SDF_isGrammarSorts(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarSymbols(SDF_Grammar arg) */

ATbool SDF_hasGrammarSymbols(SDF_Grammar arg)
{
  if (SDF_isGrammarSorts(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarKernelStartSymbols(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalStartSymbols(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarContextFreeStartSymbols(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGrammarWsAfterStartSymbols(SDF_Grammar arg) */

ATbool SDF_hasGrammarWsAfterStartSymbols(SDF_Grammar arg)
{
  if (SDF_isGrammarKernelStartSymbols(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarLexicalStartSymbols(arg)) {
    return ATtrue;
  }
  else if (SDF_isGrammarContextFreeStartSymbols(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGrammarWsAfterAliases(SDF_Grammar arg) */

SDF_OptLayout SDF_getGrammarWsAfterAliases(SDF_Grammar arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Aliases SDF_getGrammarAliases(SDF_Grammar arg) */

SDF_Aliases SDF_getGrammarAliases(SDF_Grammar arg)
{
  
    return (SDF_Aliases)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGrammarWsAfterLexical(SDF_Grammar arg) */

SDF_OptLayout SDF_getGrammarWsAfterLexical(SDF_Grammar arg)
{
  if (SDF_isGrammarLexicalSyntax(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isGrammarLexicalVariables(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isGrammarLexicalPriorities(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isGrammarLexicalRestrictions(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGrammarWsAfterSyntax(SDF_Grammar arg) */

SDF_OptLayout SDF_getGrammarWsAfterSyntax(SDF_Grammar arg)
{
  if (SDF_isGrammarLexicalSyntax(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else if (SDF_isGrammarContextFreeSyntax(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Productions SDF_getGrammarProductions(SDF_Grammar arg) */

SDF_Productions SDF_getGrammarProductions(SDF_Grammar arg)
{
  if (SDF_isGrammarLexicalSyntax(arg)) {
    return (SDF_Productions)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
  }
  else if (SDF_isGrammarContextFreeSyntax(arg)) {
    return (SDF_Productions)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
  }
  else if (SDF_isGrammarVariables(arg)) {
    return (SDF_Productions)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else if (SDF_isGrammarLexicalVariables(arg)) {
    return (SDF_Productions)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
  }
  else 
    return (SDF_Productions)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGrammarWsAfterContextFree(SDF_Grammar arg) */

SDF_OptLayout SDF_getGrammarWsAfterContextFree(SDF_Grammar arg)
{
  if (SDF_isGrammarContextFreeSyntax(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isGrammarContextFreePriorities(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isGrammarContextFreeRestrictions(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGrammarWsAfterVariables(SDF_Grammar arg) */

SDF_OptLayout SDF_getGrammarWsAfterVariables(SDF_Grammar arg)
{
  if (SDF_isGrammarVariables(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Grammar SDF_getGrammarLeft(SDF_Grammar arg) */

SDF_Grammar SDF_getGrammarLeft(SDF_Grammar arg)
{
  
    return (SDF_Grammar)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGrammarWsAfterLeft(SDF_Grammar arg) */

SDF_OptLayout SDF_getGrammarWsAfterLeft(SDF_Grammar arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Grammar SDF_getGrammarRight(SDF_Grammar arg) */

SDF_Grammar SDF_getGrammarRight(SDF_Grammar arg)
{
  
    return (SDF_Grammar)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGrammarWsAfterParenOpen(SDF_Grammar arg) */

SDF_OptLayout SDF_getGrammarWsAfterParenOpen(SDF_Grammar arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Grammar SDF_getGrammarGrammar(SDF_Grammar arg) */

SDF_Grammar SDF_getGrammarGrammar(SDF_Grammar arg)
{
  
    return (SDF_Grammar)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGrammarWsAfterGrammar(SDF_Grammar arg) */

SDF_OptLayout SDF_getGrammarWsAfterGrammar(SDF_Grammar arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_ImpSection SDF_getGrammarImpSection(SDF_Grammar arg) */

SDF_ImpSection SDF_getGrammarImpSection(SDF_Grammar arg)
{
  
    return (SDF_ImpSection)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGrammarWsAfterPriorities(SDF_Grammar arg) */

SDF_OptLayout SDF_getGrammarWsAfterPriorities(SDF_Grammar arg)
{
  if (SDF_isGrammarPriorities(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isGrammarLexicalPriorities(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Priorities SDF_getGrammarPriorities(SDF_Grammar arg) */

SDF_Priorities SDF_getGrammarPriorities(SDF_Grammar arg)
{
  if (SDF_isGrammarPriorities(arg)) {
    return (SDF_Priorities)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else if (SDF_isGrammarLexicalPriorities(arg)) {
    return (SDF_Priorities)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
  }
  else 
    return (SDF_Priorities)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGrammarWsAfterRestrictions(SDF_Grammar arg) */

SDF_OptLayout SDF_getGrammarWsAfterRestrictions(SDF_Grammar arg)
{
  if (SDF_isGrammarRestrictions(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isGrammarLexicalRestrictions(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Restrictions SDF_getGrammarRestrictions(SDF_Grammar arg) */

SDF_Restrictions SDF_getGrammarRestrictions(SDF_Grammar arg)
{
  if (SDF_isGrammarRestrictions(arg)) {
    return (SDF_Restrictions)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else if (SDF_isGrammarLexicalRestrictions(arg)) {
    return (SDF_Restrictions)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
  }
  else 
    return (SDF_Restrictions)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGrammarWsAfterSorts(SDF_Grammar arg) */

SDF_OptLayout SDF_getGrammarWsAfterSorts(SDF_Grammar arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Symbols SDF_getGrammarSymbols(SDF_Grammar arg) */

SDF_Symbols SDF_getGrammarSymbols(SDF_Grammar arg)
{
  if (SDF_isGrammarSorts(arg)) {
    return (SDF_Symbols)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else if (SDF_isGrammarKernelStartSymbols(arg)) {
    return (SDF_Symbols)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else if (SDF_isGrammarLexicalStartSymbols(arg)) {
    return (SDF_Symbols)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
  }
  else 
    return (SDF_Symbols)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGrammarWsAfterStartSymbols(SDF_Grammar arg) */

SDF_OptLayout SDF_getGrammarWsAfterStartSymbols(SDF_Grammar arg)
{
  if (SDF_isGrammarKernelStartSymbols(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isGrammarLexicalStartSymbols(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarWsAfterAliases(SDF_Grammar arg, SDF_OptLayout wsAfterAliases) */

SDF_Grammar SDF_setGrammarWsAfterAliases(SDF_Grammar arg, SDF_OptLayout wsAfterAliases)
{
  if (SDF_isGrammarAliases(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterAliases), 1), 1);
  }

  ATabort("Grammar has no WsAfterAliases: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarAliases(SDF_Grammar arg, SDF_Aliases Aliases) */

SDF_Grammar SDF_setGrammarAliases(SDF_Grammar arg, SDF_Aliases Aliases)
{
  if (SDF_isGrammarAliases(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Aliases), 2), 1);
  }

  ATabort("Grammar has no Aliases: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarWsAfterLexical(SDF_Grammar arg, SDF_OptLayout wsAfterLexical) */

SDF_Grammar SDF_setGrammarWsAfterLexical(SDF_Grammar arg, SDF_OptLayout wsAfterLexical)
{
  if (SDF_isGrammarLexicalSyntax(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLexical), 1), 1);
  }
  else if (SDF_isGrammarLexicalVariables(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLexical), 1), 1);
  }
  else if (SDF_isGrammarLexicalPriorities(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLexical), 1), 1);
  }
  else if (SDF_isGrammarLexicalRestrictions(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLexical), 1), 1);
  }
  else if (SDF_isGrammarLexicalStartSymbols(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLexical), 1), 1);
  }

  ATabort("Grammar has no WsAfterLexical: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarWsAfterSyntax(SDF_Grammar arg, SDF_OptLayout wsAfterSyntax) */

SDF_Grammar SDF_setGrammarWsAfterSyntax(SDF_Grammar arg, SDF_OptLayout wsAfterSyntax)
{
  if (SDF_isGrammarLexicalSyntax(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSyntax), 3), 1);
  }
  else if (SDF_isGrammarContextFreeSyntax(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSyntax), 3), 1);
  }
  else if (SDF_isGrammarSyntax(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSyntax), 1), 1);
  }

  ATabort("Grammar has no WsAfterSyntax: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarProductions(SDF_Grammar arg, SDF_Productions Productions) */

SDF_Grammar SDF_setGrammarProductions(SDF_Grammar arg, SDF_Productions Productions)
{
  if (SDF_isGrammarLexicalSyntax(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Productions), 4), 1);
  }
  else if (SDF_isGrammarContextFreeSyntax(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Productions), 4), 1);
  }
  else if (SDF_isGrammarVariables(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Productions), 2), 1);
  }
  else if (SDF_isGrammarLexicalVariables(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Productions), 4), 1);
  }
  else if (SDF_isGrammarSyntax(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Productions), 2), 1);
  }

  ATabort("Grammar has no Productions: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarWsAfterContextFree(SDF_Grammar arg, SDF_OptLayout wsAfterContextFree) */

SDF_Grammar SDF_setGrammarWsAfterContextFree(SDF_Grammar arg, SDF_OptLayout wsAfterContextFree)
{
  if (SDF_isGrammarContextFreeSyntax(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterContextFree), 1), 1);
  }
  else if (SDF_isGrammarContextFreePriorities(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterContextFree), 1), 1);
  }
  else if (SDF_isGrammarContextFreeRestrictions(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterContextFree), 1), 1);
  }
  else if (SDF_isGrammarContextFreeStartSymbols(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterContextFree), 1), 1);
  }

  ATabort("Grammar has no WsAfterContextFree: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarWsAfterVariables(SDF_Grammar arg, SDF_OptLayout wsAfterVariables) */

SDF_Grammar SDF_setGrammarWsAfterVariables(SDF_Grammar arg, SDF_OptLayout wsAfterVariables)
{
  if (SDF_isGrammarVariables(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterVariables), 1), 1);
  }
  else if (SDF_isGrammarLexicalVariables(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterVariables), 3), 1);
  }

  ATabort("Grammar has no WsAfterVariables: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarLeft(SDF_Grammar arg, SDF_Grammar left) */

SDF_Grammar SDF_setGrammarLeft(SDF_Grammar arg, SDF_Grammar left)
{
  if (SDF_isGrammarConcGrammars(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) left), 0), 1);
  }

  ATabort("Grammar has no Left: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarWsAfterLeft(SDF_Grammar arg, SDF_OptLayout wsAfterLeft) */

SDF_Grammar SDF_setGrammarWsAfterLeft(SDF_Grammar arg, SDF_OptLayout wsAfterLeft)
{
  if (SDF_isGrammarConcGrammars(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLeft), 1), 1);
  }

  ATabort("Grammar has no WsAfterLeft: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarRight(SDF_Grammar arg, SDF_Grammar right) */

SDF_Grammar SDF_setGrammarRight(SDF_Grammar arg, SDF_Grammar right)
{
  if (SDF_isGrammarConcGrammars(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) right), 2), 1);
  }

  ATabort("Grammar has no Right: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarWsAfterParenOpen(SDF_Grammar arg, SDF_OptLayout wsAfterParenOpen) */

SDF_Grammar SDF_setGrammarWsAfterParenOpen(SDF_Grammar arg, SDF_OptLayout wsAfterParenOpen)
{
  if (SDF_isGrammarBracket(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenOpen), 1), 1);
  }

  ATabort("Grammar has no WsAfterParenOpen: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarGrammar(SDF_Grammar arg, SDF_Grammar Grammar) */

SDF_Grammar SDF_setGrammarGrammar(SDF_Grammar arg, SDF_Grammar Grammar)
{
  if (SDF_isGrammarBracket(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Grammar), 2), 1);
  }

  ATabort("Grammar has no Grammar: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarWsAfterGrammar(SDF_Grammar arg, SDF_OptLayout wsAfterGrammar) */

SDF_Grammar SDF_setGrammarWsAfterGrammar(SDF_Grammar arg, SDF_OptLayout wsAfterGrammar)
{
  if (SDF_isGrammarBracket(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterGrammar), 3), 1);
  }

  ATabort("Grammar has no WsAfterGrammar: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarImpSection(SDF_Grammar arg, SDF_ImpSection ImpSection) */

SDF_Grammar SDF_setGrammarImpSection(SDF_Grammar arg, SDF_ImpSection ImpSection)
{
  if (SDF_isGrammarImpSection(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ImpSection), 0), 1);
  }

  ATabort("Grammar has no ImpSection: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarWsAfterPriorities(SDF_Grammar arg, SDF_OptLayout wsAfterPriorities) */

SDF_Grammar SDF_setGrammarWsAfterPriorities(SDF_Grammar arg, SDF_OptLayout wsAfterPriorities)
{
  if (SDF_isGrammarPriorities(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterPriorities), 1), 1);
  }
  else if (SDF_isGrammarLexicalPriorities(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterPriorities), 3), 1);
  }
  else if (SDF_isGrammarContextFreePriorities(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterPriorities), 3), 1);
  }

  ATabort("Grammar has no WsAfterPriorities: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarPriorities(SDF_Grammar arg, SDF_Priorities Priorities) */

SDF_Grammar SDF_setGrammarPriorities(SDF_Grammar arg, SDF_Priorities Priorities)
{
  if (SDF_isGrammarPriorities(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Priorities), 2), 1);
  }
  else if (SDF_isGrammarLexicalPriorities(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Priorities), 4), 1);
  }
  else if (SDF_isGrammarContextFreePriorities(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Priorities), 4), 1);
  }

  ATabort("Grammar has no Priorities: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarWsAfterRestrictions(SDF_Grammar arg, SDF_OptLayout wsAfterRestrictions) */

SDF_Grammar SDF_setGrammarWsAfterRestrictions(SDF_Grammar arg, SDF_OptLayout wsAfterRestrictions)
{
  if (SDF_isGrammarRestrictions(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterRestrictions), 1), 1);
  }
  else if (SDF_isGrammarLexicalRestrictions(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterRestrictions), 3), 1);
  }
  else if (SDF_isGrammarContextFreeRestrictions(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterRestrictions), 3), 1);
  }

  ATabort("Grammar has no WsAfterRestrictions: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarRestrictions(SDF_Grammar arg, SDF_Restrictions Restrictions) */

SDF_Grammar SDF_setGrammarRestrictions(SDF_Grammar arg, SDF_Restrictions Restrictions)
{
  if (SDF_isGrammarRestrictions(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Restrictions), 2), 1);
  }
  else if (SDF_isGrammarLexicalRestrictions(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Restrictions), 4), 1);
  }
  else if (SDF_isGrammarContextFreeRestrictions(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Restrictions), 4), 1);
  }

  ATabort("Grammar has no Restrictions: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarWsAfterSorts(SDF_Grammar arg, SDF_OptLayout wsAfterSorts) */

SDF_Grammar SDF_setGrammarWsAfterSorts(SDF_Grammar arg, SDF_OptLayout wsAfterSorts)
{
  if (SDF_isGrammarSorts(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSorts), 1), 1);
  }

  ATabort("Grammar has no WsAfterSorts: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarSymbols(SDF_Grammar arg, SDF_Symbols Symbols) */

SDF_Grammar SDF_setGrammarSymbols(SDF_Grammar arg, SDF_Symbols Symbols)
{
  if (SDF_isGrammarSorts(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbols), 2), 1);
  }
  else if (SDF_isGrammarKernelStartSymbols(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbols), 2), 1);
  }
  else if (SDF_isGrammarLexicalStartSymbols(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbols), 4), 1);
  }
  else if (SDF_isGrammarContextFreeStartSymbols(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbols), 4), 1);
  }

  ATabort("Grammar has no Symbols: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_setGrammarWsAfterStartSymbols(SDF_Grammar arg, SDF_OptLayout wsAfterStartSymbols) */

SDF_Grammar SDF_setGrammarWsAfterStartSymbols(SDF_Grammar arg, SDF_OptLayout wsAfterStartSymbols)
{
  if (SDF_isGrammarKernelStartSymbols(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterStartSymbols), 1), 1);
  }
  else if (SDF_isGrammarLexicalStartSymbols(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterStartSymbols), 3), 1);
  }
  else if (SDF_isGrammarContextFreeStartSymbols(arg)) {
    return (SDF_Grammar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterStartSymbols), 3), 1);
  }

  ATabort("Grammar has no WsAfterStartSymbols: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Alias accessors */

/*{{{  ATbool SDF_isValidAlias(SDF_Alias arg) */

ATbool SDF_isValidAlias(SDF_Alias arg)
{
  if (SDF_isAliasAlias(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isAliasAlias(SDF_Alias arg) */

inline ATbool SDF_isAliasAlias(SDF_Alias arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternAliasAlias, NULL, NULL, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasAliasSymbol(SDF_Alias arg) */

ATbool SDF_hasAliasSymbol(SDF_Alias arg)
{
  if (SDF_isAliasAlias(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAliasWsAfterSymbol(SDF_Alias arg) */

ATbool SDF_hasAliasWsAfterSymbol(SDF_Alias arg)
{
  if (SDF_isAliasAlias(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAliasWsAfterArrow(SDF_Alias arg) */

ATbool SDF_hasAliasWsAfterArrow(SDF_Alias arg)
{
  if (SDF_isAliasAlias(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAliasAlias(SDF_Alias arg) */

ATbool SDF_hasAliasAlias(SDF_Alias arg)
{
  if (SDF_isAliasAlias(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getAliasSymbol(SDF_Alias arg) */

SDF_Symbol SDF_getAliasSymbol(SDF_Alias arg)
{
  
    return (SDF_Symbol)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getAliasWsAfterSymbol(SDF_Alias arg) */

SDF_OptLayout SDF_getAliasWsAfterSymbol(SDF_Alias arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getAliasWsAfterArrow(SDF_Alias arg) */

SDF_OptLayout SDF_getAliasWsAfterArrow(SDF_Alias arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getAliasAlias(SDF_Alias arg) */

SDF_Symbol SDF_getAliasAlias(SDF_Alias arg)
{
  
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_Alias SDF_setAliasSymbol(SDF_Alias arg, SDF_Symbol Symbol) */

SDF_Alias SDF_setAliasSymbol(SDF_Alias arg, SDF_Symbol Symbol)
{
  if (SDF_isAliasAlias(arg)) {
    return (SDF_Alias)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbol), 0), 1);
  }

  ATabort("Alias has no Symbol: %t\n", arg);
  return (SDF_Alias)NULL;
}

/*}}}  */
/*{{{  SDF_Alias SDF_setAliasWsAfterSymbol(SDF_Alias arg, SDF_OptLayout wsAfterSymbol) */

SDF_Alias SDF_setAliasWsAfterSymbol(SDF_Alias arg, SDF_OptLayout wsAfterSymbol)
{
  if (SDF_isAliasAlias(arg)) {
    return (SDF_Alias)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSymbol), 1), 1);
  }

  ATabort("Alias has no WsAfterSymbol: %t\n", arg);
  return (SDF_Alias)NULL;
}

/*}}}  */
/*{{{  SDF_Alias SDF_setAliasWsAfterArrow(SDF_Alias arg, SDF_OptLayout wsAfterArrow) */

SDF_Alias SDF_setAliasWsAfterArrow(SDF_Alias arg, SDF_OptLayout wsAfterArrow)
{
  if (SDF_isAliasAlias(arg)) {
    return (SDF_Alias)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterArrow), 3), 1);
  }

  ATabort("Alias has no WsAfterArrow: %t\n", arg);
  return (SDF_Alias)NULL;
}

/*}}}  */
/*{{{  SDF_Alias SDF_setAliasAlias(SDF_Alias arg, SDF_Symbol alias) */

SDF_Alias SDF_setAliasAlias(SDF_Alias arg, SDF_Symbol alias)
{
  if (SDF_isAliasAlias(arg)) {
    return (SDF_Alias)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) alias), 4), 1);
  }

  ATabort("Alias has no Alias: %t\n", arg);
  return (SDF_Alias)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Aliases accessors */

/*{{{  ATbool SDF_isValidAliases(SDF_Aliases arg) */

ATbool SDF_isValidAliases(SDF_Aliases arg)
{
  if (SDF_isAliasesDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isAliasesDefault(SDF_Aliases arg) */

inline ATbool SDF_isAliasesDefault(SDF_Aliases arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternAliasesDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasAliasesList(SDF_Aliases arg) */

ATbool SDF_hasAliasesList(SDF_Aliases arg)
{
  if (SDF_isAliasesDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_AliasList SDF_getAliasesList(SDF_Aliases arg) */

SDF_AliasList SDF_getAliasesList(SDF_Aliases arg)
{
  
    return (SDF_AliasList)ATgetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), 1);
}

/*}}}  */
/*{{{  SDF_Aliases SDF_setAliasesList(SDF_Aliases arg, SDF_AliasList list) */

SDF_Aliases SDF_setAliasesList(SDF_Aliases arg, SDF_AliasList list)
{
  if (SDF_isAliasesDefault(arg)) {
    return (SDF_Aliases)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), (ATerm)((ATerm) list), 1), 0), 1);
  }

  ATabort("Aliases has no List: %t\n", arg);
  return (SDF_Aliases)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_AliasList accessors */

/*{{{  ATbool SDF_isValidAliasList(SDF_AliasList arg) */

ATbool SDF_isValidAliasList(SDF_AliasList arg)
{
  if (SDF_isAliasListEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isAliasListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isAliasListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isAliasListEmpty(SDF_AliasList arg) */

inline ATbool SDF_isAliasListEmpty(SDF_AliasList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternAliasListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isAliasListSingle(SDF_AliasList arg) */

inline ATbool SDF_isAliasListSingle(SDF_AliasList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAliasListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isAliasListMany(SDF_AliasList arg) */

inline ATbool SDF_isAliasListMany(SDF_AliasList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAliasListMany, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasAliasListHead(SDF_AliasList arg) */

ATbool SDF_hasAliasListHead(SDF_AliasList arg)
{
  if (SDF_isAliasListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isAliasListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAliasListWsAfterHead(SDF_AliasList arg) */

ATbool SDF_hasAliasListWsAfterHead(SDF_AliasList arg)
{
  if (SDF_isAliasListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAliasListTail(SDF_AliasList arg) */

ATbool SDF_hasAliasListTail(SDF_AliasList arg)
{
  if (SDF_isAliasListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_AliasList SDF_getAliasListTail(SDF_AliasList arg) */

SDF_AliasList SDF_getAliasListTail(SDF_AliasList arg)
{
  assert(!SDF_isAliasListEmpty(arg) && "getTail on an empty list");
  if (SDF_isAliasListSingle(arg)) {
    return (SDF_AliasList) SDF_makeAliasListEmpty();
  }
  else {
  
    return (SDF_AliasList)ATgetTail((ATermList)arg, 2);
  }
}

/*}}}  */
/*{{{  SDF_Alias SDF_getAliasListHead(SDF_AliasList arg) */

SDF_Alias SDF_getAliasListHead(SDF_AliasList arg)
{
  if (SDF_isAliasListSingle(arg)) {
    return (SDF_Alias)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Alias)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getAliasListWsAfterHead(SDF_AliasList arg) */

SDF_OptLayout SDF_getAliasListWsAfterHead(SDF_AliasList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_AliasList SDF_setAliasListHead(SDF_AliasList arg, SDF_Alias head) */

SDF_AliasList SDF_setAliasListHead(SDF_AliasList arg, SDF_Alias head)
{
  if (SDF_isAliasListSingle(arg)) {
    return (SDF_AliasList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isAliasListMany(arg)) {
    return (SDF_AliasList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("AliasList has no Head: %t\n", arg);
  return (SDF_AliasList)NULL;
}

/*}}}  */
/*{{{  SDF_AliasList SDF_setAliasListWsAfterHead(SDF_AliasList arg, SDF_OptLayout wsAfterHead) */

SDF_AliasList SDF_setAliasListWsAfterHead(SDF_AliasList arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isAliasListMany(arg)) {
    return (SDF_AliasList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("AliasList has no WsAfterHead: %t\n", arg);
  return (SDF_AliasList)NULL;
}

/*}}}  */
/*{{{  SDF_AliasList SDF_setAliasListTail(SDF_AliasList arg, SDF_AliasList tail) */

SDF_AliasList SDF_setAliasListTail(SDF_AliasList arg, SDF_AliasList tail)
{
  if (SDF_isAliasListMany(arg)) {
    return (SDF_AliasList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 2);
  }

  ATabort("AliasList has no Tail: %t\n", arg);
  return (SDF_AliasList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Symbol accessors */

/*{{{  ATbool SDF_isValidSymbol(SDF_Symbol arg) */

ATbool SDF_isValidSymbol(SDF_Symbol arg)
{
  if (SDF_isSymbolCf(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolLex(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolVarsym(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolLayout(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolCharClass(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolLabel(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolLifting(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolLit(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolSeq(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolOpt(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIter(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIterStar(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIterSep(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIterStarSep(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolTuple(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolPair(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolFunc(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolAlt(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolStrategy(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolStart(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolFileStart(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolSort(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolParameterizedSort(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolCf(SDF_Symbol arg) */

inline ATbool SDF_isSymbolCf(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolCf, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolLex(SDF_Symbol arg) */

inline ATbool SDF_isSymbolLex(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolLex, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolVarsym(SDF_Symbol arg) */

inline ATbool SDF_isSymbolVarsym(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolVarsym, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolLayout(SDF_Symbol arg) */

inline ATbool SDF_isSymbolLayout(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolLayout);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolCharClass(SDF_Symbol arg) */

inline ATbool SDF_isSymbolCharClass(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolCharClass, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolLabel(SDF_Symbol arg) */

inline ATbool SDF_isSymbolLabel(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolLabel, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolLifting(SDF_Symbol arg) */

inline ATbool SDF_isSymbolLifting(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolLifting, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolLit(SDF_Symbol arg) */

inline ATbool SDF_isSymbolLit(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolLit, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolEmpty(SDF_Symbol arg) */

inline ATbool SDF_isSymbolEmpty(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolEmpty, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolSeq(SDF_Symbol arg) */

inline ATbool SDF_isSymbolSeq(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolSeq, NULL, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolOpt(SDF_Symbol arg) */

inline ATbool SDF_isSymbolOpt(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolOpt, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolIter(SDF_Symbol arg) */

inline ATbool SDF_isSymbolIter(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolIter, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolIterStar(SDF_Symbol arg) */

inline ATbool SDF_isSymbolIterStar(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolIterStar, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolIterSep(SDF_Symbol arg) */

inline ATbool SDF_isSymbolIterSep(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolIterSep, NULL, NULL, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolIterStarSep(SDF_Symbol arg) */

inline ATbool SDF_isSymbolIterStarSep(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolIterStarSep, NULL, NULL, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolTuple(SDF_Symbol arg) */

inline ATbool SDF_isSymbolTuple(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolTuple, NULL, NULL, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolPair(SDF_Symbol arg) */

inline ATbool SDF_isSymbolPair(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolPair, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolFunc(SDF_Symbol arg) */

inline ATbool SDF_isSymbolFunc(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolFunc, NULL, NULL, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolAlt(SDF_Symbol arg) */

inline ATbool SDF_isSymbolAlt(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolAlt, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolStrategy(SDF_Symbol arg) */

inline ATbool SDF_isSymbolStrategy(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolStrategy, NULL, NULL, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolStart(SDF_Symbol arg) */

inline ATbool SDF_isSymbolStart(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolStart);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolFileStart(SDF_Symbol arg) */

inline ATbool SDF_isSymbolFileStart(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolFileStart);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolSort(SDF_Symbol arg) */

inline ATbool SDF_isSymbolSort(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolSort, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolParameterizedSort(SDF_Symbol arg) */

inline ATbool SDF_isSymbolParameterizedSort(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolParameterizedSort, NULL, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolBracket(SDF_Symbol arg) */

inline ATbool SDF_isSymbolBracket(SDF_Symbol arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolBracket, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterLessThan(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterLessThan(SDF_Symbol arg)
{
  if (SDF_isSymbolCf(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolLex(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolVarsym(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolTuple(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolSymbol(SDF_Symbol arg) */

ATbool SDF_hasSymbolSymbol(SDF_Symbol arg)
{
  if (SDF_isSymbolCf(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolLex(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolVarsym(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolLabel(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolLifting(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolOpt(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIter(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIterStar(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIterSep(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIterStarSep(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterSymbol(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterSymbol(SDF_Symbol arg)
{
  if (SDF_isSymbolCf(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolLex(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolVarsym(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolLifting(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolOpt(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIter(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIterStar(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIterSep(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIterStarSep(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterCF(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterCF(SDF_Symbol arg)
{
  if (SDF_isSymbolCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterLEX(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterLEX(SDF_Symbol arg)
{
  if (SDF_isSymbolLex(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterVAR(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterVAR(SDF_Symbol arg)
{
  if (SDF_isSymbolVarsym(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolCharClass(SDF_Symbol arg) */

ATbool SDF_hasSymbolCharClass(SDF_Symbol arg)
{
  if (SDF_isSymbolCharClass(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolLabel(SDF_Symbol arg) */

ATbool SDF_hasSymbolLabel(SDF_Symbol arg)
{
  if (SDF_isSymbolLabel(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterLabel(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterLabel(SDF_Symbol arg)
{
  if (SDF_isSymbolLabel(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterColon(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterColon(SDF_Symbol arg)
{
  if (SDF_isSymbolLabel(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterLeftQuote(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterLeftQuote(SDF_Symbol arg)
{
  if (SDF_isSymbolLifting(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolString(SDF_Symbol arg) */

ATbool SDF_hasSymbolString(SDF_Symbol arg)
{
  if (SDF_isSymbolLit(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterParenOpen(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterParenOpen(SDF_Symbol arg)
{
  if (SDF_isSymbolEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolSeq(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolFunc(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolStrategy(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolHead(SDF_Symbol arg) */

ATbool SDF_hasSymbolHead(SDF_Symbol arg)
{
  if (SDF_isSymbolSeq(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolTuple(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterHead(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterHead(SDF_Symbol arg)
{
  if (SDF_isSymbolSeq(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolTuple(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolTail(SDF_Symbol arg) */

ATbool SDF_hasSymbolTail(SDF_Symbol arg)
{
  if (SDF_isSymbolSeq(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterTail(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterTail(SDF_Symbol arg)
{
  if (SDF_isSymbolSeq(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterBraceOpen(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterBraceOpen(SDF_Symbol arg)
{
  if (SDF_isSymbolIterSep(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIterStarSep(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolSep(SDF_Symbol arg) */

ATbool SDF_hasSymbolSep(SDF_Symbol arg)
{
  if (SDF_isSymbolIterSep(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIterStarSep(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterSep(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterSep(SDF_Symbol arg)
{
  if (SDF_isSymbolIterSep(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIterStarSep(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterBraceClose(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterBraceClose(SDF_Symbol arg)
{
  if (SDF_isSymbolIterSep(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolIterStarSep(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterComma(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterComma(SDF_Symbol arg)
{
  if (SDF_isSymbolTuple(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolRest(SDF_Symbol arg) */

ATbool SDF_hasSymbolRest(SDF_Symbol arg)
{
  if (SDF_isSymbolTuple(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterRest(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterRest(SDF_Symbol arg)
{
  if (SDF_isSymbolTuple(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolLeft(SDF_Symbol arg) */

ATbool SDF_hasSymbolLeft(SDF_Symbol arg)
{
  if (SDF_isSymbolPair(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolAlt(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolStrategy(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterLeft(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterLeft(SDF_Symbol arg)
{
  if (SDF_isSymbolPair(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolAlt(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolStrategy(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterHash(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterHash(SDF_Symbol arg)
{
  if (SDF_isSymbolPair(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolRight(SDF_Symbol arg) */

ATbool SDF_hasSymbolRight(SDF_Symbol arg)
{
  if (SDF_isSymbolPair(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolAlt(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolStrategy(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolArguments(SDF_Symbol arg) */

ATbool SDF_hasSymbolArguments(SDF_Symbol arg)
{
  if (SDF_isSymbolFunc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterArguments(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterArguments(SDF_Symbol arg)
{
  if (SDF_isSymbolFunc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterEqualsGreaterThan(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterEqualsGreaterThan(SDF_Symbol arg)
{
  if (SDF_isSymbolFunc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolResults(SDF_Symbol arg) */

ATbool SDF_hasSymbolResults(SDF_Symbol arg)
{
  if (SDF_isSymbolFunc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterResults(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterResults(SDF_Symbol arg)
{
  if (SDF_isSymbolFunc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterBar(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterBar(SDF_Symbol arg)
{
  if (SDF_isSymbolAlt(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterGreaterThan(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterGreaterThan(SDF_Symbol arg)
{
  if (SDF_isSymbolStrategy(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterRight(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterRight(SDF_Symbol arg)
{
  if (SDF_isSymbolStrategy(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolSort(SDF_Symbol arg) */

ATbool SDF_hasSymbolSort(SDF_Symbol arg)
{
  if (SDF_isSymbolSort(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolParameterizedSort(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterSort(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterSort(SDF_Symbol arg)
{
  if (SDF_isSymbolParameterizedSort(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterBracketOpenBracketOpen(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterBracketOpenBracketOpen(SDF_Symbol arg)
{
  if (SDF_isSymbolParameterizedSort(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolParameters(SDF_Symbol arg) */

ATbool SDF_hasSymbolParameters(SDF_Symbol arg)
{
  if (SDF_isSymbolParameterizedSort(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolWsAfterParameters(SDF_Symbol arg) */

ATbool SDF_hasSymbolWsAfterParameters(SDF_Symbol arg)
{
  if (SDF_isSymbolParameterizedSort(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterLessThan(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterLessThan(SDF_Symbol arg)
{
  if (SDF_isSymbolCf(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isSymbolLex(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isSymbolVarsym(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getSymbolSymbol(SDF_Symbol arg) */

SDF_Symbol SDF_getSymbolSymbol(SDF_Symbol arg)
{
  if (SDF_isSymbolCf(arg)) {
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else if (SDF_isSymbolLex(arg)) {
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else if (SDF_isSymbolVarsym(arg)) {
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else if (SDF_isSymbolLabel(arg)) {
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
  }
  else if (SDF_isSymbolLifting(arg)) {
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else if (SDF_isSymbolOpt(arg)) {
    return (SDF_Symbol)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
  }
  else if (SDF_isSymbolIter(arg)) {
    return (SDF_Symbol)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
  }
  else if (SDF_isSymbolIterStar(arg)) {
    return (SDF_Symbol)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
  }
  else if (SDF_isSymbolIterSep(arg)) {
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else if (SDF_isSymbolIterStarSep(arg)) {
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else 
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterSymbol(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterSymbol(SDF_Symbol arg)
{
  if (SDF_isSymbolCf(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else if (SDF_isSymbolLex(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else if (SDF_isSymbolVarsym(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else if (SDF_isSymbolLifting(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else if (SDF_isSymbolOpt(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isSymbolIter(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isSymbolIterStar(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isSymbolIterSep(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else if (SDF_isSymbolIterStarSep(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterCF(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterCF(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterLEX(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterLEX(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterVAR(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterVAR(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_CharClass SDF_getSymbolCharClass(SDF_Symbol arg) */

SDF_CharClass SDF_getSymbolCharClass(SDF_Symbol arg)
{
  
    return (SDF_CharClass)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_Label SDF_getSymbolLabel(SDF_Symbol arg) */

SDF_Label SDF_getSymbolLabel(SDF_Symbol arg)
{
  
    return (SDF_Label)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterLabel(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterLabel(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterColon(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterColon(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterLeftQuote(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterLeftQuote(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_StrCon SDF_getSymbolString(SDF_Symbol arg) */

SDF_StrCon SDF_getSymbolString(SDF_Symbol arg)
{
  
    return (SDF_StrCon)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterParenOpen(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterParenOpen(SDF_Symbol arg)
{
  if (SDF_isSymbolEmpty(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isSymbolSeq(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isSymbolFunc(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isSymbolStrategy(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getSymbolHead(SDF_Symbol arg) */

SDF_Symbol SDF_getSymbolHead(SDF_Symbol arg)
{
  if (SDF_isSymbolSeq(arg)) {
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else 
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterHead(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterHead(SDF_Symbol arg)
{
  if (SDF_isSymbolSeq(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_SymbolTail SDF_getSymbolTail(SDF_Symbol arg) */

SDF_SymbolTail SDF_getSymbolTail(SDF_Symbol arg)
{
  
    return (SDF_SymbolTail)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterTail(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterTail(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterBraceOpen(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterBraceOpen(SDF_Symbol arg)
{
  if (SDF_isSymbolIterSep(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getSymbolSep(SDF_Symbol arg) */

SDF_Symbol SDF_getSymbolSep(SDF_Symbol arg)
{
  if (SDF_isSymbolIterSep(arg)) {
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
  }
  else 
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterSep(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterSep(SDF_Symbol arg)
{
  if (SDF_isSymbolIterSep(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterBraceClose(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterBraceClose(SDF_Symbol arg)
{
  if (SDF_isSymbolIterSep(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 7);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 7);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterComma(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterComma(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_SymbolRest SDF_getSymbolRest(SDF_Symbol arg) */

SDF_SymbolRest SDF_getSymbolRest(SDF_Symbol arg)
{
  
    return (SDF_SymbolRest)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 6), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterRest(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterRest(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 7);
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getSymbolLeft(SDF_Symbol arg) */

SDF_Symbol SDF_getSymbolLeft(SDF_Symbol arg)
{
  if (SDF_isSymbolPair(arg)) {
    return (SDF_Symbol)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
  }
  else if (SDF_isSymbolAlt(arg)) {
    return (SDF_Symbol)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
  }
  else 
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterLeft(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterLeft(SDF_Symbol arg)
{
  if (SDF_isSymbolPair(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isSymbolAlt(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterHash(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterHash(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getSymbolRight(SDF_Symbol arg) */

SDF_Symbol SDF_getSymbolRight(SDF_Symbol arg)
{
  if (SDF_isSymbolPair(arg)) {
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
  }
  else if (SDF_isSymbolAlt(arg)) {
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
  }
  else 
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 6);
}

/*}}}  */
/*{{{  SDF_Symbols SDF_getSymbolArguments(SDF_Symbol arg) */

SDF_Symbols SDF_getSymbolArguments(SDF_Symbol arg)
{
  
    return (SDF_Symbols)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterArguments(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterArguments(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterEqualsGreaterThan(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterEqualsGreaterThan(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getSymbolResults(SDF_Symbol arg) */

SDF_Symbol SDF_getSymbolResults(SDF_Symbol arg)
{
  
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 6);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterResults(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterResults(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 7);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterBar(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterBar(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterGreaterThan(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterGreaterThan(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterRight(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterRight(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 7);
}

/*}}}  */
/*{{{  SDF_Sort SDF_getSymbolSort(SDF_Symbol arg) */

SDF_Sort SDF_getSymbolSort(SDF_Symbol arg)
{
  if (SDF_isSymbolSort(arg)) {
    return (SDF_Sort)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
  }
  else 
    return (SDF_Sort)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterSort(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterSort(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterBracketOpenBracketOpen(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterBracketOpenBracketOpen(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_SymbolParameters SDF_getSymbolParameters(SDF_Symbol arg) */

SDF_SymbolParameters SDF_getSymbolParameters(SDF_Symbol arg)
{
  
    return (SDF_SymbolParameters)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolWsAfterParameters(SDF_Symbol arg) */

SDF_OptLayout SDF_getSymbolWsAfterParameters(SDF_Symbol arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterLessThan(SDF_Symbol arg, SDF_OptLayout wsAfterLessThan) */

SDF_Symbol SDF_setSymbolWsAfterLessThan(SDF_Symbol arg, SDF_OptLayout wsAfterLessThan)
{
  if (SDF_isSymbolCf(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLessThan), 1), 1);
  }
  else if (SDF_isSymbolLex(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLessThan), 1), 1);
  }
  else if (SDF_isSymbolVarsym(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLessThan), 1), 1);
  }
  else if (SDF_isSymbolTuple(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLessThan), 1), 1);
  }

  ATabort("Symbol has no WsAfterLessThan: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolSymbol(SDF_Symbol arg, SDF_Symbol Symbol) */

SDF_Symbol SDF_setSymbolSymbol(SDF_Symbol arg, SDF_Symbol Symbol)
{
  if (SDF_isSymbolCf(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbol), 2), 1);
  }
  else if (SDF_isSymbolLex(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbol), 2), 1);
  }
  else if (SDF_isSymbolVarsym(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbol), 2), 1);
  }
  else if (SDF_isSymbolLabel(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbol), 4), 1);
  }
  else if (SDF_isSymbolLifting(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbol), 2), 1);
  }
  else if (SDF_isSymbolOpt(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbol), 0), 1);
  }
  else if (SDF_isSymbolIter(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbol), 0), 1);
  }
  else if (SDF_isSymbolIterStar(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbol), 0), 1);
  }
  else if (SDF_isSymbolIterSep(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbol), 2), 1);
  }
  else if (SDF_isSymbolIterStarSep(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbol), 2), 1);
  }
  else if (SDF_isSymbolBracket(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbol), 2), 1);
  }

  ATabort("Symbol has no Symbol: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterSymbol(SDF_Symbol arg, SDF_OptLayout wsAfterSymbol) */

SDF_Symbol SDF_setSymbolWsAfterSymbol(SDF_Symbol arg, SDF_OptLayout wsAfterSymbol)
{
  if (SDF_isSymbolCf(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSymbol), 3), 1);
  }
  else if (SDF_isSymbolLex(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSymbol), 3), 1);
  }
  else if (SDF_isSymbolVarsym(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSymbol), 3), 1);
  }
  else if (SDF_isSymbolLifting(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSymbol), 3), 1);
  }
  else if (SDF_isSymbolOpt(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSymbol), 1), 1);
  }
  else if (SDF_isSymbolIter(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSymbol), 1), 1);
  }
  else if (SDF_isSymbolIterStar(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSymbol), 1), 1);
  }
  else if (SDF_isSymbolIterSep(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSymbol), 3), 1);
  }
  else if (SDF_isSymbolIterStarSep(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSymbol), 3), 1);
  }
  else if (SDF_isSymbolBracket(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSymbol), 3), 1);
  }

  ATabort("Symbol has no WsAfterSymbol: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterCF(SDF_Symbol arg, SDF_OptLayout wsAfterCF) */

SDF_Symbol SDF_setSymbolWsAfterCF(SDF_Symbol arg, SDF_OptLayout wsAfterCF)
{
  if (SDF_isSymbolCf(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterCF), 5), 1);
  }

  ATabort("Symbol has no WsAfterCF: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterLEX(SDF_Symbol arg, SDF_OptLayout wsAfterLEX) */

SDF_Symbol SDF_setSymbolWsAfterLEX(SDF_Symbol arg, SDF_OptLayout wsAfterLEX)
{
  if (SDF_isSymbolLex(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLEX), 5), 1);
  }

  ATabort("Symbol has no WsAfterLEX: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterVAR(SDF_Symbol arg, SDF_OptLayout wsAfterVAR) */

SDF_Symbol SDF_setSymbolWsAfterVAR(SDF_Symbol arg, SDF_OptLayout wsAfterVAR)
{
  if (SDF_isSymbolVarsym(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterVAR), 5), 1);
  }

  ATabort("Symbol has no WsAfterVAR: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolCharClass(SDF_Symbol arg, SDF_CharClass CharClass) */

SDF_Symbol SDF_setSymbolCharClass(SDF_Symbol arg, SDF_CharClass CharClass)
{
  if (SDF_isSymbolCharClass(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) CharClass), 0), 1);
  }

  ATabort("Symbol has no CharClass: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolLabel(SDF_Symbol arg, SDF_Label Label) */

SDF_Symbol SDF_setSymbolLabel(SDF_Symbol arg, SDF_Label Label)
{
  if (SDF_isSymbolLabel(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Label), 0), 1);
  }

  ATabort("Symbol has no Label: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterLabel(SDF_Symbol arg, SDF_OptLayout wsAfterLabel) */

SDF_Symbol SDF_setSymbolWsAfterLabel(SDF_Symbol arg, SDF_OptLayout wsAfterLabel)
{
  if (SDF_isSymbolLabel(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLabel), 1), 1);
  }

  ATabort("Symbol has no WsAfterLabel: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterColon(SDF_Symbol arg, SDF_OptLayout wsAfterColon) */

SDF_Symbol SDF_setSymbolWsAfterColon(SDF_Symbol arg, SDF_OptLayout wsAfterColon)
{
  if (SDF_isSymbolLabel(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterColon), 3), 1);
  }

  ATabort("Symbol has no WsAfterColon: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterLeftQuote(SDF_Symbol arg, SDF_OptLayout wsAfterLeftQuote) */

SDF_Symbol SDF_setSymbolWsAfterLeftQuote(SDF_Symbol arg, SDF_OptLayout wsAfterLeftQuote)
{
  if (SDF_isSymbolLifting(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLeftQuote), 1), 1);
  }

  ATabort("Symbol has no WsAfterLeftQuote: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolString(SDF_Symbol arg, SDF_StrCon string) */

SDF_Symbol SDF_setSymbolString(SDF_Symbol arg, SDF_StrCon string)
{
  if (SDF_isSymbolLit(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) string), 0), 1);
  }

  ATabort("Symbol has no String: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterParenOpen(SDF_Symbol arg, SDF_OptLayout wsAfterParenOpen) */

SDF_Symbol SDF_setSymbolWsAfterParenOpen(SDF_Symbol arg, SDF_OptLayout wsAfterParenOpen)
{
  if (SDF_isSymbolEmpty(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenOpen), 1), 1);
  }
  else if (SDF_isSymbolSeq(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenOpen), 1), 1);
  }
  else if (SDF_isSymbolFunc(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenOpen), 1), 1);
  }
  else if (SDF_isSymbolStrategy(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenOpen), 1), 1);
  }
  else if (SDF_isSymbolBracket(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenOpen), 1), 1);
  }

  ATabort("Symbol has no WsAfterParenOpen: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolHead(SDF_Symbol arg, SDF_Symbol head) */

SDF_Symbol SDF_setSymbolHead(SDF_Symbol arg, SDF_Symbol head)
{
  if (SDF_isSymbolSeq(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) head), 2), 1);
  }
  else if (SDF_isSymbolTuple(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) head), 2), 1);
  }

  ATabort("Symbol has no Head: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterHead(SDF_Symbol arg, SDF_OptLayout wsAfterHead) */

SDF_Symbol SDF_setSymbolWsAfterHead(SDF_Symbol arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isSymbolSeq(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterHead), 3), 1);
  }
  else if (SDF_isSymbolTuple(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterHead), 3), 1);
  }

  ATabort("Symbol has no WsAfterHead: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolTail(SDF_Symbol arg, SDF_SymbolTail tail) */

SDF_Symbol SDF_setSymbolTail(SDF_Symbol arg, SDF_SymbolTail tail)
{
  if (SDF_isSymbolSeq(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4), (ATerm)((ATerm) tail), 1), 4), 1);
  }

  ATabort("Symbol has no Tail: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterTail(SDF_Symbol arg, SDF_OptLayout wsAfterTail) */

SDF_Symbol SDF_setSymbolWsAfterTail(SDF_Symbol arg, SDF_OptLayout wsAfterTail)
{
  if (SDF_isSymbolSeq(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterTail), 5), 1);
  }

  ATabort("Symbol has no WsAfterTail: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterBraceOpen(SDF_Symbol arg, SDF_OptLayout wsAfterBraceOpen) */

SDF_Symbol SDF_setSymbolWsAfterBraceOpen(SDF_Symbol arg, SDF_OptLayout wsAfterBraceOpen)
{
  if (SDF_isSymbolIterSep(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBraceOpen), 1), 1);
  }
  else if (SDF_isSymbolIterStarSep(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBraceOpen), 1), 1);
  }

  ATabort("Symbol has no WsAfterBraceOpen: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolSep(SDF_Symbol arg, SDF_Symbol sep) */

SDF_Symbol SDF_setSymbolSep(SDF_Symbol arg, SDF_Symbol sep)
{
  if (SDF_isSymbolIterSep(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) sep), 4), 1);
  }
  else if (SDF_isSymbolIterStarSep(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) sep), 4), 1);
  }

  ATabort("Symbol has no Sep: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterSep(SDF_Symbol arg, SDF_OptLayout wsAfterSep) */

SDF_Symbol SDF_setSymbolWsAfterSep(SDF_Symbol arg, SDF_OptLayout wsAfterSep)
{
  if (SDF_isSymbolIterSep(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSep), 5), 1);
  }
  else if (SDF_isSymbolIterStarSep(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSep), 5), 1);
  }

  ATabort("Symbol has no WsAfterSep: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterBraceClose(SDF_Symbol arg, SDF_OptLayout wsAfterBraceClose) */

SDF_Symbol SDF_setSymbolWsAfterBraceClose(SDF_Symbol arg, SDF_OptLayout wsAfterBraceClose)
{
  if (SDF_isSymbolIterSep(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBraceClose), 7), 1);
  }
  else if (SDF_isSymbolIterStarSep(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBraceClose), 7), 1);
  }

  ATabort("Symbol has no WsAfterBraceClose: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterComma(SDF_Symbol arg, SDF_OptLayout wsAfterComma) */

SDF_Symbol SDF_setSymbolWsAfterComma(SDF_Symbol arg, SDF_OptLayout wsAfterComma)
{
  if (SDF_isSymbolTuple(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterComma), 5), 1);
  }

  ATabort("Symbol has no WsAfterComma: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolRest(SDF_Symbol arg, SDF_SymbolRest rest) */

SDF_Symbol SDF_setSymbolRest(SDF_Symbol arg, SDF_SymbolRest rest)
{
  if (SDF_isSymbolTuple(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 6), (ATerm)((ATerm) rest), 1), 6), 1);
  }

  ATabort("Symbol has no Rest: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterRest(SDF_Symbol arg, SDF_OptLayout wsAfterRest) */

SDF_Symbol SDF_setSymbolWsAfterRest(SDF_Symbol arg, SDF_OptLayout wsAfterRest)
{
  if (SDF_isSymbolTuple(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterRest), 7), 1);
  }

  ATabort("Symbol has no WsAfterRest: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolLeft(SDF_Symbol arg, SDF_Symbol left) */

SDF_Symbol SDF_setSymbolLeft(SDF_Symbol arg, SDF_Symbol left)
{
  if (SDF_isSymbolPair(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) left), 0), 1);
  }
  else if (SDF_isSymbolAlt(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) left), 0), 1);
  }
  else if (SDF_isSymbolStrategy(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) left), 2), 1);
  }

  ATabort("Symbol has no Left: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterLeft(SDF_Symbol arg, SDF_OptLayout wsAfterLeft) */

SDF_Symbol SDF_setSymbolWsAfterLeft(SDF_Symbol arg, SDF_OptLayout wsAfterLeft)
{
  if (SDF_isSymbolPair(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLeft), 1), 1);
  }
  else if (SDF_isSymbolAlt(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLeft), 1), 1);
  }
  else if (SDF_isSymbolStrategy(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLeft), 3), 1);
  }

  ATabort("Symbol has no WsAfterLeft: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterHash(SDF_Symbol arg, SDF_OptLayout wsAfterHash) */

SDF_Symbol SDF_setSymbolWsAfterHash(SDF_Symbol arg, SDF_OptLayout wsAfterHash)
{
  if (SDF_isSymbolPair(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterHash), 3), 1);
  }

  ATabort("Symbol has no WsAfterHash: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolRight(SDF_Symbol arg, SDF_Symbol right) */

SDF_Symbol SDF_setSymbolRight(SDF_Symbol arg, SDF_Symbol right)
{
  if (SDF_isSymbolPair(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) right), 4), 1);
  }
  else if (SDF_isSymbolAlt(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) right), 4), 1);
  }
  else if (SDF_isSymbolStrategy(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) right), 6), 1);
  }

  ATabort("Symbol has no Right: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolArguments(SDF_Symbol arg, SDF_Symbols arguments) */

SDF_Symbol SDF_setSymbolArguments(SDF_Symbol arg, SDF_Symbols arguments)
{
  if (SDF_isSymbolFunc(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) arguments), 2), 1);
  }

  ATabort("Symbol has no Arguments: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterArguments(SDF_Symbol arg, SDF_OptLayout wsAfterArguments) */

SDF_Symbol SDF_setSymbolWsAfterArguments(SDF_Symbol arg, SDF_OptLayout wsAfterArguments)
{
  if (SDF_isSymbolFunc(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterArguments), 3), 1);
  }

  ATabort("Symbol has no WsAfterArguments: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterEqualsGreaterThan(SDF_Symbol arg, SDF_OptLayout wsAfterEqualsGreaterThan) */

SDF_Symbol SDF_setSymbolWsAfterEqualsGreaterThan(SDF_Symbol arg, SDF_OptLayout wsAfterEqualsGreaterThan)
{
  if (SDF_isSymbolFunc(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterEqualsGreaterThan), 5), 1);
  }

  ATabort("Symbol has no WsAfterEqualsGreaterThan: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolResults(SDF_Symbol arg, SDF_Symbol results) */

SDF_Symbol SDF_setSymbolResults(SDF_Symbol arg, SDF_Symbol results)
{
  if (SDF_isSymbolFunc(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) results), 6), 1);
  }

  ATabort("Symbol has no Results: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterResults(SDF_Symbol arg, SDF_OptLayout wsAfterResults) */

SDF_Symbol SDF_setSymbolWsAfterResults(SDF_Symbol arg, SDF_OptLayout wsAfterResults)
{
  if (SDF_isSymbolFunc(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterResults), 7), 1);
  }

  ATabort("Symbol has no WsAfterResults: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterBar(SDF_Symbol arg, SDF_OptLayout wsAfterBar) */

SDF_Symbol SDF_setSymbolWsAfterBar(SDF_Symbol arg, SDF_OptLayout wsAfterBar)
{
  if (SDF_isSymbolAlt(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBar), 3), 1);
  }

  ATabort("Symbol has no WsAfterBar: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterGreaterThan(SDF_Symbol arg, SDF_OptLayout wsAfterGreaterThan) */

SDF_Symbol SDF_setSymbolWsAfterGreaterThan(SDF_Symbol arg, SDF_OptLayout wsAfterGreaterThan)
{
  if (SDF_isSymbolStrategy(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterGreaterThan), 5), 1);
  }

  ATabort("Symbol has no WsAfterGreaterThan: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterRight(SDF_Symbol arg, SDF_OptLayout wsAfterRight) */

SDF_Symbol SDF_setSymbolWsAfterRight(SDF_Symbol arg, SDF_OptLayout wsAfterRight)
{
  if (SDF_isSymbolStrategy(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterRight), 7), 1);
  }

  ATabort("Symbol has no WsAfterRight: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolSort(SDF_Symbol arg, SDF_Sort Sort) */

SDF_Symbol SDF_setSymbolSort(SDF_Symbol arg, SDF_Sort Sort)
{
  if (SDF_isSymbolSort(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Sort), 0), 1);
  }
  else if (SDF_isSymbolParameterizedSort(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Sort), 0), 1);
  }

  ATabort("Symbol has no Sort: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterSort(SDF_Symbol arg, SDF_OptLayout wsAfterSort) */

SDF_Symbol SDF_setSymbolWsAfterSort(SDF_Symbol arg, SDF_OptLayout wsAfterSort)
{
  if (SDF_isSymbolParameterizedSort(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSort), 1), 1);
  }

  ATabort("Symbol has no WsAfterSort: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterBracketOpenBracketOpen(SDF_Symbol arg, SDF_OptLayout wsAfterBracketOpenBracketOpen) */

SDF_Symbol SDF_setSymbolWsAfterBracketOpenBracketOpen(SDF_Symbol arg, SDF_OptLayout wsAfterBracketOpenBracketOpen)
{
  if (SDF_isSymbolParameterizedSort(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBracketOpenBracketOpen), 3), 1);
  }

  ATabort("Symbol has no WsAfterBracketOpenBracketOpen: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolParameters(SDF_Symbol arg, SDF_SymbolParameters parameters) */

SDF_Symbol SDF_setSymbolParameters(SDF_Symbol arg, SDF_SymbolParameters parameters)
{
  if (SDF_isSymbolParameterizedSort(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4), (ATerm)((ATerm) parameters), 1), 4), 1);
  }

  ATabort("Symbol has no Parameters: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_setSymbolWsAfterParameters(SDF_Symbol arg, SDF_OptLayout wsAfterParameters) */

SDF_Symbol SDF_setSymbolWsAfterParameters(SDF_Symbol arg, SDF_OptLayout wsAfterParameters)
{
  if (SDF_isSymbolParameterizedSort(arg)) {
    return (SDF_Symbol)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParameters), 5), 1);
  }

  ATabort("Symbol has no WsAfterParameters: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_CharRange accessors */

/*{{{  ATbool SDF_isValidCharRange(SDF_CharRange arg) */

ATbool SDF_isValidCharRange(SDF_CharRange arg)
{
  if (SDF_isCharRangeDefault(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharRangeRange(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharRangeDefault(SDF_CharRange arg) */

inline ATbool SDF_isCharRangeDefault(SDF_CharRange arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharRangeDefault, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharRangeRange(SDF_CharRange arg) */

inline ATbool SDF_isCharRangeRange(SDF_CharRange arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharRangeRange, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasCharRangeCharacter(SDF_CharRange arg) */

ATbool SDF_hasCharRangeCharacter(SDF_CharRange arg)
{
  if (SDF_isCharRangeDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharRangeStart(SDF_CharRange arg) */

ATbool SDF_hasCharRangeStart(SDF_CharRange arg)
{
  if (SDF_isCharRangeRange(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharRangeWsAfterStart(SDF_CharRange arg) */

ATbool SDF_hasCharRangeWsAfterStart(SDF_CharRange arg)
{
  if (SDF_isCharRangeRange(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharRangeWsAfter(SDF_CharRange arg) */

ATbool SDF_hasCharRangeWsAfter(SDF_CharRange arg)
{
  if (SDF_isCharRangeRange(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharRangeEnd(SDF_CharRange arg) */

ATbool SDF_hasCharRangeEnd(SDF_CharRange arg)
{
  if (SDF_isCharRangeRange(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_Character SDF_getCharRangeCharacter(SDF_CharRange arg) */

SDF_Character SDF_getCharRangeCharacter(SDF_CharRange arg)
{
  
    return (SDF_Character)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_Character SDF_getCharRangeStart(SDF_CharRange arg) */

SDF_Character SDF_getCharRangeStart(SDF_CharRange arg)
{
  
    return (SDF_Character)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getCharRangeWsAfterStart(SDF_CharRange arg) */

SDF_OptLayout SDF_getCharRangeWsAfterStart(SDF_CharRange arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getCharRangeWsAfter(SDF_CharRange arg) */

SDF_OptLayout SDF_getCharRangeWsAfter(SDF_CharRange arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Character SDF_getCharRangeEnd(SDF_CharRange arg) */

SDF_Character SDF_getCharRangeEnd(SDF_CharRange arg)
{
  
    return (SDF_Character)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_CharRange SDF_setCharRangeCharacter(SDF_CharRange arg, SDF_Character Character) */

SDF_CharRange SDF_setCharRangeCharacter(SDF_CharRange arg, SDF_Character Character)
{
  if (SDF_isCharRangeDefault(arg)) {
    return (SDF_CharRange)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Character), 0), 1);
  }

  ATabort("CharRange has no Character: %t\n", arg);
  return (SDF_CharRange)NULL;
}

/*}}}  */
/*{{{  SDF_CharRange SDF_setCharRangeStart(SDF_CharRange arg, SDF_Character start) */

SDF_CharRange SDF_setCharRangeStart(SDF_CharRange arg, SDF_Character start)
{
  if (SDF_isCharRangeRange(arg)) {
    return (SDF_CharRange)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) start), 0), 1);
  }

  ATabort("CharRange has no Start: %t\n", arg);
  return (SDF_CharRange)NULL;
}

/*}}}  */
/*{{{  SDF_CharRange SDF_setCharRangeWsAfterStart(SDF_CharRange arg, SDF_OptLayout wsAfterStart) */

SDF_CharRange SDF_setCharRangeWsAfterStart(SDF_CharRange arg, SDF_OptLayout wsAfterStart)
{
  if (SDF_isCharRangeRange(arg)) {
    return (SDF_CharRange)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterStart), 1), 1);
  }

  ATabort("CharRange has no WsAfterStart: %t\n", arg);
  return (SDF_CharRange)NULL;
}

/*}}}  */
/*{{{  SDF_CharRange SDF_setCharRangeWsAfter(SDF_CharRange arg, SDF_OptLayout wsAfter) */

SDF_CharRange SDF_setCharRangeWsAfter(SDF_CharRange arg, SDF_OptLayout wsAfter)
{
  if (SDF_isCharRangeRange(arg)) {
    return (SDF_CharRange)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfter), 3), 1);
  }

  ATabort("CharRange has no WsAfter: %t\n", arg);
  return (SDF_CharRange)NULL;
}

/*}}}  */
/*{{{  SDF_CharRange SDF_setCharRangeEnd(SDF_CharRange arg, SDF_Character end) */

SDF_CharRange SDF_setCharRangeEnd(SDF_CharRange arg, SDF_Character end)
{
  if (SDF_isCharRangeRange(arg)) {
    return (SDF_CharRange)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) end), 4), 1);
  }

  ATabort("CharRange has no End: %t\n", arg);
  return (SDF_CharRange)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_CharRanges accessors */

/*{{{  ATbool SDF_isValidCharRanges(SDF_CharRanges arg) */

ATbool SDF_isValidCharRanges(SDF_CharRanges arg)
{
  if (SDF_isCharRangesDefault(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharRangesConc(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharRangesBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharRangesDefault(SDF_CharRanges arg) */

inline ATbool SDF_isCharRangesDefault(SDF_CharRanges arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharRangesDefault, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharRangesConc(SDF_CharRanges arg) */

inline ATbool SDF_isCharRangesConc(SDF_CharRanges arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharRangesConc, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharRangesBracket(SDF_CharRanges arg) */

inline ATbool SDF_isCharRangesBracket(SDF_CharRanges arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharRangesBracket, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasCharRangesCharRange(SDF_CharRanges arg) */

ATbool SDF_hasCharRangesCharRange(SDF_CharRanges arg)
{
  if (SDF_isCharRangesDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharRangesLeft(SDF_CharRanges arg) */

ATbool SDF_hasCharRangesLeft(SDF_CharRanges arg)
{
  if (SDF_isCharRangesConc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharRangesWsAfterLeft(SDF_CharRanges arg) */

ATbool SDF_hasCharRangesWsAfterLeft(SDF_CharRanges arg)
{
  if (SDF_isCharRangesConc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharRangesRight(SDF_CharRanges arg) */

ATbool SDF_hasCharRangesRight(SDF_CharRanges arg)
{
  if (SDF_isCharRangesConc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharRangesWsAfterParenOpen(SDF_CharRanges arg) */

ATbool SDF_hasCharRangesWsAfterParenOpen(SDF_CharRanges arg)
{
  if (SDF_isCharRangesBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharRangesCharRanges(SDF_CharRanges arg) */

ATbool SDF_hasCharRangesCharRanges(SDF_CharRanges arg)
{
  if (SDF_isCharRangesBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharRangesWsAfterCharRanges(SDF_CharRanges arg) */

ATbool SDF_hasCharRangesWsAfterCharRanges(SDF_CharRanges arg)
{
  if (SDF_isCharRangesBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_CharRange SDF_getCharRangesCharRange(SDF_CharRanges arg) */

SDF_CharRange SDF_getCharRangesCharRange(SDF_CharRanges arg)
{
  
    return (SDF_CharRange)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_getCharRangesLeft(SDF_CharRanges arg) */

SDF_CharRanges SDF_getCharRangesLeft(SDF_CharRanges arg)
{
  
    return (SDF_CharRanges)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getCharRangesWsAfterLeft(SDF_CharRanges arg) */

SDF_OptLayout SDF_getCharRangesWsAfterLeft(SDF_CharRanges arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_getCharRangesRight(SDF_CharRanges arg) */

SDF_CharRanges SDF_getCharRangesRight(SDF_CharRanges arg)
{
  
    return (SDF_CharRanges)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getCharRangesWsAfterParenOpen(SDF_CharRanges arg) */

SDF_OptLayout SDF_getCharRangesWsAfterParenOpen(SDF_CharRanges arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_getCharRangesCharRanges(SDF_CharRanges arg) */

SDF_CharRanges SDF_getCharRangesCharRanges(SDF_CharRanges arg)
{
  
    return (SDF_CharRanges)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getCharRangesWsAfterCharRanges(SDF_CharRanges arg) */

SDF_OptLayout SDF_getCharRangesWsAfterCharRanges(SDF_CharRanges arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_setCharRangesCharRange(SDF_CharRanges arg, SDF_CharRange CharRange) */

SDF_CharRanges SDF_setCharRangesCharRange(SDF_CharRanges arg, SDF_CharRange CharRange)
{
  if (SDF_isCharRangesDefault(arg)) {
    return (SDF_CharRanges)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) CharRange), 0), 1);
  }

  ATabort("CharRanges has no CharRange: %t\n", arg);
  return (SDF_CharRanges)NULL;
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_setCharRangesLeft(SDF_CharRanges arg, SDF_CharRanges left) */

SDF_CharRanges SDF_setCharRangesLeft(SDF_CharRanges arg, SDF_CharRanges left)
{
  if (SDF_isCharRangesConc(arg)) {
    return (SDF_CharRanges)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) left), 0), 1);
  }

  ATabort("CharRanges has no Left: %t\n", arg);
  return (SDF_CharRanges)NULL;
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_setCharRangesWsAfterLeft(SDF_CharRanges arg, SDF_OptLayout wsAfterLeft) */

SDF_CharRanges SDF_setCharRangesWsAfterLeft(SDF_CharRanges arg, SDF_OptLayout wsAfterLeft)
{
  if (SDF_isCharRangesConc(arg)) {
    return (SDF_CharRanges)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLeft), 1), 1);
  }

  ATabort("CharRanges has no WsAfterLeft: %t\n", arg);
  return (SDF_CharRanges)NULL;
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_setCharRangesRight(SDF_CharRanges arg, SDF_CharRanges right) */

SDF_CharRanges SDF_setCharRangesRight(SDF_CharRanges arg, SDF_CharRanges right)
{
  if (SDF_isCharRangesConc(arg)) {
    return (SDF_CharRanges)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) right), 2), 1);
  }

  ATabort("CharRanges has no Right: %t\n", arg);
  return (SDF_CharRanges)NULL;
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_setCharRangesWsAfterParenOpen(SDF_CharRanges arg, SDF_OptLayout wsAfterParenOpen) */

SDF_CharRanges SDF_setCharRangesWsAfterParenOpen(SDF_CharRanges arg, SDF_OptLayout wsAfterParenOpen)
{
  if (SDF_isCharRangesBracket(arg)) {
    return (SDF_CharRanges)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenOpen), 1), 1);
  }

  ATabort("CharRanges has no WsAfterParenOpen: %t\n", arg);
  return (SDF_CharRanges)NULL;
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_setCharRangesCharRanges(SDF_CharRanges arg, SDF_CharRanges CharRanges) */

SDF_CharRanges SDF_setCharRangesCharRanges(SDF_CharRanges arg, SDF_CharRanges CharRanges)
{
  if (SDF_isCharRangesBracket(arg)) {
    return (SDF_CharRanges)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) CharRanges), 2), 1);
  }

  ATabort("CharRanges has no CharRanges: %t\n", arg);
  return (SDF_CharRanges)NULL;
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_setCharRangesWsAfterCharRanges(SDF_CharRanges arg, SDF_OptLayout wsAfterCharRanges) */

SDF_CharRanges SDF_setCharRangesWsAfterCharRanges(SDF_CharRanges arg, SDF_OptLayout wsAfterCharRanges)
{
  if (SDF_isCharRangesBracket(arg)) {
    return (SDF_CharRanges)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterCharRanges), 3), 1);
  }

  ATabort("CharRanges has no WsAfterCharRanges: %t\n", arg);
  return (SDF_CharRanges)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_OptCharRanges accessors */

/*{{{  ATbool SDF_isValidOptCharRanges(SDF_OptCharRanges arg) */

ATbool SDF_isValidOptCharRanges(SDF_OptCharRanges arg)
{
  if (SDF_isOptCharRangesAbsent(arg)) {
    return ATtrue;
  }
  else if (SDF_isOptCharRangesPresent(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isOptCharRangesAbsent(SDF_OptCharRanges arg) */

inline ATbool SDF_isOptCharRangesAbsent(SDF_OptCharRanges arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternOptCharRangesAbsent);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isOptCharRangesPresent(SDF_OptCharRanges arg) */

inline ATbool SDF_isOptCharRangesPresent(SDF_OptCharRanges arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternOptCharRangesPresent, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasOptCharRangesCharRanges(SDF_OptCharRanges arg) */

ATbool SDF_hasOptCharRangesCharRanges(SDF_OptCharRanges arg)
{
  if (SDF_isOptCharRangesPresent(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_getOptCharRangesCharRanges(SDF_OptCharRanges arg) */

SDF_CharRanges SDF_getOptCharRangesCharRanges(SDF_OptCharRanges arg)
{
  
    return (SDF_CharRanges)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptCharRanges SDF_setOptCharRangesCharRanges(SDF_OptCharRanges arg, SDF_CharRanges CharRanges) */

SDF_OptCharRanges SDF_setOptCharRangesCharRanges(SDF_OptCharRanges arg, SDF_CharRanges CharRanges)
{
  if (SDF_isOptCharRangesPresent(arg)) {
    return (SDF_OptCharRanges)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) CharRanges), 0), 1);
  }

  ATabort("OptCharRanges has no CharRanges: %t\n", arg);
  return (SDF_OptCharRanges)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_CharClass accessors */

/*{{{  ATbool SDF_isValidCharClass(SDF_CharClass arg) */

ATbool SDF_isValidCharClass(SDF_CharClass arg)
{
  if (SDF_isCharClassSimpleCharclass(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharClassComp(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharClassDiff(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharClassIsect(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharClassUnion(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharClassBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharClassSimpleCharclass(SDF_CharClass arg) */

inline ATbool SDF_isCharClassSimpleCharclass(SDF_CharClass arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharClassSimpleCharclass, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharClassComp(SDF_CharClass arg) */

inline ATbool SDF_isCharClassComp(SDF_CharClass arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharClassComp, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharClassDiff(SDF_CharClass arg) */

inline ATbool SDF_isCharClassDiff(SDF_CharClass arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharClassDiff, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharClassIsect(SDF_CharClass arg) */

inline ATbool SDF_isCharClassIsect(SDF_CharClass arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharClassIsect, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharClassUnion(SDF_CharClass arg) */

inline ATbool SDF_isCharClassUnion(SDF_CharClass arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharClassUnion, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharClassBracket(SDF_CharClass arg) */

inline ATbool SDF_isCharClassBracket(SDF_CharClass arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharClassBracket, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasCharClassWsAfterBracketOpen(SDF_CharClass arg) */

ATbool SDF_hasCharClassWsAfterBracketOpen(SDF_CharClass arg)
{
  if (SDF_isCharClassSimpleCharclass(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharClassOptCharRanges(SDF_CharClass arg) */

ATbool SDF_hasCharClassOptCharRanges(SDF_CharClass arg)
{
  if (SDF_isCharClassSimpleCharclass(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharClassWsAfterOptCharRanges(SDF_CharClass arg) */

ATbool SDF_hasCharClassWsAfterOptCharRanges(SDF_CharClass arg)
{
  if (SDF_isCharClassSimpleCharclass(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharClassWsAfterTilde(SDF_CharClass arg) */

ATbool SDF_hasCharClassWsAfterTilde(SDF_CharClass arg)
{
  if (SDF_isCharClassComp(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharClassCharClass(SDF_CharClass arg) */

ATbool SDF_hasCharClassCharClass(SDF_CharClass arg)
{
  if (SDF_isCharClassComp(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharClassBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharClassLeft(SDF_CharClass arg) */

ATbool SDF_hasCharClassLeft(SDF_CharClass arg)
{
  if (SDF_isCharClassDiff(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharClassIsect(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharClassUnion(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharClassWsAfterLeft(SDF_CharClass arg) */

ATbool SDF_hasCharClassWsAfterLeft(SDF_CharClass arg)
{
  if (SDF_isCharClassDiff(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharClassIsect(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharClassUnion(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharClassWsAfterSlash(SDF_CharClass arg) */

ATbool SDF_hasCharClassWsAfterSlash(SDF_CharClass arg)
{
  if (SDF_isCharClassDiff(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharClassRight(SDF_CharClass arg) */

ATbool SDF_hasCharClassRight(SDF_CharClass arg)
{
  if (SDF_isCharClassDiff(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharClassIsect(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharClassUnion(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharClassWsAfterSlashBackslash(SDF_CharClass arg) */

ATbool SDF_hasCharClassWsAfterSlashBackslash(SDF_CharClass arg)
{
  if (SDF_isCharClassIsect(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharClassWsAfterBackslashSlash(SDF_CharClass arg) */

ATbool SDF_hasCharClassWsAfterBackslashSlash(SDF_CharClass arg)
{
  if (SDF_isCharClassUnion(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharClassWsAfterParenOpen(SDF_CharClass arg) */

ATbool SDF_hasCharClassWsAfterParenOpen(SDF_CharClass arg)
{
  if (SDF_isCharClassBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharClassWsAfterCharClass(SDF_CharClass arg) */

ATbool SDF_hasCharClassWsAfterCharClass(SDF_CharClass arg)
{
  if (SDF_isCharClassBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getCharClassWsAfterBracketOpen(SDF_CharClass arg) */

SDF_OptLayout SDF_getCharClassWsAfterBracketOpen(SDF_CharClass arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptCharRanges SDF_getCharClassOptCharRanges(SDF_CharClass arg) */

SDF_OptCharRanges SDF_getCharClassOptCharRanges(SDF_CharClass arg)
{
  
    return (SDF_OptCharRanges)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getCharClassWsAfterOptCharRanges(SDF_CharClass arg) */

SDF_OptLayout SDF_getCharClassWsAfterOptCharRanges(SDF_CharClass arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getCharClassWsAfterTilde(SDF_CharClass arg) */

SDF_OptLayout SDF_getCharClassWsAfterTilde(SDF_CharClass arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_CharClass SDF_getCharClassCharClass(SDF_CharClass arg) */

SDF_CharClass SDF_getCharClassCharClass(SDF_CharClass arg)
{
  if (SDF_isCharClassComp(arg)) {
    return (SDF_CharClass)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else 
    return (SDF_CharClass)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_CharClass SDF_getCharClassLeft(SDF_CharClass arg) */

SDF_CharClass SDF_getCharClassLeft(SDF_CharClass arg)
{
  if (SDF_isCharClassDiff(arg)) {
    return (SDF_CharClass)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
  }
  else if (SDF_isCharClassIsect(arg)) {
    return (SDF_CharClass)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
  }
  else 
    return (SDF_CharClass)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getCharClassWsAfterLeft(SDF_CharClass arg) */

SDF_OptLayout SDF_getCharClassWsAfterLeft(SDF_CharClass arg)
{
  if (SDF_isCharClassDiff(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else if (SDF_isCharClassIsect(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getCharClassWsAfterSlash(SDF_CharClass arg) */

SDF_OptLayout SDF_getCharClassWsAfterSlash(SDF_CharClass arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_CharClass SDF_getCharClassRight(SDF_CharClass arg) */

SDF_CharClass SDF_getCharClassRight(SDF_CharClass arg)
{
  if (SDF_isCharClassDiff(arg)) {
    return (SDF_CharClass)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
  }
  else if (SDF_isCharClassIsect(arg)) {
    return (SDF_CharClass)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
  }
  else 
    return (SDF_CharClass)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getCharClassWsAfterSlashBackslash(SDF_CharClass arg) */

SDF_OptLayout SDF_getCharClassWsAfterSlashBackslash(SDF_CharClass arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getCharClassWsAfterBackslashSlash(SDF_CharClass arg) */

SDF_OptLayout SDF_getCharClassWsAfterBackslashSlash(SDF_CharClass arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getCharClassWsAfterParenOpen(SDF_CharClass arg) */

SDF_OptLayout SDF_getCharClassWsAfterParenOpen(SDF_CharClass arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getCharClassWsAfterCharClass(SDF_CharClass arg) */

SDF_OptLayout SDF_getCharClassWsAfterCharClass(SDF_CharClass arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_CharClass SDF_setCharClassWsAfterBracketOpen(SDF_CharClass arg, SDF_OptLayout wsAfterBracketOpen) */

SDF_CharClass SDF_setCharClassWsAfterBracketOpen(SDF_CharClass arg, SDF_OptLayout wsAfterBracketOpen)
{
  if (SDF_isCharClassSimpleCharclass(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBracketOpen), 1), 1);
  }

  ATabort("CharClass has no WsAfterBracketOpen: %t\n", arg);
  return (SDF_CharClass)NULL;
}

/*}}}  */
/*{{{  SDF_CharClass SDF_setCharClassOptCharRanges(SDF_CharClass arg, SDF_OptCharRanges OptCharRanges) */

SDF_CharClass SDF_setCharClassOptCharRanges(SDF_CharClass arg, SDF_OptCharRanges OptCharRanges)
{
  if (SDF_isCharClassSimpleCharclass(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) OptCharRanges), 2), 1);
  }

  ATabort("CharClass has no OptCharRanges: %t\n", arg);
  return (SDF_CharClass)NULL;
}

/*}}}  */
/*{{{  SDF_CharClass SDF_setCharClassWsAfterOptCharRanges(SDF_CharClass arg, SDF_OptLayout wsAfterOptCharRanges) */

SDF_CharClass SDF_setCharClassWsAfterOptCharRanges(SDF_CharClass arg, SDF_OptLayout wsAfterOptCharRanges)
{
  if (SDF_isCharClassSimpleCharclass(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterOptCharRanges), 3), 1);
  }

  ATabort("CharClass has no WsAfterOptCharRanges: %t\n", arg);
  return (SDF_CharClass)NULL;
}

/*}}}  */
/*{{{  SDF_CharClass SDF_setCharClassWsAfterTilde(SDF_CharClass arg, SDF_OptLayout wsAfterTilde) */

SDF_CharClass SDF_setCharClassWsAfterTilde(SDF_CharClass arg, SDF_OptLayout wsAfterTilde)
{
  if (SDF_isCharClassComp(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterTilde), 1), 1);
  }

  ATabort("CharClass has no WsAfterTilde: %t\n", arg);
  return (SDF_CharClass)NULL;
}

/*}}}  */
/*{{{  SDF_CharClass SDF_setCharClassCharClass(SDF_CharClass arg, SDF_CharClass CharClass) */

SDF_CharClass SDF_setCharClassCharClass(SDF_CharClass arg, SDF_CharClass CharClass)
{
  if (SDF_isCharClassComp(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) CharClass), 2), 1);
  }
  else if (SDF_isCharClassBracket(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) CharClass), 2), 1);
  }

  ATabort("CharClass has no CharClass: %t\n", arg);
  return (SDF_CharClass)NULL;
}

/*}}}  */
/*{{{  SDF_CharClass SDF_setCharClassLeft(SDF_CharClass arg, SDF_CharClass left) */

SDF_CharClass SDF_setCharClassLeft(SDF_CharClass arg, SDF_CharClass left)
{
  if (SDF_isCharClassDiff(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) left), 0), 1);
  }
  else if (SDF_isCharClassIsect(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) left), 0), 1);
  }
  else if (SDF_isCharClassUnion(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) left), 0), 1);
  }

  ATabort("CharClass has no Left: %t\n", arg);
  return (SDF_CharClass)NULL;
}

/*}}}  */
/*{{{  SDF_CharClass SDF_setCharClassWsAfterLeft(SDF_CharClass arg, SDF_OptLayout wsAfterLeft) */

SDF_CharClass SDF_setCharClassWsAfterLeft(SDF_CharClass arg, SDF_OptLayout wsAfterLeft)
{
  if (SDF_isCharClassDiff(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLeft), 1), 1);
  }
  else if (SDF_isCharClassIsect(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLeft), 1), 1);
  }
  else if (SDF_isCharClassUnion(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLeft), 1), 1);
  }

  ATabort("CharClass has no WsAfterLeft: %t\n", arg);
  return (SDF_CharClass)NULL;
}

/*}}}  */
/*{{{  SDF_CharClass SDF_setCharClassWsAfterSlash(SDF_CharClass arg, SDF_OptLayout wsAfterSlash) */

SDF_CharClass SDF_setCharClassWsAfterSlash(SDF_CharClass arg, SDF_OptLayout wsAfterSlash)
{
  if (SDF_isCharClassDiff(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSlash), 3), 1);
  }

  ATabort("CharClass has no WsAfterSlash: %t\n", arg);
  return (SDF_CharClass)NULL;
}

/*}}}  */
/*{{{  SDF_CharClass SDF_setCharClassRight(SDF_CharClass arg, SDF_CharClass right) */

SDF_CharClass SDF_setCharClassRight(SDF_CharClass arg, SDF_CharClass right)
{
  if (SDF_isCharClassDiff(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) right), 4), 1);
  }
  else if (SDF_isCharClassIsect(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) right), 4), 1);
  }
  else if (SDF_isCharClassUnion(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) right), 4), 1);
  }

  ATabort("CharClass has no Right: %t\n", arg);
  return (SDF_CharClass)NULL;
}

/*}}}  */
/*{{{  SDF_CharClass SDF_setCharClassWsAfterSlashBackslash(SDF_CharClass arg, SDF_OptLayout wsAfterSlashBackslash) */

SDF_CharClass SDF_setCharClassWsAfterSlashBackslash(SDF_CharClass arg, SDF_OptLayout wsAfterSlashBackslash)
{
  if (SDF_isCharClassIsect(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSlashBackslash), 3), 1);
  }

  ATabort("CharClass has no WsAfterSlashBackslash: %t\n", arg);
  return (SDF_CharClass)NULL;
}

/*}}}  */
/*{{{  SDF_CharClass SDF_setCharClassWsAfterBackslashSlash(SDF_CharClass arg, SDF_OptLayout wsAfterBackslashSlash) */

SDF_CharClass SDF_setCharClassWsAfterBackslashSlash(SDF_CharClass arg, SDF_OptLayout wsAfterBackslashSlash)
{
  if (SDF_isCharClassUnion(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBackslashSlash), 3), 1);
  }

  ATabort("CharClass has no WsAfterBackslashSlash: %t\n", arg);
  return (SDF_CharClass)NULL;
}

/*}}}  */
/*{{{  SDF_CharClass SDF_setCharClassWsAfterParenOpen(SDF_CharClass arg, SDF_OptLayout wsAfterParenOpen) */

SDF_CharClass SDF_setCharClassWsAfterParenOpen(SDF_CharClass arg, SDF_OptLayout wsAfterParenOpen)
{
  if (SDF_isCharClassBracket(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenOpen), 1), 1);
  }

  ATabort("CharClass has no WsAfterParenOpen: %t\n", arg);
  return (SDF_CharClass)NULL;
}

/*}}}  */
/*{{{  SDF_CharClass SDF_setCharClassWsAfterCharClass(SDF_CharClass arg, SDF_OptLayout wsAfterCharClass) */

SDF_CharClass SDF_setCharClassWsAfterCharClass(SDF_CharClass arg, SDF_OptLayout wsAfterCharClass)
{
  if (SDF_isCharClassBracket(arg)) {
    return (SDF_CharClass)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterCharClass), 3), 1);
  }

  ATabort("CharClass has no WsAfterCharClass: %t\n", arg);
  return (SDF_CharClass)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_LexNumChar accessors */

/*{{{  ATbool SDF_isValidLexNumChar(SDF_LexNumChar arg) */

ATbool SDF_isValidLexNumChar(SDF_LexNumChar arg)
{
  if (SDF_isLexNumCharDigits(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexNumCharDigits(SDF_LexNumChar arg) */

inline ATbool SDF_isLexNumCharDigits(SDF_LexNumChar arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternLexNumCharDigits, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexNumCharNumber(SDF_LexNumChar arg) */

ATbool SDF_hasLexNumCharNumber(SDF_LexNumChar arg)
{
  if (SDF_isLexNumCharDigits(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char* SDF_getLexNumCharNumber(SDF_LexNumChar arg) */

char* SDF_getLexNumCharNumber(SDF_LexNumChar arg)
{
  
    return (char*)SDF_charsToString((ATerm)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1), 1));
}

/*}}}  */
/*{{{  SDF_LexNumChar SDF_setLexNumCharNumber(SDF_LexNumChar arg, const char* number) */

SDF_LexNumChar SDF_setLexNumCharNumber(SDF_LexNumChar arg, const char* number)
{
  if (SDF_isLexNumCharDigits(arg)) {
    return (SDF_LexNumChar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1), (ATerm)((ATerm) ((ATerm) SDF_stringToChars(number))), 1), 1), 1);
  }

  ATabort("LexNumChar has no Number: %t\n", arg);
  return (SDF_LexNumChar)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_NumChar accessors */

/*{{{  ATbool SDF_isValidNumChar(SDF_NumChar arg) */

ATbool SDF_isValidNumChar(SDF_NumChar arg)
{
  if (SDF_isNumCharLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isNumCharLexToCf(SDF_NumChar arg) */

inline ATbool SDF_isNumCharLexToCf(SDF_NumChar arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternNumCharLexToCf, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasNumCharNumChar(SDF_NumChar arg) */

ATbool SDF_hasNumCharNumChar(SDF_NumChar arg)
{
  if (SDF_isNumCharLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_LexNumChar SDF_getNumCharNumChar(SDF_NumChar arg) */

SDF_LexNumChar SDF_getNumCharNumChar(SDF_NumChar arg)
{
  
    return (SDF_LexNumChar)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_NumChar SDF_setNumCharNumChar(SDF_NumChar arg, SDF_LexNumChar NumChar) */

SDF_NumChar SDF_setNumCharNumChar(SDF_NumChar arg, SDF_LexNumChar NumChar)
{
  if (SDF_isNumCharLexToCf(arg)) {
    return (SDF_NumChar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) NumChar), 0), 1);
  }

  ATabort("NumChar has no NumChar: %t\n", arg);
  return (SDF_NumChar)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_LexShortChar accessors */

/*{{{  ATbool SDF_isValidLexShortChar(SDF_LexShortChar arg) */

ATbool SDF_isValidLexShortChar(SDF_LexShortChar arg)
{
  if (SDF_isLexShortCharRegular(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexShortCharEscaped(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexShortCharRegular(SDF_LexShortChar arg) */

inline ATbool SDF_isLexShortCharRegular(SDF_LexShortChar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexShortCharRegular, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexShortCharEscaped(SDF_LexShortChar arg) */

inline ATbool SDF_isLexShortCharEscaped(SDF_LexShortChar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexShortCharEscaped, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasLexShortCharCharacter(SDF_LexShortChar arg) */

ATbool SDF_hasLexShortCharCharacter(SDF_LexShortChar arg)
{
  if (SDF_isLexShortCharRegular(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexShortCharEscape(SDF_LexShortChar arg) */

ATbool SDF_hasLexShortCharEscape(SDF_LexShortChar arg)
{
  if (SDF_isLexShortCharEscaped(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char SDF_getLexShortCharCharacter(SDF_LexShortChar arg) */

char SDF_getLexShortCharCharacter(SDF_LexShortChar arg)
{
  
    return (char)SDF_charToByte((ATerm)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)));
}

/*}}}  */
/*{{{  char SDF_getLexShortCharEscape(SDF_LexShortChar arg) */

char SDF_getLexShortCharEscape(SDF_LexShortChar arg)
{
  
    return (char)SDF_charToByte((ATerm)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1));
}

/*}}}  */
/*{{{  SDF_LexShortChar SDF_setLexShortCharCharacter(SDF_LexShortChar arg, char character) */

SDF_LexShortChar SDF_setLexShortCharCharacter(SDF_LexShortChar arg, char character)
{
  if (SDF_isLexShortCharRegular(arg)) {
    return (SDF_LexShortChar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ((ATerm) SDF_byteToChar(character))), 0), 1);
  }

  ATabort("LexShortChar has no Character: %t\n", arg);
  return (SDF_LexShortChar)NULL;
}

/*}}}  */
/*{{{  SDF_LexShortChar SDF_setLexShortCharEscape(SDF_LexShortChar arg, char escape) */

SDF_LexShortChar SDF_setLexShortCharEscape(SDF_LexShortChar arg, char escape)
{
  if (SDF_isLexShortCharEscaped(arg)) {
    return (SDF_LexShortChar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ((ATerm) SDF_byteToChar(escape))), 1), 1);
  }

  ATabort("LexShortChar has no Escape: %t\n", arg);
  return (SDF_LexShortChar)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_ShortChar accessors */

/*{{{  ATbool SDF_isValidShortChar(SDF_ShortChar arg) */

ATbool SDF_isValidShortChar(SDF_ShortChar arg)
{
  if (SDF_isShortCharLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isShortCharLexToCf(SDF_ShortChar arg) */

inline ATbool SDF_isShortCharLexToCf(SDF_ShortChar arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternShortCharLexToCf, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasShortCharShortChar(SDF_ShortChar arg) */

ATbool SDF_hasShortCharShortChar(SDF_ShortChar arg)
{
  if (SDF_isShortCharLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_LexShortChar SDF_getShortCharShortChar(SDF_ShortChar arg) */

SDF_LexShortChar SDF_getShortCharShortChar(SDF_ShortChar arg)
{
  
    return (SDF_LexShortChar)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_ShortChar SDF_setShortCharShortChar(SDF_ShortChar arg, SDF_LexShortChar ShortChar) */

SDF_ShortChar SDF_setShortCharShortChar(SDF_ShortChar arg, SDF_LexShortChar ShortChar)
{
  if (SDF_isShortCharLexToCf(arg)) {
    return (SDF_ShortChar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ShortChar), 0), 1);
  }

  ATabort("ShortChar has no ShortChar: %t\n", arg);
  return (SDF_ShortChar)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Character accessors */

/*{{{  ATbool SDF_isValidCharacter(SDF_Character arg) */

ATbool SDF_isValidCharacter(SDF_Character arg)
{
  if (SDF_isCharacterNumeric(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharacterShort(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharacterTop(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharacterEof(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharacterBot(arg)) {
    return ATtrue;
  }
  else if (SDF_isCharacterLabelUnderscoreStart(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharacterNumeric(SDF_Character arg) */

inline ATbool SDF_isCharacterNumeric(SDF_Character arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharacterNumeric, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharacterShort(SDF_Character arg) */

inline ATbool SDF_isCharacterShort(SDF_Character arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharacterShort, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharacterTop(SDF_Character arg) */

inline ATbool SDF_isCharacterTop(SDF_Character arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharacterTop);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharacterEof(SDF_Character arg) */

inline ATbool SDF_isCharacterEof(SDF_Character arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharacterEof);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharacterBot(SDF_Character arg) */

inline ATbool SDF_isCharacterBot(SDF_Character arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharacterBot);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isCharacterLabelUnderscoreStart(SDF_Character arg) */

inline ATbool SDF_isCharacterLabelUnderscoreStart(SDF_Character arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternCharacterLabelUnderscoreStart);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasCharacterNumChar(SDF_Character arg) */

ATbool SDF_hasCharacterNumChar(SDF_Character arg)
{
  if (SDF_isCharacterNumeric(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasCharacterShortChar(SDF_Character arg) */

ATbool SDF_hasCharacterShortChar(SDF_Character arg)
{
  if (SDF_isCharacterShort(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_NumChar SDF_getCharacterNumChar(SDF_Character arg) */

SDF_NumChar SDF_getCharacterNumChar(SDF_Character arg)
{
  
    return (SDF_NumChar)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_ShortChar SDF_getCharacterShortChar(SDF_Character arg) */

SDF_ShortChar SDF_getCharacterShortChar(SDF_Character arg)
{
  
    return (SDF_ShortChar)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_Character SDF_setCharacterNumChar(SDF_Character arg, SDF_NumChar NumChar) */

SDF_Character SDF_setCharacterNumChar(SDF_Character arg, SDF_NumChar NumChar)
{
  if (SDF_isCharacterNumeric(arg)) {
    return (SDF_Character)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) NumChar), 0), 1);
  }

  ATabort("Character has no NumChar: %t\n", arg);
  return (SDF_Character)NULL;
}

/*}}}  */
/*{{{  SDF_Character SDF_setCharacterShortChar(SDF_Character arg, SDF_ShortChar ShortChar) */

SDF_Character SDF_setCharacterShortChar(SDF_Character arg, SDF_ShortChar ShortChar)
{
  if (SDF_isCharacterShort(arg)) {
    return (SDF_Character)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ShortChar), 0), 1);
  }

  ATabort("Character has no ShortChar: %t\n", arg);
  return (SDF_Character)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_ATermAttribute accessors */

/*{{{  ATbool SDF_isValidATermAttribute(SDF_ATermAttribute arg) */

ATbool SDF_isValidATermAttribute(SDF_ATermAttribute arg)
{
  if (SDF_isATermAttributeDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isATermAttributeDefault(SDF_ATermAttribute arg) */

inline ATbool SDF_isATermAttributeDefault(SDF_ATermAttribute arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternATermAttributeDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasATermAttributeAterm(SDF_ATermAttribute arg) */

ATbool SDF_hasATermAttributeAterm(SDF_ATermAttribute arg)
{
  if (SDF_isATermAttributeDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_getATermAttributeAterm(SDF_ATermAttribute arg) */

SDF_ATerm SDF_getATermAttributeAterm(SDF_ATermAttribute arg)
{
  
    return (SDF_ATerm)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_ATermAttribute SDF_setATermAttributeAterm(SDF_ATermAttribute arg, SDF_ATerm aterm) */

SDF_ATermAttribute SDF_setATermAttributeAterm(SDF_ATermAttribute arg, SDF_ATerm aterm)
{
  if (SDF_isATermAttributeDefault(arg)) {
    return (SDF_ATermAttribute)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) aterm), 0), 1);
  }

  ATabort("ATermAttribute has no Aterm: %t\n", arg);
  return (SDF_ATermAttribute)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Attribute accessors */

/*{{{  ATbool SDF_isValidAttribute(SDF_Attribute arg) */

ATbool SDF_isValidAttribute(SDF_Attribute arg)
{
  if (SDF_isAttributeTerm(arg)) {
    return ATtrue;
  }
  else if (SDF_isAttributeId(arg)) {
    return ATtrue;
  }
  else if (SDF_isAttributeBracket(arg)) {
    return ATtrue;
  }
  else if (SDF_isAttributeAssoc(arg)) {
    return ATtrue;
  }
  else if (SDF_isAttributeReject(arg)) {
    return ATtrue;
  }
  else if (SDF_isAttributePrefer(arg)) {
    return ATtrue;
  }
  else if (SDF_isAttributeAvoid(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isAttributeTerm(SDF_Attribute arg) */

inline ATbool SDF_isAttributeTerm(SDF_Attribute arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAttributeTerm, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isAttributeId(SDF_Attribute arg) */

inline ATbool SDF_isAttributeId(SDF_Attribute arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAttributeId, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isAttributeBracket(SDF_Attribute arg) */

inline ATbool SDF_isAttributeBracket(SDF_Attribute arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAttributeBracket);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isAttributeAssoc(SDF_Attribute arg) */

inline ATbool SDF_isAttributeAssoc(SDF_Attribute arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAttributeAssoc, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isAttributeReject(SDF_Attribute arg) */

inline ATbool SDF_isAttributeReject(SDF_Attribute arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAttributeReject);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isAttributePrefer(SDF_Attribute arg) */

inline ATbool SDF_isAttributePrefer(SDF_Attribute arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAttributePrefer);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isAttributeAvoid(SDF_Attribute arg) */

inline ATbool SDF_isAttributeAvoid(SDF_Attribute arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAttributeAvoid);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasAttributeATermAttribute(SDF_Attribute arg) */

ATbool SDF_hasAttributeATermAttribute(SDF_Attribute arg)
{
  if (SDF_isAttributeTerm(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAttributeWsAfterId(SDF_Attribute arg) */

ATbool SDF_hasAttributeWsAfterId(SDF_Attribute arg)
{
  if (SDF_isAttributeId(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAttributeWsAfterParenOpen(SDF_Attribute arg) */

ATbool SDF_hasAttributeWsAfterParenOpen(SDF_Attribute arg)
{
  if (SDF_isAttributeId(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAttributeModuleName(SDF_Attribute arg) */

ATbool SDF_hasAttributeModuleName(SDF_Attribute arg)
{
  if (SDF_isAttributeId(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAttributeWsAfterModuleName(SDF_Attribute arg) */

ATbool SDF_hasAttributeWsAfterModuleName(SDF_Attribute arg)
{
  if (SDF_isAttributeId(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAttributeAssociativity(SDF_Attribute arg) */

ATbool SDF_hasAttributeAssociativity(SDF_Attribute arg)
{
  if (SDF_isAttributeAssoc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_ATermAttribute SDF_getAttributeATermAttribute(SDF_Attribute arg) */

SDF_ATermAttribute SDF_getAttributeATermAttribute(SDF_Attribute arg)
{
  
    return (SDF_ATermAttribute)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getAttributeWsAfterId(SDF_Attribute arg) */

SDF_OptLayout SDF_getAttributeWsAfterId(SDF_Attribute arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getAttributeWsAfterParenOpen(SDF_Attribute arg) */

SDF_OptLayout SDF_getAttributeWsAfterParenOpen(SDF_Attribute arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_ModuleName SDF_getAttributeModuleName(SDF_Attribute arg) */

SDF_ModuleName SDF_getAttributeModuleName(SDF_Attribute arg)
{
  
    return (SDF_ModuleName)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getAttributeWsAfterModuleName(SDF_Attribute arg) */

SDF_OptLayout SDF_getAttributeWsAfterModuleName(SDF_Attribute arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_Associativity SDF_getAttributeAssociativity(SDF_Attribute arg) */

SDF_Associativity SDF_getAttributeAssociativity(SDF_Attribute arg)
{
  
    return (SDF_Associativity)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_Attribute SDF_setAttributeATermAttribute(SDF_Attribute arg, SDF_ATermAttribute ATermAttribute) */

SDF_Attribute SDF_setAttributeATermAttribute(SDF_Attribute arg, SDF_ATermAttribute ATermAttribute)
{
  if (SDF_isAttributeTerm(arg)) {
    return (SDF_Attribute)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ATermAttribute), 0), 1);
  }

  ATabort("Attribute has no ATermAttribute: %t\n", arg);
  return (SDF_Attribute)NULL;
}

/*}}}  */
/*{{{  SDF_Attribute SDF_setAttributeWsAfterId(SDF_Attribute arg, SDF_OptLayout wsAfterId) */

SDF_Attribute SDF_setAttributeWsAfterId(SDF_Attribute arg, SDF_OptLayout wsAfterId)
{
  if (SDF_isAttributeId(arg)) {
    return (SDF_Attribute)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterId), 1), 1);
  }

  ATabort("Attribute has no WsAfterId: %t\n", arg);
  return (SDF_Attribute)NULL;
}

/*}}}  */
/*{{{  SDF_Attribute SDF_setAttributeWsAfterParenOpen(SDF_Attribute arg, SDF_OptLayout wsAfterParenOpen) */

SDF_Attribute SDF_setAttributeWsAfterParenOpen(SDF_Attribute arg, SDF_OptLayout wsAfterParenOpen)
{
  if (SDF_isAttributeId(arg)) {
    return (SDF_Attribute)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenOpen), 3), 1);
  }

  ATabort("Attribute has no WsAfterParenOpen: %t\n", arg);
  return (SDF_Attribute)NULL;
}

/*}}}  */
/*{{{  SDF_Attribute SDF_setAttributeModuleName(SDF_Attribute arg, SDF_ModuleName ModuleName) */

SDF_Attribute SDF_setAttributeModuleName(SDF_Attribute arg, SDF_ModuleName ModuleName)
{
  if (SDF_isAttributeId(arg)) {
    return (SDF_Attribute)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ModuleName), 4), 1);
  }

  ATabort("Attribute has no ModuleName: %t\n", arg);
  return (SDF_Attribute)NULL;
}

/*}}}  */
/*{{{  SDF_Attribute SDF_setAttributeWsAfterModuleName(SDF_Attribute arg, SDF_OptLayout wsAfterModuleName) */

SDF_Attribute SDF_setAttributeWsAfterModuleName(SDF_Attribute arg, SDF_OptLayout wsAfterModuleName)
{
  if (SDF_isAttributeId(arg)) {
    return (SDF_Attribute)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterModuleName), 5), 1);
  }

  ATabort("Attribute has no WsAfterModuleName: %t\n", arg);
  return (SDF_Attribute)NULL;
}

/*}}}  */
/*{{{  SDF_Attribute SDF_setAttributeAssociativity(SDF_Attribute arg, SDF_Associativity Associativity) */

SDF_Attribute SDF_setAttributeAssociativity(SDF_Attribute arg, SDF_Associativity Associativity)
{
  if (SDF_isAttributeAssoc(arg)) {
    return (SDF_Attribute)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Associativity), 0), 1);
  }

  ATabort("Attribute has no Associativity: %t\n", arg);
  return (SDF_Attribute)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Attributes accessors */

/*{{{  ATbool SDF_isValidAttributes(SDF_Attributes arg) */

ATbool SDF_isValidAttributes(SDF_Attributes arg)
{
  if (SDF_isAttributesAttrs(arg)) {
    return ATtrue;
  }
  else if (SDF_isAttributesNoAttrs(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isAttributesAttrs(SDF_Attributes arg) */

inline ATbool SDF_isAttributesAttrs(SDF_Attributes arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAttributesAttrs, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isAttributesNoAttrs(SDF_Attributes arg) */

inline ATbool SDF_isAttributesNoAttrs(SDF_Attributes arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAttributesNoAttrs);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasAttributesWsAfterBraceOpen(SDF_Attributes arg) */

ATbool SDF_hasAttributesWsAfterBraceOpen(SDF_Attributes arg)
{
  if (SDF_isAttributesAttrs(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAttributesList(SDF_Attributes arg) */

ATbool SDF_hasAttributesList(SDF_Attributes arg)
{
  if (SDF_isAttributesAttrs(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAttributesWsAfterList(SDF_Attributes arg) */

ATbool SDF_hasAttributesWsAfterList(SDF_Attributes arg)
{
  if (SDF_isAttributesAttrs(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getAttributesWsAfterBraceOpen(SDF_Attributes arg) */

SDF_OptLayout SDF_getAttributesWsAfterBraceOpen(SDF_Attributes arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_AttributeList SDF_getAttributesList(SDF_Attributes arg) */

SDF_AttributeList SDF_getAttributesList(SDF_Attributes arg)
{
  
    return (SDF_AttributeList)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getAttributesWsAfterList(SDF_Attributes arg) */

SDF_OptLayout SDF_getAttributesWsAfterList(SDF_Attributes arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Attributes SDF_setAttributesWsAfterBraceOpen(SDF_Attributes arg, SDF_OptLayout wsAfterBraceOpen) */

SDF_Attributes SDF_setAttributesWsAfterBraceOpen(SDF_Attributes arg, SDF_OptLayout wsAfterBraceOpen)
{
  if (SDF_isAttributesAttrs(arg)) {
    return (SDF_Attributes)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBraceOpen), 1), 1);
  }

  ATabort("Attributes has no WsAfterBraceOpen: %t\n", arg);
  return (SDF_Attributes)NULL;
}

/*}}}  */
/*{{{  SDF_Attributes SDF_setAttributesList(SDF_Attributes arg, SDF_AttributeList list) */

SDF_Attributes SDF_setAttributesList(SDF_Attributes arg, SDF_AttributeList list)
{
  if (SDF_isAttributesAttrs(arg)) {
    return (SDF_Attributes)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), (ATerm)((ATerm) list), 1), 2), 1);
  }

  ATabort("Attributes has no List: %t\n", arg);
  return (SDF_Attributes)NULL;
}

/*}}}  */
/*{{{  SDF_Attributes SDF_setAttributesWsAfterList(SDF_Attributes arg, SDF_OptLayout wsAfterList) */

SDF_Attributes SDF_setAttributesWsAfterList(SDF_Attributes arg, SDF_OptLayout wsAfterList)
{
  if (SDF_isAttributesAttrs(arg)) {
    return (SDF_Attributes)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterList), 3), 1);
  }

  ATabort("Attributes has no WsAfterList: %t\n", arg);
  return (SDF_Attributes)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Production accessors */

/*{{{  ATbool SDF_isValidProduction(SDF_Production arg) */

ATbool SDF_isValidProduction(SDF_Production arg)
{
  if (SDF_isProductionProd(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionUnquotedFun(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isProductionProd(SDF_Production arg) */

inline ATbool SDF_isProductionProd(SDF_Production arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternProductionProd, NULL, NULL, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isProductionUnquotedFun(SDF_Production arg) */

inline ATbool SDF_isProductionUnquotedFun(SDF_Production arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternProductionUnquotedFun, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isProductionQuotedFun(SDF_Production arg) */

inline ATbool SDF_isProductionQuotedFun(SDF_Production arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternProductionQuotedFun, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionSymbols(SDF_Production arg) */

ATbool SDF_hasProductionSymbols(SDF_Production arg)
{
  if (SDF_isProductionProd(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionWsAfterSymbols(SDF_Production arg) */

ATbool SDF_hasProductionWsAfterSymbols(SDF_Production arg)
{
  if (SDF_isProductionProd(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionWsAfterGreaterThan(SDF_Production arg) */

ATbool SDF_hasProductionWsAfterGreaterThan(SDF_Production arg)
{
  if (SDF_isProductionProd(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionUnquotedFun(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionResult(SDF_Production arg) */

ATbool SDF_hasProductionResult(SDF_Production arg)
{
  if (SDF_isProductionProd(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionUnquotedFun(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionWsAfterResult(SDF_Production arg) */

ATbool SDF_hasProductionWsAfterResult(SDF_Production arg)
{
  if (SDF_isProductionProd(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionUnquotedFun(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionAttributes(SDF_Production arg) */

ATbool SDF_hasProductionAttributes(SDF_Production arg)
{
  if (SDF_isProductionProd(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionUnquotedFun(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionFun(SDF_Production arg) */

ATbool SDF_hasProductionFun(SDF_Production arg)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionWsAfterFun(SDF_Production arg) */

ATbool SDF_hasProductionWsAfterFun(SDF_Production arg)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionWsAfterParenOpen(SDF_Production arg) */

ATbool SDF_hasProductionWsAfterParenOpen(SDF_Production arg)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionArguments(SDF_Production arg) */

ATbool SDF_hasProductionArguments(SDF_Production arg)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionWsAfterArguments(SDF_Production arg) */

ATbool SDF_hasProductionWsAfterArguments(SDF_Production arg)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionWsAfterParenClose(SDF_Production arg) */

ATbool SDF_hasProductionWsAfterParenClose(SDF_Production arg)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionStr(SDF_Production arg) */

ATbool SDF_hasProductionStr(SDF_Production arg)
{
  if (SDF_isProductionQuotedFun(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionWsAfterStr(SDF_Production arg) */

ATbool SDF_hasProductionWsAfterStr(SDF_Production arg)
{
  if (SDF_isProductionQuotedFun(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_Symbols SDF_getProductionSymbols(SDF_Production arg) */

SDF_Symbols SDF_getProductionSymbols(SDF_Production arg)
{
  
    return (SDF_Symbols)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getProductionWsAfterSymbols(SDF_Production arg) */

SDF_OptLayout SDF_getProductionWsAfterSymbols(SDF_Production arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getProductionWsAfterGreaterThan(SDF_Production arg) */

SDF_OptLayout SDF_getProductionWsAfterGreaterThan(SDF_Production arg)
{
  if (SDF_isProductionProd(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 9);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 9);
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getProductionResult(SDF_Production arg) */

SDF_Symbol SDF_getProductionResult(SDF_Production arg)
{
  if (SDF_isProductionProd(arg)) {
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
  }
  else if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 10);
  }
  else 
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 10);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getProductionWsAfterResult(SDF_Production arg) */

SDF_OptLayout SDF_getProductionWsAfterResult(SDF_Production arg)
{
  if (SDF_isProductionProd(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
  }
  else if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 11);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 11);
}

/*}}}  */
/*{{{  SDF_Attributes SDF_getProductionAttributes(SDF_Production arg) */

SDF_Attributes SDF_getProductionAttributes(SDF_Production arg)
{
  if (SDF_isProductionProd(arg)) {
    return (SDF_Attributes)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 6);
  }
  else if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_Attributes)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 12);
  }
  else 
    return (SDF_Attributes)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 12);
}

/*}}}  */
/*{{{  SDF_IdCon SDF_getProductionFun(SDF_Production arg) */

SDF_IdCon SDF_getProductionFun(SDF_Production arg)
{
  
    return (SDF_IdCon)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getProductionWsAfterFun(SDF_Production arg) */

SDF_OptLayout SDF_getProductionWsAfterFun(SDF_Production arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getProductionWsAfterParenOpen(SDF_Production arg) */

SDF_OptLayout SDF_getProductionWsAfterParenOpen(SDF_Production arg)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_SymbolArguments SDF_getProductionArguments(SDF_Production arg) */

SDF_SymbolArguments SDF_getProductionArguments(SDF_Production arg)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_SymbolArguments)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4), 1);
  }
  else 
    return (SDF_SymbolArguments)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getProductionWsAfterArguments(SDF_Production arg) */

SDF_OptLayout SDF_getProductionWsAfterArguments(SDF_Production arg)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getProductionWsAfterParenClose(SDF_Production arg) */

SDF_OptLayout SDF_getProductionWsAfterParenClose(SDF_Production arg)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 7);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 7);
}

/*}}}  */
/*{{{  SDF_StrCon SDF_getProductionStr(SDF_Production arg) */

SDF_StrCon SDF_getProductionStr(SDF_Production arg)
{
  
    return (SDF_StrCon)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getProductionWsAfterStr(SDF_Production arg) */

SDF_OptLayout SDF_getProductionWsAfterStr(SDF_Production arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Production SDF_setProductionSymbols(SDF_Production arg, SDF_Symbols Symbols) */

SDF_Production SDF_setProductionSymbols(SDF_Production arg, SDF_Symbols Symbols)
{
  if (SDF_isProductionProd(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbols), 0), 1);
  }

  ATabort("Production has no Symbols: %t\n", arg);
  return (SDF_Production)NULL;
}

/*}}}  */
/*{{{  SDF_Production SDF_setProductionWsAfterSymbols(SDF_Production arg, SDF_OptLayout wsAfterSymbols) */

SDF_Production SDF_setProductionWsAfterSymbols(SDF_Production arg, SDF_OptLayout wsAfterSymbols)
{
  if (SDF_isProductionProd(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSymbols), 1), 1);
  }

  ATabort("Production has no WsAfterSymbols: %t\n", arg);
  return (SDF_Production)NULL;
}

/*}}}  */
/*{{{  SDF_Production SDF_setProductionWsAfterGreaterThan(SDF_Production arg, SDF_OptLayout wsAfterGreaterThan) */

SDF_Production SDF_setProductionWsAfterGreaterThan(SDF_Production arg, SDF_OptLayout wsAfterGreaterThan)
{
  if (SDF_isProductionProd(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterGreaterThan), 3), 1);
  }
  else if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterGreaterThan), 9), 1);
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterGreaterThan), 9), 1);
  }

  ATabort("Production has no WsAfterGreaterThan: %t\n", arg);
  return (SDF_Production)NULL;
}

/*}}}  */
/*{{{  SDF_Production SDF_setProductionResult(SDF_Production arg, SDF_Symbol result) */

SDF_Production SDF_setProductionResult(SDF_Production arg, SDF_Symbol result)
{
  if (SDF_isProductionProd(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) result), 4), 1);
  }
  else if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) result), 10), 1);
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) result), 10), 1);
  }

  ATabort("Production has no Result: %t\n", arg);
  return (SDF_Production)NULL;
}

/*}}}  */
/*{{{  SDF_Production SDF_setProductionWsAfterResult(SDF_Production arg, SDF_OptLayout wsAfterResult) */

SDF_Production SDF_setProductionWsAfterResult(SDF_Production arg, SDF_OptLayout wsAfterResult)
{
  if (SDF_isProductionProd(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterResult), 5), 1);
  }
  else if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterResult), 11), 1);
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterResult), 11), 1);
  }

  ATabort("Production has no WsAfterResult: %t\n", arg);
  return (SDF_Production)NULL;
}

/*}}}  */
/*{{{  SDF_Production SDF_setProductionAttributes(SDF_Production arg, SDF_Attributes Attributes) */

SDF_Production SDF_setProductionAttributes(SDF_Production arg, SDF_Attributes Attributes)
{
  if (SDF_isProductionProd(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Attributes), 6), 1);
  }
  else if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Attributes), 12), 1);
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Attributes), 12), 1);
  }

  ATabort("Production has no Attributes: %t\n", arg);
  return (SDF_Production)NULL;
}

/*}}}  */
/*{{{  SDF_Production SDF_setProductionFun(SDF_Production arg, SDF_IdCon fun) */

SDF_Production SDF_setProductionFun(SDF_Production arg, SDF_IdCon fun)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) fun), 0), 1);
  }

  ATabort("Production has no Fun: %t\n", arg);
  return (SDF_Production)NULL;
}

/*}}}  */
/*{{{  SDF_Production SDF_setProductionWsAfterFun(SDF_Production arg, SDF_OptLayout wsAfterFun) */

SDF_Production SDF_setProductionWsAfterFun(SDF_Production arg, SDF_OptLayout wsAfterFun)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterFun), 1), 1);
  }

  ATabort("Production has no WsAfterFun: %t\n", arg);
  return (SDF_Production)NULL;
}

/*}}}  */
/*{{{  SDF_Production SDF_setProductionWsAfterParenOpen(SDF_Production arg, SDF_OptLayout wsAfterParenOpen) */

SDF_Production SDF_setProductionWsAfterParenOpen(SDF_Production arg, SDF_OptLayout wsAfterParenOpen)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenOpen), 3), 1);
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenOpen), 3), 1);
  }

  ATabort("Production has no WsAfterParenOpen: %t\n", arg);
  return (SDF_Production)NULL;
}

/*}}}  */
/*{{{  SDF_Production SDF_setProductionArguments(SDF_Production arg, SDF_SymbolArguments arguments) */

SDF_Production SDF_setProductionArguments(SDF_Production arg, SDF_SymbolArguments arguments)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4), (ATerm)((ATerm) arguments), 1), 4), 1);
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4), (ATerm)((ATerm) arguments), 1), 4), 1);
  }

  ATabort("Production has no Arguments: %t\n", arg);
  return (SDF_Production)NULL;
}

/*}}}  */
/*{{{  SDF_Production SDF_setProductionWsAfterArguments(SDF_Production arg, SDF_OptLayout wsAfterArguments) */

SDF_Production SDF_setProductionWsAfterArguments(SDF_Production arg, SDF_OptLayout wsAfterArguments)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterArguments), 5), 1);
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterArguments), 5), 1);
  }

  ATabort("Production has no WsAfterArguments: %t\n", arg);
  return (SDF_Production)NULL;
}

/*}}}  */
/*{{{  SDF_Production SDF_setProductionWsAfterParenClose(SDF_Production arg, SDF_OptLayout wsAfterParenClose) */

SDF_Production SDF_setProductionWsAfterParenClose(SDF_Production arg, SDF_OptLayout wsAfterParenClose)
{
  if (SDF_isProductionUnquotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenClose), 7), 1);
  }
  else if (SDF_isProductionQuotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenClose), 7), 1);
  }

  ATabort("Production has no WsAfterParenClose: %t\n", arg);
  return (SDF_Production)NULL;
}

/*}}}  */
/*{{{  SDF_Production SDF_setProductionStr(SDF_Production arg, SDF_StrCon str) */

SDF_Production SDF_setProductionStr(SDF_Production arg, SDF_StrCon str)
{
  if (SDF_isProductionQuotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) str), 0), 1);
  }

  ATabort("Production has no Str: %t\n", arg);
  return (SDF_Production)NULL;
}

/*}}}  */
/*{{{  SDF_Production SDF_setProductionWsAfterStr(SDF_Production arg, SDF_OptLayout wsAfterStr) */

SDF_Production SDF_setProductionWsAfterStr(SDF_Production arg, SDF_OptLayout wsAfterStr)
{
  if (SDF_isProductionQuotedFun(arg)) {
    return (SDF_Production)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterStr), 1), 1);
  }

  ATabort("Production has no WsAfterStr: %t\n", arg);
  return (SDF_Production)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Productions accessors */

/*{{{  ATbool SDF_isValidProductions(SDF_Productions arg) */

ATbool SDF_isValidProductions(SDF_Productions arg)
{
  if (SDF_isProductionsDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isProductionsDefault(SDF_Productions arg) */

inline ATbool SDF_isProductionsDefault(SDF_Productions arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternProductionsDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionsList(SDF_Productions arg) */

ATbool SDF_hasProductionsList(SDF_Productions arg)
{
  if (SDF_isProductionsDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_ProductionList SDF_getProductionsList(SDF_Productions arg) */

SDF_ProductionList SDF_getProductionsList(SDF_Productions arg)
{
  
    return (SDF_ProductionList)ATgetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), 1);
}

/*}}}  */
/*{{{  SDF_Productions SDF_setProductionsList(SDF_Productions arg, SDF_ProductionList list) */

SDF_Productions SDF_setProductionsList(SDF_Productions arg, SDF_ProductionList list)
{
  if (SDF_isProductionsDefault(arg)) {
    return (SDF_Productions)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), (ATerm)((ATerm) list), 1), 0), 1);
  }

  ATabort("Productions has no List: %t\n", arg);
  return (SDF_Productions)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_AttributeList accessors */

/*{{{  ATbool SDF_isValidAttributeList(SDF_AttributeList arg) */

ATbool SDF_isValidAttributeList(SDF_AttributeList arg)
{
  if (SDF_isAttributeListEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isAttributeListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isAttributeListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isAttributeListEmpty(SDF_AttributeList arg) */

inline ATbool SDF_isAttributeListEmpty(SDF_AttributeList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternAttributeListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isAttributeListSingle(SDF_AttributeList arg) */

inline ATbool SDF_isAttributeListSingle(SDF_AttributeList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAttributeListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isAttributeListMany(SDF_AttributeList arg) */

inline ATbool SDF_isAttributeListMany(SDF_AttributeList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAttributeListMany, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasAttributeListHead(SDF_AttributeList arg) */

ATbool SDF_hasAttributeListHead(SDF_AttributeList arg)
{
  if (SDF_isAttributeListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isAttributeListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAttributeListWsAfterHead(SDF_AttributeList arg) */

ATbool SDF_hasAttributeListWsAfterHead(SDF_AttributeList arg)
{
  if (SDF_isAttributeListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAttributeListWsAfterSep(SDF_AttributeList arg) */

ATbool SDF_hasAttributeListWsAfterSep(SDF_AttributeList arg)
{
  if (SDF_isAttributeListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasAttributeListTail(SDF_AttributeList arg) */

ATbool SDF_hasAttributeListTail(SDF_AttributeList arg)
{
  if (SDF_isAttributeListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_AttributeList SDF_getAttributeListTail(SDF_AttributeList arg) */

SDF_AttributeList SDF_getAttributeListTail(SDF_AttributeList arg)
{
  assert(!SDF_isAttributeListEmpty(arg) && "getTail on an empty list");
  if (SDF_isAttributeListSingle(arg)) {
    return (SDF_AttributeList) SDF_makeAttributeListEmpty();
  }
  else {
  
    return (SDF_AttributeList)ATgetTail((ATermList)arg, 4);
  }
}

/*}}}  */
/*{{{  SDF_Attribute SDF_getAttributeListHead(SDF_AttributeList arg) */

SDF_Attribute SDF_getAttributeListHead(SDF_AttributeList arg)
{
  if (SDF_isAttributeListSingle(arg)) {
    return (SDF_Attribute)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Attribute)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getAttributeListWsAfterHead(SDF_AttributeList arg) */

SDF_OptLayout SDF_getAttributeListWsAfterHead(SDF_AttributeList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getAttributeListWsAfterSep(SDF_AttributeList arg) */

SDF_OptLayout SDF_getAttributeListWsAfterSep(SDF_AttributeList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 3);
}

/*}}}  */
/*{{{  SDF_AttributeList SDF_setAttributeListHead(SDF_AttributeList arg, SDF_Attribute head) */

SDF_AttributeList SDF_setAttributeListHead(SDF_AttributeList arg, SDF_Attribute head)
{
  if (SDF_isAttributeListSingle(arg)) {
    return (SDF_AttributeList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isAttributeListMany(arg)) {
    return (SDF_AttributeList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("AttributeList has no Head: %t\n", arg);
  return (SDF_AttributeList)NULL;
}

/*}}}  */
/*{{{  SDF_AttributeList SDF_setAttributeListWsAfterHead(SDF_AttributeList arg, SDF_OptLayout wsAfterHead) */

SDF_AttributeList SDF_setAttributeListWsAfterHead(SDF_AttributeList arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isAttributeListMany(arg)) {
    return (SDF_AttributeList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("AttributeList has no WsAfterHead: %t\n", arg);
  return (SDF_AttributeList)NULL;
}

/*}}}  */
/*{{{  SDF_AttributeList SDF_setAttributeListWsAfterSep(SDF_AttributeList arg, SDF_OptLayout wsAfterSep) */

SDF_AttributeList SDF_setAttributeListWsAfterSep(SDF_AttributeList arg, SDF_OptLayout wsAfterSep)
{
  if (SDF_isAttributeListMany(arg)) {
    return (SDF_AttributeList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterSep), 3);
  }

  ATabort("AttributeList has no WsAfterSep: %t\n", arg);
  return (SDF_AttributeList)NULL;
}

/*}}}  */
/*{{{  SDF_AttributeList SDF_setAttributeListTail(SDF_AttributeList arg, SDF_AttributeList tail) */

SDF_AttributeList SDF_setAttributeListTail(SDF_AttributeList arg, SDF_AttributeList tail)
{
  if (SDF_isAttributeListMany(arg)) {
    return (SDF_AttributeList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 4);
  }

  ATabort("AttributeList has no Tail: %t\n", arg);
  return (SDF_AttributeList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_ProductionList accessors */

/*{{{  ATbool SDF_isValidProductionList(SDF_ProductionList arg) */

ATbool SDF_isValidProductionList(SDF_ProductionList arg)
{
  if (SDF_isProductionListEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isProductionListEmpty(SDF_ProductionList arg) */

inline ATbool SDF_isProductionListEmpty(SDF_ProductionList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternProductionListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isProductionListSingle(SDF_ProductionList arg) */

inline ATbool SDF_isProductionListSingle(SDF_ProductionList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternProductionListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isProductionListMany(SDF_ProductionList arg) */

inline ATbool SDF_isProductionListMany(SDF_ProductionList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternProductionListMany, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionListHead(SDF_ProductionList arg) */

ATbool SDF_hasProductionListHead(SDF_ProductionList arg)
{
  if (SDF_isProductionListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isProductionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionListWsAfterHead(SDF_ProductionList arg) */

ATbool SDF_hasProductionListWsAfterHead(SDF_ProductionList arg)
{
  if (SDF_isProductionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasProductionListTail(SDF_ProductionList arg) */

ATbool SDF_hasProductionListTail(SDF_ProductionList arg)
{
  if (SDF_isProductionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_ProductionList SDF_getProductionListTail(SDF_ProductionList arg) */

SDF_ProductionList SDF_getProductionListTail(SDF_ProductionList arg)
{
  assert(!SDF_isProductionListEmpty(arg) && "getTail on an empty list");
  if (SDF_isProductionListSingle(arg)) {
    return (SDF_ProductionList) SDF_makeProductionListEmpty();
  }
  else {
  
    return (SDF_ProductionList)ATgetTail((ATermList)arg, 2);
  }
}

/*}}}  */
/*{{{  SDF_Production SDF_getProductionListHead(SDF_ProductionList arg) */

SDF_Production SDF_getProductionListHead(SDF_ProductionList arg)
{
  if (SDF_isProductionListSingle(arg)) {
    return (SDF_Production)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Production)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getProductionListWsAfterHead(SDF_ProductionList arg) */

SDF_OptLayout SDF_getProductionListWsAfterHead(SDF_ProductionList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_ProductionList SDF_setProductionListHead(SDF_ProductionList arg, SDF_Production head) */

SDF_ProductionList SDF_setProductionListHead(SDF_ProductionList arg, SDF_Production head)
{
  if (SDF_isProductionListSingle(arg)) {
    return (SDF_ProductionList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isProductionListMany(arg)) {
    return (SDF_ProductionList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("ProductionList has no Head: %t\n", arg);
  return (SDF_ProductionList)NULL;
}

/*}}}  */
/*{{{  SDF_ProductionList SDF_setProductionListWsAfterHead(SDF_ProductionList arg, SDF_OptLayout wsAfterHead) */

SDF_ProductionList SDF_setProductionListWsAfterHead(SDF_ProductionList arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isProductionListMany(arg)) {
    return (SDF_ProductionList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("ProductionList has no WsAfterHead: %t\n", arg);
  return (SDF_ProductionList)NULL;
}

/*}}}  */
/*{{{  SDF_ProductionList SDF_setProductionListTail(SDF_ProductionList arg, SDF_ProductionList tail) */

SDF_ProductionList SDF_setProductionListTail(SDF_ProductionList arg, SDF_ProductionList tail)
{
  if (SDF_isProductionListMany(arg)) {
    return (SDF_ProductionList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 2);
  }

  ATabort("ProductionList has no Tail: %t\n", arg);
  return (SDF_ProductionList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Label accessors */

/*{{{  ATbool SDF_isValidLabel(SDF_Label arg) */

ATbool SDF_isValidLabel(SDF_Label arg)
{
  if (SDF_isLabelQuoted(arg)) {
    return ATtrue;
  }
  else if (SDF_isLabelUnquoted(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLabelQuoted(SDF_Label arg) */

inline ATbool SDF_isLabelQuoted(SDF_Label arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLabelQuoted, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLabelUnquoted(SDF_Label arg) */

inline ATbool SDF_isLabelUnquoted(SDF_Label arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLabelUnquoted, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasLabelStrCon(SDF_Label arg) */

ATbool SDF_hasLabelStrCon(SDF_Label arg)
{
  if (SDF_isLabelQuoted(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLabelIdCon(SDF_Label arg) */

ATbool SDF_hasLabelIdCon(SDF_Label arg)
{
  if (SDF_isLabelUnquoted(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_StrCon SDF_getLabelStrCon(SDF_Label arg) */

SDF_StrCon SDF_getLabelStrCon(SDF_Label arg)
{
  
    return (SDF_StrCon)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_IdCon SDF_getLabelIdCon(SDF_Label arg) */

SDF_IdCon SDF_getLabelIdCon(SDF_Label arg)
{
  
    return (SDF_IdCon)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_Label SDF_setLabelStrCon(SDF_Label arg, SDF_StrCon StrCon) */

SDF_Label SDF_setLabelStrCon(SDF_Label arg, SDF_StrCon StrCon)
{
  if (SDF_isLabelQuoted(arg)) {
    return (SDF_Label)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) StrCon), 0), 1);
  }

  ATabort("Label has no StrCon: %t\n", arg);
  return (SDF_Label)NULL;
}

/*}}}  */
/*{{{  SDF_Label SDF_setLabelIdCon(SDF_Label arg, SDF_IdCon IdCon) */

SDF_Label SDF_setLabelIdCon(SDF_Label arg, SDF_IdCon IdCon)
{
  if (SDF_isLabelUnquoted(arg)) {
    return (SDF_Label)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) IdCon), 0), 1);
  }

  ATabort("Label has no IdCon: %t\n", arg);
  return (SDF_Label)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_SymbolArguments accessors */

/*{{{  ATbool SDF_isValidSymbolArguments(SDF_SymbolArguments arg) */

ATbool SDF_isValidSymbolArguments(SDF_SymbolArguments arg)
{
  if (SDF_isSymbolArgumentsEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolArgumentsSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolArgumentsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolArgumentsEmpty(SDF_SymbolArguments arg) */

inline ATbool SDF_isSymbolArgumentsEmpty(SDF_SymbolArguments arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternSymbolArgumentsEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolArgumentsSingle(SDF_SymbolArguments arg) */

inline ATbool SDF_isSymbolArgumentsSingle(SDF_SymbolArguments arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolArgumentsSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolArgumentsMany(SDF_SymbolArguments arg) */

inline ATbool SDF_isSymbolArgumentsMany(SDF_SymbolArguments arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolArgumentsMany, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolArgumentsHead(SDF_SymbolArguments arg) */

ATbool SDF_hasSymbolArgumentsHead(SDF_SymbolArguments arg)
{
  if (SDF_isSymbolArgumentsSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolArgumentsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolArgumentsWsAfterHead(SDF_SymbolArguments arg) */

ATbool SDF_hasSymbolArgumentsWsAfterHead(SDF_SymbolArguments arg)
{
  if (SDF_isSymbolArgumentsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolArgumentsWsAfterSep(SDF_SymbolArguments arg) */

ATbool SDF_hasSymbolArgumentsWsAfterSep(SDF_SymbolArguments arg)
{
  if (SDF_isSymbolArgumentsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolArgumentsTail(SDF_SymbolArguments arg) */

ATbool SDF_hasSymbolArgumentsTail(SDF_SymbolArguments arg)
{
  if (SDF_isSymbolArgumentsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_SymbolArguments SDF_getSymbolArgumentsTail(SDF_SymbolArguments arg) */

SDF_SymbolArguments SDF_getSymbolArgumentsTail(SDF_SymbolArguments arg)
{
  assert(!SDF_isSymbolArgumentsEmpty(arg) && "getTail on an empty list");
  if (SDF_isSymbolArgumentsSingle(arg)) {
    return (SDF_SymbolArguments) SDF_makeSymbolArgumentsEmpty();
  }
  else {
  
    return (SDF_SymbolArguments)ATgetTail((ATermList)arg, 4);
  }
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getSymbolArgumentsHead(SDF_SymbolArguments arg) */

SDF_Symbol SDF_getSymbolArgumentsHead(SDF_SymbolArguments arg)
{
  if (SDF_isSymbolArgumentsSingle(arg)) {
    return (SDF_Symbol)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Symbol)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolArgumentsWsAfterHead(SDF_SymbolArguments arg) */

SDF_OptLayout SDF_getSymbolArgumentsWsAfterHead(SDF_SymbolArguments arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolArgumentsWsAfterSep(SDF_SymbolArguments arg) */

SDF_OptLayout SDF_getSymbolArgumentsWsAfterSep(SDF_SymbolArguments arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 3);
}

/*}}}  */
/*{{{  SDF_SymbolArguments SDF_setSymbolArgumentsHead(SDF_SymbolArguments arg, SDF_Symbol head) */

SDF_SymbolArguments SDF_setSymbolArgumentsHead(SDF_SymbolArguments arg, SDF_Symbol head)
{
  if (SDF_isSymbolArgumentsSingle(arg)) {
    return (SDF_SymbolArguments)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isSymbolArgumentsMany(arg)) {
    return (SDF_SymbolArguments)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("SymbolArguments has no Head: %t\n", arg);
  return (SDF_SymbolArguments)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolArguments SDF_setSymbolArgumentsWsAfterHead(SDF_SymbolArguments arg, SDF_OptLayout wsAfterHead) */

SDF_SymbolArguments SDF_setSymbolArgumentsWsAfterHead(SDF_SymbolArguments arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isSymbolArgumentsMany(arg)) {
    return (SDF_SymbolArguments)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("SymbolArguments has no WsAfterHead: %t\n", arg);
  return (SDF_SymbolArguments)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolArguments SDF_setSymbolArgumentsWsAfterSep(SDF_SymbolArguments arg, SDF_OptLayout wsAfterSep) */

SDF_SymbolArguments SDF_setSymbolArgumentsWsAfterSep(SDF_SymbolArguments arg, SDF_OptLayout wsAfterSep)
{
  if (SDF_isSymbolArgumentsMany(arg)) {
    return (SDF_SymbolArguments)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterSep), 3);
  }

  ATabort("SymbolArguments has no WsAfterSep: %t\n", arg);
  return (SDF_SymbolArguments)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolArguments SDF_setSymbolArgumentsTail(SDF_SymbolArguments arg, SDF_SymbolArguments tail) */

SDF_SymbolArguments SDF_setSymbolArgumentsTail(SDF_SymbolArguments arg, SDF_SymbolArguments tail)
{
  if (SDF_isSymbolArgumentsMany(arg)) {
    return (SDF_SymbolArguments)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 4);
  }

  ATabort("SymbolArguments has no Tail: %t\n", arg);
  return (SDF_SymbolArguments)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_LexModuleWord accessors */

/*{{{  ATbool SDF_isValidLexModuleWord(SDF_LexModuleWord arg) */

ATbool SDF_isValidLexModuleWord(SDF_LexModuleWord arg)
{
  if (SDF_isLexModuleWordWord(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexModuleWordWord(SDF_LexModuleWord arg) */

inline ATbool SDF_isLexModuleWordWord(SDF_LexModuleWord arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternLexModuleWordWord, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexModuleWordLetters(SDF_LexModuleWord arg) */

ATbool SDF_hasLexModuleWordLetters(SDF_LexModuleWord arg)
{
  if (SDF_isLexModuleWordWord(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char* SDF_getLexModuleWordLetters(SDF_LexModuleWord arg) */

char* SDF_getLexModuleWordLetters(SDF_LexModuleWord arg)
{
  
    return (char*)SDF_charsToString((ATerm)ATgetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), 1));
}

/*}}}  */
/*{{{  SDF_LexModuleWord SDF_setLexModuleWordLetters(SDF_LexModuleWord arg, const char* letters) */

SDF_LexModuleWord SDF_setLexModuleWordLetters(SDF_LexModuleWord arg, const char* letters)
{
  if (SDF_isLexModuleWordWord(arg)) {
    return (SDF_LexModuleWord)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), (ATerm)((ATerm) ((ATerm) SDF_stringToChars(letters))), 1), 0), 1);
  }

  ATabort("LexModuleWord has no Letters: %t\n", arg);
  return (SDF_LexModuleWord)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_ModuleWord accessors */

/*{{{  ATbool SDF_isValidModuleWord(SDF_ModuleWord arg) */

ATbool SDF_isValidModuleWord(SDF_ModuleWord arg)
{
  if (SDF_isModuleWordLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isModuleWordLexToCf(SDF_ModuleWord arg) */

inline ATbool SDF_isModuleWordLexToCf(SDF_ModuleWord arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternModuleWordLexToCf, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleWordModuleWord(SDF_ModuleWord arg) */

ATbool SDF_hasModuleWordModuleWord(SDF_ModuleWord arg)
{
  if (SDF_isModuleWordLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_LexModuleWord SDF_getModuleWordModuleWord(SDF_ModuleWord arg) */

SDF_LexModuleWord SDF_getModuleWordModuleWord(SDF_ModuleWord arg)
{
  
    return (SDF_LexModuleWord)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_ModuleWord SDF_setModuleWordModuleWord(SDF_ModuleWord arg, SDF_LexModuleWord ModuleWord) */

SDF_ModuleWord SDF_setModuleWordModuleWord(SDF_ModuleWord arg, SDF_LexModuleWord ModuleWord)
{
  if (SDF_isModuleWordLexToCf(arg)) {
    return (SDF_ModuleWord)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ModuleWord), 0), 1);
  }

  ATabort("ModuleWord has no ModuleWord: %t\n", arg);
  return (SDF_ModuleWord)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_LexModuleId accessors */

/*{{{  ATbool SDF_isValidLexModuleId(SDF_LexModuleId arg) */

ATbool SDF_isValidLexModuleId(SDF_LexModuleId arg)
{
  if (SDF_isLexModuleIdLeaf(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexModuleIdRoot(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexModuleIdPath(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexModuleIdLeaf(SDF_LexModuleId arg) */

inline ATbool SDF_isLexModuleIdLeaf(SDF_LexModuleId arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexModuleIdLeaf, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexModuleIdRoot(SDF_LexModuleId arg) */

inline ATbool SDF_isLexModuleIdRoot(SDF_LexModuleId arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexModuleIdRoot, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexModuleIdPath(SDF_LexModuleId arg) */

inline ATbool SDF_isLexModuleIdPath(SDF_LexModuleId arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexModuleIdPath, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasLexModuleIdModuleWord(SDF_LexModuleId arg) */

ATbool SDF_hasLexModuleIdModuleWord(SDF_LexModuleId arg)
{
  if (SDF_isLexModuleIdLeaf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexModuleIdBasename(SDF_LexModuleId arg) */

ATbool SDF_hasLexModuleIdBasename(SDF_LexModuleId arg)
{
  if (SDF_isLexModuleIdRoot(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexModuleIdPath(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexModuleIdDirname(SDF_LexModuleId arg) */

ATbool SDF_hasLexModuleIdDirname(SDF_LexModuleId arg)
{
  if (SDF_isLexModuleIdPath(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_LexModuleWord SDF_getLexModuleIdModuleWord(SDF_LexModuleId arg) */

SDF_LexModuleWord SDF_getLexModuleIdModuleWord(SDF_LexModuleId arg)
{
  
    return (SDF_LexModuleWord)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_LexModuleId SDF_getLexModuleIdBasename(SDF_LexModuleId arg) */

SDF_LexModuleId SDF_getLexModuleIdBasename(SDF_LexModuleId arg)
{
  if (SDF_isLexModuleIdRoot(arg)) {
    return (SDF_LexModuleId)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else 
    return (SDF_LexModuleId)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_LexModuleWord SDF_getLexModuleIdDirname(SDF_LexModuleId arg) */

SDF_LexModuleWord SDF_getLexModuleIdDirname(SDF_LexModuleId arg)
{
  
    return (SDF_LexModuleWord)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_LexModuleId SDF_setLexModuleIdModuleWord(SDF_LexModuleId arg, SDF_LexModuleWord ModuleWord) */

SDF_LexModuleId SDF_setLexModuleIdModuleWord(SDF_LexModuleId arg, SDF_LexModuleWord ModuleWord)
{
  if (SDF_isLexModuleIdLeaf(arg)) {
    return (SDF_LexModuleId)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ModuleWord), 0), 1);
  }

  ATabort("LexModuleId has no ModuleWord: %t\n", arg);
  return (SDF_LexModuleId)NULL;
}

/*}}}  */
/*{{{  SDF_LexModuleId SDF_setLexModuleIdBasename(SDF_LexModuleId arg, SDF_LexModuleId basename) */

SDF_LexModuleId SDF_setLexModuleIdBasename(SDF_LexModuleId arg, SDF_LexModuleId basename)
{
  if (SDF_isLexModuleIdRoot(arg)) {
    return (SDF_LexModuleId)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) basename), 1), 1);
  }
  else if (SDF_isLexModuleIdPath(arg)) {
    return (SDF_LexModuleId)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) basename), 2), 1);
  }

  ATabort("LexModuleId has no Basename: %t\n", arg);
  return (SDF_LexModuleId)NULL;
}

/*}}}  */
/*{{{  SDF_LexModuleId SDF_setLexModuleIdDirname(SDF_LexModuleId arg, SDF_LexModuleWord dirname) */

SDF_LexModuleId SDF_setLexModuleIdDirname(SDF_LexModuleId arg, SDF_LexModuleWord dirname)
{
  if (SDF_isLexModuleIdPath(arg)) {
    return (SDF_LexModuleId)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) dirname), 0), 1);
  }

  ATabort("LexModuleId has no Dirname: %t\n", arg);
  return (SDF_LexModuleId)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_ModuleId accessors */

/*{{{  ATbool SDF_isValidModuleId(SDF_ModuleId arg) */

ATbool SDF_isValidModuleId(SDF_ModuleId arg)
{
  if (SDF_isModuleIdLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isModuleIdLexToCf(SDF_ModuleId arg) */

inline ATbool SDF_isModuleIdLexToCf(SDF_ModuleId arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternModuleIdLexToCf, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleIdModuleId(SDF_ModuleId arg) */

ATbool SDF_hasModuleIdModuleId(SDF_ModuleId arg)
{
  if (SDF_isModuleIdLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_LexModuleId SDF_getModuleIdModuleId(SDF_ModuleId arg) */

SDF_LexModuleId SDF_getModuleIdModuleId(SDF_ModuleId arg)
{
  
    return (SDF_LexModuleId)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_ModuleId SDF_setModuleIdModuleId(SDF_ModuleId arg, SDF_LexModuleId ModuleId) */

SDF_ModuleId SDF_setModuleIdModuleId(SDF_ModuleId arg, SDF_LexModuleId ModuleId)
{
  if (SDF_isModuleIdLexToCf(arg)) {
    return (SDF_ModuleId)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ModuleId), 0), 1);
  }

  ATabort("ModuleId has no ModuleId: %t\n", arg);
  return (SDF_ModuleId)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Definition accessors */

/*{{{  ATbool SDF_isValidDefinition(SDF_Definition arg) */

ATbool SDF_isValidDefinition(SDF_Definition arg)
{
  if (SDF_isDefinitionDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isDefinitionDefault(SDF_Definition arg) */

inline ATbool SDF_isDefinitionDefault(SDF_Definition arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternDefinitionDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasDefinitionList(SDF_Definition arg) */

ATbool SDF_hasDefinitionList(SDF_Definition arg)
{
  if (SDF_isDefinitionDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_ModuleList SDF_getDefinitionList(SDF_Definition arg) */

SDF_ModuleList SDF_getDefinitionList(SDF_Definition arg)
{
  
    return (SDF_ModuleList)ATgetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), 1);
}

/*}}}  */
/*{{{  SDF_Definition SDF_setDefinitionList(SDF_Definition arg, SDF_ModuleList list) */

SDF_Definition SDF_setDefinitionList(SDF_Definition arg, SDF_ModuleList list)
{
  if (SDF_isDefinitionDefault(arg)) {
    return (SDF_Definition)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), (ATerm)((ATerm) list), 1), 0), 1);
  }

  ATabort("Definition has no List: %t\n", arg);
  return (SDF_Definition)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Module accessors */

/*{{{  ATbool SDF_isValidModule(SDF_Module arg) */

ATbool SDF_isValidModule(SDF_Module arg)
{
  if (SDF_isModuleModule(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isModuleModule(SDF_Module arg) */

inline ATbool SDF_isModuleModule(SDF_Module arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternModuleModule, NULL, NULL, NULL, NULL, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleWsAfterModule(SDF_Module arg) */

ATbool SDF_hasModuleWsAfterModule(SDF_Module arg)
{
  if (SDF_isModuleModule(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleModuleName(SDF_Module arg) */

ATbool SDF_hasModuleModuleName(SDF_Module arg)
{
  if (SDF_isModuleModule(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleWsAfterModuleName(SDF_Module arg) */

ATbool SDF_hasModuleWsAfterModuleName(SDF_Module arg)
{
  if (SDF_isModuleModule(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleList(SDF_Module arg) */

ATbool SDF_hasModuleList(SDF_Module arg)
{
  if (SDF_isModuleModule(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleWsAfterList(SDF_Module arg) */

ATbool SDF_hasModuleWsAfterList(SDF_Module arg)
{
  if (SDF_isModuleModule(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleSections(SDF_Module arg) */

ATbool SDF_hasModuleSections(SDF_Module arg)
{
  if (SDF_isModuleModule(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getModuleWsAfterModule(SDF_Module arg) */

SDF_OptLayout SDF_getModuleWsAfterModule(SDF_Module arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_ModuleName SDF_getModuleModuleName(SDF_Module arg) */

SDF_ModuleName SDF_getModuleModuleName(SDF_Module arg)
{
  
    return (SDF_ModuleName)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getModuleWsAfterModuleName(SDF_Module arg) */

SDF_OptLayout SDF_getModuleWsAfterModuleName(SDF_Module arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_ImpSectionList SDF_getModuleList(SDF_Module arg) */

SDF_ImpSectionList SDF_getModuleList(SDF_Module arg)
{
  
    return (SDF_ImpSectionList)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getModuleWsAfterList(SDF_Module arg) */

SDF_OptLayout SDF_getModuleWsAfterList(SDF_Module arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_Sections SDF_getModuleSections(SDF_Module arg) */

SDF_Sections SDF_getModuleSections(SDF_Module arg)
{
  
    return (SDF_Sections)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 6);
}

/*}}}  */
/*{{{  SDF_Module SDF_setModuleWsAfterModule(SDF_Module arg, SDF_OptLayout wsAfterModule) */

SDF_Module SDF_setModuleWsAfterModule(SDF_Module arg, SDF_OptLayout wsAfterModule)
{
  if (SDF_isModuleModule(arg)) {
    return (SDF_Module)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterModule), 1), 1);
  }

  ATabort("Module has no WsAfterModule: %t\n", arg);
  return (SDF_Module)NULL;
}

/*}}}  */
/*{{{  SDF_Module SDF_setModuleModuleName(SDF_Module arg, SDF_ModuleName ModuleName) */

SDF_Module SDF_setModuleModuleName(SDF_Module arg, SDF_ModuleName ModuleName)
{
  if (SDF_isModuleModule(arg)) {
    return (SDF_Module)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ModuleName), 2), 1);
  }

  ATabort("Module has no ModuleName: %t\n", arg);
  return (SDF_Module)NULL;
}

/*}}}  */
/*{{{  SDF_Module SDF_setModuleWsAfterModuleName(SDF_Module arg, SDF_OptLayout wsAfterModuleName) */

SDF_Module SDF_setModuleWsAfterModuleName(SDF_Module arg, SDF_OptLayout wsAfterModuleName)
{
  if (SDF_isModuleModule(arg)) {
    return (SDF_Module)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterModuleName), 3), 1);
  }

  ATabort("Module has no WsAfterModuleName: %t\n", arg);
  return (SDF_Module)NULL;
}

/*}}}  */
/*{{{  SDF_Module SDF_setModuleList(SDF_Module arg, SDF_ImpSectionList list) */

SDF_Module SDF_setModuleList(SDF_Module arg, SDF_ImpSectionList list)
{
  if (SDF_isModuleModule(arg)) {
    return (SDF_Module)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4), (ATerm)((ATerm) list), 1), 4), 1);
  }

  ATabort("Module has no List: %t\n", arg);
  return (SDF_Module)NULL;
}

/*}}}  */
/*{{{  SDF_Module SDF_setModuleWsAfterList(SDF_Module arg, SDF_OptLayout wsAfterList) */

SDF_Module SDF_setModuleWsAfterList(SDF_Module arg, SDF_OptLayout wsAfterList)
{
  if (SDF_isModuleModule(arg)) {
    return (SDF_Module)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterList), 5), 1);
  }

  ATabort("Module has no WsAfterList: %t\n", arg);
  return (SDF_Module)NULL;
}

/*}}}  */
/*{{{  SDF_Module SDF_setModuleSections(SDF_Module arg, SDF_Sections Sections) */

SDF_Module SDF_setModuleSections(SDF_Module arg, SDF_Sections Sections)
{
  if (SDF_isModuleModule(arg)) {
    return (SDF_Module)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Sections), 6), 1);
  }

  ATabort("Module has no Sections: %t\n", arg);
  return (SDF_Module)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Section accessors */

/*{{{  ATbool SDF_isValidSection(SDF_Section arg) */

ATbool SDF_isValidSection(SDF_Section arg)
{
  if (SDF_isSectionExports(arg)) {
    return ATtrue;
  }
  else if (SDF_isSectionHiddens(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSectionExports(SDF_Section arg) */

inline ATbool SDF_isSectionExports(SDF_Section arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSectionExports, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSectionHiddens(SDF_Section arg) */

inline ATbool SDF_isSectionHiddens(SDF_Section arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSectionHiddens, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasSectionWsAfterExports(SDF_Section arg) */

ATbool SDF_hasSectionWsAfterExports(SDF_Section arg)
{
  if (SDF_isSectionExports(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSectionGrammar(SDF_Section arg) */

ATbool SDF_hasSectionGrammar(SDF_Section arg)
{
  if (SDF_isSectionExports(arg)) {
    return ATtrue;
  }
  else if (SDF_isSectionHiddens(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSectionWsAfterHiddens(SDF_Section arg) */

ATbool SDF_hasSectionWsAfterHiddens(SDF_Section arg)
{
  if (SDF_isSectionHiddens(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSectionWsAfterExports(SDF_Section arg) */

SDF_OptLayout SDF_getSectionWsAfterExports(SDF_Section arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Grammar SDF_getSectionGrammar(SDF_Section arg) */

SDF_Grammar SDF_getSectionGrammar(SDF_Section arg)
{
  if (SDF_isSectionExports(arg)) {
    return (SDF_Grammar)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else 
    return (SDF_Grammar)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSectionWsAfterHiddens(SDF_Section arg) */

SDF_OptLayout SDF_getSectionWsAfterHiddens(SDF_Section arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Section SDF_setSectionWsAfterExports(SDF_Section arg, SDF_OptLayout wsAfterExports) */

SDF_Section SDF_setSectionWsAfterExports(SDF_Section arg, SDF_OptLayout wsAfterExports)
{
  if (SDF_isSectionExports(arg)) {
    return (SDF_Section)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterExports), 1), 1);
  }

  ATabort("Section has no WsAfterExports: %t\n", arg);
  return (SDF_Section)NULL;
}

/*}}}  */
/*{{{  SDF_Section SDF_setSectionGrammar(SDF_Section arg, SDF_Grammar Grammar) */

SDF_Section SDF_setSectionGrammar(SDF_Section arg, SDF_Grammar Grammar)
{
  if (SDF_isSectionExports(arg)) {
    return (SDF_Section)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Grammar), 2), 1);
  }
  else if (SDF_isSectionHiddens(arg)) {
    return (SDF_Section)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Grammar), 2), 1);
  }

  ATabort("Section has no Grammar: %t\n", arg);
  return (SDF_Section)NULL;
}

/*}}}  */
/*{{{  SDF_Section SDF_setSectionWsAfterHiddens(SDF_Section arg, SDF_OptLayout wsAfterHiddens) */

SDF_Section SDF_setSectionWsAfterHiddens(SDF_Section arg, SDF_OptLayout wsAfterHiddens)
{
  if (SDF_isSectionHiddens(arg)) {
    return (SDF_Section)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterHiddens), 1), 1);
  }

  ATabort("Section has no WsAfterHiddens: %t\n", arg);
  return (SDF_Section)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Sections accessors */

/*{{{  ATbool SDF_isValidSections(SDF_Sections arg) */

ATbool SDF_isValidSections(SDF_Sections arg)
{
  if (SDF_isSectionsDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSectionsDefault(SDF_Sections arg) */

inline ATbool SDF_isSectionsDefault(SDF_Sections arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternSectionsDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasSectionsList(SDF_Sections arg) */

ATbool SDF_hasSectionsList(SDF_Sections arg)
{
  if (SDF_isSectionsDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_SectionList SDF_getSectionsList(SDF_Sections arg) */

SDF_SectionList SDF_getSectionsList(SDF_Sections arg)
{
  
    return (SDF_SectionList)ATgetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), 1);
}

/*}}}  */
/*{{{  SDF_Sections SDF_setSectionsList(SDF_Sections arg, SDF_SectionList list) */

SDF_Sections SDF_setSectionsList(SDF_Sections arg, SDF_SectionList list)
{
  if (SDF_isSectionsDefault(arg)) {
    return (SDF_Sections)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), (ATerm)((ATerm) list), 1), 0), 1);
  }

  ATabort("Sections has no List: %t\n", arg);
  return (SDF_Sections)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_ModuleName accessors */

/*{{{  ATbool SDF_isValidModuleName(SDF_ModuleName arg) */

ATbool SDF_isValidModuleName(SDF_ModuleName arg)
{
  if (SDF_isModuleNameUnparameterized(arg)) {
    return ATtrue;
  }
  else if (SDF_isModuleNameParameterized(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isModuleNameUnparameterized(SDF_ModuleName arg) */

inline ATbool SDF_isModuleNameUnparameterized(SDF_ModuleName arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternModuleNameUnparameterized, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isModuleNameParameterized(SDF_ModuleName arg) */

inline ATbool SDF_isModuleNameParameterized(SDF_ModuleName arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternModuleNameParameterized, NULL, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleNameModuleId(SDF_ModuleName arg) */

ATbool SDF_hasModuleNameModuleId(SDF_ModuleName arg)
{
  if (SDF_isModuleNameUnparameterized(arg)) {
    return ATtrue;
  }
  else if (SDF_isModuleNameParameterized(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleNameWsAfterModuleId(SDF_ModuleName arg) */

ATbool SDF_hasModuleNameWsAfterModuleId(SDF_ModuleName arg)
{
  if (SDF_isModuleNameParameterized(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleNameWsAfterBracketOpen(SDF_ModuleName arg) */

ATbool SDF_hasModuleNameWsAfterBracketOpen(SDF_ModuleName arg)
{
  if (SDF_isModuleNameParameterized(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleNameParams(SDF_ModuleName arg) */

ATbool SDF_hasModuleNameParams(SDF_ModuleName arg)
{
  if (SDF_isModuleNameParameterized(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleNameWsAfterParams(SDF_ModuleName arg) */

ATbool SDF_hasModuleNameWsAfterParams(SDF_ModuleName arg)
{
  if (SDF_isModuleNameParameterized(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_ModuleId SDF_getModuleNameModuleId(SDF_ModuleName arg) */

SDF_ModuleId SDF_getModuleNameModuleId(SDF_ModuleName arg)
{
  if (SDF_isModuleNameUnparameterized(arg)) {
    return (SDF_ModuleId)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
  }
  else 
    return (SDF_ModuleId)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getModuleNameWsAfterModuleId(SDF_ModuleName arg) */

SDF_OptLayout SDF_getModuleNameWsAfterModuleId(SDF_ModuleName arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getModuleNameWsAfterBracketOpen(SDF_ModuleName arg) */

SDF_OptLayout SDF_getModuleNameWsAfterBracketOpen(SDF_ModuleName arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Symbols SDF_getModuleNameParams(SDF_ModuleName arg) */

SDF_Symbols SDF_getModuleNameParams(SDF_ModuleName arg)
{
  
    return (SDF_Symbols)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getModuleNameWsAfterParams(SDF_ModuleName arg) */

SDF_OptLayout SDF_getModuleNameWsAfterParams(SDF_ModuleName arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_ModuleName SDF_setModuleNameModuleId(SDF_ModuleName arg, SDF_ModuleId ModuleId) */

SDF_ModuleName SDF_setModuleNameModuleId(SDF_ModuleName arg, SDF_ModuleId ModuleId)
{
  if (SDF_isModuleNameUnparameterized(arg)) {
    return (SDF_ModuleName)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ModuleId), 0), 1);
  }
  else if (SDF_isModuleNameParameterized(arg)) {
    return (SDF_ModuleName)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ModuleId), 0), 1);
  }

  ATabort("ModuleName has no ModuleId: %t\n", arg);
  return (SDF_ModuleName)NULL;
}

/*}}}  */
/*{{{  SDF_ModuleName SDF_setModuleNameWsAfterModuleId(SDF_ModuleName arg, SDF_OptLayout wsAfterModuleId) */

SDF_ModuleName SDF_setModuleNameWsAfterModuleId(SDF_ModuleName arg, SDF_OptLayout wsAfterModuleId)
{
  if (SDF_isModuleNameParameterized(arg)) {
    return (SDF_ModuleName)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterModuleId), 1), 1);
  }

  ATabort("ModuleName has no WsAfterModuleId: %t\n", arg);
  return (SDF_ModuleName)NULL;
}

/*}}}  */
/*{{{  SDF_ModuleName SDF_setModuleNameWsAfterBracketOpen(SDF_ModuleName arg, SDF_OptLayout wsAfterBracketOpen) */

SDF_ModuleName SDF_setModuleNameWsAfterBracketOpen(SDF_ModuleName arg, SDF_OptLayout wsAfterBracketOpen)
{
  if (SDF_isModuleNameParameterized(arg)) {
    return (SDF_ModuleName)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBracketOpen), 3), 1);
  }

  ATabort("ModuleName has no WsAfterBracketOpen: %t\n", arg);
  return (SDF_ModuleName)NULL;
}

/*}}}  */
/*{{{  SDF_ModuleName SDF_setModuleNameParams(SDF_ModuleName arg, SDF_Symbols params) */

SDF_ModuleName SDF_setModuleNameParams(SDF_ModuleName arg, SDF_Symbols params)
{
  if (SDF_isModuleNameParameterized(arg)) {
    return (SDF_ModuleName)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) params), 4), 1);
  }

  ATabort("ModuleName has no Params: %t\n", arg);
  return (SDF_ModuleName)NULL;
}

/*}}}  */
/*{{{  SDF_ModuleName SDF_setModuleNameWsAfterParams(SDF_ModuleName arg, SDF_OptLayout wsAfterParams) */

SDF_ModuleName SDF_setModuleNameWsAfterParams(SDF_ModuleName arg, SDF_OptLayout wsAfterParams)
{
  if (SDF_isModuleNameParameterized(arg)) {
    return (SDF_ModuleName)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParams), 5), 1);
  }

  ATabort("ModuleName has no WsAfterParams: %t\n", arg);
  return (SDF_ModuleName)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_ImpSection accessors */

/*{{{  ATbool SDF_isValidImpSection(SDF_ImpSection arg) */

ATbool SDF_isValidImpSection(SDF_ImpSection arg)
{
  if (SDF_isImpSectionImports(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isImpSectionImports(SDF_ImpSection arg) */

inline ATbool SDF_isImpSectionImports(SDF_ImpSection arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternImpSectionImports, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasImpSectionWsAfterImports(SDF_ImpSection arg) */

ATbool SDF_hasImpSectionWsAfterImports(SDF_ImpSection arg)
{
  if (SDF_isImpSectionImports(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasImpSectionList(SDF_ImpSection arg) */

ATbool SDF_hasImpSectionList(SDF_ImpSection arg)
{
  if (SDF_isImpSectionImports(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getImpSectionWsAfterImports(SDF_ImpSection arg) */

SDF_OptLayout SDF_getImpSectionWsAfterImports(SDF_ImpSection arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Imports SDF_getImpSectionList(SDF_ImpSection arg) */

SDF_Imports SDF_getImpSectionList(SDF_ImpSection arg)
{
  
    return (SDF_Imports)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_ImpSection SDF_setImpSectionWsAfterImports(SDF_ImpSection arg, SDF_OptLayout wsAfterImports) */

SDF_ImpSection SDF_setImpSectionWsAfterImports(SDF_ImpSection arg, SDF_OptLayout wsAfterImports)
{
  if (SDF_isImpSectionImports(arg)) {
    return (SDF_ImpSection)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterImports), 1), 1);
  }

  ATabort("ImpSection has no WsAfterImports: %t\n", arg);
  return (SDF_ImpSection)NULL;
}

/*}}}  */
/*{{{  SDF_ImpSection SDF_setImpSectionList(SDF_ImpSection arg, SDF_Imports list) */

SDF_ImpSection SDF_setImpSectionList(SDF_ImpSection arg, SDF_Imports list)
{
  if (SDF_isImpSectionImports(arg)) {
    return (SDF_ImpSection)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) list), 2), 1);
  }

  ATabort("ImpSection has no List: %t\n", arg);
  return (SDF_ImpSection)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Imports accessors */

/*{{{  ATbool SDF_isValidImports(SDF_Imports arg) */

ATbool SDF_isValidImports(SDF_Imports arg)
{
  if (SDF_isImportsDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isImportsDefault(SDF_Imports arg) */

inline ATbool SDF_isImportsDefault(SDF_Imports arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternImportsDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasImportsList(SDF_Imports arg) */

ATbool SDF_hasImportsList(SDF_Imports arg)
{
  if (SDF_isImportsDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_ImportList SDF_getImportsList(SDF_Imports arg) */

SDF_ImportList SDF_getImportsList(SDF_Imports arg)
{
  
    return (SDF_ImportList)ATgetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), 1);
}

/*}}}  */
/*{{{  SDF_Imports SDF_setImportsList(SDF_Imports arg, SDF_ImportList list) */

SDF_Imports SDF_setImportsList(SDF_Imports arg, SDF_ImportList list)
{
  if (SDF_isImportsDefault(arg)) {
    return (SDF_Imports)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), (ATerm)((ATerm) list), 1), 0), 1);
  }

  ATabort("Imports has no List: %t\n", arg);
  return (SDF_Imports)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Import accessors */

/*{{{  ATbool SDF_isValidImport(SDF_Import arg) */

ATbool SDF_isValidImport(SDF_Import arg)
{
  if (SDF_isImportModule(arg)) {
    return ATtrue;
  }
  else if (SDF_isImportRenamedModule(arg)) {
    return ATtrue;
  }
  else if (SDF_isImportBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isImportModule(SDF_Import arg) */

inline ATbool SDF_isImportModule(SDF_Import arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternImportModule, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isImportRenamedModule(SDF_Import arg) */

inline ATbool SDF_isImportRenamedModule(SDF_Import arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternImportRenamedModule, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isImportBracket(SDF_Import arg) */

inline ATbool SDF_isImportBracket(SDF_Import arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternImportBracket, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasImportModuleName(SDF_Import arg) */

ATbool SDF_hasImportModuleName(SDF_Import arg)
{
  if (SDF_isImportModule(arg)) {
    return ATtrue;
  }
  else if (SDF_isImportRenamedModule(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasImportWsAfterModuleName(SDF_Import arg) */

ATbool SDF_hasImportWsAfterModuleName(SDF_Import arg)
{
  if (SDF_isImportRenamedModule(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasImportRenamings(SDF_Import arg) */

ATbool SDF_hasImportRenamings(SDF_Import arg)
{
  if (SDF_isImportRenamedModule(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasImportWsAfterParenOpen(SDF_Import arg) */

ATbool SDF_hasImportWsAfterParenOpen(SDF_Import arg)
{
  if (SDF_isImportBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasImportImport(SDF_Import arg) */

ATbool SDF_hasImportImport(SDF_Import arg)
{
  if (SDF_isImportBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasImportWsAfterImport(SDF_Import arg) */

ATbool SDF_hasImportWsAfterImport(SDF_Import arg)
{
  if (SDF_isImportBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_ModuleName SDF_getImportModuleName(SDF_Import arg) */

SDF_ModuleName SDF_getImportModuleName(SDF_Import arg)
{
  if (SDF_isImportModule(arg)) {
    return (SDF_ModuleName)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
  }
  else 
    return (SDF_ModuleName)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getImportWsAfterModuleName(SDF_Import arg) */

SDF_OptLayout SDF_getImportWsAfterModuleName(SDF_Import arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Renamings SDF_getImportRenamings(SDF_Import arg) */

SDF_Renamings SDF_getImportRenamings(SDF_Import arg)
{
  
    return (SDF_Renamings)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getImportWsAfterParenOpen(SDF_Import arg) */

SDF_OptLayout SDF_getImportWsAfterParenOpen(SDF_Import arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Import SDF_getImportImport(SDF_Import arg) */

SDF_Import SDF_getImportImport(SDF_Import arg)
{
  
    return (SDF_Import)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getImportWsAfterImport(SDF_Import arg) */

SDF_OptLayout SDF_getImportWsAfterImport(SDF_Import arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Import SDF_setImportModuleName(SDF_Import arg, SDF_ModuleName ModuleName) */

SDF_Import SDF_setImportModuleName(SDF_Import arg, SDF_ModuleName ModuleName)
{
  if (SDF_isImportModule(arg)) {
    return (SDF_Import)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ModuleName), 0), 1);
  }
  else if (SDF_isImportRenamedModule(arg)) {
    return (SDF_Import)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ModuleName), 0), 1);
  }

  ATabort("Import has no ModuleName: %t\n", arg);
  return (SDF_Import)NULL;
}

/*}}}  */
/*{{{  SDF_Import SDF_setImportWsAfterModuleName(SDF_Import arg, SDF_OptLayout wsAfterModuleName) */

SDF_Import SDF_setImportWsAfterModuleName(SDF_Import arg, SDF_OptLayout wsAfterModuleName)
{
  if (SDF_isImportRenamedModule(arg)) {
    return (SDF_Import)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterModuleName), 1), 1);
  }

  ATabort("Import has no WsAfterModuleName: %t\n", arg);
  return (SDF_Import)NULL;
}

/*}}}  */
/*{{{  SDF_Import SDF_setImportRenamings(SDF_Import arg, SDF_Renamings Renamings) */

SDF_Import SDF_setImportRenamings(SDF_Import arg, SDF_Renamings Renamings)
{
  if (SDF_isImportRenamedModule(arg)) {
    return (SDF_Import)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Renamings), 2), 1);
  }

  ATabort("Import has no Renamings: %t\n", arg);
  return (SDF_Import)NULL;
}

/*}}}  */
/*{{{  SDF_Import SDF_setImportWsAfterParenOpen(SDF_Import arg, SDF_OptLayout wsAfterParenOpen) */

SDF_Import SDF_setImportWsAfterParenOpen(SDF_Import arg, SDF_OptLayout wsAfterParenOpen)
{
  if (SDF_isImportBracket(arg)) {
    return (SDF_Import)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenOpen), 1), 1);
  }

  ATabort("Import has no WsAfterParenOpen: %t\n", arg);
  return (SDF_Import)NULL;
}

/*}}}  */
/*{{{  SDF_Import SDF_setImportImport(SDF_Import arg, SDF_Import Import) */

SDF_Import SDF_setImportImport(SDF_Import arg, SDF_Import Import)
{
  if (SDF_isImportBracket(arg)) {
    return (SDF_Import)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Import), 2), 1);
  }

  ATabort("Import has no Import: %t\n", arg);
  return (SDF_Import)NULL;
}

/*}}}  */
/*{{{  SDF_Import SDF_setImportWsAfterImport(SDF_Import arg, SDF_OptLayout wsAfterImport) */

SDF_Import SDF_setImportWsAfterImport(SDF_Import arg, SDF_OptLayout wsAfterImport)
{
  if (SDF_isImportBracket(arg)) {
    return (SDF_Import)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterImport), 3), 1);
  }

  ATabort("Import has no WsAfterImport: %t\n", arg);
  return (SDF_Import)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_ModuleList accessors */

/*{{{  ATbool SDF_isValidModuleList(SDF_ModuleList arg) */

ATbool SDF_isValidModuleList(SDF_ModuleList arg)
{
  if (SDF_isModuleListEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isModuleListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isModuleListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isModuleListEmpty(SDF_ModuleList arg) */

inline ATbool SDF_isModuleListEmpty(SDF_ModuleList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternModuleListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isModuleListSingle(SDF_ModuleList arg) */

inline ATbool SDF_isModuleListSingle(SDF_ModuleList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternModuleListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isModuleListMany(SDF_ModuleList arg) */

inline ATbool SDF_isModuleListMany(SDF_ModuleList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternModuleListMany, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleListHead(SDF_ModuleList arg) */

ATbool SDF_hasModuleListHead(SDF_ModuleList arg)
{
  if (SDF_isModuleListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isModuleListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleListWsAfterHead(SDF_ModuleList arg) */

ATbool SDF_hasModuleListWsAfterHead(SDF_ModuleList arg)
{
  if (SDF_isModuleListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasModuleListTail(SDF_ModuleList arg) */

ATbool SDF_hasModuleListTail(SDF_ModuleList arg)
{
  if (SDF_isModuleListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_ModuleList SDF_getModuleListTail(SDF_ModuleList arg) */

SDF_ModuleList SDF_getModuleListTail(SDF_ModuleList arg)
{
  assert(!SDF_isModuleListEmpty(arg) && "getTail on an empty list");
  if (SDF_isModuleListSingle(arg)) {
    return (SDF_ModuleList) SDF_makeModuleListEmpty();
  }
  else {
  
    return (SDF_ModuleList)ATgetTail((ATermList)arg, 2);
  }
}

/*}}}  */
/*{{{  SDF_Module SDF_getModuleListHead(SDF_ModuleList arg) */

SDF_Module SDF_getModuleListHead(SDF_ModuleList arg)
{
  if (SDF_isModuleListSingle(arg)) {
    return (SDF_Module)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Module)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getModuleListWsAfterHead(SDF_ModuleList arg) */

SDF_OptLayout SDF_getModuleListWsAfterHead(SDF_ModuleList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_ModuleList SDF_setModuleListHead(SDF_ModuleList arg, SDF_Module head) */

SDF_ModuleList SDF_setModuleListHead(SDF_ModuleList arg, SDF_Module head)
{
  if (SDF_isModuleListSingle(arg)) {
    return (SDF_ModuleList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isModuleListMany(arg)) {
    return (SDF_ModuleList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("ModuleList has no Head: %t\n", arg);
  return (SDF_ModuleList)NULL;
}

/*}}}  */
/*{{{  SDF_ModuleList SDF_setModuleListWsAfterHead(SDF_ModuleList arg, SDF_OptLayout wsAfterHead) */

SDF_ModuleList SDF_setModuleListWsAfterHead(SDF_ModuleList arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isModuleListMany(arg)) {
    return (SDF_ModuleList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("ModuleList has no WsAfterHead: %t\n", arg);
  return (SDF_ModuleList)NULL;
}

/*}}}  */
/*{{{  SDF_ModuleList SDF_setModuleListTail(SDF_ModuleList arg, SDF_ModuleList tail) */

SDF_ModuleList SDF_setModuleListTail(SDF_ModuleList arg, SDF_ModuleList tail)
{
  if (SDF_isModuleListMany(arg)) {
    return (SDF_ModuleList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 2);
  }

  ATabort("ModuleList has no Tail: %t\n", arg);
  return (SDF_ModuleList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_ImpSectionList accessors */

/*{{{  ATbool SDF_isValidImpSectionList(SDF_ImpSectionList arg) */

ATbool SDF_isValidImpSectionList(SDF_ImpSectionList arg)
{
  if (SDF_isImpSectionListEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isImpSectionListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isImpSectionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isImpSectionListEmpty(SDF_ImpSectionList arg) */

inline ATbool SDF_isImpSectionListEmpty(SDF_ImpSectionList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternImpSectionListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isImpSectionListSingle(SDF_ImpSectionList arg) */

inline ATbool SDF_isImpSectionListSingle(SDF_ImpSectionList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternImpSectionListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isImpSectionListMany(SDF_ImpSectionList arg) */

inline ATbool SDF_isImpSectionListMany(SDF_ImpSectionList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternImpSectionListMany, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasImpSectionListHead(SDF_ImpSectionList arg) */

ATbool SDF_hasImpSectionListHead(SDF_ImpSectionList arg)
{
  if (SDF_isImpSectionListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isImpSectionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasImpSectionListWsAfterHead(SDF_ImpSectionList arg) */

ATbool SDF_hasImpSectionListWsAfterHead(SDF_ImpSectionList arg)
{
  if (SDF_isImpSectionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasImpSectionListTail(SDF_ImpSectionList arg) */

ATbool SDF_hasImpSectionListTail(SDF_ImpSectionList arg)
{
  if (SDF_isImpSectionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_ImpSectionList SDF_getImpSectionListTail(SDF_ImpSectionList arg) */

SDF_ImpSectionList SDF_getImpSectionListTail(SDF_ImpSectionList arg)
{
  assert(!SDF_isImpSectionListEmpty(arg) && "getTail on an empty list");
  if (SDF_isImpSectionListSingle(arg)) {
    return (SDF_ImpSectionList) SDF_makeImpSectionListEmpty();
  }
  else {
  
    return (SDF_ImpSectionList)ATgetTail((ATermList)arg, 2);
  }
}

/*}}}  */
/*{{{  SDF_ImpSection SDF_getImpSectionListHead(SDF_ImpSectionList arg) */

SDF_ImpSection SDF_getImpSectionListHead(SDF_ImpSectionList arg)
{
  if (SDF_isImpSectionListSingle(arg)) {
    return (SDF_ImpSection)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_ImpSection)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getImpSectionListWsAfterHead(SDF_ImpSectionList arg) */

SDF_OptLayout SDF_getImpSectionListWsAfterHead(SDF_ImpSectionList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_ImpSectionList SDF_setImpSectionListHead(SDF_ImpSectionList arg, SDF_ImpSection head) */

SDF_ImpSectionList SDF_setImpSectionListHead(SDF_ImpSectionList arg, SDF_ImpSection head)
{
  if (SDF_isImpSectionListSingle(arg)) {
    return (SDF_ImpSectionList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isImpSectionListMany(arg)) {
    return (SDF_ImpSectionList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("ImpSectionList has no Head: %t\n", arg);
  return (SDF_ImpSectionList)NULL;
}

/*}}}  */
/*{{{  SDF_ImpSectionList SDF_setImpSectionListWsAfterHead(SDF_ImpSectionList arg, SDF_OptLayout wsAfterHead) */

SDF_ImpSectionList SDF_setImpSectionListWsAfterHead(SDF_ImpSectionList arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isImpSectionListMany(arg)) {
    return (SDF_ImpSectionList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("ImpSectionList has no WsAfterHead: %t\n", arg);
  return (SDF_ImpSectionList)NULL;
}

/*}}}  */
/*{{{  SDF_ImpSectionList SDF_setImpSectionListTail(SDF_ImpSectionList arg, SDF_ImpSectionList tail) */

SDF_ImpSectionList SDF_setImpSectionListTail(SDF_ImpSectionList arg, SDF_ImpSectionList tail)
{
  if (SDF_isImpSectionListMany(arg)) {
    return (SDF_ImpSectionList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 2);
  }

  ATabort("ImpSectionList has no Tail: %t\n", arg);
  return (SDF_ImpSectionList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_SectionList accessors */

/*{{{  ATbool SDF_isValidSectionList(SDF_SectionList arg) */

ATbool SDF_isValidSectionList(SDF_SectionList arg)
{
  if (SDF_isSectionListEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isSectionListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isSectionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSectionListEmpty(SDF_SectionList arg) */

inline ATbool SDF_isSectionListEmpty(SDF_SectionList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternSectionListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSectionListSingle(SDF_SectionList arg) */

inline ATbool SDF_isSectionListSingle(SDF_SectionList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSectionListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSectionListMany(SDF_SectionList arg) */

inline ATbool SDF_isSectionListMany(SDF_SectionList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSectionListMany, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasSectionListHead(SDF_SectionList arg) */

ATbool SDF_hasSectionListHead(SDF_SectionList arg)
{
  if (SDF_isSectionListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isSectionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSectionListWsAfterHead(SDF_SectionList arg) */

ATbool SDF_hasSectionListWsAfterHead(SDF_SectionList arg)
{
  if (SDF_isSectionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSectionListTail(SDF_SectionList arg) */

ATbool SDF_hasSectionListTail(SDF_SectionList arg)
{
  if (SDF_isSectionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_SectionList SDF_getSectionListTail(SDF_SectionList arg) */

SDF_SectionList SDF_getSectionListTail(SDF_SectionList arg)
{
  assert(!SDF_isSectionListEmpty(arg) && "getTail on an empty list");
  if (SDF_isSectionListSingle(arg)) {
    return (SDF_SectionList) SDF_makeSectionListEmpty();
  }
  else {
  
    return (SDF_SectionList)ATgetTail((ATermList)arg, 2);
  }
}

/*}}}  */
/*{{{  SDF_Section SDF_getSectionListHead(SDF_SectionList arg) */

SDF_Section SDF_getSectionListHead(SDF_SectionList arg)
{
  if (SDF_isSectionListSingle(arg)) {
    return (SDF_Section)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Section)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSectionListWsAfterHead(SDF_SectionList arg) */

SDF_OptLayout SDF_getSectionListWsAfterHead(SDF_SectionList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_SectionList SDF_setSectionListHead(SDF_SectionList arg, SDF_Section head) */

SDF_SectionList SDF_setSectionListHead(SDF_SectionList arg, SDF_Section head)
{
  if (SDF_isSectionListSingle(arg)) {
    return (SDF_SectionList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isSectionListMany(arg)) {
    return (SDF_SectionList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("SectionList has no Head: %t\n", arg);
  return (SDF_SectionList)NULL;
}

/*}}}  */
/*{{{  SDF_SectionList SDF_setSectionListWsAfterHead(SDF_SectionList arg, SDF_OptLayout wsAfterHead) */

SDF_SectionList SDF_setSectionListWsAfterHead(SDF_SectionList arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isSectionListMany(arg)) {
    return (SDF_SectionList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("SectionList has no WsAfterHead: %t\n", arg);
  return (SDF_SectionList)NULL;
}

/*}}}  */
/*{{{  SDF_SectionList SDF_setSectionListTail(SDF_SectionList arg, SDF_SectionList tail) */

SDF_SectionList SDF_setSectionListTail(SDF_SectionList arg, SDF_SectionList tail)
{
  if (SDF_isSectionListMany(arg)) {
    return (SDF_SectionList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 2);
  }

  ATabort("SectionList has no Tail: %t\n", arg);
  return (SDF_SectionList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_ImportList accessors */

/*{{{  ATbool SDF_isValidImportList(SDF_ImportList arg) */

ATbool SDF_isValidImportList(SDF_ImportList arg)
{
  if (SDF_isImportListEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isImportListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isImportListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isImportListEmpty(SDF_ImportList arg) */

inline ATbool SDF_isImportListEmpty(SDF_ImportList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternImportListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isImportListSingle(SDF_ImportList arg) */

inline ATbool SDF_isImportListSingle(SDF_ImportList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternImportListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isImportListMany(SDF_ImportList arg) */

inline ATbool SDF_isImportListMany(SDF_ImportList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternImportListMany, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasImportListHead(SDF_ImportList arg) */

ATbool SDF_hasImportListHead(SDF_ImportList arg)
{
  if (SDF_isImportListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isImportListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasImportListWsAfterHead(SDF_ImportList arg) */

ATbool SDF_hasImportListWsAfterHead(SDF_ImportList arg)
{
  if (SDF_isImportListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasImportListTail(SDF_ImportList arg) */

ATbool SDF_hasImportListTail(SDF_ImportList arg)
{
  if (SDF_isImportListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_ImportList SDF_getImportListTail(SDF_ImportList arg) */

SDF_ImportList SDF_getImportListTail(SDF_ImportList arg)
{
  assert(!SDF_isImportListEmpty(arg) && "getTail on an empty list");
  if (SDF_isImportListSingle(arg)) {
    return (SDF_ImportList) SDF_makeImportListEmpty();
  }
  else {
  
    return (SDF_ImportList)ATgetTail((ATermList)arg, 2);
  }
}

/*}}}  */
/*{{{  SDF_Import SDF_getImportListHead(SDF_ImportList arg) */

SDF_Import SDF_getImportListHead(SDF_ImportList arg)
{
  if (SDF_isImportListSingle(arg)) {
    return (SDF_Import)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Import)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getImportListWsAfterHead(SDF_ImportList arg) */

SDF_OptLayout SDF_getImportListWsAfterHead(SDF_ImportList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_ImportList SDF_setImportListHead(SDF_ImportList arg, SDF_Import head) */

SDF_ImportList SDF_setImportListHead(SDF_ImportList arg, SDF_Import head)
{
  if (SDF_isImportListSingle(arg)) {
    return (SDF_ImportList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isImportListMany(arg)) {
    return (SDF_ImportList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("ImportList has no Head: %t\n", arg);
  return (SDF_ImportList)NULL;
}

/*}}}  */
/*{{{  SDF_ImportList SDF_setImportListWsAfterHead(SDF_ImportList arg, SDF_OptLayout wsAfterHead) */

SDF_ImportList SDF_setImportListWsAfterHead(SDF_ImportList arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isImportListMany(arg)) {
    return (SDF_ImportList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("ImportList has no WsAfterHead: %t\n", arg);
  return (SDF_ImportList)NULL;
}

/*}}}  */
/*{{{  SDF_ImportList SDF_setImportListTail(SDF_ImportList arg, SDF_ImportList tail) */

SDF_ImportList SDF_setImportListTail(SDF_ImportList arg, SDF_ImportList tail)
{
  if (SDF_isImportListMany(arg)) {
    return (SDF_ImportList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 2);
  }

  ATabort("ImportList has no Tail: %t\n", arg);
  return (SDF_ImportList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Associativity accessors */

/*{{{  ATbool SDF_isValidAssociativity(SDF_Associativity arg) */

ATbool SDF_isValidAssociativity(SDF_Associativity arg)
{
  if (SDF_isAssociativityLeft(arg)) {
    return ATtrue;
  }
  else if (SDF_isAssociativityRight(arg)) {
    return ATtrue;
  }
  else if (SDF_isAssociativityNonAssoc(arg)) {
    return ATtrue;
  }
  else if (SDF_isAssociativityAssoc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isAssociativityLeft(SDF_Associativity arg) */

inline ATbool SDF_isAssociativityLeft(SDF_Associativity arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAssociativityLeft);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isAssociativityRight(SDF_Associativity arg) */

inline ATbool SDF_isAssociativityRight(SDF_Associativity arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAssociativityRight);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isAssociativityNonAssoc(SDF_Associativity arg) */

inline ATbool SDF_isAssociativityNonAssoc(SDF_Associativity arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAssociativityNonAssoc);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isAssociativityAssoc(SDF_Associativity arg) */

inline ATbool SDF_isAssociativityAssoc(SDF_Associativity arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternAssociativityAssoc);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */

/*}}}  */
/*{{{  SDF_ArgumentIndicator accessors */

/*{{{  ATbool SDF_isValidArgumentIndicator(SDF_ArgumentIndicator arg) */

ATbool SDF_isValidArgumentIndicator(SDF_ArgumentIndicator arg)
{
  if (SDF_isArgumentIndicatorDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isArgumentIndicatorDefault(SDF_ArgumentIndicator arg) */

inline ATbool SDF_isArgumentIndicatorDefault(SDF_ArgumentIndicator arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternArgumentIndicatorDefault, NULL, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasArgumentIndicatorWsAfterLessThan(SDF_ArgumentIndicator arg) */

ATbool SDF_hasArgumentIndicatorWsAfterLessThan(SDF_ArgumentIndicator arg)
{
  if (SDF_isArgumentIndicatorDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasArgumentIndicatorArguments(SDF_ArgumentIndicator arg) */

ATbool SDF_hasArgumentIndicatorArguments(SDF_ArgumentIndicator arg)
{
  if (SDF_isArgumentIndicatorDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasArgumentIndicatorWsAfterArguments(SDF_ArgumentIndicator arg) */

ATbool SDF_hasArgumentIndicatorWsAfterArguments(SDF_ArgumentIndicator arg)
{
  if (SDF_isArgumentIndicatorDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getArgumentIndicatorWsAfterLessThan(SDF_ArgumentIndicator arg) */

SDF_OptLayout SDF_getArgumentIndicatorWsAfterLessThan(SDF_ArgumentIndicator arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_NatConArguments SDF_getArgumentIndicatorArguments(SDF_ArgumentIndicator arg) */

SDF_NatConArguments SDF_getArgumentIndicatorArguments(SDF_ArgumentIndicator arg)
{
  
    return (SDF_NatConArguments)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getArgumentIndicatorWsAfterArguments(SDF_ArgumentIndicator arg) */

SDF_OptLayout SDF_getArgumentIndicatorWsAfterArguments(SDF_ArgumentIndicator arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_ArgumentIndicator SDF_setArgumentIndicatorWsAfterLessThan(SDF_ArgumentIndicator arg, SDF_OptLayout wsAfterLessThan) */

SDF_ArgumentIndicator SDF_setArgumentIndicatorWsAfterLessThan(SDF_ArgumentIndicator arg, SDF_OptLayout wsAfterLessThan)
{
  if (SDF_isArgumentIndicatorDefault(arg)) {
    return (SDF_ArgumentIndicator)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLessThan), 1), 1);
  }

  ATabort("ArgumentIndicator has no WsAfterLessThan: %t\n", arg);
  return (SDF_ArgumentIndicator)NULL;
}

/*}}}  */
/*{{{  SDF_ArgumentIndicator SDF_setArgumentIndicatorArguments(SDF_ArgumentIndicator arg, SDF_NatConArguments arguments) */

SDF_ArgumentIndicator SDF_setArgumentIndicatorArguments(SDF_ArgumentIndicator arg, SDF_NatConArguments arguments)
{
  if (SDF_isArgumentIndicatorDefault(arg)) {
    return (SDF_ArgumentIndicator)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), (ATerm)((ATerm) arguments), 1), 2), 1);
  }

  ATabort("ArgumentIndicator has no Arguments: %t\n", arg);
  return (SDF_ArgumentIndicator)NULL;
}

/*}}}  */
/*{{{  SDF_ArgumentIndicator SDF_setArgumentIndicatorWsAfterArguments(SDF_ArgumentIndicator arg, SDF_OptLayout wsAfterArguments) */

SDF_ArgumentIndicator SDF_setArgumentIndicatorWsAfterArguments(SDF_ArgumentIndicator arg, SDF_OptLayout wsAfterArguments)
{
  if (SDF_isArgumentIndicatorDefault(arg)) {
    return (SDF_ArgumentIndicator)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterArguments), 3), 1);
  }

  ATabort("ArgumentIndicator has no WsAfterArguments: %t\n", arg);
  return (SDF_ArgumentIndicator)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Group accessors */

/*{{{  ATbool SDF_isValidGroup(SDF_Group arg) */

ATbool SDF_isValidGroup(SDF_Group arg)
{
  if (SDF_isGroupWithArguments(arg)) {
    return ATtrue;
  }
  else if (SDF_isGroupSimpleGroup(arg)) {
    return ATtrue;
  }
  else if (SDF_isGroupProdsGroup(arg)) {
    return ATtrue;
  }
  else if (SDF_isGroupAssocGroup(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isGroupWithArguments(SDF_Group arg) */

inline ATbool SDF_isGroupWithArguments(SDF_Group arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGroupWithArguments, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGroupSimpleGroup(SDF_Group arg) */

inline ATbool SDF_isGroupSimpleGroup(SDF_Group arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGroupSimpleGroup, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGroupProdsGroup(SDF_Group arg) */

inline ATbool SDF_isGroupProdsGroup(SDF_Group arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGroupProdsGroup, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGroupAssocGroup(SDF_Group arg) */

inline ATbool SDF_isGroupAssocGroup(SDF_Group arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGroupAssocGroup, NULL, NULL, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasGroupGroup(SDF_Group arg) */

ATbool SDF_hasGroupGroup(SDF_Group arg)
{
  if (SDF_isGroupWithArguments(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGroupWsAfterGroup(SDF_Group arg) */

ATbool SDF_hasGroupWsAfterGroup(SDF_Group arg)
{
  if (SDF_isGroupWithArguments(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGroupArgumentIndicator(SDF_Group arg) */

ATbool SDF_hasGroupArgumentIndicator(SDF_Group arg)
{
  if (SDF_isGroupWithArguments(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGroupProduction(SDF_Group arg) */

ATbool SDF_hasGroupProduction(SDF_Group arg)
{
  if (SDF_isGroupSimpleGroup(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGroupWsAfterBraceOpen(SDF_Group arg) */

ATbool SDF_hasGroupWsAfterBraceOpen(SDF_Group arg)
{
  if (SDF_isGroupProdsGroup(arg)) {
    return ATtrue;
  }
  else if (SDF_isGroupAssocGroup(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGroupProductions(SDF_Group arg) */

ATbool SDF_hasGroupProductions(SDF_Group arg)
{
  if (SDF_isGroupProdsGroup(arg)) {
    return ATtrue;
  }
  else if (SDF_isGroupAssocGroup(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGroupWsAfterProductions(SDF_Group arg) */

ATbool SDF_hasGroupWsAfterProductions(SDF_Group arg)
{
  if (SDF_isGroupProdsGroup(arg)) {
    return ATtrue;
  }
  else if (SDF_isGroupAssocGroup(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGroupAssociativity(SDF_Group arg) */

ATbool SDF_hasGroupAssociativity(SDF_Group arg)
{
  if (SDF_isGroupAssocGroup(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGroupWsAfterAssociativity(SDF_Group arg) */

ATbool SDF_hasGroupWsAfterAssociativity(SDF_Group arg)
{
  if (SDF_isGroupAssocGroup(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGroupWsAfterColon(SDF_Group arg) */

ATbool SDF_hasGroupWsAfterColon(SDF_Group arg)
{
  if (SDF_isGroupAssocGroup(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_Group SDF_getGroupGroup(SDF_Group arg) */

SDF_Group SDF_getGroupGroup(SDF_Group arg)
{
  
    return (SDF_Group)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGroupWsAfterGroup(SDF_Group arg) */

SDF_OptLayout SDF_getGroupWsAfterGroup(SDF_Group arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_ArgumentIndicator SDF_getGroupArgumentIndicator(SDF_Group arg) */

SDF_ArgumentIndicator SDF_getGroupArgumentIndicator(SDF_Group arg)
{
  
    return (SDF_ArgumentIndicator)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_Production SDF_getGroupProduction(SDF_Group arg) */

SDF_Production SDF_getGroupProduction(SDF_Group arg)
{
  
    return (SDF_Production)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGroupWsAfterBraceOpen(SDF_Group arg) */

SDF_OptLayout SDF_getGroupWsAfterBraceOpen(SDF_Group arg)
{
  if (SDF_isGroupProdsGroup(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Productions SDF_getGroupProductions(SDF_Group arg) */

SDF_Productions SDF_getGroupProductions(SDF_Group arg)
{
  if (SDF_isGroupProdsGroup(arg)) {
    return (SDF_Productions)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
  }
  else 
    return (SDF_Productions)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 6);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGroupWsAfterProductions(SDF_Group arg) */

SDF_OptLayout SDF_getGroupWsAfterProductions(SDF_Group arg)
{
  if (SDF_isGroupProdsGroup(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 7);
}

/*}}}  */
/*{{{  SDF_Associativity SDF_getGroupAssociativity(SDF_Group arg) */

SDF_Associativity SDF_getGroupAssociativity(SDF_Group arg)
{
  
    return (SDF_Associativity)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGroupWsAfterAssociativity(SDF_Group arg) */

SDF_OptLayout SDF_getGroupWsAfterAssociativity(SDF_Group arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGroupWsAfterColon(SDF_Group arg) */

SDF_OptLayout SDF_getGroupWsAfterColon(SDF_Group arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 5);
}

/*}}}  */
/*{{{  SDF_Group SDF_setGroupGroup(SDF_Group arg, SDF_Group Group) */

SDF_Group SDF_setGroupGroup(SDF_Group arg, SDF_Group Group)
{
  if (SDF_isGroupWithArguments(arg)) {
    return (SDF_Group)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Group), 0), 1);
  }

  ATabort("Group has no Group: %t\n", arg);
  return (SDF_Group)NULL;
}

/*}}}  */
/*{{{  SDF_Group SDF_setGroupWsAfterGroup(SDF_Group arg, SDF_OptLayout wsAfterGroup) */

SDF_Group SDF_setGroupWsAfterGroup(SDF_Group arg, SDF_OptLayout wsAfterGroup)
{
  if (SDF_isGroupWithArguments(arg)) {
    return (SDF_Group)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterGroup), 1), 1);
  }

  ATabort("Group has no WsAfterGroup: %t\n", arg);
  return (SDF_Group)NULL;
}

/*}}}  */
/*{{{  SDF_Group SDF_setGroupArgumentIndicator(SDF_Group arg, SDF_ArgumentIndicator ArgumentIndicator) */

SDF_Group SDF_setGroupArgumentIndicator(SDF_Group arg, SDF_ArgumentIndicator ArgumentIndicator)
{
  if (SDF_isGroupWithArguments(arg)) {
    return (SDF_Group)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ArgumentIndicator), 2), 1);
  }

  ATabort("Group has no ArgumentIndicator: %t\n", arg);
  return (SDF_Group)NULL;
}

/*}}}  */
/*{{{  SDF_Group SDF_setGroupProduction(SDF_Group arg, SDF_Production Production) */

SDF_Group SDF_setGroupProduction(SDF_Group arg, SDF_Production Production)
{
  if (SDF_isGroupSimpleGroup(arg)) {
    return (SDF_Group)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Production), 0), 1);
  }

  ATabort("Group has no Production: %t\n", arg);
  return (SDF_Group)NULL;
}

/*}}}  */
/*{{{  SDF_Group SDF_setGroupWsAfterBraceOpen(SDF_Group arg, SDF_OptLayout wsAfterBraceOpen) */

SDF_Group SDF_setGroupWsAfterBraceOpen(SDF_Group arg, SDF_OptLayout wsAfterBraceOpen)
{
  if (SDF_isGroupProdsGroup(arg)) {
    return (SDF_Group)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBraceOpen), 1), 1);
  }
  else if (SDF_isGroupAssocGroup(arg)) {
    return (SDF_Group)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBraceOpen), 1), 1);
  }

  ATabort("Group has no WsAfterBraceOpen: %t\n", arg);
  return (SDF_Group)NULL;
}

/*}}}  */
/*{{{  SDF_Group SDF_setGroupProductions(SDF_Group arg, SDF_Productions Productions) */

SDF_Group SDF_setGroupProductions(SDF_Group arg, SDF_Productions Productions)
{
  if (SDF_isGroupProdsGroup(arg)) {
    return (SDF_Group)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Productions), 2), 1);
  }
  else if (SDF_isGroupAssocGroup(arg)) {
    return (SDF_Group)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Productions), 6), 1);
  }

  ATabort("Group has no Productions: %t\n", arg);
  return (SDF_Group)NULL;
}

/*}}}  */
/*{{{  SDF_Group SDF_setGroupWsAfterProductions(SDF_Group arg, SDF_OptLayout wsAfterProductions) */

SDF_Group SDF_setGroupWsAfterProductions(SDF_Group arg, SDF_OptLayout wsAfterProductions)
{
  if (SDF_isGroupProdsGroup(arg)) {
    return (SDF_Group)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterProductions), 3), 1);
  }
  else if (SDF_isGroupAssocGroup(arg)) {
    return (SDF_Group)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterProductions), 7), 1);
  }

  ATabort("Group has no WsAfterProductions: %t\n", arg);
  return (SDF_Group)NULL;
}

/*}}}  */
/*{{{  SDF_Group SDF_setGroupAssociativity(SDF_Group arg, SDF_Associativity Associativity) */

SDF_Group SDF_setGroupAssociativity(SDF_Group arg, SDF_Associativity Associativity)
{
  if (SDF_isGroupAssocGroup(arg)) {
    return (SDF_Group)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Associativity), 2), 1);
  }

  ATabort("Group has no Associativity: %t\n", arg);
  return (SDF_Group)NULL;
}

/*}}}  */
/*{{{  SDF_Group SDF_setGroupWsAfterAssociativity(SDF_Group arg, SDF_OptLayout wsAfterAssociativity) */

SDF_Group SDF_setGroupWsAfterAssociativity(SDF_Group arg, SDF_OptLayout wsAfterAssociativity)
{
  if (SDF_isGroupAssocGroup(arg)) {
    return (SDF_Group)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterAssociativity), 3), 1);
  }

  ATabort("Group has no WsAfterAssociativity: %t\n", arg);
  return (SDF_Group)NULL;
}

/*}}}  */
/*{{{  SDF_Group SDF_setGroupWsAfterColon(SDF_Group arg, SDF_OptLayout wsAfterColon) */

SDF_Group SDF_setGroupWsAfterColon(SDF_Group arg, SDF_OptLayout wsAfterColon)
{
  if (SDF_isGroupAssocGroup(arg)) {
    return (SDF_Group)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterColon), 5), 1);
  }

  ATabort("Group has no WsAfterColon: %t\n", arg);
  return (SDF_Group)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Priority accessors */

/*{{{  ATbool SDF_isValidPriority(SDF_Priority arg) */

ATbool SDF_isValidPriority(SDF_Priority arg)
{
  if (SDF_isPriorityChain(arg)) {
    return ATtrue;
  }
  else if (SDF_isPriorityAssoc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isPriorityChain(SDF_Priority arg) */

inline ATbool SDF_isPriorityChain(SDF_Priority arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternPriorityChain, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isPriorityAssoc(SDF_Priority arg) */

inline ATbool SDF_isPriorityAssoc(SDF_Priority arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternPriorityAssoc, NULL, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasPriorityList(SDF_Priority arg) */

ATbool SDF_hasPriorityList(SDF_Priority arg)
{
  if (SDF_isPriorityChain(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasPriorityLeft(SDF_Priority arg) */

ATbool SDF_hasPriorityLeft(SDF_Priority arg)
{
  if (SDF_isPriorityAssoc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasPriorityWsAfterLeft(SDF_Priority arg) */

ATbool SDF_hasPriorityWsAfterLeft(SDF_Priority arg)
{
  if (SDF_isPriorityAssoc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasPriorityAssociativity(SDF_Priority arg) */

ATbool SDF_hasPriorityAssociativity(SDF_Priority arg)
{
  if (SDF_isPriorityAssoc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasPriorityWsAfterAssociativity(SDF_Priority arg) */

ATbool SDF_hasPriorityWsAfterAssociativity(SDF_Priority arg)
{
  if (SDF_isPriorityAssoc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasPriorityRight(SDF_Priority arg) */

ATbool SDF_hasPriorityRight(SDF_Priority arg)
{
  if (SDF_isPriorityAssoc(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_GroupList SDF_getPriorityList(SDF_Priority arg) */

SDF_GroupList SDF_getPriorityList(SDF_Priority arg)
{
  
    return (SDF_GroupList)ATgetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), 1);
}

/*}}}  */
/*{{{  SDF_Group SDF_getPriorityLeft(SDF_Priority arg) */

SDF_Group SDF_getPriorityLeft(SDF_Priority arg)
{
  
    return (SDF_Group)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getPriorityWsAfterLeft(SDF_Priority arg) */

SDF_OptLayout SDF_getPriorityWsAfterLeft(SDF_Priority arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Associativity SDF_getPriorityAssociativity(SDF_Priority arg) */

SDF_Associativity SDF_getPriorityAssociativity(SDF_Priority arg)
{
  
    return (SDF_Associativity)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getPriorityWsAfterAssociativity(SDF_Priority arg) */

SDF_OptLayout SDF_getPriorityWsAfterAssociativity(SDF_Priority arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Group SDF_getPriorityRight(SDF_Priority arg) */

SDF_Group SDF_getPriorityRight(SDF_Priority arg)
{
  
    return (SDF_Group)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_Priority SDF_setPriorityList(SDF_Priority arg, SDF_GroupList list) */

SDF_Priority SDF_setPriorityList(SDF_Priority arg, SDF_GroupList list)
{
  if (SDF_isPriorityChain(arg)) {
    return (SDF_Priority)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), (ATerm)((ATerm) list), 1), 0), 1);
  }

  ATabort("Priority has no List: %t\n", arg);
  return (SDF_Priority)NULL;
}

/*}}}  */
/*{{{  SDF_Priority SDF_setPriorityLeft(SDF_Priority arg, SDF_Group left) */

SDF_Priority SDF_setPriorityLeft(SDF_Priority arg, SDF_Group left)
{
  if (SDF_isPriorityAssoc(arg)) {
    return (SDF_Priority)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) left), 0), 1);
  }

  ATabort("Priority has no Left: %t\n", arg);
  return (SDF_Priority)NULL;
}

/*}}}  */
/*{{{  SDF_Priority SDF_setPriorityWsAfterLeft(SDF_Priority arg, SDF_OptLayout wsAfterLeft) */

SDF_Priority SDF_setPriorityWsAfterLeft(SDF_Priority arg, SDF_OptLayout wsAfterLeft)
{
  if (SDF_isPriorityAssoc(arg)) {
    return (SDF_Priority)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLeft), 1), 1);
  }

  ATabort("Priority has no WsAfterLeft: %t\n", arg);
  return (SDF_Priority)NULL;
}

/*}}}  */
/*{{{  SDF_Priority SDF_setPriorityAssociativity(SDF_Priority arg, SDF_Associativity Associativity) */

SDF_Priority SDF_setPriorityAssociativity(SDF_Priority arg, SDF_Associativity Associativity)
{
  if (SDF_isPriorityAssoc(arg)) {
    return (SDF_Priority)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Associativity), 2), 1);
  }

  ATabort("Priority has no Associativity: %t\n", arg);
  return (SDF_Priority)NULL;
}

/*}}}  */
/*{{{  SDF_Priority SDF_setPriorityWsAfterAssociativity(SDF_Priority arg, SDF_OptLayout wsAfterAssociativity) */

SDF_Priority SDF_setPriorityWsAfterAssociativity(SDF_Priority arg, SDF_OptLayout wsAfterAssociativity)
{
  if (SDF_isPriorityAssoc(arg)) {
    return (SDF_Priority)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterAssociativity), 3), 1);
  }

  ATabort("Priority has no WsAfterAssociativity: %t\n", arg);
  return (SDF_Priority)NULL;
}

/*}}}  */
/*{{{  SDF_Priority SDF_setPriorityRight(SDF_Priority arg, SDF_Group right) */

SDF_Priority SDF_setPriorityRight(SDF_Priority arg, SDF_Group right)
{
  if (SDF_isPriorityAssoc(arg)) {
    return (SDF_Priority)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) right), 4), 1);
  }

  ATabort("Priority has no Right: %t\n", arg);
  return (SDF_Priority)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Priorities accessors */

/*{{{  ATbool SDF_isValidPriorities(SDF_Priorities arg) */

ATbool SDF_isValidPriorities(SDF_Priorities arg)
{
  if (SDF_isPrioritiesDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isPrioritiesDefault(SDF_Priorities arg) */

inline ATbool SDF_isPrioritiesDefault(SDF_Priorities arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternPrioritiesDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasPrioritiesList(SDF_Priorities arg) */

ATbool SDF_hasPrioritiesList(SDF_Priorities arg)
{
  if (SDF_isPrioritiesDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_PriorityList SDF_getPrioritiesList(SDF_Priorities arg) */

SDF_PriorityList SDF_getPrioritiesList(SDF_Priorities arg)
{
  
    return (SDF_PriorityList)ATgetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), 1);
}

/*}}}  */
/*{{{  SDF_Priorities SDF_setPrioritiesList(SDF_Priorities arg, SDF_PriorityList list) */

SDF_Priorities SDF_setPrioritiesList(SDF_Priorities arg, SDF_PriorityList list)
{
  if (SDF_isPrioritiesDefault(arg)) {
    return (SDF_Priorities)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), (ATerm)((ATerm) list), 1), 0), 1);
  }

  ATabort("Priorities has no List: %t\n", arg);
  return (SDF_Priorities)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_NatConArguments accessors */

/*{{{  ATbool SDF_isValidNatConArguments(SDF_NatConArguments arg) */

ATbool SDF_isValidNatConArguments(SDF_NatConArguments arg)
{
  if (SDF_isNatConArgumentsEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isNatConArgumentsSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isNatConArgumentsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isNatConArgumentsEmpty(SDF_NatConArguments arg) */

inline ATbool SDF_isNatConArgumentsEmpty(SDF_NatConArguments arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternNatConArgumentsEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isNatConArgumentsSingle(SDF_NatConArguments arg) */

inline ATbool SDF_isNatConArgumentsSingle(SDF_NatConArguments arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternNatConArgumentsSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isNatConArgumentsMany(SDF_NatConArguments arg) */

inline ATbool SDF_isNatConArgumentsMany(SDF_NatConArguments arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternNatConArgumentsMany, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasNatConArgumentsHead(SDF_NatConArguments arg) */

ATbool SDF_hasNatConArgumentsHead(SDF_NatConArguments arg)
{
  if (SDF_isNatConArgumentsSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isNatConArgumentsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasNatConArgumentsWsAfterHead(SDF_NatConArguments arg) */

ATbool SDF_hasNatConArgumentsWsAfterHead(SDF_NatConArguments arg)
{
  if (SDF_isNatConArgumentsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasNatConArgumentsWsAfterSep(SDF_NatConArguments arg) */

ATbool SDF_hasNatConArgumentsWsAfterSep(SDF_NatConArguments arg)
{
  if (SDF_isNatConArgumentsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasNatConArgumentsTail(SDF_NatConArguments arg) */

ATbool SDF_hasNatConArgumentsTail(SDF_NatConArguments arg)
{
  if (SDF_isNatConArgumentsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_NatConArguments SDF_getNatConArgumentsTail(SDF_NatConArguments arg) */

SDF_NatConArguments SDF_getNatConArgumentsTail(SDF_NatConArguments arg)
{
  assert(!SDF_isNatConArgumentsEmpty(arg) && "getTail on an empty list");
  if (SDF_isNatConArgumentsSingle(arg)) {
    return (SDF_NatConArguments) SDF_makeNatConArgumentsEmpty();
  }
  else {
  
    return (SDF_NatConArguments)ATgetTail((ATermList)arg, 4);
  }
}

/*}}}  */
/*{{{  SDF_NatCon SDF_getNatConArgumentsHead(SDF_NatConArguments arg) */

SDF_NatCon SDF_getNatConArgumentsHead(SDF_NatConArguments arg)
{
  if (SDF_isNatConArgumentsSingle(arg)) {
    return (SDF_NatCon)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_NatCon)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getNatConArgumentsWsAfterHead(SDF_NatConArguments arg) */

SDF_OptLayout SDF_getNatConArgumentsWsAfterHead(SDF_NatConArguments arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getNatConArgumentsWsAfterSep(SDF_NatConArguments arg) */

SDF_OptLayout SDF_getNatConArgumentsWsAfterSep(SDF_NatConArguments arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 3);
}

/*}}}  */
/*{{{  SDF_NatConArguments SDF_setNatConArgumentsHead(SDF_NatConArguments arg, SDF_NatCon head) */

SDF_NatConArguments SDF_setNatConArgumentsHead(SDF_NatConArguments arg, SDF_NatCon head)
{
  if (SDF_isNatConArgumentsSingle(arg)) {
    return (SDF_NatConArguments)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isNatConArgumentsMany(arg)) {
    return (SDF_NatConArguments)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("NatConArguments has no Head: %t\n", arg);
  return (SDF_NatConArguments)NULL;
}

/*}}}  */
/*{{{  SDF_NatConArguments SDF_setNatConArgumentsWsAfterHead(SDF_NatConArguments arg, SDF_OptLayout wsAfterHead) */

SDF_NatConArguments SDF_setNatConArgumentsWsAfterHead(SDF_NatConArguments arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isNatConArgumentsMany(arg)) {
    return (SDF_NatConArguments)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("NatConArguments has no WsAfterHead: %t\n", arg);
  return (SDF_NatConArguments)NULL;
}

/*}}}  */
/*{{{  SDF_NatConArguments SDF_setNatConArgumentsWsAfterSep(SDF_NatConArguments arg, SDF_OptLayout wsAfterSep) */

SDF_NatConArguments SDF_setNatConArgumentsWsAfterSep(SDF_NatConArguments arg, SDF_OptLayout wsAfterSep)
{
  if (SDF_isNatConArgumentsMany(arg)) {
    return (SDF_NatConArguments)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterSep), 3);
  }

  ATabort("NatConArguments has no WsAfterSep: %t\n", arg);
  return (SDF_NatConArguments)NULL;
}

/*}}}  */
/*{{{  SDF_NatConArguments SDF_setNatConArgumentsTail(SDF_NatConArguments arg, SDF_NatConArguments tail) */

SDF_NatConArguments SDF_setNatConArgumentsTail(SDF_NatConArguments arg, SDF_NatConArguments tail)
{
  if (SDF_isNatConArgumentsMany(arg)) {
    return (SDF_NatConArguments)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 4);
  }

  ATabort("NatConArguments has no Tail: %t\n", arg);
  return (SDF_NatConArguments)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_GroupList accessors */

/*{{{  ATbool SDF_isValidGroupList(SDF_GroupList arg) */

ATbool SDF_isValidGroupList(SDF_GroupList arg)
{
  if (SDF_isGroupListEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isGroupListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isGroupListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isGroupListEmpty(SDF_GroupList arg) */

inline ATbool SDF_isGroupListEmpty(SDF_GroupList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternGroupListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isGroupListSingle(SDF_GroupList arg) */

inline ATbool SDF_isGroupListSingle(SDF_GroupList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGroupListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isGroupListMany(SDF_GroupList arg) */

inline ATbool SDF_isGroupListMany(SDF_GroupList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternGroupListMany, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasGroupListHead(SDF_GroupList arg) */

ATbool SDF_hasGroupListHead(SDF_GroupList arg)
{
  if (SDF_isGroupListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isGroupListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGroupListWsAfterHead(SDF_GroupList arg) */

ATbool SDF_hasGroupListWsAfterHead(SDF_GroupList arg)
{
  if (SDF_isGroupListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGroupListWsAfterSep(SDF_GroupList arg) */

ATbool SDF_hasGroupListWsAfterSep(SDF_GroupList arg)
{
  if (SDF_isGroupListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasGroupListTail(SDF_GroupList arg) */

ATbool SDF_hasGroupListTail(SDF_GroupList arg)
{
  if (SDF_isGroupListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_GroupList SDF_getGroupListTail(SDF_GroupList arg) */

SDF_GroupList SDF_getGroupListTail(SDF_GroupList arg)
{
  assert(!SDF_isGroupListEmpty(arg) && "getTail on an empty list");
  if (SDF_isGroupListSingle(arg)) {
    return (SDF_GroupList) SDF_makeGroupListEmpty();
  }
  else {
  
    return (SDF_GroupList)ATgetTail((ATermList)arg, 4);
  }
}

/*}}}  */
/*{{{  SDF_Group SDF_getGroupListHead(SDF_GroupList arg) */

SDF_Group SDF_getGroupListHead(SDF_GroupList arg)
{
  if (SDF_isGroupListSingle(arg)) {
    return (SDF_Group)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Group)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGroupListWsAfterHead(SDF_GroupList arg) */

SDF_OptLayout SDF_getGroupListWsAfterHead(SDF_GroupList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getGroupListWsAfterSep(SDF_GroupList arg) */

SDF_OptLayout SDF_getGroupListWsAfterSep(SDF_GroupList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 3);
}

/*}}}  */
/*{{{  SDF_GroupList SDF_setGroupListHead(SDF_GroupList arg, SDF_Group head) */

SDF_GroupList SDF_setGroupListHead(SDF_GroupList arg, SDF_Group head)
{
  if (SDF_isGroupListSingle(arg)) {
    return (SDF_GroupList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isGroupListMany(arg)) {
    return (SDF_GroupList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("GroupList has no Head: %t\n", arg);
  return (SDF_GroupList)NULL;
}

/*}}}  */
/*{{{  SDF_GroupList SDF_setGroupListWsAfterHead(SDF_GroupList arg, SDF_OptLayout wsAfterHead) */

SDF_GroupList SDF_setGroupListWsAfterHead(SDF_GroupList arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isGroupListMany(arg)) {
    return (SDF_GroupList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("GroupList has no WsAfterHead: %t\n", arg);
  return (SDF_GroupList)NULL;
}

/*}}}  */
/*{{{  SDF_GroupList SDF_setGroupListWsAfterSep(SDF_GroupList arg, SDF_OptLayout wsAfterSep) */

SDF_GroupList SDF_setGroupListWsAfterSep(SDF_GroupList arg, SDF_OptLayout wsAfterSep)
{
  if (SDF_isGroupListMany(arg)) {
    return (SDF_GroupList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterSep), 3);
  }

  ATabort("GroupList has no WsAfterSep: %t\n", arg);
  return (SDF_GroupList)NULL;
}

/*}}}  */
/*{{{  SDF_GroupList SDF_setGroupListTail(SDF_GroupList arg, SDF_GroupList tail) */

SDF_GroupList SDF_setGroupListTail(SDF_GroupList arg, SDF_GroupList tail)
{
  if (SDF_isGroupListMany(arg)) {
    return (SDF_GroupList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 4);
  }

  ATabort("GroupList has no Tail: %t\n", arg);
  return (SDF_GroupList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_PriorityList accessors */

/*{{{  ATbool SDF_isValidPriorityList(SDF_PriorityList arg) */

ATbool SDF_isValidPriorityList(SDF_PriorityList arg)
{
  if (SDF_isPriorityListEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isPriorityListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isPriorityListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isPriorityListEmpty(SDF_PriorityList arg) */

inline ATbool SDF_isPriorityListEmpty(SDF_PriorityList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternPriorityListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isPriorityListSingle(SDF_PriorityList arg) */

inline ATbool SDF_isPriorityListSingle(SDF_PriorityList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternPriorityListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isPriorityListMany(SDF_PriorityList arg) */

inline ATbool SDF_isPriorityListMany(SDF_PriorityList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternPriorityListMany, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasPriorityListHead(SDF_PriorityList arg) */

ATbool SDF_hasPriorityListHead(SDF_PriorityList arg)
{
  if (SDF_isPriorityListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isPriorityListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasPriorityListWsAfterHead(SDF_PriorityList arg) */

ATbool SDF_hasPriorityListWsAfterHead(SDF_PriorityList arg)
{
  if (SDF_isPriorityListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasPriorityListWsAfterSep(SDF_PriorityList arg) */

ATbool SDF_hasPriorityListWsAfterSep(SDF_PriorityList arg)
{
  if (SDF_isPriorityListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasPriorityListTail(SDF_PriorityList arg) */

ATbool SDF_hasPriorityListTail(SDF_PriorityList arg)
{
  if (SDF_isPriorityListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_PriorityList SDF_getPriorityListTail(SDF_PriorityList arg) */

SDF_PriorityList SDF_getPriorityListTail(SDF_PriorityList arg)
{
  assert(!SDF_isPriorityListEmpty(arg) && "getTail on an empty list");
  if (SDF_isPriorityListSingle(arg)) {
    return (SDF_PriorityList) SDF_makePriorityListEmpty();
  }
  else {
  
    return (SDF_PriorityList)ATgetTail((ATermList)arg, 4);
  }
}

/*}}}  */
/*{{{  SDF_Priority SDF_getPriorityListHead(SDF_PriorityList arg) */

SDF_Priority SDF_getPriorityListHead(SDF_PriorityList arg)
{
  if (SDF_isPriorityListSingle(arg)) {
    return (SDF_Priority)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Priority)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getPriorityListWsAfterHead(SDF_PriorityList arg) */

SDF_OptLayout SDF_getPriorityListWsAfterHead(SDF_PriorityList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getPriorityListWsAfterSep(SDF_PriorityList arg) */

SDF_OptLayout SDF_getPriorityListWsAfterSep(SDF_PriorityList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 3);
}

/*}}}  */
/*{{{  SDF_PriorityList SDF_setPriorityListHead(SDF_PriorityList arg, SDF_Priority head) */

SDF_PriorityList SDF_setPriorityListHead(SDF_PriorityList arg, SDF_Priority head)
{
  if (SDF_isPriorityListSingle(arg)) {
    return (SDF_PriorityList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isPriorityListMany(arg)) {
    return (SDF_PriorityList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("PriorityList has no Head: %t\n", arg);
  return (SDF_PriorityList)NULL;
}

/*}}}  */
/*{{{  SDF_PriorityList SDF_setPriorityListWsAfterHead(SDF_PriorityList arg, SDF_OptLayout wsAfterHead) */

SDF_PriorityList SDF_setPriorityListWsAfterHead(SDF_PriorityList arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isPriorityListMany(arg)) {
    return (SDF_PriorityList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("PriorityList has no WsAfterHead: %t\n", arg);
  return (SDF_PriorityList)NULL;
}

/*}}}  */
/*{{{  SDF_PriorityList SDF_setPriorityListWsAfterSep(SDF_PriorityList arg, SDF_OptLayout wsAfterSep) */

SDF_PriorityList SDF_setPriorityListWsAfterSep(SDF_PriorityList arg, SDF_OptLayout wsAfterSep)
{
  if (SDF_isPriorityListMany(arg)) {
    return (SDF_PriorityList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterSep), 3);
  }

  ATabort("PriorityList has no WsAfterSep: %t\n", arg);
  return (SDF_PriorityList)NULL;
}

/*}}}  */
/*{{{  SDF_PriorityList SDF_setPriorityListTail(SDF_PriorityList arg, SDF_PriorityList tail) */

SDF_PriorityList SDF_setPriorityListTail(SDF_PriorityList arg, SDF_PriorityList tail)
{
  if (SDF_isPriorityListMany(arg)) {
    return (SDF_PriorityList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 4);
  }

  ATabort("PriorityList has no Tail: %t\n", arg);
  return (SDF_PriorityList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_SymbolTail accessors */

/*{{{  ATbool SDF_isValidSymbolTail(SDF_SymbolTail arg) */

ATbool SDF_isValidSymbolTail(SDF_SymbolTail arg)
{
  if (SDF_isSymbolTailEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolTailSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolTailMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolTailEmpty(SDF_SymbolTail arg) */

inline ATbool SDF_isSymbolTailEmpty(SDF_SymbolTail arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternSymbolTailEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolTailSingle(SDF_SymbolTail arg) */

inline ATbool SDF_isSymbolTailSingle(SDF_SymbolTail arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolTailSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolTailMany(SDF_SymbolTail arg) */

inline ATbool SDF_isSymbolTailMany(SDF_SymbolTail arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolTailMany, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolTailHead(SDF_SymbolTail arg) */

ATbool SDF_hasSymbolTailHead(SDF_SymbolTail arg)
{
  if (SDF_isSymbolTailSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolTailMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolTailWsAfterHead(SDF_SymbolTail arg) */

ATbool SDF_hasSymbolTailWsAfterHead(SDF_SymbolTail arg)
{
  if (SDF_isSymbolTailMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolTailTail(SDF_SymbolTail arg) */

ATbool SDF_hasSymbolTailTail(SDF_SymbolTail arg)
{
  if (SDF_isSymbolTailMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_SymbolTail SDF_getSymbolTailTail(SDF_SymbolTail arg) */

SDF_SymbolTail SDF_getSymbolTailTail(SDF_SymbolTail arg)
{
  assert(!SDF_isSymbolTailEmpty(arg) && "getTail on an empty list");
  if (SDF_isSymbolTailSingle(arg)) {
    return (SDF_SymbolTail) SDF_makeSymbolTailEmpty();
  }
  else {
  
    return (SDF_SymbolTail)ATgetTail((ATermList)arg, 2);
  }
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getSymbolTailHead(SDF_SymbolTail arg) */

SDF_Symbol SDF_getSymbolTailHead(SDF_SymbolTail arg)
{
  if (SDF_isSymbolTailSingle(arg)) {
    return (SDF_Symbol)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Symbol)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolTailWsAfterHead(SDF_SymbolTail arg) */

SDF_OptLayout SDF_getSymbolTailWsAfterHead(SDF_SymbolTail arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_SymbolTail SDF_setSymbolTailHead(SDF_SymbolTail arg, SDF_Symbol head) */

SDF_SymbolTail SDF_setSymbolTailHead(SDF_SymbolTail arg, SDF_Symbol head)
{
  if (SDF_isSymbolTailSingle(arg)) {
    return (SDF_SymbolTail)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isSymbolTailMany(arg)) {
    return (SDF_SymbolTail)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("SymbolTail has no Head: %t\n", arg);
  return (SDF_SymbolTail)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolTail SDF_setSymbolTailWsAfterHead(SDF_SymbolTail arg, SDF_OptLayout wsAfterHead) */

SDF_SymbolTail SDF_setSymbolTailWsAfterHead(SDF_SymbolTail arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isSymbolTailMany(arg)) {
    return (SDF_SymbolTail)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("SymbolTail has no WsAfterHead: %t\n", arg);
  return (SDF_SymbolTail)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolTail SDF_setSymbolTailTail(SDF_SymbolTail arg, SDF_SymbolTail tail) */

SDF_SymbolTail SDF_setSymbolTailTail(SDF_SymbolTail arg, SDF_SymbolTail tail)
{
  if (SDF_isSymbolTailMany(arg)) {
    return (SDF_SymbolTail)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 2);
  }

  ATabort("SymbolTail has no Tail: %t\n", arg);
  return (SDF_SymbolTail)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_SymbolRest accessors */

/*{{{  ATbool SDF_isValidSymbolRest(SDF_SymbolRest arg) */

ATbool SDF_isValidSymbolRest(SDF_SymbolRest arg)
{
  if (SDF_isSymbolRestEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolRestSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolRestMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolRestEmpty(SDF_SymbolRest arg) */

inline ATbool SDF_isSymbolRestEmpty(SDF_SymbolRest arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternSymbolRestEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolRestSingle(SDF_SymbolRest arg) */

inline ATbool SDF_isSymbolRestSingle(SDF_SymbolRest arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolRestSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolRestMany(SDF_SymbolRest arg) */

inline ATbool SDF_isSymbolRestMany(SDF_SymbolRest arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolRestMany, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolRestHead(SDF_SymbolRest arg) */

ATbool SDF_hasSymbolRestHead(SDF_SymbolRest arg)
{
  if (SDF_isSymbolRestSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolRestMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolRestWsAfterHead(SDF_SymbolRest arg) */

ATbool SDF_hasSymbolRestWsAfterHead(SDF_SymbolRest arg)
{
  if (SDF_isSymbolRestMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolRestWsAfterSep(SDF_SymbolRest arg) */

ATbool SDF_hasSymbolRestWsAfterSep(SDF_SymbolRest arg)
{
  if (SDF_isSymbolRestMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolRestTail(SDF_SymbolRest arg) */

ATbool SDF_hasSymbolRestTail(SDF_SymbolRest arg)
{
  if (SDF_isSymbolRestMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_SymbolRest SDF_getSymbolRestTail(SDF_SymbolRest arg) */

SDF_SymbolRest SDF_getSymbolRestTail(SDF_SymbolRest arg)
{
  assert(!SDF_isSymbolRestEmpty(arg) && "getTail on an empty list");
  if (SDF_isSymbolRestSingle(arg)) {
    return (SDF_SymbolRest) SDF_makeSymbolRestEmpty();
  }
  else {
  
    return (SDF_SymbolRest)ATgetTail((ATermList)arg, 4);
  }
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getSymbolRestHead(SDF_SymbolRest arg) */

SDF_Symbol SDF_getSymbolRestHead(SDF_SymbolRest arg)
{
  if (SDF_isSymbolRestSingle(arg)) {
    return (SDF_Symbol)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Symbol)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolRestWsAfterHead(SDF_SymbolRest arg) */

SDF_OptLayout SDF_getSymbolRestWsAfterHead(SDF_SymbolRest arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolRestWsAfterSep(SDF_SymbolRest arg) */

SDF_OptLayout SDF_getSymbolRestWsAfterSep(SDF_SymbolRest arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 3);
}

/*}}}  */
/*{{{  SDF_SymbolRest SDF_setSymbolRestHead(SDF_SymbolRest arg, SDF_Symbol head) */

SDF_SymbolRest SDF_setSymbolRestHead(SDF_SymbolRest arg, SDF_Symbol head)
{
  if (SDF_isSymbolRestSingle(arg)) {
    return (SDF_SymbolRest)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isSymbolRestMany(arg)) {
    return (SDF_SymbolRest)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("SymbolRest has no Head: %t\n", arg);
  return (SDF_SymbolRest)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolRest SDF_setSymbolRestWsAfterHead(SDF_SymbolRest arg, SDF_OptLayout wsAfterHead) */

SDF_SymbolRest SDF_setSymbolRestWsAfterHead(SDF_SymbolRest arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isSymbolRestMany(arg)) {
    return (SDF_SymbolRest)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("SymbolRest has no WsAfterHead: %t\n", arg);
  return (SDF_SymbolRest)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolRest SDF_setSymbolRestWsAfterSep(SDF_SymbolRest arg, SDF_OptLayout wsAfterSep) */

SDF_SymbolRest SDF_setSymbolRestWsAfterSep(SDF_SymbolRest arg, SDF_OptLayout wsAfterSep)
{
  if (SDF_isSymbolRestMany(arg)) {
    return (SDF_SymbolRest)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterSep), 3);
  }

  ATabort("SymbolRest has no WsAfterSep: %t\n", arg);
  return (SDF_SymbolRest)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolRest SDF_setSymbolRestTail(SDF_SymbolRest arg, SDF_SymbolRest tail) */

SDF_SymbolRest SDF_setSymbolRestTail(SDF_SymbolRest arg, SDF_SymbolRest tail)
{
  if (SDF_isSymbolRestMany(arg)) {
    return (SDF_SymbolRest)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 4);
  }

  ATabort("SymbolRest has no Tail: %t\n", arg);
  return (SDF_SymbolRest)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Renamings accessors */

/*{{{  ATbool SDF_isValidRenamings(SDF_Renamings arg) */

ATbool SDF_isValidRenamings(SDF_Renamings arg)
{
  if (SDF_isRenamingsRenamings(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isRenamingsRenamings(SDF_Renamings arg) */

inline ATbool SDF_isRenamingsRenamings(SDF_Renamings arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternRenamingsRenamings, NULL, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasRenamingsWsAfterBracketOpen(SDF_Renamings arg) */

ATbool SDF_hasRenamingsWsAfterBracketOpen(SDF_Renamings arg)
{
  if (SDF_isRenamingsRenamings(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRenamingsList(SDF_Renamings arg) */

ATbool SDF_hasRenamingsList(SDF_Renamings arg)
{
  if (SDF_isRenamingsRenamings(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRenamingsWsAfterList(SDF_Renamings arg) */

ATbool SDF_hasRenamingsWsAfterList(SDF_Renamings arg)
{
  if (SDF_isRenamingsRenamings(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getRenamingsWsAfterBracketOpen(SDF_Renamings arg) */

SDF_OptLayout SDF_getRenamingsWsAfterBracketOpen(SDF_Renamings arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_RenamingList SDF_getRenamingsList(SDF_Renamings arg) */

SDF_RenamingList SDF_getRenamingsList(SDF_Renamings arg)
{
  
    return (SDF_RenamingList)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getRenamingsWsAfterList(SDF_Renamings arg) */

SDF_OptLayout SDF_getRenamingsWsAfterList(SDF_Renamings arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Renamings SDF_setRenamingsWsAfterBracketOpen(SDF_Renamings arg, SDF_OptLayout wsAfterBracketOpen) */

SDF_Renamings SDF_setRenamingsWsAfterBracketOpen(SDF_Renamings arg, SDF_OptLayout wsAfterBracketOpen)
{
  if (SDF_isRenamingsRenamings(arg)) {
    return (SDF_Renamings)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBracketOpen), 1), 1);
  }

  ATabort("Renamings has no WsAfterBracketOpen: %t\n", arg);
  return (SDF_Renamings)NULL;
}

/*}}}  */
/*{{{  SDF_Renamings SDF_setRenamingsList(SDF_Renamings arg, SDF_RenamingList list) */

SDF_Renamings SDF_setRenamingsList(SDF_Renamings arg, SDF_RenamingList list)
{
  if (SDF_isRenamingsRenamings(arg)) {
    return (SDF_Renamings)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), (ATerm)((ATerm) list), 1), 2), 1);
  }

  ATabort("Renamings has no List: %t\n", arg);
  return (SDF_Renamings)NULL;
}

/*}}}  */
/*{{{  SDF_Renamings SDF_setRenamingsWsAfterList(SDF_Renamings arg, SDF_OptLayout wsAfterList) */

SDF_Renamings SDF_setRenamingsWsAfterList(SDF_Renamings arg, SDF_OptLayout wsAfterList)
{
  if (SDF_isRenamingsRenamings(arg)) {
    return (SDF_Renamings)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterList), 3), 1);
  }

  ATabort("Renamings has no WsAfterList: %t\n", arg);
  return (SDF_Renamings)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Renaming accessors */

/*{{{  ATbool SDF_isValidRenaming(SDF_Renaming arg) */

ATbool SDF_isValidRenaming(SDF_Renaming arg)
{
  if (SDF_isRenamingSymbol(arg)) {
    return ATtrue;
  }
  else if (SDF_isRenamingProduction(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isRenamingSymbol(SDF_Renaming arg) */

inline ATbool SDF_isRenamingSymbol(SDF_Renaming arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternRenamingSymbol, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isRenamingProduction(SDF_Renaming arg) */

inline ATbool SDF_isRenamingProduction(SDF_Renaming arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternRenamingProduction, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasRenamingFrom(SDF_Renaming arg) */

ATbool SDF_hasRenamingFrom(SDF_Renaming arg)
{
  if (SDF_isRenamingSymbol(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRenamingWsAfterFrom(SDF_Renaming arg) */

ATbool SDF_hasRenamingWsAfterFrom(SDF_Renaming arg)
{
  if (SDF_isRenamingSymbol(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRenamingWsAfterEqualsGreaterThan(SDF_Renaming arg) */

ATbool SDF_hasRenamingWsAfterEqualsGreaterThan(SDF_Renaming arg)
{
  if (SDF_isRenamingSymbol(arg)) {
    return ATtrue;
  }
  else if (SDF_isRenamingProduction(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRenamingTo(SDF_Renaming arg) */

ATbool SDF_hasRenamingTo(SDF_Renaming arg)
{
  if (SDF_isRenamingSymbol(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRenamingFromProd(SDF_Renaming arg) */

ATbool SDF_hasRenamingFromProd(SDF_Renaming arg)
{
  if (SDF_isRenamingProduction(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRenamingWsAfterFromProd(SDF_Renaming arg) */

ATbool SDF_hasRenamingWsAfterFromProd(SDF_Renaming arg)
{
  if (SDF_isRenamingProduction(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRenamingToProd(SDF_Renaming arg) */

ATbool SDF_hasRenamingToProd(SDF_Renaming arg)
{
  if (SDF_isRenamingProduction(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getRenamingFrom(SDF_Renaming arg) */

SDF_Symbol SDF_getRenamingFrom(SDF_Renaming arg)
{
  
    return (SDF_Symbol)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getRenamingWsAfterFrom(SDF_Renaming arg) */

SDF_OptLayout SDF_getRenamingWsAfterFrom(SDF_Renaming arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getRenamingWsAfterEqualsGreaterThan(SDF_Renaming arg) */

SDF_OptLayout SDF_getRenamingWsAfterEqualsGreaterThan(SDF_Renaming arg)
{
  if (SDF_isRenamingSymbol(arg)) {
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
  }
  else 
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getRenamingTo(SDF_Renaming arg) */

SDF_Symbol SDF_getRenamingTo(SDF_Renaming arg)
{
  
    return (SDF_Symbol)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_Production SDF_getRenamingFromProd(SDF_Renaming arg) */

SDF_Production SDF_getRenamingFromProd(SDF_Renaming arg)
{
  
    return (SDF_Production)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getRenamingWsAfterFromProd(SDF_Renaming arg) */

SDF_OptLayout SDF_getRenamingWsAfterFromProd(SDF_Renaming arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Production SDF_getRenamingToProd(SDF_Renaming arg) */

SDF_Production SDF_getRenamingToProd(SDF_Renaming arg)
{
  
    return (SDF_Production)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_Renaming SDF_setRenamingFrom(SDF_Renaming arg, SDF_Symbol from) */

SDF_Renaming SDF_setRenamingFrom(SDF_Renaming arg, SDF_Symbol from)
{
  if (SDF_isRenamingSymbol(arg)) {
    return (SDF_Renaming)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) from), 0), 1);
  }

  ATabort("Renaming has no From: %t\n", arg);
  return (SDF_Renaming)NULL;
}

/*}}}  */
/*{{{  SDF_Renaming SDF_setRenamingWsAfterFrom(SDF_Renaming arg, SDF_OptLayout wsAfterFrom) */

SDF_Renaming SDF_setRenamingWsAfterFrom(SDF_Renaming arg, SDF_OptLayout wsAfterFrom)
{
  if (SDF_isRenamingSymbol(arg)) {
    return (SDF_Renaming)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterFrom), 1), 1);
  }

  ATabort("Renaming has no WsAfterFrom: %t\n", arg);
  return (SDF_Renaming)NULL;
}

/*}}}  */
/*{{{  SDF_Renaming SDF_setRenamingWsAfterEqualsGreaterThan(SDF_Renaming arg, SDF_OptLayout wsAfterEqualsGreaterThan) */

SDF_Renaming SDF_setRenamingWsAfterEqualsGreaterThan(SDF_Renaming arg, SDF_OptLayout wsAfterEqualsGreaterThan)
{
  if (SDF_isRenamingSymbol(arg)) {
    return (SDF_Renaming)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterEqualsGreaterThan), 3), 1);
  }
  else if (SDF_isRenamingProduction(arg)) {
    return (SDF_Renaming)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterEqualsGreaterThan), 3), 1);
  }

  ATabort("Renaming has no WsAfterEqualsGreaterThan: %t\n", arg);
  return (SDF_Renaming)NULL;
}

/*}}}  */
/*{{{  SDF_Renaming SDF_setRenamingTo(SDF_Renaming arg, SDF_Symbol to) */

SDF_Renaming SDF_setRenamingTo(SDF_Renaming arg, SDF_Symbol to)
{
  if (SDF_isRenamingSymbol(arg)) {
    return (SDF_Renaming)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) to), 4), 1);
  }

  ATabort("Renaming has no To: %t\n", arg);
  return (SDF_Renaming)NULL;
}

/*}}}  */
/*{{{  SDF_Renaming SDF_setRenamingFromProd(SDF_Renaming arg, SDF_Production fromProd) */

SDF_Renaming SDF_setRenamingFromProd(SDF_Renaming arg, SDF_Production fromProd)
{
  if (SDF_isRenamingProduction(arg)) {
    return (SDF_Renaming)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) fromProd), 0), 1);
  }

  ATabort("Renaming has no FromProd: %t\n", arg);
  return (SDF_Renaming)NULL;
}

/*}}}  */
/*{{{  SDF_Renaming SDF_setRenamingWsAfterFromProd(SDF_Renaming arg, SDF_OptLayout wsAfterFromProd) */

SDF_Renaming SDF_setRenamingWsAfterFromProd(SDF_Renaming arg, SDF_OptLayout wsAfterFromProd)
{
  if (SDF_isRenamingProduction(arg)) {
    return (SDF_Renaming)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterFromProd), 1), 1);
  }

  ATabort("Renaming has no WsAfterFromProd: %t\n", arg);
  return (SDF_Renaming)NULL;
}

/*}}}  */
/*{{{  SDF_Renaming SDF_setRenamingToProd(SDF_Renaming arg, SDF_Production toProd) */

SDF_Renaming SDF_setRenamingToProd(SDF_Renaming arg, SDF_Production toProd)
{
  if (SDF_isRenamingProduction(arg)) {
    return (SDF_Renaming)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) toProd), 4), 1);
  }

  ATabort("Renaming has no ToProd: %t\n", arg);
  return (SDF_Renaming)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_RenamingList accessors */

/*{{{  ATbool SDF_isValidRenamingList(SDF_RenamingList arg) */

ATbool SDF_isValidRenamingList(SDF_RenamingList arg)
{
  if (SDF_isRenamingListEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isRenamingListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isRenamingListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isRenamingListEmpty(SDF_RenamingList arg) */

inline ATbool SDF_isRenamingListEmpty(SDF_RenamingList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternRenamingListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isRenamingListSingle(SDF_RenamingList arg) */

inline ATbool SDF_isRenamingListSingle(SDF_RenamingList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternRenamingListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isRenamingListMany(SDF_RenamingList arg) */

inline ATbool SDF_isRenamingListMany(SDF_RenamingList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternRenamingListMany, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasRenamingListHead(SDF_RenamingList arg) */

ATbool SDF_hasRenamingListHead(SDF_RenamingList arg)
{
  if (SDF_isRenamingListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isRenamingListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRenamingListWsAfterHead(SDF_RenamingList arg) */

ATbool SDF_hasRenamingListWsAfterHead(SDF_RenamingList arg)
{
  if (SDF_isRenamingListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRenamingListTail(SDF_RenamingList arg) */

ATbool SDF_hasRenamingListTail(SDF_RenamingList arg)
{
  if (SDF_isRenamingListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_RenamingList SDF_getRenamingListTail(SDF_RenamingList arg) */

SDF_RenamingList SDF_getRenamingListTail(SDF_RenamingList arg)
{
  assert(!SDF_isRenamingListEmpty(arg) && "getTail on an empty list");
  if (SDF_isRenamingListSingle(arg)) {
    return (SDF_RenamingList) SDF_makeRenamingListEmpty();
  }
  else {
  
    return (SDF_RenamingList)ATgetTail((ATermList)arg, 2);
  }
}

/*}}}  */
/*{{{  SDF_Renaming SDF_getRenamingListHead(SDF_RenamingList arg) */

SDF_Renaming SDF_getRenamingListHead(SDF_RenamingList arg)
{
  if (SDF_isRenamingListSingle(arg)) {
    return (SDF_Renaming)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Renaming)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getRenamingListWsAfterHead(SDF_RenamingList arg) */

SDF_OptLayout SDF_getRenamingListWsAfterHead(SDF_RenamingList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_RenamingList SDF_setRenamingListHead(SDF_RenamingList arg, SDF_Renaming head) */

SDF_RenamingList SDF_setRenamingListHead(SDF_RenamingList arg, SDF_Renaming head)
{
  if (SDF_isRenamingListSingle(arg)) {
    return (SDF_RenamingList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isRenamingListMany(arg)) {
    return (SDF_RenamingList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("RenamingList has no Head: %t\n", arg);
  return (SDF_RenamingList)NULL;
}

/*}}}  */
/*{{{  SDF_RenamingList SDF_setRenamingListWsAfterHead(SDF_RenamingList arg, SDF_OptLayout wsAfterHead) */

SDF_RenamingList SDF_setRenamingListWsAfterHead(SDF_RenamingList arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isRenamingListMany(arg)) {
    return (SDF_RenamingList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("RenamingList has no WsAfterHead: %t\n", arg);
  return (SDF_RenamingList)NULL;
}

/*}}}  */
/*{{{  SDF_RenamingList SDF_setRenamingListTail(SDF_RenamingList arg, SDF_RenamingList tail) */

SDF_RenamingList SDF_setRenamingListTail(SDF_RenamingList arg, SDF_RenamingList tail)
{
  if (SDF_isRenamingListMany(arg)) {
    return (SDF_RenamingList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 2);
  }

  ATabort("RenamingList has no Tail: %t\n", arg);
  return (SDF_RenamingList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Lookahead accessors */

/*{{{  ATbool SDF_isValidLookahead(SDF_Lookahead arg) */

ATbool SDF_isValidLookahead(SDF_Lookahead arg)
{
  if (SDF_isLookaheadCharClass(arg)) {
    return ATtrue;
  }
  else if (SDF_isLookaheadSeq(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLookaheadCharClass(SDF_Lookahead arg) */

inline ATbool SDF_isLookaheadCharClass(SDF_Lookahead arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLookaheadCharClass, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLookaheadSeq(SDF_Lookahead arg) */

inline ATbool SDF_isLookaheadSeq(SDF_Lookahead arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLookaheadSeq, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadCharClass(SDF_Lookahead arg) */

ATbool SDF_hasLookaheadCharClass(SDF_Lookahead arg)
{
  if (SDF_isLookaheadCharClass(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadHead(SDF_Lookahead arg) */

ATbool SDF_hasLookaheadHead(SDF_Lookahead arg)
{
  if (SDF_isLookaheadSeq(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadWsAfterHead(SDF_Lookahead arg) */

ATbool SDF_hasLookaheadWsAfterHead(SDF_Lookahead arg)
{
  if (SDF_isLookaheadSeq(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadWsAfterPeriod(SDF_Lookahead arg) */

ATbool SDF_hasLookaheadWsAfterPeriod(SDF_Lookahead arg)
{
  if (SDF_isLookaheadSeq(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadTail(SDF_Lookahead arg) */

ATbool SDF_hasLookaheadTail(SDF_Lookahead arg)
{
  if (SDF_isLookaheadSeq(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_CharClass SDF_getLookaheadCharClass(SDF_Lookahead arg) */

SDF_CharClass SDF_getLookaheadCharClass(SDF_Lookahead arg)
{
  
    return (SDF_CharClass)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_CharClass SDF_getLookaheadHead(SDF_Lookahead arg) */

SDF_CharClass SDF_getLookaheadHead(SDF_Lookahead arg)
{
  
    return (SDF_CharClass)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getLookaheadWsAfterHead(SDF_Lookahead arg) */

SDF_OptLayout SDF_getLookaheadWsAfterHead(SDF_Lookahead arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getLookaheadWsAfterPeriod(SDF_Lookahead arg) */

SDF_OptLayout SDF_getLookaheadWsAfterPeriod(SDF_Lookahead arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_getLookaheadTail(SDF_Lookahead arg) */

SDF_Lookaheads SDF_getLookaheadTail(SDF_Lookahead arg)
{
  
    return (SDF_Lookaheads)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_Lookahead SDF_setLookaheadCharClass(SDF_Lookahead arg, SDF_CharClass CharClass) */

SDF_Lookahead SDF_setLookaheadCharClass(SDF_Lookahead arg, SDF_CharClass CharClass)
{
  if (SDF_isLookaheadCharClass(arg)) {
    return (SDF_Lookahead)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) CharClass), 0), 1);
  }

  ATabort("Lookahead has no CharClass: %t\n", arg);
  return (SDF_Lookahead)NULL;
}

/*}}}  */
/*{{{  SDF_Lookahead SDF_setLookaheadHead(SDF_Lookahead arg, SDF_CharClass head) */

SDF_Lookahead SDF_setLookaheadHead(SDF_Lookahead arg, SDF_CharClass head)
{
  if (SDF_isLookaheadSeq(arg)) {
    return (SDF_Lookahead)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) head), 0), 1);
  }

  ATabort("Lookahead has no Head: %t\n", arg);
  return (SDF_Lookahead)NULL;
}

/*}}}  */
/*{{{  SDF_Lookahead SDF_setLookaheadWsAfterHead(SDF_Lookahead arg, SDF_OptLayout wsAfterHead) */

SDF_Lookahead SDF_setLookaheadWsAfterHead(SDF_Lookahead arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isLookaheadSeq(arg)) {
    return (SDF_Lookahead)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterHead), 1), 1);
  }

  ATabort("Lookahead has no WsAfterHead: %t\n", arg);
  return (SDF_Lookahead)NULL;
}

/*}}}  */
/*{{{  SDF_Lookahead SDF_setLookaheadWsAfterPeriod(SDF_Lookahead arg, SDF_OptLayout wsAfterPeriod) */

SDF_Lookahead SDF_setLookaheadWsAfterPeriod(SDF_Lookahead arg, SDF_OptLayout wsAfterPeriod)
{
  if (SDF_isLookaheadSeq(arg)) {
    return (SDF_Lookahead)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterPeriod), 3), 1);
  }

  ATabort("Lookahead has no WsAfterPeriod: %t\n", arg);
  return (SDF_Lookahead)NULL;
}

/*}}}  */
/*{{{  SDF_Lookahead SDF_setLookaheadTail(SDF_Lookahead arg, SDF_Lookaheads tail) */

SDF_Lookahead SDF_setLookaheadTail(SDF_Lookahead arg, SDF_Lookaheads tail)
{
  if (SDF_isLookaheadSeq(arg)) {
    return (SDF_Lookahead)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) tail), 4), 1);
  }

  ATabort("Lookahead has no Tail: %t\n", arg);
  return (SDF_Lookahead)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Lookaheads accessors */

/*{{{  ATbool SDF_isValidLookaheads(SDF_Lookaheads arg) */

ATbool SDF_isValidLookaheads(SDF_Lookaheads arg)
{
  if (SDF_isLookaheadsSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isLookaheadsAlt(arg)) {
    return ATtrue;
  }
  else if (SDF_isLookaheadsBracket(arg)) {
    return ATtrue;
  }
  else if (SDF_isLookaheadsList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLookaheadsSingle(SDF_Lookaheads arg) */

inline ATbool SDF_isLookaheadsSingle(SDF_Lookaheads arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLookaheadsSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLookaheadsAlt(SDF_Lookaheads arg) */

inline ATbool SDF_isLookaheadsAlt(SDF_Lookaheads arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLookaheadsAlt, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLookaheadsBracket(SDF_Lookaheads arg) */

inline ATbool SDF_isLookaheadsBracket(SDF_Lookaheads arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLookaheadsBracket, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLookaheadsList(SDF_Lookaheads arg) */

inline ATbool SDF_isLookaheadsList(SDF_Lookaheads arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLookaheadsList, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadsLookahead(SDF_Lookaheads arg) */

ATbool SDF_hasLookaheadsLookahead(SDF_Lookaheads arg)
{
  if (SDF_isLookaheadsSingle(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadsLeft(SDF_Lookaheads arg) */

ATbool SDF_hasLookaheadsLeft(SDF_Lookaheads arg)
{
  if (SDF_isLookaheadsAlt(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadsWsAfterLeft(SDF_Lookaheads arg) */

ATbool SDF_hasLookaheadsWsAfterLeft(SDF_Lookaheads arg)
{
  if (SDF_isLookaheadsAlt(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadsWsAfterBar(SDF_Lookaheads arg) */

ATbool SDF_hasLookaheadsWsAfterBar(SDF_Lookaheads arg)
{
  if (SDF_isLookaheadsAlt(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadsRight(SDF_Lookaheads arg) */

ATbool SDF_hasLookaheadsRight(SDF_Lookaheads arg)
{
  if (SDF_isLookaheadsAlt(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadsWsAfterParenOpen(SDF_Lookaheads arg) */

ATbool SDF_hasLookaheadsWsAfterParenOpen(SDF_Lookaheads arg)
{
  if (SDF_isLookaheadsBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadsLookaheads(SDF_Lookaheads arg) */

ATbool SDF_hasLookaheadsLookaheads(SDF_Lookaheads arg)
{
  if (SDF_isLookaheadsBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadsWsAfterLookaheads(SDF_Lookaheads arg) */

ATbool SDF_hasLookaheadsWsAfterLookaheads(SDF_Lookaheads arg)
{
  if (SDF_isLookaheadsBracket(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadsWsAfterBracketOpenBracketOpen(SDF_Lookaheads arg) */

ATbool SDF_hasLookaheadsWsAfterBracketOpenBracketOpen(SDF_Lookaheads arg)
{
  if (SDF_isLookaheadsList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadsList(SDF_Lookaheads arg) */

ATbool SDF_hasLookaheadsList(SDF_Lookaheads arg)
{
  if (SDF_isLookaheadsList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadsWsAfterList(SDF_Lookaheads arg) */

ATbool SDF_hasLookaheadsWsAfterList(SDF_Lookaheads arg)
{
  if (SDF_isLookaheadsList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_Lookahead SDF_getLookaheadsLookahead(SDF_Lookaheads arg) */

SDF_Lookahead SDF_getLookaheadsLookahead(SDF_Lookaheads arg)
{
  
    return (SDF_Lookahead)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_getLookaheadsLeft(SDF_Lookaheads arg) */

SDF_Lookaheads SDF_getLookaheadsLeft(SDF_Lookaheads arg)
{
  
    return (SDF_Lookaheads)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getLookaheadsWsAfterLeft(SDF_Lookaheads arg) */

SDF_OptLayout SDF_getLookaheadsWsAfterLeft(SDF_Lookaheads arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getLookaheadsWsAfterBar(SDF_Lookaheads arg) */

SDF_OptLayout SDF_getLookaheadsWsAfterBar(SDF_Lookaheads arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_getLookaheadsRight(SDF_Lookaheads arg) */

SDF_Lookaheads SDF_getLookaheadsRight(SDF_Lookaheads arg)
{
  
    return (SDF_Lookaheads)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getLookaheadsWsAfterParenOpen(SDF_Lookaheads arg) */

SDF_OptLayout SDF_getLookaheadsWsAfterParenOpen(SDF_Lookaheads arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_getLookaheadsLookaheads(SDF_Lookaheads arg) */

SDF_Lookaheads SDF_getLookaheadsLookaheads(SDF_Lookaheads arg)
{
  
    return (SDF_Lookaheads)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getLookaheadsWsAfterLookaheads(SDF_Lookaheads arg) */

SDF_OptLayout SDF_getLookaheadsWsAfterLookaheads(SDF_Lookaheads arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getLookaheadsWsAfterBracketOpenBracketOpen(SDF_Lookaheads arg) */

SDF_OptLayout SDF_getLookaheadsWsAfterBracketOpenBracketOpen(SDF_Lookaheads arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_LookaheadList SDF_getLookaheadsList(SDF_Lookaheads arg) */

SDF_LookaheadList SDF_getLookaheadsList(SDF_Lookaheads arg)
{
  
    return (SDF_LookaheadList)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getLookaheadsWsAfterList(SDF_Lookaheads arg) */

SDF_OptLayout SDF_getLookaheadsWsAfterList(SDF_Lookaheads arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_setLookaheadsLookahead(SDF_Lookaheads arg, SDF_Lookahead Lookahead) */

SDF_Lookaheads SDF_setLookaheadsLookahead(SDF_Lookaheads arg, SDF_Lookahead Lookahead)
{
  if (SDF_isLookaheadsSingle(arg)) {
    return (SDF_Lookaheads)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Lookahead), 0), 1);
  }

  ATabort("Lookaheads has no Lookahead: %t\n", arg);
  return (SDF_Lookaheads)NULL;
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_setLookaheadsLeft(SDF_Lookaheads arg, SDF_Lookaheads left) */

SDF_Lookaheads SDF_setLookaheadsLeft(SDF_Lookaheads arg, SDF_Lookaheads left)
{
  if (SDF_isLookaheadsAlt(arg)) {
    return (SDF_Lookaheads)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) left), 0), 1);
  }

  ATabort("Lookaheads has no Left: %t\n", arg);
  return (SDF_Lookaheads)NULL;
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_setLookaheadsWsAfterLeft(SDF_Lookaheads arg, SDF_OptLayout wsAfterLeft) */

SDF_Lookaheads SDF_setLookaheadsWsAfterLeft(SDF_Lookaheads arg, SDF_OptLayout wsAfterLeft)
{
  if (SDF_isLookaheadsAlt(arg)) {
    return (SDF_Lookaheads)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLeft), 1), 1);
  }

  ATabort("Lookaheads has no WsAfterLeft: %t\n", arg);
  return (SDF_Lookaheads)NULL;
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_setLookaheadsWsAfterBar(SDF_Lookaheads arg, SDF_OptLayout wsAfterBar) */

SDF_Lookaheads SDF_setLookaheadsWsAfterBar(SDF_Lookaheads arg, SDF_OptLayout wsAfterBar)
{
  if (SDF_isLookaheadsAlt(arg)) {
    return (SDF_Lookaheads)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBar), 3), 1);
  }

  ATabort("Lookaheads has no WsAfterBar: %t\n", arg);
  return (SDF_Lookaheads)NULL;
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_setLookaheadsRight(SDF_Lookaheads arg, SDF_Lookaheads right) */

SDF_Lookaheads SDF_setLookaheadsRight(SDF_Lookaheads arg, SDF_Lookaheads right)
{
  if (SDF_isLookaheadsAlt(arg)) {
    return (SDF_Lookaheads)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) right), 4), 1);
  }

  ATabort("Lookaheads has no Right: %t\n", arg);
  return (SDF_Lookaheads)NULL;
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_setLookaheadsWsAfterParenOpen(SDF_Lookaheads arg, SDF_OptLayout wsAfterParenOpen) */

SDF_Lookaheads SDF_setLookaheadsWsAfterParenOpen(SDF_Lookaheads arg, SDF_OptLayout wsAfterParenOpen)
{
  if (SDF_isLookaheadsBracket(arg)) {
    return (SDF_Lookaheads)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterParenOpen), 1), 1);
  }

  ATabort("Lookaheads has no WsAfterParenOpen: %t\n", arg);
  return (SDF_Lookaheads)NULL;
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_setLookaheadsLookaheads(SDF_Lookaheads arg, SDF_Lookaheads Lookaheads) */

SDF_Lookaheads SDF_setLookaheadsLookaheads(SDF_Lookaheads arg, SDF_Lookaheads Lookaheads)
{
  if (SDF_isLookaheadsBracket(arg)) {
    return (SDF_Lookaheads)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Lookaheads), 2), 1);
  }

  ATabort("Lookaheads has no Lookaheads: %t\n", arg);
  return (SDF_Lookaheads)NULL;
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_setLookaheadsWsAfterLookaheads(SDF_Lookaheads arg, SDF_OptLayout wsAfterLookaheads) */

SDF_Lookaheads SDF_setLookaheadsWsAfterLookaheads(SDF_Lookaheads arg, SDF_OptLayout wsAfterLookaheads)
{
  if (SDF_isLookaheadsBracket(arg)) {
    return (SDF_Lookaheads)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterLookaheads), 3), 1);
  }

  ATabort("Lookaheads has no WsAfterLookaheads: %t\n", arg);
  return (SDF_Lookaheads)NULL;
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_setLookaheadsWsAfterBracketOpenBracketOpen(SDF_Lookaheads arg, SDF_OptLayout wsAfterBracketOpenBracketOpen) */

SDF_Lookaheads SDF_setLookaheadsWsAfterBracketOpenBracketOpen(SDF_Lookaheads arg, SDF_OptLayout wsAfterBracketOpenBracketOpen)
{
  if (SDF_isLookaheadsList(arg)) {
    return (SDF_Lookaheads)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterBracketOpenBracketOpen), 1), 1);
  }

  ATabort("Lookaheads has no WsAfterBracketOpenBracketOpen: %t\n", arg);
  return (SDF_Lookaheads)NULL;
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_setLookaheadsList(SDF_Lookaheads arg, SDF_LookaheadList list) */

SDF_Lookaheads SDF_setLookaheadsList(SDF_Lookaheads arg, SDF_LookaheadList list)
{
  if (SDF_isLookaheadsList(arg)) {
    return (SDF_Lookaheads)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2), (ATerm)((ATerm) list), 1), 2), 1);
  }

  ATabort("Lookaheads has no List: %t\n", arg);
  return (SDF_Lookaheads)NULL;
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_setLookaheadsWsAfterList(SDF_Lookaheads arg, SDF_OptLayout wsAfterList) */

SDF_Lookaheads SDF_setLookaheadsWsAfterList(SDF_Lookaheads arg, SDF_OptLayout wsAfterList)
{
  if (SDF_isLookaheadsList(arg)) {
    return (SDF_Lookaheads)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterList), 3), 1);
  }

  ATabort("Lookaheads has no WsAfterList: %t\n", arg);
  return (SDF_Lookaheads)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Restriction accessors */

/*{{{  ATbool SDF_isValidRestriction(SDF_Restriction arg) */

ATbool SDF_isValidRestriction(SDF_Restriction arg)
{
  if (SDF_isRestrictionFollow(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isRestrictionFollow(SDF_Restriction arg) */

inline ATbool SDF_isRestrictionFollow(SDF_Restriction arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternRestrictionFollow, NULL, NULL, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasRestrictionSymbols(SDF_Restriction arg) */

ATbool SDF_hasRestrictionSymbols(SDF_Restriction arg)
{
  if (SDF_isRestrictionFollow(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRestrictionWsAfterSymbols(SDF_Restriction arg) */

ATbool SDF_hasRestrictionWsAfterSymbols(SDF_Restriction arg)
{
  if (SDF_isRestrictionFollow(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRestrictionWsAfterSlash(SDF_Restriction arg) */

ATbool SDF_hasRestrictionWsAfterSlash(SDF_Restriction arg)
{
  if (SDF_isRestrictionFollow(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRestrictionLookaheads(SDF_Restriction arg) */

ATbool SDF_hasRestrictionLookaheads(SDF_Restriction arg)
{
  if (SDF_isRestrictionFollow(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_Symbols SDF_getRestrictionSymbols(SDF_Restriction arg) */

SDF_Symbols SDF_getRestrictionSymbols(SDF_Restriction arg)
{
  
    return (SDF_Symbols)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getRestrictionWsAfterSymbols(SDF_Restriction arg) */

SDF_OptLayout SDF_getRestrictionWsAfterSymbols(SDF_Restriction arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getRestrictionWsAfterSlash(SDF_Restriction arg) */

SDF_OptLayout SDF_getRestrictionWsAfterSlash(SDF_Restriction arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3);
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_getRestrictionLookaheads(SDF_Restriction arg) */

SDF_Lookaheads SDF_getRestrictionLookaheads(SDF_Restriction arg)
{
  
    return (SDF_Lookaheads)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 4);
}

/*}}}  */
/*{{{  SDF_Restriction SDF_setRestrictionSymbols(SDF_Restriction arg, SDF_Symbols Symbols) */

SDF_Restriction SDF_setRestrictionSymbols(SDF_Restriction arg, SDF_Symbols Symbols)
{
  if (SDF_isRestrictionFollow(arg)) {
    return (SDF_Restriction)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Symbols), 0), 1);
  }

  ATabort("Restriction has no Symbols: %t\n", arg);
  return (SDF_Restriction)NULL;
}

/*}}}  */
/*{{{  SDF_Restriction SDF_setRestrictionWsAfterSymbols(SDF_Restriction arg, SDF_OptLayout wsAfterSymbols) */

SDF_Restriction SDF_setRestrictionWsAfterSymbols(SDF_Restriction arg, SDF_OptLayout wsAfterSymbols)
{
  if (SDF_isRestrictionFollow(arg)) {
    return (SDF_Restriction)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSymbols), 1), 1);
  }

  ATabort("Restriction has no WsAfterSymbols: %t\n", arg);
  return (SDF_Restriction)NULL;
}

/*}}}  */
/*{{{  SDF_Restriction SDF_setRestrictionWsAfterSlash(SDF_Restriction arg, SDF_OptLayout wsAfterSlash) */

SDF_Restriction SDF_setRestrictionWsAfterSlash(SDF_Restriction arg, SDF_OptLayout wsAfterSlash)
{
  if (SDF_isRestrictionFollow(arg)) {
    return (SDF_Restriction)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterSlash), 3), 1);
  }

  ATabort("Restriction has no WsAfterSlash: %t\n", arg);
  return (SDF_Restriction)NULL;
}

/*}}}  */
/*{{{  SDF_Restriction SDF_setRestrictionLookaheads(SDF_Restriction arg, SDF_Lookaheads Lookaheads) */

SDF_Restriction SDF_setRestrictionLookaheads(SDF_Restriction arg, SDF_Lookaheads Lookaheads)
{
  if (SDF_isRestrictionFollow(arg)) {
    return (SDF_Restriction)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Lookaheads), 4), 1);
  }

  ATabort("Restriction has no Lookaheads: %t\n", arg);
  return (SDF_Restriction)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Restrictions accessors */

/*{{{  ATbool SDF_isValidRestrictions(SDF_Restrictions arg) */

ATbool SDF_isValidRestrictions(SDF_Restrictions arg)
{
  if (SDF_isRestrictionsDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isRestrictionsDefault(SDF_Restrictions arg) */

inline ATbool SDF_isRestrictionsDefault(SDF_Restrictions arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternRestrictionsDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasRestrictionsList(SDF_Restrictions arg) */

ATbool SDF_hasRestrictionsList(SDF_Restrictions arg)
{
  if (SDF_isRestrictionsDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_RestrictionList SDF_getRestrictionsList(SDF_Restrictions arg) */

SDF_RestrictionList SDF_getRestrictionsList(SDF_Restrictions arg)
{
  
    return (SDF_RestrictionList)ATgetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), 1);
}

/*}}}  */
/*{{{  SDF_Restrictions SDF_setRestrictionsList(SDF_Restrictions arg, SDF_RestrictionList list) */

SDF_Restrictions SDF_setRestrictionsList(SDF_Restrictions arg, SDF_RestrictionList list)
{
  if (SDF_isRestrictionsDefault(arg)) {
    return (SDF_Restrictions)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), (ATerm)((ATerm) list), 1), 0), 1);
  }

  ATabort("Restrictions has no List: %t\n", arg);
  return (SDF_Restrictions)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_LookaheadList accessors */

/*{{{  ATbool SDF_isValidLookaheadList(SDF_LookaheadList arg) */

ATbool SDF_isValidLookaheadList(SDF_LookaheadList arg)
{
  if (SDF_isLookaheadListEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isLookaheadListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isLookaheadListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLookaheadListEmpty(SDF_LookaheadList arg) */

inline ATbool SDF_isLookaheadListEmpty(SDF_LookaheadList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternLookaheadListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLookaheadListSingle(SDF_LookaheadList arg) */

inline ATbool SDF_isLookaheadListSingle(SDF_LookaheadList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLookaheadListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLookaheadListMany(SDF_LookaheadList arg) */

inline ATbool SDF_isLookaheadListMany(SDF_LookaheadList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLookaheadListMany, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadListHead(SDF_LookaheadList arg) */

ATbool SDF_hasLookaheadListHead(SDF_LookaheadList arg)
{
  if (SDF_isLookaheadListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isLookaheadListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadListWsAfterHead(SDF_LookaheadList arg) */

ATbool SDF_hasLookaheadListWsAfterHead(SDF_LookaheadList arg)
{
  if (SDF_isLookaheadListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadListWsAfterSep(SDF_LookaheadList arg) */

ATbool SDF_hasLookaheadListWsAfterSep(SDF_LookaheadList arg)
{
  if (SDF_isLookaheadListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLookaheadListTail(SDF_LookaheadList arg) */

ATbool SDF_hasLookaheadListTail(SDF_LookaheadList arg)
{
  if (SDF_isLookaheadListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_LookaheadList SDF_getLookaheadListTail(SDF_LookaheadList arg) */

SDF_LookaheadList SDF_getLookaheadListTail(SDF_LookaheadList arg)
{
  assert(!SDF_isLookaheadListEmpty(arg) && "getTail on an empty list");
  if (SDF_isLookaheadListSingle(arg)) {
    return (SDF_LookaheadList) SDF_makeLookaheadListEmpty();
  }
  else {
  
    return (SDF_LookaheadList)ATgetTail((ATermList)arg, 4);
  }
}

/*}}}  */
/*{{{  SDF_Lookahead SDF_getLookaheadListHead(SDF_LookaheadList arg) */

SDF_Lookahead SDF_getLookaheadListHead(SDF_LookaheadList arg)
{
  if (SDF_isLookaheadListSingle(arg)) {
    return (SDF_Lookahead)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Lookahead)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getLookaheadListWsAfterHead(SDF_LookaheadList arg) */

SDF_OptLayout SDF_getLookaheadListWsAfterHead(SDF_LookaheadList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getLookaheadListWsAfterSep(SDF_LookaheadList arg) */

SDF_OptLayout SDF_getLookaheadListWsAfterSep(SDF_LookaheadList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 3);
}

/*}}}  */
/*{{{  SDF_LookaheadList SDF_setLookaheadListHead(SDF_LookaheadList arg, SDF_Lookahead head) */

SDF_LookaheadList SDF_setLookaheadListHead(SDF_LookaheadList arg, SDF_Lookahead head)
{
  if (SDF_isLookaheadListSingle(arg)) {
    return (SDF_LookaheadList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isLookaheadListMany(arg)) {
    return (SDF_LookaheadList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("LookaheadList has no Head: %t\n", arg);
  return (SDF_LookaheadList)NULL;
}

/*}}}  */
/*{{{  SDF_LookaheadList SDF_setLookaheadListWsAfterHead(SDF_LookaheadList arg, SDF_OptLayout wsAfterHead) */

SDF_LookaheadList SDF_setLookaheadListWsAfterHead(SDF_LookaheadList arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isLookaheadListMany(arg)) {
    return (SDF_LookaheadList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("LookaheadList has no WsAfterHead: %t\n", arg);
  return (SDF_LookaheadList)NULL;
}

/*}}}  */
/*{{{  SDF_LookaheadList SDF_setLookaheadListWsAfterSep(SDF_LookaheadList arg, SDF_OptLayout wsAfterSep) */

SDF_LookaheadList SDF_setLookaheadListWsAfterSep(SDF_LookaheadList arg, SDF_OptLayout wsAfterSep)
{
  if (SDF_isLookaheadListMany(arg)) {
    return (SDF_LookaheadList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterSep), 3);
  }

  ATabort("LookaheadList has no WsAfterSep: %t\n", arg);
  return (SDF_LookaheadList)NULL;
}

/*}}}  */
/*{{{  SDF_LookaheadList SDF_setLookaheadListTail(SDF_LookaheadList arg, SDF_LookaheadList tail) */

SDF_LookaheadList SDF_setLookaheadListTail(SDF_LookaheadList arg, SDF_LookaheadList tail)
{
  if (SDF_isLookaheadListMany(arg)) {
    return (SDF_LookaheadList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 4);
  }

  ATabort("LookaheadList has no Tail: %t\n", arg);
  return (SDF_LookaheadList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_RestrictionList accessors */

/*{{{  ATbool SDF_isValidRestrictionList(SDF_RestrictionList arg) */

ATbool SDF_isValidRestrictionList(SDF_RestrictionList arg)
{
  if (SDF_isRestrictionListEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isRestrictionListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isRestrictionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isRestrictionListEmpty(SDF_RestrictionList arg) */

inline ATbool SDF_isRestrictionListEmpty(SDF_RestrictionList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternRestrictionListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isRestrictionListSingle(SDF_RestrictionList arg) */

inline ATbool SDF_isRestrictionListSingle(SDF_RestrictionList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternRestrictionListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isRestrictionListMany(SDF_RestrictionList arg) */

inline ATbool SDF_isRestrictionListMany(SDF_RestrictionList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternRestrictionListMany, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasRestrictionListHead(SDF_RestrictionList arg) */

ATbool SDF_hasRestrictionListHead(SDF_RestrictionList arg)
{
  if (SDF_isRestrictionListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isRestrictionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRestrictionListWsAfterHead(SDF_RestrictionList arg) */

ATbool SDF_hasRestrictionListWsAfterHead(SDF_RestrictionList arg)
{
  if (SDF_isRestrictionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasRestrictionListTail(SDF_RestrictionList arg) */

ATbool SDF_hasRestrictionListTail(SDF_RestrictionList arg)
{
  if (SDF_isRestrictionListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_RestrictionList SDF_getRestrictionListTail(SDF_RestrictionList arg) */

SDF_RestrictionList SDF_getRestrictionListTail(SDF_RestrictionList arg)
{
  assert(!SDF_isRestrictionListEmpty(arg) && "getTail on an empty list");
  if (SDF_isRestrictionListSingle(arg)) {
    return (SDF_RestrictionList) SDF_makeRestrictionListEmpty();
  }
  else {
  
    return (SDF_RestrictionList)ATgetTail((ATermList)arg, 2);
  }
}

/*}}}  */
/*{{{  SDF_Restriction SDF_getRestrictionListHead(SDF_RestrictionList arg) */

SDF_Restriction SDF_getRestrictionListHead(SDF_RestrictionList arg)
{
  if (SDF_isRestrictionListSingle(arg)) {
    return (SDF_Restriction)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Restriction)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getRestrictionListWsAfterHead(SDF_RestrictionList arg) */

SDF_OptLayout SDF_getRestrictionListWsAfterHead(SDF_RestrictionList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_RestrictionList SDF_setRestrictionListHead(SDF_RestrictionList arg, SDF_Restriction head) */

SDF_RestrictionList SDF_setRestrictionListHead(SDF_RestrictionList arg, SDF_Restriction head)
{
  if (SDF_isRestrictionListSingle(arg)) {
    return (SDF_RestrictionList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isRestrictionListMany(arg)) {
    return (SDF_RestrictionList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("RestrictionList has no Head: %t\n", arg);
  return (SDF_RestrictionList)NULL;
}

/*}}}  */
/*{{{  SDF_RestrictionList SDF_setRestrictionListWsAfterHead(SDF_RestrictionList arg, SDF_OptLayout wsAfterHead) */

SDF_RestrictionList SDF_setRestrictionListWsAfterHead(SDF_RestrictionList arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isRestrictionListMany(arg)) {
    return (SDF_RestrictionList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("RestrictionList has no WsAfterHead: %t\n", arg);
  return (SDF_RestrictionList)NULL;
}

/*}}}  */
/*{{{  SDF_RestrictionList SDF_setRestrictionListTail(SDF_RestrictionList arg, SDF_RestrictionList tail) */

SDF_RestrictionList SDF_setRestrictionListTail(SDF_RestrictionList arg, SDF_RestrictionList tail)
{
  if (SDF_isRestrictionListMany(arg)) {
    return (SDF_RestrictionList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 2);
  }

  ATabort("RestrictionList has no Tail: %t\n", arg);
  return (SDF_RestrictionList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_SDF accessors */

/*{{{  ATbool SDF_isValidSDF(SDF_SDF arg) */

ATbool SDF_isValidSDF(SDF_SDF arg)
{
  if (SDF_isSDFDefinition(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSDFDefinition(SDF_SDF arg) */

inline ATbool SDF_isSDFDefinition(SDF_SDF arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternSDFDefinition, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasSDFWsAfterDefinition(SDF_SDF arg) */

ATbool SDF_hasSDFWsAfterDefinition(SDF_SDF arg)
{
  if (SDF_isSDFDefinition(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSDFDefinition(SDF_SDF arg) */

ATbool SDF_hasSDFDefinition(SDF_SDF arg)
{
  if (SDF_isSDFDefinition(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSDFWsAfterDefinition(SDF_SDF arg) */

SDF_OptLayout SDF_getSDFWsAfterDefinition(SDF_SDF arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1);
}

/*}}}  */
/*{{{  SDF_Definition SDF_getSDFDefinition(SDF_SDF arg) */

SDF_Definition SDF_getSDFDefinition(SDF_SDF arg)
{
  
    return (SDF_Definition)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2);
}

/*}}}  */
/*{{{  SDF_SDF SDF_setSDFWsAfterDefinition(SDF_SDF arg, SDF_OptLayout wsAfterDefinition) */

SDF_SDF SDF_setSDFWsAfterDefinition(SDF_SDF arg, SDF_OptLayout wsAfterDefinition)
{
  if (SDF_isSDFDefinition(arg)) {
    return (SDF_SDF)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) wsAfterDefinition), 1), 1);
  }

  ATabort("SDF has no WsAfterDefinition: %t\n", arg);
  return (SDF_SDF)NULL;
}

/*}}}  */
/*{{{  SDF_SDF SDF_setSDFDefinition(SDF_SDF arg, SDF_Definition Definition) */

SDF_SDF SDF_setSDFDefinition(SDF_SDF arg, SDF_Definition Definition)
{
  if (SDF_isSDFDefinition(arg)) {
    return (SDF_SDF)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Definition), 2), 1);
  }

  ATabort("SDF has no Definition: %t\n", arg);
  return (SDF_SDF)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Sort accessors */

/*{{{  ATbool SDF_isValidSort(SDF_Sort arg) */

ATbool SDF_isValidSort(SDF_Sort arg)
{
  if (SDF_isSortLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSortLexToCf(SDF_Sort arg) */

inline ATbool SDF_isSortLexToCf(SDF_Sort arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternSortLexToCf, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasSortSort(SDF_Sort arg) */

ATbool SDF_hasSortSort(SDF_Sort arg)
{
  if (SDF_isSortLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_LexSort SDF_getSortSort(SDF_Sort arg) */

SDF_LexSort SDF_getSortSort(SDF_Sort arg)
{
  
    return (SDF_LexSort)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_Sort SDF_setSortSort(SDF_Sort arg, SDF_LexSort Sort) */

SDF_Sort SDF_setSortSort(SDF_Sort arg, SDF_LexSort Sort)
{
  if (SDF_isSortLexToCf(arg)) {
    return (SDF_Sort)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) Sort), 0), 1);
  }

  ATabort("Sort has no Sort: %t\n", arg);
  return (SDF_Sort)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_LexSort accessors */

/*{{{  ATbool SDF_isValidLexSort(SDF_LexSort arg) */

ATbool SDF_isValidLexSort(SDF_LexSort arg)
{
  if (SDF_isLexSortOneChar(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexSortMoreChars(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexSortOneChar(SDF_LexSort arg) */

inline ATbool SDF_isLexSortOneChar(SDF_LexSort arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexSortOneChar, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexSortMoreChars(SDF_LexSort arg) */

inline ATbool SDF_isLexSortMoreChars(SDF_LexSort arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexSortMoreChars, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasLexSortHead(SDF_LexSort arg) */

ATbool SDF_hasLexSortHead(SDF_LexSort arg)
{
  if (SDF_isLexSortOneChar(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexSortMoreChars(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexSortMiddle(SDF_LexSort arg) */

ATbool SDF_hasLexSortMiddle(SDF_LexSort arg)
{
  if (SDF_isLexSortMoreChars(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexSortLast(SDF_LexSort arg) */

ATbool SDF_hasLexSortLast(SDF_LexSort arg)
{
  if (SDF_isLexSortMoreChars(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char SDF_getLexSortHead(SDF_LexSort arg) */

char SDF_getLexSortHead(SDF_LexSort arg)
{
  if (SDF_isLexSortOneChar(arg)) {
    return (char)SDF_charToByte((ATerm)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)));
  }
  else 
    return (char)SDF_charToByte((ATerm)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)));
}

/*}}}  */
/*{{{  char* SDF_getLexSortMiddle(SDF_LexSort arg) */

char* SDF_getLexSortMiddle(SDF_LexSort arg)
{
  
    return (char*)SDF_charsToString((ATerm)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1), 1));
}

/*}}}  */
/*{{{  char SDF_getLexSortLast(SDF_LexSort arg) */

char SDF_getLexSortLast(SDF_LexSort arg)
{
  
    return (char)SDF_charToByte((ATerm)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2));
}

/*}}}  */
/*{{{  SDF_LexSort SDF_setLexSortHead(SDF_LexSort arg, char head) */

SDF_LexSort SDF_setLexSortHead(SDF_LexSort arg, char head)
{
  if (SDF_isLexSortOneChar(arg)) {
    return (SDF_LexSort)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ((ATerm) SDF_byteToChar(head))), 0), 1);
  }
  else if (SDF_isLexSortMoreChars(arg)) {
    return (SDF_LexSort)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ((ATerm) SDF_byteToChar(head))), 0), 1);
  }

  ATabort("LexSort has no Head: %t\n", arg);
  return (SDF_LexSort)NULL;
}

/*}}}  */
/*{{{  SDF_LexSort SDF_setLexSortMiddle(SDF_LexSort arg, const char* middle) */

SDF_LexSort SDF_setLexSortMiddle(SDF_LexSort arg, const char* middle)
{
  if (SDF_isLexSortMoreChars(arg)) {
    return (SDF_LexSort)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1), (ATerm)((ATerm) ((ATerm) SDF_stringToChars(middle))), 1), 1), 1);
  }

  ATabort("LexSort has no Middle: %t\n", arg);
  return (SDF_LexSort)NULL;
}

/*}}}  */
/*{{{  SDF_LexSort SDF_setLexSortLast(SDF_LexSort arg, char last) */

SDF_LexSort SDF_setLexSortLast(SDF_LexSort arg, char last)
{
  if (SDF_isLexSortMoreChars(arg)) {
    return (SDF_LexSort)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ((ATerm) SDF_byteToChar(last))), 2), 1);
  }

  ATabort("LexSort has no Last: %t\n", arg);
  return (SDF_LexSort)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_SymbolParameters accessors */

/*{{{  ATbool SDF_isValidSymbolParameters(SDF_SymbolParameters arg) */

ATbool SDF_isValidSymbolParameters(SDF_SymbolParameters arg)
{
  if (SDF_isSymbolParametersEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolParametersSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolParametersMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolParametersEmpty(SDF_SymbolParameters arg) */

inline ATbool SDF_isSymbolParametersEmpty(SDF_SymbolParameters arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternSymbolParametersEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolParametersSingle(SDF_SymbolParameters arg) */

inline ATbool SDF_isSymbolParametersSingle(SDF_SymbolParameters arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolParametersSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolParametersMany(SDF_SymbolParameters arg) */

inline ATbool SDF_isSymbolParametersMany(SDF_SymbolParameters arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolParametersMany, NULL, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolParametersHead(SDF_SymbolParameters arg) */

ATbool SDF_hasSymbolParametersHead(SDF_SymbolParameters arg)
{
  if (SDF_isSymbolParametersSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolParametersMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolParametersWsAfterHead(SDF_SymbolParameters arg) */

ATbool SDF_hasSymbolParametersWsAfterHead(SDF_SymbolParameters arg)
{
  if (SDF_isSymbolParametersMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolParametersWsAfterSep(SDF_SymbolParameters arg) */

ATbool SDF_hasSymbolParametersWsAfterSep(SDF_SymbolParameters arg)
{
  if (SDF_isSymbolParametersMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolParametersTail(SDF_SymbolParameters arg) */

ATbool SDF_hasSymbolParametersTail(SDF_SymbolParameters arg)
{
  if (SDF_isSymbolParametersMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_SymbolParameters SDF_getSymbolParametersTail(SDF_SymbolParameters arg) */

SDF_SymbolParameters SDF_getSymbolParametersTail(SDF_SymbolParameters arg)
{
  assert(!SDF_isSymbolParametersEmpty(arg) && "getTail on an empty list");
  if (SDF_isSymbolParametersSingle(arg)) {
    return (SDF_SymbolParameters) SDF_makeSymbolParametersEmpty();
  }
  else {
  
    return (SDF_SymbolParameters)ATgetTail((ATermList)arg, 4);
  }
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getSymbolParametersHead(SDF_SymbolParameters arg) */

SDF_Symbol SDF_getSymbolParametersHead(SDF_SymbolParameters arg)
{
  if (SDF_isSymbolParametersSingle(arg)) {
    return (SDF_Symbol)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Symbol)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolParametersWsAfterHead(SDF_SymbolParameters arg) */

SDF_OptLayout SDF_getSymbolParametersWsAfterHead(SDF_SymbolParameters arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolParametersWsAfterSep(SDF_SymbolParameters arg) */

SDF_OptLayout SDF_getSymbolParametersWsAfterSep(SDF_SymbolParameters arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 3);
}

/*}}}  */
/*{{{  SDF_SymbolParameters SDF_setSymbolParametersHead(SDF_SymbolParameters arg, SDF_Symbol head) */

SDF_SymbolParameters SDF_setSymbolParametersHead(SDF_SymbolParameters arg, SDF_Symbol head)
{
  if (SDF_isSymbolParametersSingle(arg)) {
    return (SDF_SymbolParameters)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isSymbolParametersMany(arg)) {
    return (SDF_SymbolParameters)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("SymbolParameters has no Head: %t\n", arg);
  return (SDF_SymbolParameters)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolParameters SDF_setSymbolParametersWsAfterHead(SDF_SymbolParameters arg, SDF_OptLayout wsAfterHead) */

SDF_SymbolParameters SDF_setSymbolParametersWsAfterHead(SDF_SymbolParameters arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isSymbolParametersMany(arg)) {
    return (SDF_SymbolParameters)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("SymbolParameters has no WsAfterHead: %t\n", arg);
  return (SDF_SymbolParameters)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolParameters SDF_setSymbolParametersWsAfterSep(SDF_SymbolParameters arg, SDF_OptLayout wsAfterSep) */

SDF_SymbolParameters SDF_setSymbolParametersWsAfterSep(SDF_SymbolParameters arg, SDF_OptLayout wsAfterSep)
{
  if (SDF_isSymbolParametersMany(arg)) {
    return (SDF_SymbolParameters)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterSep), 3);
  }

  ATabort("SymbolParameters has no WsAfterSep: %t\n", arg);
  return (SDF_SymbolParameters)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolParameters SDF_setSymbolParametersTail(SDF_SymbolParameters arg, SDF_SymbolParameters tail) */

SDF_SymbolParameters SDF_setSymbolParametersTail(SDF_SymbolParameters arg, SDF_SymbolParameters tail)
{
  if (SDF_isSymbolParametersMany(arg)) {
    return (SDF_SymbolParameters)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 4);
  }

  ATabort("SymbolParameters has no Tail: %t\n", arg);
  return (SDF_SymbolParameters)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_Symbols accessors */

/*{{{  ATbool SDF_isValidSymbols(SDF_Symbols arg) */

ATbool SDF_isValidSymbols(SDF_Symbols arg)
{
  if (SDF_isSymbolsDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolsDefault(SDF_Symbols arg) */

inline ATbool SDF_isSymbolsDefault(SDF_Symbols arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternSymbolsDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolsList(SDF_Symbols arg) */

ATbool SDF_hasSymbolsList(SDF_Symbols arg)
{
  if (SDF_isSymbolsDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_SymbolList SDF_getSymbolsList(SDF_Symbols arg) */

SDF_SymbolList SDF_getSymbolsList(SDF_Symbols arg)
{
  
    return (SDF_SymbolList)ATgetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), 1);
}

/*}}}  */
/*{{{  SDF_Symbols SDF_setSymbolsList(SDF_Symbols arg, SDF_SymbolList list) */

SDF_Symbols SDF_setSymbolsList(SDF_Symbols arg, SDF_SymbolList list)
{
  if (SDF_isSymbolsDefault(arg)) {
    return (SDF_Symbols)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), (ATerm)((ATerm) list), 1), 0), 1);
  }

  ATabort("Symbols has no List: %t\n", arg);
  return (SDF_Symbols)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_SymbolList accessors */

/*{{{  ATbool SDF_isValidSymbolList(SDF_SymbolList arg) */

ATbool SDF_isValidSymbolList(SDF_SymbolList arg)
{
  if (SDF_isSymbolListEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolListEmpty(SDF_SymbolList arg) */

inline ATbool SDF_isSymbolListEmpty(SDF_SymbolList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternSymbolListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolListSingle(SDF_SymbolList arg) */

inline ATbool SDF_isSymbolListSingle(SDF_SymbolList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolListSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isSymbolListMany(SDF_SymbolList arg) */

inline ATbool SDF_isSymbolListMany(SDF_SymbolList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternSymbolListMany, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolListHead(SDF_SymbolList arg) */

ATbool SDF_hasSymbolListHead(SDF_SymbolList arg)
{
  if (SDF_isSymbolListSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isSymbolListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolListWsAfterHead(SDF_SymbolList arg) */

ATbool SDF_hasSymbolListWsAfterHead(SDF_SymbolList arg)
{
  if (SDF_isSymbolListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasSymbolListTail(SDF_SymbolList arg) */

ATbool SDF_hasSymbolListTail(SDF_SymbolList arg)
{
  if (SDF_isSymbolListMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_SymbolList SDF_getSymbolListTail(SDF_SymbolList arg) */

SDF_SymbolList SDF_getSymbolListTail(SDF_SymbolList arg)
{
  assert(!SDF_isSymbolListEmpty(arg) && "getTail on an empty list");
  if (SDF_isSymbolListSingle(arg)) {
    return (SDF_SymbolList) SDF_makeSymbolListEmpty();
  }
  else {
  
    return (SDF_SymbolList)ATgetTail((ATermList)arg, 2);
  }
}

/*}}}  */
/*{{{  SDF_Symbol SDF_getSymbolListHead(SDF_SymbolList arg) */

SDF_Symbol SDF_getSymbolListHead(SDF_SymbolList arg)
{
  if (SDF_isSymbolListSingle(arg)) {
    return (SDF_Symbol)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_Symbol)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_OptLayout SDF_getSymbolListWsAfterHead(SDF_SymbolList arg) */

SDF_OptLayout SDF_getSymbolListWsAfterHead(SDF_SymbolList arg)
{
  
    return (SDF_OptLayout)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  SDF_SymbolList SDF_setSymbolListHead(SDF_SymbolList arg, SDF_Symbol head) */

SDF_SymbolList SDF_setSymbolListHead(SDF_SymbolList arg, SDF_Symbol head)
{
  if (SDF_isSymbolListSingle(arg)) {
    return (SDF_SymbolList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isSymbolListMany(arg)) {
    return (SDF_SymbolList)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("SymbolList has no Head: %t\n", arg);
  return (SDF_SymbolList)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolList SDF_setSymbolListWsAfterHead(SDF_SymbolList arg, SDF_OptLayout wsAfterHead) */

SDF_SymbolList SDF_setSymbolListWsAfterHead(SDF_SymbolList arg, SDF_OptLayout wsAfterHead)
{
  if (SDF_isSymbolListMany(arg)) {
    return (SDF_SymbolList)ATreplace((ATermList)arg, (ATerm)((ATerm) wsAfterHead), 1);
  }

  ATabort("SymbolList has no WsAfterHead: %t\n", arg);
  return (SDF_SymbolList)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolList SDF_setSymbolListTail(SDF_SymbolList arg, SDF_SymbolList tail) */

SDF_SymbolList SDF_setSymbolListTail(SDF_SymbolList arg, SDF_SymbolList tail)
{
  if (SDF_isSymbolListMany(arg)) {
    return (SDF_SymbolList)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 2);
  }

  ATabort("SymbolList has no Tail: %t\n", arg);
  return (SDF_SymbolList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_LexLayout accessors */

/*{{{  ATbool SDF_isValidLexLayout(SDF_LexLayout arg) */

ATbool SDF_isValidLexLayout(SDF_LexLayout arg)
{
  if (SDF_isLexLayoutWhitespace(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexLayoutLine(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexLayoutNested(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexLayoutWhitespace(SDF_LexLayout arg) */

inline ATbool SDF_isLexLayoutWhitespace(SDF_LexLayout arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexLayoutWhitespace, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexLayoutLine(SDF_LexLayout arg) */

inline ATbool SDF_isLexLayoutLine(SDF_LexLayout arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexLayoutLine, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexLayoutNested(SDF_LexLayout arg) */

inline ATbool SDF_isLexLayoutNested(SDF_LexLayout arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexLayoutNested, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasLexLayoutCh(SDF_LexLayout arg) */

ATbool SDF_hasLexLayoutCh(SDF_LexLayout arg)
{
  if (SDF_isLexLayoutWhitespace(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexLayoutLine(SDF_LexLayout arg) */

ATbool SDF_hasLexLayoutLine(SDF_LexLayout arg)
{
  if (SDF_isLexLayoutLine(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexLayoutContent(SDF_LexLayout arg) */

ATbool SDF_hasLexLayoutContent(SDF_LexLayout arg)
{
  if (SDF_isLexLayoutNested(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char SDF_getLexLayoutCh(SDF_LexLayout arg) */

char SDF_getLexLayoutCh(SDF_LexLayout arg)
{
  
    return (char)SDF_charToByte((ATerm)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)));
}

/*}}}  */
/*{{{  char* SDF_getLexLayoutLine(SDF_LexLayout arg) */

char* SDF_getLexLayoutLine(SDF_LexLayout arg)
{
  
    return (char*)SDF_charsToString((ATerm)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1), 1));
}

/*}}}  */
/*{{{  char* SDF_getLexLayoutContent(SDF_LexLayout arg) */

char* SDF_getLexLayoutContent(SDF_LexLayout arg)
{
  
    return (char*)SDF_charsToString((ATerm)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1), 1));
}

/*}}}  */
/*{{{  SDF_LexLayout SDF_setLexLayoutCh(SDF_LexLayout arg, char ch) */

SDF_LexLayout SDF_setLexLayoutCh(SDF_LexLayout arg, char ch)
{
  if (SDF_isLexLayoutWhitespace(arg)) {
    return (SDF_LexLayout)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ((ATerm) SDF_byteToChar(ch))), 0), 1);
  }

  ATabort("LexLayout has no Ch: %t\n", arg);
  return (SDF_LexLayout)NULL;
}

/*}}}  */
/*{{{  SDF_LexLayout SDF_setLexLayoutLine(SDF_LexLayout arg, const char* line) */

SDF_LexLayout SDF_setLexLayoutLine(SDF_LexLayout arg, const char* line)
{
  if (SDF_isLexLayoutLine(arg)) {
    return (SDF_LexLayout)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1), (ATerm)((ATerm) ((ATerm) SDF_stringToChars(line))), 1), 1), 1);
  }

  ATabort("LexLayout has no Line: %t\n", arg);
  return (SDF_LexLayout)NULL;
}

/*}}}  */
/*{{{  SDF_LexLayout SDF_setLexLayoutContent(SDF_LexLayout arg, const char* content) */

SDF_LexLayout SDF_setLexLayoutContent(SDF_LexLayout arg, const char* content)
{
  if (SDF_isLexLayoutNested(arg)) {
    return (SDF_LexLayout)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1), (ATerm)((ATerm) ((ATerm) SDF_stringToChars(content))), 1), 1), 1);
  }

  ATabort("LexLayout has no Content: %t\n", arg);
  return (SDF_LexLayout)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_LexStrChar accessors */

/*{{{  ATbool SDF_isValidLexStrChar(SDF_LexStrChar arg) */

ATbool SDF_isValidLexStrChar(SDF_LexStrChar arg)
{
  if (SDF_isLexStrCharNewline(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexStrCharTab(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexStrCharQuote(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexStrCharBackslash(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexStrCharDecimal(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexStrCharNormal(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexStrCharNewline(SDF_LexStrChar arg) */

inline ATbool SDF_isLexStrCharNewline(SDF_LexStrChar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexStrCharNewline);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexStrCharTab(SDF_LexStrChar arg) */

inline ATbool SDF_isLexStrCharTab(SDF_LexStrChar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexStrCharTab);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexStrCharQuote(SDF_LexStrChar arg) */

inline ATbool SDF_isLexStrCharQuote(SDF_LexStrChar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexStrCharQuote);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexStrCharBackslash(SDF_LexStrChar arg) */

inline ATbool SDF_isLexStrCharBackslash(SDF_LexStrChar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexStrCharBackslash);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexStrCharDecimal(SDF_LexStrChar arg) */

inline ATbool SDF_isLexStrCharDecimal(SDF_LexStrChar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexStrCharDecimal, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexStrCharNormal(SDF_LexStrChar arg) */

inline ATbool SDF_isLexStrCharNormal(SDF_LexStrChar arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexStrCharNormal, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasLexStrCharA(SDF_LexStrChar arg) */

ATbool SDF_hasLexStrCharA(SDF_LexStrChar arg)
{
  if (SDF_isLexStrCharDecimal(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexStrCharB(SDF_LexStrChar arg) */

ATbool SDF_hasLexStrCharB(SDF_LexStrChar arg)
{
  if (SDF_isLexStrCharDecimal(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexStrCharC(SDF_LexStrChar arg) */

ATbool SDF_hasLexStrCharC(SDF_LexStrChar arg)
{
  if (SDF_isLexStrCharDecimal(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexStrCharCh(SDF_LexStrChar arg) */

ATbool SDF_hasLexStrCharCh(SDF_LexStrChar arg)
{
  if (SDF_isLexStrCharNormal(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char SDF_getLexStrCharA(SDF_LexStrChar arg) */

char SDF_getLexStrCharA(SDF_LexStrChar arg)
{
  
    return (char)SDF_charToByte((ATerm)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1));
}

/*}}}  */
/*{{{  char SDF_getLexStrCharB(SDF_LexStrChar arg) */

char SDF_getLexStrCharB(SDF_LexStrChar arg)
{
  
    return (char)SDF_charToByte((ATerm)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 2));
}

/*}}}  */
/*{{{  char SDF_getLexStrCharC(SDF_LexStrChar arg) */

char SDF_getLexStrCharC(SDF_LexStrChar arg)
{
  
    return (char)SDF_charToByte((ATerm)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 3));
}

/*}}}  */
/*{{{  char SDF_getLexStrCharCh(SDF_LexStrChar arg) */

char SDF_getLexStrCharCh(SDF_LexStrChar arg)
{
  
    return (char)SDF_charToByte((ATerm)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)));
}

/*}}}  */
/*{{{  SDF_LexStrChar SDF_setLexStrCharA(SDF_LexStrChar arg, char a) */

SDF_LexStrChar SDF_setLexStrCharA(SDF_LexStrChar arg, char a)
{
  if (SDF_isLexStrCharDecimal(arg)) {
    return (SDF_LexStrChar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ((ATerm) SDF_byteToChar(a))), 1), 1);
  }

  ATabort("LexStrChar has no A: %t\n", arg);
  return (SDF_LexStrChar)NULL;
}

/*}}}  */
/*{{{  SDF_LexStrChar SDF_setLexStrCharB(SDF_LexStrChar arg, char b) */

SDF_LexStrChar SDF_setLexStrCharB(SDF_LexStrChar arg, char b)
{
  if (SDF_isLexStrCharDecimal(arg)) {
    return (SDF_LexStrChar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ((ATerm) SDF_byteToChar(b))), 2), 1);
  }

  ATabort("LexStrChar has no B: %t\n", arg);
  return (SDF_LexStrChar)NULL;
}

/*}}}  */
/*{{{  SDF_LexStrChar SDF_setLexStrCharC(SDF_LexStrChar arg, char c) */

SDF_LexStrChar SDF_setLexStrCharC(SDF_LexStrChar arg, char c)
{
  if (SDF_isLexStrCharDecimal(arg)) {
    return (SDF_LexStrChar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ((ATerm) SDF_byteToChar(c))), 3), 1);
  }

  ATabort("LexStrChar has no C: %t\n", arg);
  return (SDF_LexStrChar)NULL;
}

/*}}}  */
/*{{{  SDF_LexStrChar SDF_setLexStrCharCh(SDF_LexStrChar arg, char ch) */

SDF_LexStrChar SDF_setLexStrCharCh(SDF_LexStrChar arg, char ch)
{
  if (SDF_isLexStrCharNormal(arg)) {
    return (SDF_LexStrChar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ((ATerm) SDF_byteToChar(ch))), 0), 1);
  }

  ATabort("LexStrChar has no Ch: %t\n", arg);
  return (SDF_LexStrChar)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_StrChar accessors */

/*{{{  ATbool SDF_isValidStrChar(SDF_StrChar arg) */

ATbool SDF_isValidStrChar(SDF_StrChar arg)
{
  if (SDF_isStrCharLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isStrCharLexToCf(SDF_StrChar arg) */

inline ATbool SDF_isStrCharLexToCf(SDF_StrChar arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternStrCharLexToCf, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasStrCharStrChar(SDF_StrChar arg) */

ATbool SDF_hasStrCharStrChar(SDF_StrChar arg)
{
  if (SDF_isStrCharLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_LexStrChar SDF_getStrCharStrChar(SDF_StrChar arg) */

SDF_LexStrChar SDF_getStrCharStrChar(SDF_StrChar arg)
{
  
    return (SDF_LexStrChar)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_StrChar SDF_setStrCharStrChar(SDF_StrChar arg, SDF_LexStrChar StrChar) */

SDF_StrChar SDF_setStrCharStrChar(SDF_StrChar arg, SDF_LexStrChar StrChar)
{
  if (SDF_isStrCharLexToCf(arg)) {
    return (SDF_StrChar)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) StrChar), 0), 1);
  }

  ATabort("StrChar has no StrChar: %t\n", arg);
  return (SDF_StrChar)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_LexStrCon accessors */

/*{{{  ATbool SDF_isValidLexStrCon(SDF_LexStrCon arg) */

ATbool SDF_isValidLexStrCon(SDF_LexStrCon arg)
{
  if (SDF_isLexStrConDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexStrConDefault(SDF_LexStrCon arg) */

inline ATbool SDF_isLexStrConDefault(SDF_LexStrCon arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternLexStrConDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexStrConChars(SDF_LexStrCon arg) */

ATbool SDF_hasLexStrConChars(SDF_LexStrCon arg)
{
  if (SDF_isLexStrConDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_LexStrCharChars SDF_getLexStrConChars(SDF_LexStrCon arg) */

SDF_LexStrCharChars SDF_getLexStrConChars(SDF_LexStrCon arg)
{
  
    return (SDF_LexStrCharChars)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1), 1);
}

/*}}}  */
/*{{{  SDF_LexStrCon SDF_setLexStrConChars(SDF_LexStrCon arg, SDF_LexStrCharChars chars) */

SDF_LexStrCon SDF_setLexStrConChars(SDF_LexStrCon arg, SDF_LexStrCharChars chars)
{
  if (SDF_isLexStrConDefault(arg)) {
    return (SDF_LexStrCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1), (ATerm)((ATerm) chars), 1), 1), 1);
  }

  ATabort("LexStrCon has no Chars: %t\n", arg);
  return (SDF_LexStrCon)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_StrCon accessors */

/*{{{  ATbool SDF_isValidStrCon(SDF_StrCon arg) */

ATbool SDF_isValidStrCon(SDF_StrCon arg)
{
  if (SDF_isStrConLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isStrConLexToCf(SDF_StrCon arg) */

inline ATbool SDF_isStrConLexToCf(SDF_StrCon arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternStrConLexToCf, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasStrConStrCon(SDF_StrCon arg) */

ATbool SDF_hasStrConStrCon(SDF_StrCon arg)
{
  if (SDF_isStrConLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_LexStrCon SDF_getStrConStrCon(SDF_StrCon arg) */

SDF_LexStrCon SDF_getStrConStrCon(SDF_StrCon arg)
{
  
    return (SDF_LexStrCon)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_StrCon SDF_setStrConStrCon(SDF_StrCon arg, SDF_LexStrCon StrCon) */

SDF_StrCon SDF_setStrConStrCon(SDF_StrCon arg, SDF_LexStrCon StrCon)
{
  if (SDF_isStrConLexToCf(arg)) {
    return (SDF_StrCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) StrCon), 0), 1);
  }

  ATabort("StrCon has no StrCon: %t\n", arg);
  return (SDF_StrCon)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_LexStrCharChars accessors */

/*{{{  ATbool SDF_isValidLexStrCharChars(SDF_LexStrCharChars arg) */

ATbool SDF_isValidLexStrCharChars(SDF_LexStrCharChars arg)
{
  if (SDF_isLexStrCharCharsEmpty(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexStrCharCharsSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexStrCharCharsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexStrCharCharsEmpty(SDF_LexStrCharChars arg) */

inline ATbool SDF_isLexStrCharCharsEmpty(SDF_LexStrCharChars arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternLexStrCharCharsEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexStrCharCharsSingle(SDF_LexStrCharChars arg) */

inline ATbool SDF_isLexStrCharCharsSingle(SDF_LexStrCharChars arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexStrCharCharsSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexStrCharCharsMany(SDF_LexStrCharChars arg) */

inline ATbool SDF_isLexStrCharCharsMany(SDF_LexStrCharChars arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, SDF_patternLexStrCharCharsMany, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool SDF_hasLexStrCharCharsHead(SDF_LexStrCharChars arg) */

ATbool SDF_hasLexStrCharCharsHead(SDF_LexStrCharChars arg)
{
  if (SDF_isLexStrCharCharsSingle(arg)) {
    return ATtrue;
  }
  else if (SDF_isLexStrCharCharsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexStrCharCharsTail(SDF_LexStrCharChars arg) */

ATbool SDF_hasLexStrCharCharsTail(SDF_LexStrCharChars arg)
{
  if (SDF_isLexStrCharCharsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_LexStrChar SDF_getLexStrCharCharsHead(SDF_LexStrCharChars arg) */

SDF_LexStrChar SDF_getLexStrCharCharsHead(SDF_LexStrCharChars arg)
{
  if (SDF_isLexStrCharCharsSingle(arg)) {
    return (SDF_LexStrChar)ATgetFirst((ATermList)arg);
  }
  else 
    return (SDF_LexStrChar)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_LexStrCharChars SDF_getLexStrCharCharsTail(SDF_LexStrCharChars arg) */

SDF_LexStrCharChars SDF_getLexStrCharCharsTail(SDF_LexStrCharChars arg)
{
  
    return (SDF_LexStrCharChars)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  SDF_LexStrCharChars SDF_setLexStrCharCharsHead(SDF_LexStrCharChars arg, SDF_LexStrChar head) */

SDF_LexStrCharChars SDF_setLexStrCharCharsHead(SDF_LexStrCharChars arg, SDF_LexStrChar head)
{
  if (SDF_isLexStrCharCharsSingle(arg)) {
    return (SDF_LexStrCharChars)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (SDF_isLexStrCharCharsMany(arg)) {
    return (SDF_LexStrCharChars)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("LexStrCharChars has no Head: %t\n", arg);
  return (SDF_LexStrCharChars)NULL;
}

/*}}}  */
/*{{{  SDF_LexStrCharChars SDF_setLexStrCharCharsTail(SDF_LexStrCharChars arg, SDF_LexStrCharChars tail) */

SDF_LexStrCharChars SDF_setLexStrCharCharsTail(SDF_LexStrCharChars arg, SDF_LexStrCharChars tail)
{
  if (SDF_isLexStrCharCharsMany(arg)) {
    return (SDF_LexStrCharChars)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 1);
  }

  ATabort("LexStrCharChars has no Tail: %t\n", arg);
  return (SDF_LexStrCharChars)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_LexNatCon accessors */

/*{{{  ATbool SDF_isValidLexNatCon(SDF_LexNatCon arg) */

ATbool SDF_isValidLexNatCon(SDF_LexNatCon arg)
{
  if (SDF_isLexNatConDigits(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexNatConDigits(SDF_LexNatCon arg) */

inline ATbool SDF_isLexNatConDigits(SDF_LexNatCon arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternLexNatConDigits, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexNatConList(SDF_LexNatCon arg) */

ATbool SDF_hasLexNatConList(SDF_LexNatCon arg)
{
  if (SDF_isLexNatConDigits(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char* SDF_getLexNatConList(SDF_LexNatCon arg) */

char* SDF_getLexNatConList(SDF_LexNatCon arg)
{
  
    return (char*)SDF_charsToString((ATerm)ATgetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), 1));
}

/*}}}  */
/*{{{  SDF_LexNatCon SDF_setLexNatConList(SDF_LexNatCon arg, const char* list) */

SDF_LexNatCon SDF_setLexNatConList(SDF_LexNatCon arg, const char* list)
{
  if (SDF_isLexNatConDigits(arg)) {
    return (SDF_LexNatCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)), (ATerm)((ATerm) ((ATerm) SDF_stringToChars(list))), 1), 0), 1);
  }

  ATabort("LexNatCon has no List: %t\n", arg);
  return (SDF_LexNatCon)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_NatCon accessors */

/*{{{  ATbool SDF_isValidNatCon(SDF_NatCon arg) */

ATbool SDF_isValidNatCon(SDF_NatCon arg)
{
  if (SDF_isNatConLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isNatConLexToCf(SDF_NatCon arg) */

inline ATbool SDF_isNatConLexToCf(SDF_NatCon arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternNatConLexToCf, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasNatConNatCon(SDF_NatCon arg) */

ATbool SDF_hasNatConNatCon(SDF_NatCon arg)
{
  if (SDF_isNatConLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_LexNatCon SDF_getNatConNatCon(SDF_NatCon arg) */

SDF_LexNatCon SDF_getNatConNatCon(SDF_NatCon arg)
{
  
    return (SDF_LexNatCon)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_NatCon SDF_setNatConNatCon(SDF_NatCon arg, SDF_LexNatCon NatCon) */

SDF_NatCon SDF_setNatConNatCon(SDF_NatCon arg, SDF_LexNatCon NatCon)
{
  if (SDF_isNatConLexToCf(arg)) {
    return (SDF_NatCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) NatCon), 0), 1);
  }

  ATabort("NatCon has no NatCon: %t\n", arg);
  return (SDF_NatCon)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_LexIdCon accessors */

/*{{{  ATbool SDF_isValidLexIdCon(SDF_LexIdCon arg) */

ATbool SDF_isValidLexIdCon(SDF_LexIdCon arg)
{
  if (SDF_isLexIdConDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isLexIdConDefault(SDF_LexIdCon arg) */

inline ATbool SDF_isLexIdConDefault(SDF_LexIdCon arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternLexIdConDefault, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexIdConHead(SDF_LexIdCon arg) */

ATbool SDF_hasLexIdConHead(SDF_LexIdCon arg)
{
  if (SDF_isLexIdConDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool SDF_hasLexIdConTail(SDF_LexIdCon arg) */

ATbool SDF_hasLexIdConTail(SDF_LexIdCon arg)
{
  if (SDF_isLexIdConDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char SDF_getLexIdConHead(SDF_LexIdCon arg) */

char SDF_getLexIdConHead(SDF_LexIdCon arg)
{
  
    return (char)SDF_charToByte((ATerm)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1)));
}

/*}}}  */
/*{{{  char* SDF_getLexIdConTail(SDF_LexIdCon arg) */

char* SDF_getLexIdConTail(SDF_LexIdCon arg)
{
  
    return (char*)SDF_charsToString((ATerm)ATgetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1), 1));
}

/*}}}  */
/*{{{  SDF_LexIdCon SDF_setLexIdConHead(SDF_LexIdCon arg, char head) */

SDF_LexIdCon SDF_setLexIdConHead(SDF_LexIdCon arg, char head)
{
  if (SDF_isLexIdConDefault(arg)) {
    return (SDF_LexIdCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) ((ATerm) SDF_byteToChar(head))), 0), 1);
  }

  ATabort("LexIdCon has no Head: %t\n", arg);
  return (SDF_LexIdCon)NULL;
}

/*}}}  */
/*{{{  SDF_LexIdCon SDF_setLexIdConTail(SDF_LexIdCon arg, const char* tail) */

SDF_LexIdCon SDF_setLexIdConTail(SDF_LexIdCon arg, const char* tail)
{
  if (SDF_isLexIdConDefault(arg)) {
    return (SDF_LexIdCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)ATsetArgument((ATermAppl)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 1), 1), (ATerm)((ATerm) ((ATerm) SDF_stringToChars(tail))), 1), 1), 1);
  }

  ATabort("LexIdCon has no Tail: %t\n", arg);
  return (SDF_LexIdCon)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  SDF_IdCon accessors */

/*{{{  ATbool SDF_isValidIdCon(SDF_IdCon arg) */

ATbool SDF_isValidIdCon(SDF_IdCon arg)
{
  if (SDF_isIdConLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool SDF_isIdConLexToCf(SDF_IdCon arg) */

inline ATbool SDF_isIdConLexToCf(SDF_IdCon arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, SDF_patternIdConLexToCf, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool SDF_hasIdConIdCon(SDF_IdCon arg) */

ATbool SDF_hasIdConIdCon(SDF_IdCon arg)
{
  if (SDF_isIdConLexToCf(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  SDF_LexIdCon SDF_getIdConIdCon(SDF_IdCon arg) */

SDF_LexIdCon SDF_getIdConIdCon(SDF_IdCon arg)
{
  
    return (SDF_LexIdCon)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 1));
}

/*}}}  */
/*{{{  SDF_IdCon SDF_setIdConIdCon(SDF_IdCon arg, SDF_LexIdCon IdCon) */

SDF_IdCon SDF_setIdConIdCon(SDF_IdCon arg, SDF_LexIdCon IdCon)
{
  if (SDF_isIdConLexToCf(arg)) {
    return (SDF_IdCon)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 1), (ATerm)((ATerm) IdCon), 0), 1);
  }

  ATabort("IdCon has no IdCon: %t\n", arg);
  return (SDF_IdCon)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  sort visitors */

/*{{{  SDF_OptLayout SDF_visitOptLayout(SDF_OptLayout arg, SDF_Layout (*acceptLayout)(SDF_Layout)) */

SDF_OptLayout SDF_visitOptLayout(SDF_OptLayout arg, SDF_Layout (*acceptLayout)(SDF_Layout))
{
  if (SDF_isOptLayoutAbsent(arg)) {
    return SDF_makeOptLayoutAbsent();
  }
  if (SDF_isOptLayoutPresent(arg)) {
    return SDF_makeOptLayoutPresent(
        acceptLayout ? acceptLayout(SDF_getOptLayoutLayout(arg)) : SDF_getOptLayoutLayout(arg));
  }
  ATabort("not a OptLayout: %t\n", arg);
  return (SDF_OptLayout)NULL;
}

/*}}}  */
/*{{{  SDF_Layout SDF_visitLayout(SDF_Layout arg, SDF_LexLayoutList (*acceptList)(SDF_LexLayoutList)) */

SDF_Layout SDF_visitLayout(SDF_Layout arg, SDF_LexLayoutList (*acceptList)(SDF_LexLayoutList))
{
  if (SDF_isLayoutLexToCf(arg)) {
    return SDF_makeLayoutLexToCf(
        acceptList ? acceptList(SDF_getLayoutList(arg)) : SDF_getLayoutList(arg));
  }
  ATabort("not a Layout: %t\n", arg);
  return (SDF_Layout)NULL;
}

/*}}}  */
/*{{{  SDF_LexLayoutList SDF_visitLexLayoutList(SDF_LexLayoutList arg, SDF_LexLayout (*acceptHead)(SDF_LexLayout)) */

SDF_LexLayoutList SDF_visitLexLayoutList(SDF_LexLayoutList arg, SDF_LexLayout (*acceptHead)(SDF_LexLayout))
{
  if (SDF_isLexLayoutListEmpty(arg)) {
    return SDF_makeLexLayoutListEmpty();
  }
  if (SDF_isLexLayoutListSingle(arg)) {
    return SDF_makeLexLayoutListSingle(
        acceptHead ? acceptHead(SDF_getLexLayoutListHead(arg)) : SDF_getLexLayoutListHead(arg));
  }
  if (SDF_isLexLayoutListMany(arg)) {
    return SDF_makeLexLayoutListMany(
        acceptHead ? acceptHead(SDF_getLexLayoutListHead(arg)) : SDF_getLexLayoutListHead(arg),
        SDF_visitLexLayoutList(SDF_getLexLayoutListTail(arg), acceptHead));
  }
  ATabort("not a LexLayoutList: %t\n", arg);
  return (SDF_LexLayoutList)NULL;
}

/*}}}  */
/*{{{  SDF_AFun SDF_visitAFun(SDF_AFun arg, SDF_StrCon (*acceptStrCon)(SDF_StrCon), SDF_IdCon (*acceptIdCon)(SDF_IdCon)) */

SDF_AFun SDF_visitAFun(SDF_AFun arg, SDF_StrCon (*acceptStrCon)(SDF_StrCon), SDF_IdCon (*acceptIdCon)(SDF_IdCon))
{
  if (SDF_isAFunQuoted(arg)) {
    return SDF_makeAFunQuoted(
        acceptStrCon ? acceptStrCon(SDF_getAFunStrCon(arg)) : SDF_getAFunStrCon(arg));
  }
  if (SDF_isAFunUnquoted(arg)) {
    return SDF_makeAFunUnquoted(
        acceptIdCon ? acceptIdCon(SDF_getAFunIdCon(arg)) : SDF_getAFunIdCon(arg));
  }
  ATabort("not a AFun: %t\n", arg);
  return (SDF_AFun)NULL;
}

/*}}}  */
/*{{{  SDF_ATerm SDF_visitATerm(SDF_ATerm arg, SDF_IntCon (*acceptIntCon)(SDF_IntCon), SDF_RealCon (*acceptRealCon)(SDF_RealCon), SDF_AFun (*acceptFun)(SDF_AFun), SDF_OptLayout (*acceptWsAfterFun)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_ATermArgs (*acceptArgs)(SDF_ATermArgs), SDF_OptLayout (*acceptWsAfterArgs)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterLessThan)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterType)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBracketOpen)(SDF_OptLayout), SDF_ATermElems (*acceptElems)(SDF_ATermElems), SDF_OptLayout (*acceptWsAfterElems)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterTrm)(SDF_OptLayout), SDF_Annotation (*acceptAnnotation)(SDF_Annotation)) */

SDF_ATerm SDF_visitATerm(SDF_ATerm arg, SDF_IntCon (*acceptIntCon)(SDF_IntCon), SDF_RealCon (*acceptRealCon)(SDF_RealCon), SDF_AFun (*acceptFun)(SDF_AFun), SDF_OptLayout (*acceptWsAfterFun)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_ATermArgs (*acceptArgs)(SDF_ATermArgs), SDF_OptLayout (*acceptWsAfterArgs)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterLessThan)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterType)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBracketOpen)(SDF_OptLayout), SDF_ATermElems (*acceptElems)(SDF_ATermElems), SDF_OptLayout (*acceptWsAfterElems)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterTrm)(SDF_OptLayout), SDF_Annotation (*acceptAnnotation)(SDF_Annotation))
{
  if (SDF_isATermInt(arg)) {
    return SDF_makeATermInt(
        acceptIntCon ? acceptIntCon(SDF_getATermIntCon(arg)) : SDF_getATermIntCon(arg));
  }
  if (SDF_isATermReal(arg)) {
    return SDF_makeATermReal(
        acceptRealCon ? acceptRealCon(SDF_getATermRealCon(arg)) : SDF_getATermRealCon(arg));
  }
  if (SDF_isATermFun(arg)) {
    return SDF_makeATermFun(
        acceptFun ? acceptFun(SDF_getATermFun(arg)) : SDF_getATermFun(arg));
  }
  if (SDF_isATermAppl(arg)) {
    return SDF_makeATermAppl(
        acceptFun ? acceptFun(SDF_getATermFun(arg)) : SDF_getATermFun(arg),
        acceptWsAfterFun ? acceptWsAfterFun(SDF_getATermWsAfterFun(arg)) : SDF_getATermWsAfterFun(arg),
        acceptWsAfterParenOpen ? acceptWsAfterParenOpen(SDF_getATermWsAfterParenOpen(arg)) : SDF_getATermWsAfterParenOpen(arg),
        acceptArgs ? acceptArgs(SDF_getATermArgs(arg)) : SDF_getATermArgs(arg),
        acceptWsAfterArgs ? acceptWsAfterArgs(SDF_getATermWsAfterArgs(arg)) : SDF_getATermWsAfterArgs(arg));
  }
  if (SDF_isATermPlaceholder(arg)) {
    return SDF_makeATermPlaceholder(
        acceptWsAfterLessThan ? acceptWsAfterLessThan(SDF_getATermWsAfterLessThan(arg)) : SDF_getATermWsAfterLessThan(arg),
        SDF_visitATerm(SDF_getATermType(arg), acceptIntCon, acceptRealCon, acceptFun, acceptWsAfterFun, acceptWsAfterParenOpen, acceptArgs, acceptWsAfterArgs, acceptWsAfterLessThan, acceptWsAfterType, acceptWsAfterBracketOpen, acceptElems, acceptWsAfterElems, acceptWsAfterTrm, acceptAnnotation),
        acceptWsAfterType ? acceptWsAfterType(SDF_getATermWsAfterType(arg)) : SDF_getATermWsAfterType(arg));
  }
  if (SDF_isATermList(arg)) {
    return SDF_makeATermList(
        acceptWsAfterBracketOpen ? acceptWsAfterBracketOpen(SDF_getATermWsAfterBracketOpen(arg)) : SDF_getATermWsAfterBracketOpen(arg),
        acceptElems ? acceptElems(SDF_getATermElems(arg)) : SDF_getATermElems(arg),
        acceptWsAfterElems ? acceptWsAfterElems(SDF_getATermWsAfterElems(arg)) : SDF_getATermWsAfterElems(arg));
  }
  if (SDF_isATermAnnotated(arg)) {
    return SDF_makeATermAnnotated(
        SDF_visitATerm(SDF_getATermTrm(arg), acceptIntCon, acceptRealCon, acceptFun, acceptWsAfterFun, acceptWsAfterParenOpen, acceptArgs, acceptWsAfterArgs, acceptWsAfterLessThan, acceptWsAfterType, acceptWsAfterBracketOpen, acceptElems, acceptWsAfterElems, acceptWsAfterTrm, acceptAnnotation),
        acceptWsAfterTrm ? acceptWsAfterTrm(SDF_getATermWsAfterTrm(arg)) : SDF_getATermWsAfterTrm(arg),
        acceptAnnotation ? acceptAnnotation(SDF_getATermAnnotation(arg)) : SDF_getATermAnnotation(arg));
  }
  ATabort("not a ATerm: %t\n", arg);
  return (SDF_ATerm)NULL;
}

/*}}}  */
/*{{{  SDF_Annotation SDF_visitAnnotation(SDF_Annotation arg, SDF_OptLayout (*acceptWsAfterBraceOpen)(SDF_OptLayout), SDF_ATermAnnos (*acceptAnnos)(SDF_ATermAnnos), SDF_OptLayout (*acceptWsAfterAnnos)(SDF_OptLayout)) */

SDF_Annotation SDF_visitAnnotation(SDF_Annotation arg, SDF_OptLayout (*acceptWsAfterBraceOpen)(SDF_OptLayout), SDF_ATermAnnos (*acceptAnnos)(SDF_ATermAnnos), SDF_OptLayout (*acceptWsAfterAnnos)(SDF_OptLayout))
{
  if (SDF_isAnnotationDefault(arg)) {
    return SDF_makeAnnotationDefault(
        acceptWsAfterBraceOpen ? acceptWsAfterBraceOpen(SDF_getAnnotationWsAfterBraceOpen(arg)) : SDF_getAnnotationWsAfterBraceOpen(arg),
        acceptAnnos ? acceptAnnos(SDF_getAnnotationAnnos(arg)) : SDF_getAnnotationAnnos(arg),
        acceptWsAfterAnnos ? acceptWsAfterAnnos(SDF_getAnnotationWsAfterAnnos(arg)) : SDF_getAnnotationWsAfterAnnos(arg));
  }
  ATabort("not a Annotation: %t\n", arg);
  return (SDF_Annotation)NULL;
}

/*}}}  */
/*{{{  SDF_Start SDF_visitStart(SDF_Start arg, SDF_OptLayout (*acceptWsBefore)(SDF_OptLayout), SDF_ATerm (*acceptTopATerm)(SDF_ATerm), SDF_OptLayout (*acceptWsAfter)(SDF_OptLayout), int (*acceptAmbCnt)(int), SDF_IntCon (*acceptTopIntCon)(SDF_IntCon), SDF_RealCon (*acceptTopRealCon)(SDF_RealCon), SDF_Module (*acceptTopModule)(SDF_Module), SDF_SDF (*acceptTopSDF)(SDF_SDF)) */

SDF_Start SDF_visitStart(SDF_Start arg, SDF_OptLayout (*acceptWsBefore)(SDF_OptLayout), SDF_ATerm (*acceptTopATerm)(SDF_ATerm), SDF_OptLayout (*acceptWsAfter)(SDF_OptLayout), int (*acceptAmbCnt)(int), SDF_IntCon (*acceptTopIntCon)(SDF_IntCon), SDF_RealCon (*acceptTopRealCon)(SDF_RealCon), SDF_Module (*acceptTopModule)(SDF_Module), SDF_SDF (*acceptTopSDF)(SDF_SDF))
{
  if (SDF_isStartATerm(arg)) {
    return SDF_makeStartATerm(
        acceptWsBefore ? acceptWsBefore(SDF_getStartWsBefore(arg)) : SDF_getStartWsBefore(arg),
        acceptTopATerm ? acceptTopATerm(SDF_getStartTopATerm(arg)) : SDF_getStartTopATerm(arg),
        acceptWsAfter ? acceptWsAfter(SDF_getStartWsAfter(arg)) : SDF_getStartWsAfter(arg),
        acceptAmbCnt ? acceptAmbCnt(SDF_getStartAmbCnt(arg)) : SDF_getStartAmbCnt(arg));
  }
  if (SDF_isStartIntCon(arg)) {
    return SDF_makeStartIntCon(
        acceptWsBefore ? acceptWsBefore(SDF_getStartWsBefore(arg)) : SDF_getStartWsBefore(arg),
        acceptTopIntCon ? acceptTopIntCon(SDF_getStartTopIntCon(arg)) : SDF_getStartTopIntCon(arg),
        acceptWsAfter ? acceptWsAfter(SDF_getStartWsAfter(arg)) : SDF_getStartWsAfter(arg),
        acceptAmbCnt ? acceptAmbCnt(SDF_getStartAmbCnt(arg)) : SDF_getStartAmbCnt(arg));
  }
  if (SDF_isStartRealCon(arg)) {
    return SDF_makeStartRealCon(
        acceptWsBefore ? acceptWsBefore(SDF_getStartWsBefore(arg)) : SDF_getStartWsBefore(arg),
        acceptTopRealCon ? acceptTopRealCon(SDF_getStartTopRealCon(arg)) : SDF_getStartTopRealCon(arg),
        acceptWsAfter ? acceptWsAfter(SDF_getStartWsAfter(arg)) : SDF_getStartWsAfter(arg),
        acceptAmbCnt ? acceptAmbCnt(SDF_getStartAmbCnt(arg)) : SDF_getStartAmbCnt(arg));
  }
  if (SDF_isStartModule(arg)) {
    return SDF_makeStartModule(
        acceptWsBefore ? acceptWsBefore(SDF_getStartWsBefore(arg)) : SDF_getStartWsBefore(arg),
        acceptTopModule ? acceptTopModule(SDF_getStartTopModule(arg)) : SDF_getStartTopModule(arg),
        acceptWsAfter ? acceptWsAfter(SDF_getStartWsAfter(arg)) : SDF_getStartWsAfter(arg),
        acceptAmbCnt ? acceptAmbCnt(SDF_getStartAmbCnt(arg)) : SDF_getStartAmbCnt(arg));
  }
  if (SDF_isStartSDF(arg)) {
    return SDF_makeStartSDF(
        acceptWsBefore ? acceptWsBefore(SDF_getStartWsBefore(arg)) : SDF_getStartWsBefore(arg),
        acceptTopSDF ? acceptTopSDF(SDF_getStartTopSDF(arg)) : SDF_getStartTopSDF(arg),
        acceptWsAfter ? acceptWsAfter(SDF_getStartWsAfter(arg)) : SDF_getStartWsAfter(arg),
        acceptAmbCnt ? acceptAmbCnt(SDF_getStartAmbCnt(arg)) : SDF_getStartAmbCnt(arg));
  }
  ATabort("not a Start: %t\n", arg);
  return (SDF_Start)NULL;
}

/*}}}  */
/*{{{  SDF_ATermArgs SDF_visitATermArgs(SDF_ATermArgs arg, SDF_ATerm (*acceptHead)(SDF_ATerm), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout)) */

SDF_ATermArgs SDF_visitATermArgs(SDF_ATermArgs arg, SDF_ATerm (*acceptHead)(SDF_ATerm), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout))
{
  if (SDF_isATermArgsEmpty(arg)) {
    return SDF_makeATermArgsEmpty();
  }
  if (SDF_isATermArgsSingle(arg)) {
    return SDF_makeATermArgsSingle(
        acceptHead ? acceptHead(SDF_getATermArgsHead(arg)) : SDF_getATermArgsHead(arg));
  }
  if (SDF_isATermArgsMany(arg)) {
    return SDF_makeATermArgsMany(
        acceptHead ? acceptHead(SDF_getATermArgsHead(arg)) : SDF_getATermArgsHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getATermArgsWsAfterHead(arg)) : SDF_getATermArgsWsAfterHead(arg),
        acceptWsAfterSep ? acceptWsAfterSep(SDF_getATermArgsWsAfterSep(arg)) : SDF_getATermArgsWsAfterSep(arg),
        SDF_visitATermArgs(SDF_getATermArgsTail(arg), acceptHead, acceptWsAfterHead, acceptWsAfterSep));
  }
  ATabort("not a ATermArgs: %t\n", arg);
  return (SDF_ATermArgs)NULL;
}

/*}}}  */
/*{{{  SDF_ATermElems SDF_visitATermElems(SDF_ATermElems arg, SDF_ATerm (*acceptHead)(SDF_ATerm), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout)) */

SDF_ATermElems SDF_visitATermElems(SDF_ATermElems arg, SDF_ATerm (*acceptHead)(SDF_ATerm), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout))
{
  if (SDF_isATermElemsEmpty(arg)) {
    return SDF_makeATermElemsEmpty();
  }
  if (SDF_isATermElemsSingle(arg)) {
    return SDF_makeATermElemsSingle(
        acceptHead ? acceptHead(SDF_getATermElemsHead(arg)) : SDF_getATermElemsHead(arg));
  }
  if (SDF_isATermElemsMany(arg)) {
    return SDF_makeATermElemsMany(
        acceptHead ? acceptHead(SDF_getATermElemsHead(arg)) : SDF_getATermElemsHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getATermElemsWsAfterHead(arg)) : SDF_getATermElemsWsAfterHead(arg),
        acceptWsAfterSep ? acceptWsAfterSep(SDF_getATermElemsWsAfterSep(arg)) : SDF_getATermElemsWsAfterSep(arg),
        SDF_visitATermElems(SDF_getATermElemsTail(arg), acceptHead, acceptWsAfterHead, acceptWsAfterSep));
  }
  ATabort("not a ATermElems: %t\n", arg);
  return (SDF_ATermElems)NULL;
}

/*}}}  */
/*{{{  SDF_ATermAnnos SDF_visitATermAnnos(SDF_ATermAnnos arg, SDF_ATerm (*acceptHead)(SDF_ATerm), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout)) */

SDF_ATermAnnos SDF_visitATermAnnos(SDF_ATermAnnos arg, SDF_ATerm (*acceptHead)(SDF_ATerm), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout))
{
  if (SDF_isATermAnnosEmpty(arg)) {
    return SDF_makeATermAnnosEmpty();
  }
  if (SDF_isATermAnnosSingle(arg)) {
    return SDF_makeATermAnnosSingle(
        acceptHead ? acceptHead(SDF_getATermAnnosHead(arg)) : SDF_getATermAnnosHead(arg));
  }
  if (SDF_isATermAnnosMany(arg)) {
    return SDF_makeATermAnnosMany(
        acceptHead ? acceptHead(SDF_getATermAnnosHead(arg)) : SDF_getATermAnnosHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getATermAnnosWsAfterHead(arg)) : SDF_getATermAnnosWsAfterHead(arg),
        acceptWsAfterSep ? acceptWsAfterSep(SDF_getATermAnnosWsAfterSep(arg)) : SDF_getATermAnnosWsAfterSep(arg),
        SDF_visitATermAnnos(SDF_getATermAnnosTail(arg), acceptHead, acceptWsAfterHead, acceptWsAfterSep));
  }
  ATabort("not a ATermAnnos: %t\n", arg);
  return (SDF_ATermAnnos)NULL;
}

/*}}}  */
/*{{{  SDF_IntCon SDF_visitIntCon(SDF_IntCon arg, SDF_NatCon (*acceptNatCon)(SDF_NatCon), SDF_OptLayout (*acceptWsAfterPos)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterNeg)(SDF_OptLayout)) */

SDF_IntCon SDF_visitIntCon(SDF_IntCon arg, SDF_NatCon (*acceptNatCon)(SDF_NatCon), SDF_OptLayout (*acceptWsAfterPos)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterNeg)(SDF_OptLayout))
{
  if (SDF_isIntConNatural(arg)) {
    return SDF_makeIntConNatural(
        acceptNatCon ? acceptNatCon(SDF_getIntConNatCon(arg)) : SDF_getIntConNatCon(arg));
  }
  if (SDF_isIntConPositive(arg)) {
    return SDF_makeIntConPositive(
        acceptWsAfterPos ? acceptWsAfterPos(SDF_getIntConWsAfterPos(arg)) : SDF_getIntConWsAfterPos(arg),
        acceptNatCon ? acceptNatCon(SDF_getIntConNatCon(arg)) : SDF_getIntConNatCon(arg));
  }
  if (SDF_isIntConNegative(arg)) {
    return SDF_makeIntConNegative(
        acceptWsAfterNeg ? acceptWsAfterNeg(SDF_getIntConWsAfterNeg(arg)) : SDF_getIntConWsAfterNeg(arg),
        acceptNatCon ? acceptNatCon(SDF_getIntConNatCon(arg)) : SDF_getIntConNatCon(arg));
  }
  ATabort("not a IntCon: %t\n", arg);
  return (SDF_IntCon)NULL;
}

/*}}}  */
/*{{{  SDF_OptExp SDF_visitOptExp(SDF_OptExp arg, SDF_OptLayout (*acceptWsAfterE)(SDF_OptLayout), SDF_IntCon (*acceptIntCon)(SDF_IntCon)) */

SDF_OptExp SDF_visitOptExp(SDF_OptExp arg, SDF_OptLayout (*acceptWsAfterE)(SDF_OptLayout), SDF_IntCon (*acceptIntCon)(SDF_IntCon))
{
  if (SDF_isOptExpPresent(arg)) {
    return SDF_makeOptExpPresent(
        acceptWsAfterE ? acceptWsAfterE(SDF_getOptExpWsAfterE(arg)) : SDF_getOptExpWsAfterE(arg),
        acceptIntCon ? acceptIntCon(SDF_getOptExpIntCon(arg)) : SDF_getOptExpIntCon(arg));
  }
  if (SDF_isOptExpAbsent(arg)) {
    return SDF_makeOptExpAbsent();
  }
  ATabort("not a OptExp: %t\n", arg);
  return (SDF_OptExp)NULL;
}

/*}}}  */
/*{{{  SDF_RealCon SDF_visitRealCon(SDF_RealCon arg, SDF_IntCon (*acceptBase)(SDF_IntCon), SDF_OptLayout (*acceptWsAfterBase)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterPeriod)(SDF_OptLayout), SDF_NatCon (*acceptDecimal)(SDF_NatCon), SDF_OptLayout (*acceptWsAfterDecimal)(SDF_OptLayout), SDF_OptExp (*acceptExp)(SDF_OptExp)) */

SDF_RealCon SDF_visitRealCon(SDF_RealCon arg, SDF_IntCon (*acceptBase)(SDF_IntCon), SDF_OptLayout (*acceptWsAfterBase)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterPeriod)(SDF_OptLayout), SDF_NatCon (*acceptDecimal)(SDF_NatCon), SDF_OptLayout (*acceptWsAfterDecimal)(SDF_OptLayout), SDF_OptExp (*acceptExp)(SDF_OptExp))
{
  if (SDF_isRealConRealCon(arg)) {
    return SDF_makeRealConRealCon(
        acceptBase ? acceptBase(SDF_getRealConBase(arg)) : SDF_getRealConBase(arg),
        acceptWsAfterBase ? acceptWsAfterBase(SDF_getRealConWsAfterBase(arg)) : SDF_getRealConWsAfterBase(arg),
        acceptWsAfterPeriod ? acceptWsAfterPeriod(SDF_getRealConWsAfterPeriod(arg)) : SDF_getRealConWsAfterPeriod(arg),
        acceptDecimal ? acceptDecimal(SDF_getRealConDecimal(arg)) : SDF_getRealConDecimal(arg),
        acceptWsAfterDecimal ? acceptWsAfterDecimal(SDF_getRealConWsAfterDecimal(arg)) : SDF_getRealConWsAfterDecimal(arg),
        acceptExp ? acceptExp(SDF_getRealConExp(arg)) : SDF_getRealConExp(arg));
  }
  ATabort("not a RealCon: %t\n", arg);
  return (SDF_RealCon)NULL;
}

/*}}}  */
/*{{{  SDF_Grammar SDF_visitGrammar(SDF_Grammar arg, SDF_OptLayout (*acceptWsAfterAliases)(SDF_OptLayout), SDF_Aliases (*acceptAliases)(SDF_Aliases), SDF_OptLayout (*acceptWsAfterLexical)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSyntax)(SDF_OptLayout), SDF_Productions (*acceptProductions)(SDF_Productions), SDF_OptLayout (*acceptWsAfterContextFree)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterVariables)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterLeft)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterGrammar)(SDF_OptLayout), SDF_ImpSection (*acceptImpSection)(SDF_ImpSection), SDF_OptLayout (*acceptWsAfterPriorities)(SDF_OptLayout), SDF_Priorities (*acceptPriorities)(SDF_Priorities), SDF_OptLayout (*acceptWsAfterRestrictions)(SDF_OptLayout), SDF_Restrictions (*acceptRestrictions)(SDF_Restrictions), SDF_OptLayout (*acceptWsAfterSorts)(SDF_OptLayout), SDF_Symbols (*acceptSymbols)(SDF_Symbols), SDF_OptLayout (*acceptWsAfterStartSymbols)(SDF_OptLayout)) */

SDF_Grammar SDF_visitGrammar(SDF_Grammar arg, SDF_OptLayout (*acceptWsAfterAliases)(SDF_OptLayout), SDF_Aliases (*acceptAliases)(SDF_Aliases), SDF_OptLayout (*acceptWsAfterLexical)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSyntax)(SDF_OptLayout), SDF_Productions (*acceptProductions)(SDF_Productions), SDF_OptLayout (*acceptWsAfterContextFree)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterVariables)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterLeft)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterGrammar)(SDF_OptLayout), SDF_ImpSection (*acceptImpSection)(SDF_ImpSection), SDF_OptLayout (*acceptWsAfterPriorities)(SDF_OptLayout), SDF_Priorities (*acceptPriorities)(SDF_Priorities), SDF_OptLayout (*acceptWsAfterRestrictions)(SDF_OptLayout), SDF_Restrictions (*acceptRestrictions)(SDF_Restrictions), SDF_OptLayout (*acceptWsAfterSorts)(SDF_OptLayout), SDF_Symbols (*acceptSymbols)(SDF_Symbols), SDF_OptLayout (*acceptWsAfterStartSymbols)(SDF_OptLayout))
{
  if (SDF_isGrammarAliases(arg)) {
    return SDF_makeGrammarAliases(
        acceptWsAfterAliases ? acceptWsAfterAliases(SDF_getGrammarWsAfterAliases(arg)) : SDF_getGrammarWsAfterAliases(arg),
        acceptAliases ? acceptAliases(SDF_getGrammarAliases(arg)) : SDF_getGrammarAliases(arg));
  }
  if (SDF_isGrammarLexicalSyntax(arg)) {
    return SDF_makeGrammarLexicalSyntax(
        acceptWsAfterLexical ? acceptWsAfterLexical(SDF_getGrammarWsAfterLexical(arg)) : SDF_getGrammarWsAfterLexical(arg),
        acceptWsAfterSyntax ? acceptWsAfterSyntax(SDF_getGrammarWsAfterSyntax(arg)) : SDF_getGrammarWsAfterSyntax(arg),
        acceptProductions ? acceptProductions(SDF_getGrammarProductions(arg)) : SDF_getGrammarProductions(arg));
  }
  if (SDF_isGrammarContextFreeSyntax(arg)) {
    return SDF_makeGrammarContextFreeSyntax(
        acceptWsAfterContextFree ? acceptWsAfterContextFree(SDF_getGrammarWsAfterContextFree(arg)) : SDF_getGrammarWsAfterContextFree(arg),
        acceptWsAfterSyntax ? acceptWsAfterSyntax(SDF_getGrammarWsAfterSyntax(arg)) : SDF_getGrammarWsAfterSyntax(arg),
        acceptProductions ? acceptProductions(SDF_getGrammarProductions(arg)) : SDF_getGrammarProductions(arg));
  }
  if (SDF_isGrammarVariables(arg)) {
    return SDF_makeGrammarVariables(
        acceptWsAfterVariables ? acceptWsAfterVariables(SDF_getGrammarWsAfterVariables(arg)) : SDF_getGrammarWsAfterVariables(arg),
        acceptProductions ? acceptProductions(SDF_getGrammarProductions(arg)) : SDF_getGrammarProductions(arg));
  }
  if (SDF_isGrammarLexicalVariables(arg)) {
    return SDF_makeGrammarLexicalVariables(
        acceptWsAfterLexical ? acceptWsAfterLexical(SDF_getGrammarWsAfterLexical(arg)) : SDF_getGrammarWsAfterLexical(arg),
        acceptWsAfterVariables ? acceptWsAfterVariables(SDF_getGrammarWsAfterVariables(arg)) : SDF_getGrammarWsAfterVariables(arg),
        acceptProductions ? acceptProductions(SDF_getGrammarProductions(arg)) : SDF_getGrammarProductions(arg));
  }
  if (SDF_isGrammarEmptyGrammar(arg)) {
    return SDF_makeGrammarEmptyGrammar();
  }
  if (SDF_isGrammarConcGrammars(arg)) {
    return SDF_makeGrammarConcGrammars(
        SDF_visitGrammar(SDF_getGrammarLeft(arg), acceptWsAfterAliases, acceptAliases, acceptWsAfterLexical, acceptWsAfterSyntax, acceptProductions, acceptWsAfterContextFree, acceptWsAfterVariables, acceptWsAfterLeft, acceptWsAfterParenOpen, acceptWsAfterGrammar, acceptImpSection, acceptWsAfterPriorities, acceptPriorities, acceptWsAfterRestrictions, acceptRestrictions, acceptWsAfterSorts, acceptSymbols, acceptWsAfterStartSymbols),
        acceptWsAfterLeft ? acceptWsAfterLeft(SDF_getGrammarWsAfterLeft(arg)) : SDF_getGrammarWsAfterLeft(arg),
        SDF_visitGrammar(SDF_getGrammarRight(arg), acceptWsAfterAliases, acceptAliases, acceptWsAfterLexical, acceptWsAfterSyntax, acceptProductions, acceptWsAfterContextFree, acceptWsAfterVariables, acceptWsAfterLeft, acceptWsAfterParenOpen, acceptWsAfterGrammar, acceptImpSection, acceptWsAfterPriorities, acceptPriorities, acceptWsAfterRestrictions, acceptRestrictions, acceptWsAfterSorts, acceptSymbols, acceptWsAfterStartSymbols));
  }
  if (SDF_isGrammarBracket(arg)) {
    return SDF_makeGrammarBracket(
        acceptWsAfterParenOpen ? acceptWsAfterParenOpen(SDF_getGrammarWsAfterParenOpen(arg)) : SDF_getGrammarWsAfterParenOpen(arg),
        SDF_visitGrammar(SDF_getGrammarGrammar(arg), acceptWsAfterAliases, acceptAliases, acceptWsAfterLexical, acceptWsAfterSyntax, acceptProductions, acceptWsAfterContextFree, acceptWsAfterVariables, acceptWsAfterLeft, acceptWsAfterParenOpen, acceptWsAfterGrammar, acceptImpSection, acceptWsAfterPriorities, acceptPriorities, acceptWsAfterRestrictions, acceptRestrictions, acceptWsAfterSorts, acceptSymbols, acceptWsAfterStartSymbols),
        acceptWsAfterGrammar ? acceptWsAfterGrammar(SDF_getGrammarWsAfterGrammar(arg)) : SDF_getGrammarWsAfterGrammar(arg));
  }
  if (SDF_isGrammarSyntax(arg)) {
    return SDF_makeGrammarSyntax(
        acceptWsAfterSyntax ? acceptWsAfterSyntax(SDF_getGrammarWsAfterSyntax(arg)) : SDF_getGrammarWsAfterSyntax(arg),
        acceptProductions ? acceptProductions(SDF_getGrammarProductions(arg)) : SDF_getGrammarProductions(arg));
  }
  if (SDF_isGrammarImpSection(arg)) {
    return SDF_makeGrammarImpSection(
        acceptImpSection ? acceptImpSection(SDF_getGrammarImpSection(arg)) : SDF_getGrammarImpSection(arg));
  }
  if (SDF_isGrammarPriorities(arg)) {
    return SDF_makeGrammarPriorities(
        acceptWsAfterPriorities ? acceptWsAfterPriorities(SDF_getGrammarWsAfterPriorities(arg)) : SDF_getGrammarWsAfterPriorities(arg),
        acceptPriorities ? acceptPriorities(SDF_getGrammarPriorities(arg)) : SDF_getGrammarPriorities(arg));
  }
  if (SDF_isGrammarRestrictions(arg)) {
    return SDF_makeGrammarRestrictions(
        acceptWsAfterRestrictions ? acceptWsAfterRestrictions(SDF_getGrammarWsAfterRestrictions(arg)) : SDF_getGrammarWsAfterRestrictions(arg),
        acceptRestrictions ? acceptRestrictions(SDF_getGrammarRestrictions(arg)) : SDF_getGrammarRestrictions(arg));
  }
  if (SDF_isGrammarLexicalPriorities(arg)) {
    return SDF_makeGrammarLexicalPriorities(
        acceptWsAfterLexical ? acceptWsAfterLexical(SDF_getGrammarWsAfterLexical(arg)) : SDF_getGrammarWsAfterLexical(arg),
        acceptWsAfterPriorities ? acceptWsAfterPriorities(SDF_getGrammarWsAfterPriorities(arg)) : SDF_getGrammarWsAfterPriorities(arg),
        acceptPriorities ? acceptPriorities(SDF_getGrammarPriorities(arg)) : SDF_getGrammarPriorities(arg));
  }
  if (SDF_isGrammarContextFreePriorities(arg)) {
    return SDF_makeGrammarContextFreePriorities(
        acceptWsAfterContextFree ? acceptWsAfterContextFree(SDF_getGrammarWsAfterContextFree(arg)) : SDF_getGrammarWsAfterContextFree(arg),
        acceptWsAfterPriorities ? acceptWsAfterPriorities(SDF_getGrammarWsAfterPriorities(arg)) : SDF_getGrammarWsAfterPriorities(arg),
        acceptPriorities ? acceptPriorities(SDF_getGrammarPriorities(arg)) : SDF_getGrammarPriorities(arg));
  }
  if (SDF_isGrammarLexicalRestrictions(arg)) {
    return SDF_makeGrammarLexicalRestrictions(
        acceptWsAfterLexical ? acceptWsAfterLexical(SDF_getGrammarWsAfterLexical(arg)) : SDF_getGrammarWsAfterLexical(arg),
        acceptWsAfterRestrictions ? acceptWsAfterRestrictions(SDF_getGrammarWsAfterRestrictions(arg)) : SDF_getGrammarWsAfterRestrictions(arg),
        acceptRestrictions ? acceptRestrictions(SDF_getGrammarRestrictions(arg)) : SDF_getGrammarRestrictions(arg));
  }
  if (SDF_isGrammarContextFreeRestrictions(arg)) {
    return SDF_makeGrammarContextFreeRestrictions(
        acceptWsAfterContextFree ? acceptWsAfterContextFree(SDF_getGrammarWsAfterContextFree(arg)) : SDF_getGrammarWsAfterContextFree(arg),
        acceptWsAfterRestrictions ? acceptWsAfterRestrictions(SDF_getGrammarWsAfterRestrictions(arg)) : SDF_getGrammarWsAfterRestrictions(arg),
        acceptRestrictions ? acceptRestrictions(SDF_getGrammarRestrictions(arg)) : SDF_getGrammarRestrictions(arg));
  }
  if (SDF_isGrammarSorts(arg)) {
    return SDF_makeGrammarSorts(
        acceptWsAfterSorts ? acceptWsAfterSorts(SDF_getGrammarWsAfterSorts(arg)) : SDF_getGrammarWsAfterSorts(arg),
        acceptSymbols ? acceptSymbols(SDF_getGrammarSymbols(arg)) : SDF_getGrammarSymbols(arg));
  }
  if (SDF_isGrammarKernelStartSymbols(arg)) {
    return SDF_makeGrammarKernelStartSymbols(
        acceptWsAfterStartSymbols ? acceptWsAfterStartSymbols(SDF_getGrammarWsAfterStartSymbols(arg)) : SDF_getGrammarWsAfterStartSymbols(arg),
        acceptSymbols ? acceptSymbols(SDF_getGrammarSymbols(arg)) : SDF_getGrammarSymbols(arg));
  }
  if (SDF_isGrammarLexicalStartSymbols(arg)) {
    return SDF_makeGrammarLexicalStartSymbols(
        acceptWsAfterLexical ? acceptWsAfterLexical(SDF_getGrammarWsAfterLexical(arg)) : SDF_getGrammarWsAfterLexical(arg),
        acceptWsAfterStartSymbols ? acceptWsAfterStartSymbols(SDF_getGrammarWsAfterStartSymbols(arg)) : SDF_getGrammarWsAfterStartSymbols(arg),
        acceptSymbols ? acceptSymbols(SDF_getGrammarSymbols(arg)) : SDF_getGrammarSymbols(arg));
  }
  if (SDF_isGrammarContextFreeStartSymbols(arg)) {
    return SDF_makeGrammarContextFreeStartSymbols(
        acceptWsAfterContextFree ? acceptWsAfterContextFree(SDF_getGrammarWsAfterContextFree(arg)) : SDF_getGrammarWsAfterContextFree(arg),
        acceptWsAfterStartSymbols ? acceptWsAfterStartSymbols(SDF_getGrammarWsAfterStartSymbols(arg)) : SDF_getGrammarWsAfterStartSymbols(arg),
        acceptSymbols ? acceptSymbols(SDF_getGrammarSymbols(arg)) : SDF_getGrammarSymbols(arg));
  }
  ATabort("not a Grammar: %t\n", arg);
  return (SDF_Grammar)NULL;
}

/*}}}  */
/*{{{  SDF_Alias SDF_visitAlias(SDF_Alias arg, SDF_Symbol (*acceptSymbol)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterSymbol)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterArrow)(SDF_OptLayout), SDF_Symbol (*acceptAlias)(SDF_Symbol)) */

SDF_Alias SDF_visitAlias(SDF_Alias arg, SDF_Symbol (*acceptSymbol)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterSymbol)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterArrow)(SDF_OptLayout), SDF_Symbol (*acceptAlias)(SDF_Symbol))
{
  if (SDF_isAliasAlias(arg)) {
    return SDF_makeAliasAlias(
        acceptSymbol ? acceptSymbol(SDF_getAliasSymbol(arg)) : SDF_getAliasSymbol(arg),
        acceptWsAfterSymbol ? acceptWsAfterSymbol(SDF_getAliasWsAfterSymbol(arg)) : SDF_getAliasWsAfterSymbol(arg),
        acceptWsAfterArrow ? acceptWsAfterArrow(SDF_getAliasWsAfterArrow(arg)) : SDF_getAliasWsAfterArrow(arg),
        acceptAlias ? acceptAlias(SDF_getAliasAlias(arg)) : SDF_getAliasAlias(arg));
  }
  ATabort("not a Alias: %t\n", arg);
  return (SDF_Alias)NULL;
}

/*}}}  */
/*{{{  SDF_Aliases SDF_visitAliases(SDF_Aliases arg, SDF_AliasList (*acceptList)(SDF_AliasList)) */

SDF_Aliases SDF_visitAliases(SDF_Aliases arg, SDF_AliasList (*acceptList)(SDF_AliasList))
{
  if (SDF_isAliasesDefault(arg)) {
    return SDF_makeAliasesDefault(
        acceptList ? acceptList(SDF_getAliasesList(arg)) : SDF_getAliasesList(arg));
  }
  ATabort("not a Aliases: %t\n", arg);
  return (SDF_Aliases)NULL;
}

/*}}}  */
/*{{{  SDF_AliasList SDF_visitAliasList(SDF_AliasList arg, SDF_Alias (*acceptHead)(SDF_Alias), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout)) */

SDF_AliasList SDF_visitAliasList(SDF_AliasList arg, SDF_Alias (*acceptHead)(SDF_Alias), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout))
{
  if (SDF_isAliasListEmpty(arg)) {
    return SDF_makeAliasListEmpty();
  }
  if (SDF_isAliasListSingle(arg)) {
    return SDF_makeAliasListSingle(
        acceptHead ? acceptHead(SDF_getAliasListHead(arg)) : SDF_getAliasListHead(arg));
  }
  if (SDF_isAliasListMany(arg)) {
    return SDF_makeAliasListMany(
        acceptHead ? acceptHead(SDF_getAliasListHead(arg)) : SDF_getAliasListHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getAliasListWsAfterHead(arg)) : SDF_getAliasListWsAfterHead(arg),
        SDF_visitAliasList(SDF_getAliasListTail(arg), acceptHead, acceptWsAfterHead));
  }
  ATabort("not a AliasList: %t\n", arg);
  return (SDF_AliasList)NULL;
}

/*}}}  */
/*{{{  SDF_Symbol SDF_visitSymbol(SDF_Symbol arg, SDF_OptLayout (*acceptWsAfterLessThan)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSymbol)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterCF)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterLEX)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterVAR)(SDF_OptLayout), SDF_CharClass (*acceptCharClass)(SDF_CharClass), SDF_Label (*acceptLabel)(SDF_Label), SDF_OptLayout (*acceptWsAfterLabel)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterColon)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterLeftQuote)(SDF_OptLayout), SDF_StrCon (*acceptString)(SDF_StrCon), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_SymbolTail (*acceptTail)(SDF_SymbolTail), SDF_OptLayout (*acceptWsAfterTail)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBraceOpen)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBraceClose)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterComma)(SDF_OptLayout), SDF_SymbolRest (*acceptRest)(SDF_SymbolRest), SDF_OptLayout (*acceptWsAfterRest)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterLeft)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterHash)(SDF_OptLayout), SDF_Symbols (*acceptArguments)(SDF_Symbols), SDF_OptLayout (*acceptWsAfterArguments)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterEqualsGreaterThan)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterResults)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBar)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterGreaterThan)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterRight)(SDF_OptLayout), SDF_Sort (*acceptSort)(SDF_Sort), SDF_OptLayout (*acceptWsAfterSort)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBracketOpenBracketOpen)(SDF_OptLayout), SDF_SymbolParameters (*acceptParameters)(SDF_SymbolParameters), SDF_OptLayout (*acceptWsAfterParameters)(SDF_OptLayout)) */

SDF_Symbol SDF_visitSymbol(SDF_Symbol arg, SDF_OptLayout (*acceptWsAfterLessThan)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSymbol)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterCF)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterLEX)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterVAR)(SDF_OptLayout), SDF_CharClass (*acceptCharClass)(SDF_CharClass), SDF_Label (*acceptLabel)(SDF_Label), SDF_OptLayout (*acceptWsAfterLabel)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterColon)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterLeftQuote)(SDF_OptLayout), SDF_StrCon (*acceptString)(SDF_StrCon), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_SymbolTail (*acceptTail)(SDF_SymbolTail), SDF_OptLayout (*acceptWsAfterTail)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBraceOpen)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBraceClose)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterComma)(SDF_OptLayout), SDF_SymbolRest (*acceptRest)(SDF_SymbolRest), SDF_OptLayout (*acceptWsAfterRest)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterLeft)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterHash)(SDF_OptLayout), SDF_Symbols (*acceptArguments)(SDF_Symbols), SDF_OptLayout (*acceptWsAfterArguments)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterEqualsGreaterThan)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterResults)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBar)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterGreaterThan)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterRight)(SDF_OptLayout), SDF_Sort (*acceptSort)(SDF_Sort), SDF_OptLayout (*acceptWsAfterSort)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBracketOpenBracketOpen)(SDF_OptLayout), SDF_SymbolParameters (*acceptParameters)(SDF_SymbolParameters), SDF_OptLayout (*acceptWsAfterParameters)(SDF_OptLayout))
{
  if (SDF_isSymbolCf(arg)) {
    return SDF_makeSymbolCf(
        acceptWsAfterLessThan ? acceptWsAfterLessThan(SDF_getSymbolWsAfterLessThan(arg)) : SDF_getSymbolWsAfterLessThan(arg),
        SDF_visitSymbol(SDF_getSymbolSymbol(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterSymbol ? acceptWsAfterSymbol(SDF_getSymbolWsAfterSymbol(arg)) : SDF_getSymbolWsAfterSymbol(arg),
        acceptWsAfterCF ? acceptWsAfterCF(SDF_getSymbolWsAfterCF(arg)) : SDF_getSymbolWsAfterCF(arg));
  }
  if (SDF_isSymbolLex(arg)) {
    return SDF_makeSymbolLex(
        acceptWsAfterLessThan ? acceptWsAfterLessThan(SDF_getSymbolWsAfterLessThan(arg)) : SDF_getSymbolWsAfterLessThan(arg),
        SDF_visitSymbol(SDF_getSymbolSymbol(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterSymbol ? acceptWsAfterSymbol(SDF_getSymbolWsAfterSymbol(arg)) : SDF_getSymbolWsAfterSymbol(arg),
        acceptWsAfterLEX ? acceptWsAfterLEX(SDF_getSymbolWsAfterLEX(arg)) : SDF_getSymbolWsAfterLEX(arg));
  }
  if (SDF_isSymbolVarsym(arg)) {
    return SDF_makeSymbolVarsym(
        acceptWsAfterLessThan ? acceptWsAfterLessThan(SDF_getSymbolWsAfterLessThan(arg)) : SDF_getSymbolWsAfterLessThan(arg),
        SDF_visitSymbol(SDF_getSymbolSymbol(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterSymbol ? acceptWsAfterSymbol(SDF_getSymbolWsAfterSymbol(arg)) : SDF_getSymbolWsAfterSymbol(arg),
        acceptWsAfterVAR ? acceptWsAfterVAR(SDF_getSymbolWsAfterVAR(arg)) : SDF_getSymbolWsAfterVAR(arg));
  }
  if (SDF_isSymbolLayout(arg)) {
    return SDF_makeSymbolLayout();
  }
  if (SDF_isSymbolCharClass(arg)) {
    return SDF_makeSymbolCharClass(
        acceptCharClass ? acceptCharClass(SDF_getSymbolCharClass(arg)) : SDF_getSymbolCharClass(arg));
  }
  if (SDF_isSymbolLabel(arg)) {
    return SDF_makeSymbolLabel(
        acceptLabel ? acceptLabel(SDF_getSymbolLabel(arg)) : SDF_getSymbolLabel(arg),
        acceptWsAfterLabel ? acceptWsAfterLabel(SDF_getSymbolWsAfterLabel(arg)) : SDF_getSymbolWsAfterLabel(arg),
        acceptWsAfterColon ? acceptWsAfterColon(SDF_getSymbolWsAfterColon(arg)) : SDF_getSymbolWsAfterColon(arg),
        SDF_visitSymbol(SDF_getSymbolSymbol(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters));
  }
  if (SDF_isSymbolLifting(arg)) {
    return SDF_makeSymbolLifting(
        acceptWsAfterLeftQuote ? acceptWsAfterLeftQuote(SDF_getSymbolWsAfterLeftQuote(arg)) : SDF_getSymbolWsAfterLeftQuote(arg),
        SDF_visitSymbol(SDF_getSymbolSymbol(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterSymbol ? acceptWsAfterSymbol(SDF_getSymbolWsAfterSymbol(arg)) : SDF_getSymbolWsAfterSymbol(arg));
  }
  if (SDF_isSymbolLit(arg)) {
    return SDF_makeSymbolLit(
        acceptString ? acceptString(SDF_getSymbolString(arg)) : SDF_getSymbolString(arg));
  }
  if (SDF_isSymbolEmpty(arg)) {
    return SDF_makeSymbolEmpty(
        acceptWsAfterParenOpen ? acceptWsAfterParenOpen(SDF_getSymbolWsAfterParenOpen(arg)) : SDF_getSymbolWsAfterParenOpen(arg));
  }
  if (SDF_isSymbolSeq(arg)) {
    return SDF_makeSymbolSeq(
        acceptWsAfterParenOpen ? acceptWsAfterParenOpen(SDF_getSymbolWsAfterParenOpen(arg)) : SDF_getSymbolWsAfterParenOpen(arg),
        SDF_visitSymbol(SDF_getSymbolHead(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getSymbolWsAfterHead(arg)) : SDF_getSymbolWsAfterHead(arg),
        acceptTail ? acceptTail(SDF_getSymbolTail(arg)) : SDF_getSymbolTail(arg),
        acceptWsAfterTail ? acceptWsAfterTail(SDF_getSymbolWsAfterTail(arg)) : SDF_getSymbolWsAfterTail(arg));
  }
  if (SDF_isSymbolOpt(arg)) {
    return SDF_makeSymbolOpt(
        SDF_visitSymbol(SDF_getSymbolSymbol(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterSymbol ? acceptWsAfterSymbol(SDF_getSymbolWsAfterSymbol(arg)) : SDF_getSymbolWsAfterSymbol(arg));
  }
  if (SDF_isSymbolIter(arg)) {
    return SDF_makeSymbolIter(
        SDF_visitSymbol(SDF_getSymbolSymbol(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterSymbol ? acceptWsAfterSymbol(SDF_getSymbolWsAfterSymbol(arg)) : SDF_getSymbolWsAfterSymbol(arg));
  }
  if (SDF_isSymbolIterStar(arg)) {
    return SDF_makeSymbolIterStar(
        SDF_visitSymbol(SDF_getSymbolSymbol(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterSymbol ? acceptWsAfterSymbol(SDF_getSymbolWsAfterSymbol(arg)) : SDF_getSymbolWsAfterSymbol(arg));
  }
  if (SDF_isSymbolIterSep(arg)) {
    return SDF_makeSymbolIterSep(
        acceptWsAfterBraceOpen ? acceptWsAfterBraceOpen(SDF_getSymbolWsAfterBraceOpen(arg)) : SDF_getSymbolWsAfterBraceOpen(arg),
        SDF_visitSymbol(SDF_getSymbolSymbol(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterSymbol ? acceptWsAfterSymbol(SDF_getSymbolWsAfterSymbol(arg)) : SDF_getSymbolWsAfterSymbol(arg),
        SDF_visitSymbol(SDF_getSymbolSep(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterSep ? acceptWsAfterSep(SDF_getSymbolWsAfterSep(arg)) : SDF_getSymbolWsAfterSep(arg),
        acceptWsAfterBraceClose ? acceptWsAfterBraceClose(SDF_getSymbolWsAfterBraceClose(arg)) : SDF_getSymbolWsAfterBraceClose(arg));
  }
  if (SDF_isSymbolIterStarSep(arg)) {
    return SDF_makeSymbolIterStarSep(
        acceptWsAfterBraceOpen ? acceptWsAfterBraceOpen(SDF_getSymbolWsAfterBraceOpen(arg)) : SDF_getSymbolWsAfterBraceOpen(arg),
        SDF_visitSymbol(SDF_getSymbolSymbol(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterSymbol ? acceptWsAfterSymbol(SDF_getSymbolWsAfterSymbol(arg)) : SDF_getSymbolWsAfterSymbol(arg),
        SDF_visitSymbol(SDF_getSymbolSep(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterSep ? acceptWsAfterSep(SDF_getSymbolWsAfterSep(arg)) : SDF_getSymbolWsAfterSep(arg),
        acceptWsAfterBraceClose ? acceptWsAfterBraceClose(SDF_getSymbolWsAfterBraceClose(arg)) : SDF_getSymbolWsAfterBraceClose(arg));
  }
  if (SDF_isSymbolTuple(arg)) {
    return SDF_makeSymbolTuple(
        acceptWsAfterLessThan ? acceptWsAfterLessThan(SDF_getSymbolWsAfterLessThan(arg)) : SDF_getSymbolWsAfterLessThan(arg),
        SDF_visitSymbol(SDF_getSymbolHead(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getSymbolWsAfterHead(arg)) : SDF_getSymbolWsAfterHead(arg),
        acceptWsAfterComma ? acceptWsAfterComma(SDF_getSymbolWsAfterComma(arg)) : SDF_getSymbolWsAfterComma(arg),
        acceptRest ? acceptRest(SDF_getSymbolRest(arg)) : SDF_getSymbolRest(arg),
        acceptWsAfterRest ? acceptWsAfterRest(SDF_getSymbolWsAfterRest(arg)) : SDF_getSymbolWsAfterRest(arg));
  }
  if (SDF_isSymbolPair(arg)) {
    return SDF_makeSymbolPair(
        SDF_visitSymbol(SDF_getSymbolLeft(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterLeft ? acceptWsAfterLeft(SDF_getSymbolWsAfterLeft(arg)) : SDF_getSymbolWsAfterLeft(arg),
        acceptWsAfterHash ? acceptWsAfterHash(SDF_getSymbolWsAfterHash(arg)) : SDF_getSymbolWsAfterHash(arg),
        SDF_visitSymbol(SDF_getSymbolRight(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters));
  }
  if (SDF_isSymbolFunc(arg)) {
    return SDF_makeSymbolFunc(
        acceptWsAfterParenOpen ? acceptWsAfterParenOpen(SDF_getSymbolWsAfterParenOpen(arg)) : SDF_getSymbolWsAfterParenOpen(arg),
        acceptArguments ? acceptArguments(SDF_getSymbolArguments(arg)) : SDF_getSymbolArguments(arg),
        acceptWsAfterArguments ? acceptWsAfterArguments(SDF_getSymbolWsAfterArguments(arg)) : SDF_getSymbolWsAfterArguments(arg),
        acceptWsAfterEqualsGreaterThan ? acceptWsAfterEqualsGreaterThan(SDF_getSymbolWsAfterEqualsGreaterThan(arg)) : SDF_getSymbolWsAfterEqualsGreaterThan(arg),
        SDF_visitSymbol(SDF_getSymbolResults(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterResults ? acceptWsAfterResults(SDF_getSymbolWsAfterResults(arg)) : SDF_getSymbolWsAfterResults(arg));
  }
  if (SDF_isSymbolAlt(arg)) {
    return SDF_makeSymbolAlt(
        SDF_visitSymbol(SDF_getSymbolLeft(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterLeft ? acceptWsAfterLeft(SDF_getSymbolWsAfterLeft(arg)) : SDF_getSymbolWsAfterLeft(arg),
        acceptWsAfterBar ? acceptWsAfterBar(SDF_getSymbolWsAfterBar(arg)) : SDF_getSymbolWsAfterBar(arg),
        SDF_visitSymbol(SDF_getSymbolRight(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters));
  }
  if (SDF_isSymbolStrategy(arg)) {
    return SDF_makeSymbolStrategy(
        acceptWsAfterParenOpen ? acceptWsAfterParenOpen(SDF_getSymbolWsAfterParenOpen(arg)) : SDF_getSymbolWsAfterParenOpen(arg),
        SDF_visitSymbol(SDF_getSymbolLeft(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterLeft ? acceptWsAfterLeft(SDF_getSymbolWsAfterLeft(arg)) : SDF_getSymbolWsAfterLeft(arg),
        acceptWsAfterGreaterThan ? acceptWsAfterGreaterThan(SDF_getSymbolWsAfterGreaterThan(arg)) : SDF_getSymbolWsAfterGreaterThan(arg),
        SDF_visitSymbol(SDF_getSymbolRight(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterRight ? acceptWsAfterRight(SDF_getSymbolWsAfterRight(arg)) : SDF_getSymbolWsAfterRight(arg));
  }
  if (SDF_isSymbolStart(arg)) {
    return SDF_makeSymbolStart();
  }
  if (SDF_isSymbolFileStart(arg)) {
    return SDF_makeSymbolFileStart();
  }
  if (SDF_isSymbolSort(arg)) {
    return SDF_makeSymbolSort(
        acceptSort ? acceptSort(SDF_getSymbolSort(arg)) : SDF_getSymbolSort(arg));
  }
  if (SDF_isSymbolParameterizedSort(arg)) {
    return SDF_makeSymbolParameterizedSort(
        acceptSort ? acceptSort(SDF_getSymbolSort(arg)) : SDF_getSymbolSort(arg),
        acceptWsAfterSort ? acceptWsAfterSort(SDF_getSymbolWsAfterSort(arg)) : SDF_getSymbolWsAfterSort(arg),
        acceptWsAfterBracketOpenBracketOpen ? acceptWsAfterBracketOpenBracketOpen(SDF_getSymbolWsAfterBracketOpenBracketOpen(arg)) : SDF_getSymbolWsAfterBracketOpenBracketOpen(arg),
        acceptParameters ? acceptParameters(SDF_getSymbolParameters(arg)) : SDF_getSymbolParameters(arg),
        acceptWsAfterParameters ? acceptWsAfterParameters(SDF_getSymbolWsAfterParameters(arg)) : SDF_getSymbolWsAfterParameters(arg));
  }
  if (SDF_isSymbolBracket(arg)) {
    return SDF_makeSymbolBracket(
        acceptWsAfterParenOpen ? acceptWsAfterParenOpen(SDF_getSymbolWsAfterParenOpen(arg)) : SDF_getSymbolWsAfterParenOpen(arg),
        SDF_visitSymbol(SDF_getSymbolSymbol(arg), acceptWsAfterLessThan, acceptWsAfterSymbol, acceptWsAfterCF, acceptWsAfterLEX, acceptWsAfterVAR, acceptCharClass, acceptLabel, acceptWsAfterLabel, acceptWsAfterColon, acceptWsAfterLeftQuote, acceptString, acceptWsAfterParenOpen, acceptWsAfterHead, acceptTail, acceptWsAfterTail, acceptWsAfterBraceOpen, acceptWsAfterSep, acceptWsAfterBraceClose, acceptWsAfterComma, acceptRest, acceptWsAfterRest, acceptWsAfterLeft, acceptWsAfterHash, acceptArguments, acceptWsAfterArguments, acceptWsAfterEqualsGreaterThan, acceptWsAfterResults, acceptWsAfterBar, acceptWsAfterGreaterThan, acceptWsAfterRight, acceptSort, acceptWsAfterSort, acceptWsAfterBracketOpenBracketOpen, acceptParameters, acceptWsAfterParameters),
        acceptWsAfterSymbol ? acceptWsAfterSymbol(SDF_getSymbolWsAfterSymbol(arg)) : SDF_getSymbolWsAfterSymbol(arg));
  }
  ATabort("not a Symbol: %t\n", arg);
  return (SDF_Symbol)NULL;
}

/*}}}  */
/*{{{  SDF_CharRange SDF_visitCharRange(SDF_CharRange arg, SDF_Character (*acceptCharacter)(SDF_Character), SDF_Character (*acceptStart)(SDF_Character), SDF_OptLayout (*acceptWsAfterStart)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfter)(SDF_OptLayout), SDF_Character (*acceptEnd)(SDF_Character)) */

SDF_CharRange SDF_visitCharRange(SDF_CharRange arg, SDF_Character (*acceptCharacter)(SDF_Character), SDF_Character (*acceptStart)(SDF_Character), SDF_OptLayout (*acceptWsAfterStart)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfter)(SDF_OptLayout), SDF_Character (*acceptEnd)(SDF_Character))
{
  if (SDF_isCharRangeDefault(arg)) {
    return SDF_makeCharRangeDefault(
        acceptCharacter ? acceptCharacter(SDF_getCharRangeCharacter(arg)) : SDF_getCharRangeCharacter(arg));
  }
  if (SDF_isCharRangeRange(arg)) {
    return SDF_makeCharRangeRange(
        acceptStart ? acceptStart(SDF_getCharRangeStart(arg)) : SDF_getCharRangeStart(arg),
        acceptWsAfterStart ? acceptWsAfterStart(SDF_getCharRangeWsAfterStart(arg)) : SDF_getCharRangeWsAfterStart(arg),
        acceptWsAfter ? acceptWsAfter(SDF_getCharRangeWsAfter(arg)) : SDF_getCharRangeWsAfter(arg),
        acceptEnd ? acceptEnd(SDF_getCharRangeEnd(arg)) : SDF_getCharRangeEnd(arg));
  }
  ATabort("not a CharRange: %t\n", arg);
  return (SDF_CharRange)NULL;
}

/*}}}  */
/*{{{  SDF_CharRanges SDF_visitCharRanges(SDF_CharRanges arg, SDF_CharRange (*acceptCharRange)(SDF_CharRange), SDF_OptLayout (*acceptWsAfterLeft)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterCharRanges)(SDF_OptLayout)) */

SDF_CharRanges SDF_visitCharRanges(SDF_CharRanges arg, SDF_CharRange (*acceptCharRange)(SDF_CharRange), SDF_OptLayout (*acceptWsAfterLeft)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterCharRanges)(SDF_OptLayout))
{
  if (SDF_isCharRangesDefault(arg)) {
    return SDF_makeCharRangesDefault(
        acceptCharRange ? acceptCharRange(SDF_getCharRangesCharRange(arg)) : SDF_getCharRangesCharRange(arg));
  }
  if (SDF_isCharRangesConc(arg)) {
    return SDF_makeCharRangesConc(
        SDF_visitCharRanges(SDF_getCharRangesLeft(arg), acceptCharRange, acceptWsAfterLeft, acceptWsAfterParenOpen, acceptWsAfterCharRanges),
        acceptWsAfterLeft ? acceptWsAfterLeft(SDF_getCharRangesWsAfterLeft(arg)) : SDF_getCharRangesWsAfterLeft(arg),
        SDF_visitCharRanges(SDF_getCharRangesRight(arg), acceptCharRange, acceptWsAfterLeft, acceptWsAfterParenOpen, acceptWsAfterCharRanges));
  }
  if (SDF_isCharRangesBracket(arg)) {
    return SDF_makeCharRangesBracket(
        acceptWsAfterParenOpen ? acceptWsAfterParenOpen(SDF_getCharRangesWsAfterParenOpen(arg)) : SDF_getCharRangesWsAfterParenOpen(arg),
        SDF_visitCharRanges(SDF_getCharRangesCharRanges(arg), acceptCharRange, acceptWsAfterLeft, acceptWsAfterParenOpen, acceptWsAfterCharRanges),
        acceptWsAfterCharRanges ? acceptWsAfterCharRanges(SDF_getCharRangesWsAfterCharRanges(arg)) : SDF_getCharRangesWsAfterCharRanges(arg));
  }
  ATabort("not a CharRanges: %t\n", arg);
  return (SDF_CharRanges)NULL;
}

/*}}}  */
/*{{{  SDF_OptCharRanges SDF_visitOptCharRanges(SDF_OptCharRanges arg, SDF_CharRanges (*acceptCharRanges)(SDF_CharRanges)) */

SDF_OptCharRanges SDF_visitOptCharRanges(SDF_OptCharRanges arg, SDF_CharRanges (*acceptCharRanges)(SDF_CharRanges))
{
  if (SDF_isOptCharRangesAbsent(arg)) {
    return SDF_makeOptCharRangesAbsent();
  }
  if (SDF_isOptCharRangesPresent(arg)) {
    return SDF_makeOptCharRangesPresent(
        acceptCharRanges ? acceptCharRanges(SDF_getOptCharRangesCharRanges(arg)) : SDF_getOptCharRangesCharRanges(arg));
  }
  ATabort("not a OptCharRanges: %t\n", arg);
  return (SDF_OptCharRanges)NULL;
}

/*}}}  */
/*{{{  SDF_CharClass SDF_visitCharClass(SDF_CharClass arg, SDF_OptLayout (*acceptWsAfterBracketOpen)(SDF_OptLayout), SDF_OptCharRanges (*acceptOptCharRanges)(SDF_OptCharRanges), SDF_OptLayout (*acceptWsAfterOptCharRanges)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterTilde)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterLeft)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSlash)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSlashBackslash)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBackslashSlash)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterCharClass)(SDF_OptLayout)) */

SDF_CharClass SDF_visitCharClass(SDF_CharClass arg, SDF_OptLayout (*acceptWsAfterBracketOpen)(SDF_OptLayout), SDF_OptCharRanges (*acceptOptCharRanges)(SDF_OptCharRanges), SDF_OptLayout (*acceptWsAfterOptCharRanges)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterTilde)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterLeft)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSlash)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSlashBackslash)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBackslashSlash)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterCharClass)(SDF_OptLayout))
{
  if (SDF_isCharClassSimpleCharclass(arg)) {
    return SDF_makeCharClassSimpleCharclass(
        acceptWsAfterBracketOpen ? acceptWsAfterBracketOpen(SDF_getCharClassWsAfterBracketOpen(arg)) : SDF_getCharClassWsAfterBracketOpen(arg),
        acceptOptCharRanges ? acceptOptCharRanges(SDF_getCharClassOptCharRanges(arg)) : SDF_getCharClassOptCharRanges(arg),
        acceptWsAfterOptCharRanges ? acceptWsAfterOptCharRanges(SDF_getCharClassWsAfterOptCharRanges(arg)) : SDF_getCharClassWsAfterOptCharRanges(arg));
  }
  if (SDF_isCharClassComp(arg)) {
    return SDF_makeCharClassComp(
        acceptWsAfterTilde ? acceptWsAfterTilde(SDF_getCharClassWsAfterTilde(arg)) : SDF_getCharClassWsAfterTilde(arg),
        SDF_visitCharClass(SDF_getCharClassCharClass(arg), acceptWsAfterBracketOpen, acceptOptCharRanges, acceptWsAfterOptCharRanges, acceptWsAfterTilde, acceptWsAfterLeft, acceptWsAfterSlash, acceptWsAfterSlashBackslash, acceptWsAfterBackslashSlash, acceptWsAfterParenOpen, acceptWsAfterCharClass));
  }
  if (SDF_isCharClassDiff(arg)) {
    return SDF_makeCharClassDiff(
        SDF_visitCharClass(SDF_getCharClassLeft(arg), acceptWsAfterBracketOpen, acceptOptCharRanges, acceptWsAfterOptCharRanges, acceptWsAfterTilde, acceptWsAfterLeft, acceptWsAfterSlash, acceptWsAfterSlashBackslash, acceptWsAfterBackslashSlash, acceptWsAfterParenOpen, acceptWsAfterCharClass),
        acceptWsAfterLeft ? acceptWsAfterLeft(SDF_getCharClassWsAfterLeft(arg)) : SDF_getCharClassWsAfterLeft(arg),
        acceptWsAfterSlash ? acceptWsAfterSlash(SDF_getCharClassWsAfterSlash(arg)) : SDF_getCharClassWsAfterSlash(arg),
        SDF_visitCharClass(SDF_getCharClassRight(arg), acceptWsAfterBracketOpen, acceptOptCharRanges, acceptWsAfterOptCharRanges, acceptWsAfterTilde, acceptWsAfterLeft, acceptWsAfterSlash, acceptWsAfterSlashBackslash, acceptWsAfterBackslashSlash, acceptWsAfterParenOpen, acceptWsAfterCharClass));
  }
  if (SDF_isCharClassIsect(arg)) {
    return SDF_makeCharClassIsect(
        SDF_visitCharClass(SDF_getCharClassLeft(arg), acceptWsAfterBracketOpen, acceptOptCharRanges, acceptWsAfterOptCharRanges, acceptWsAfterTilde, acceptWsAfterLeft, acceptWsAfterSlash, acceptWsAfterSlashBackslash, acceptWsAfterBackslashSlash, acceptWsAfterParenOpen, acceptWsAfterCharClass),
        acceptWsAfterLeft ? acceptWsAfterLeft(SDF_getCharClassWsAfterLeft(arg)) : SDF_getCharClassWsAfterLeft(arg),
        acceptWsAfterSlashBackslash ? acceptWsAfterSlashBackslash(SDF_getCharClassWsAfterSlashBackslash(arg)) : SDF_getCharClassWsAfterSlashBackslash(arg),
        SDF_visitCharClass(SDF_getCharClassRight(arg), acceptWsAfterBracketOpen, acceptOptCharRanges, acceptWsAfterOptCharRanges, acceptWsAfterTilde, acceptWsAfterLeft, acceptWsAfterSlash, acceptWsAfterSlashBackslash, acceptWsAfterBackslashSlash, acceptWsAfterParenOpen, acceptWsAfterCharClass));
  }
  if (SDF_isCharClassUnion(arg)) {
    return SDF_makeCharClassUnion(
        SDF_visitCharClass(SDF_getCharClassLeft(arg), acceptWsAfterBracketOpen, acceptOptCharRanges, acceptWsAfterOptCharRanges, acceptWsAfterTilde, acceptWsAfterLeft, acceptWsAfterSlash, acceptWsAfterSlashBackslash, acceptWsAfterBackslashSlash, acceptWsAfterParenOpen, acceptWsAfterCharClass),
        acceptWsAfterLeft ? acceptWsAfterLeft(SDF_getCharClassWsAfterLeft(arg)) : SDF_getCharClassWsAfterLeft(arg),
        acceptWsAfterBackslashSlash ? acceptWsAfterBackslashSlash(SDF_getCharClassWsAfterBackslashSlash(arg)) : SDF_getCharClassWsAfterBackslashSlash(arg),
        SDF_visitCharClass(SDF_getCharClassRight(arg), acceptWsAfterBracketOpen, acceptOptCharRanges, acceptWsAfterOptCharRanges, acceptWsAfterTilde, acceptWsAfterLeft, acceptWsAfterSlash, acceptWsAfterSlashBackslash, acceptWsAfterBackslashSlash, acceptWsAfterParenOpen, acceptWsAfterCharClass));
  }
  if (SDF_isCharClassBracket(arg)) {
    return SDF_makeCharClassBracket(
        acceptWsAfterParenOpen ? acceptWsAfterParenOpen(SDF_getCharClassWsAfterParenOpen(arg)) : SDF_getCharClassWsAfterParenOpen(arg),
        SDF_visitCharClass(SDF_getCharClassCharClass(arg), acceptWsAfterBracketOpen, acceptOptCharRanges, acceptWsAfterOptCharRanges, acceptWsAfterTilde, acceptWsAfterLeft, acceptWsAfterSlash, acceptWsAfterSlashBackslash, acceptWsAfterBackslashSlash, acceptWsAfterParenOpen, acceptWsAfterCharClass),
        acceptWsAfterCharClass ? acceptWsAfterCharClass(SDF_getCharClassWsAfterCharClass(arg)) : SDF_getCharClassWsAfterCharClass(arg));
  }
  ATabort("not a CharClass: %t\n", arg);
  return (SDF_CharClass)NULL;
}

/*}}}  */
/*{{{  SDF_LexNumChar SDF_visitLexNumChar(SDF_LexNumChar arg, char* (*acceptNumber)(char*)) */

SDF_LexNumChar SDF_visitLexNumChar(SDF_LexNumChar arg, char* (*acceptNumber)(char*))
{
  if (SDF_isLexNumCharDigits(arg)) {
    return SDF_makeLexNumCharDigits(
        acceptNumber ? acceptNumber(SDF_getLexNumCharNumber(arg)) : SDF_getLexNumCharNumber(arg));
  }
  ATabort("not a LexNumChar: %t\n", arg);
  return (SDF_LexNumChar)NULL;
}

/*}}}  */
/*{{{  SDF_NumChar SDF_visitNumChar(SDF_NumChar arg, SDF_LexNumChar (*acceptNumChar)(SDF_LexNumChar)) */

SDF_NumChar SDF_visitNumChar(SDF_NumChar arg, SDF_LexNumChar (*acceptNumChar)(SDF_LexNumChar))
{
  if (SDF_isNumCharLexToCf(arg)) {
    return SDF_makeNumCharLexToCf(
        acceptNumChar ? acceptNumChar(SDF_getNumCharNumChar(arg)) : SDF_getNumCharNumChar(arg));
  }
  ATabort("not a NumChar: %t\n", arg);
  return (SDF_NumChar)NULL;
}

/*}}}  */
/*{{{  SDF_LexShortChar SDF_visitLexShortChar(SDF_LexShortChar arg, char (*acceptCharacter)(char), char (*acceptEscape)(char)) */

SDF_LexShortChar SDF_visitLexShortChar(SDF_LexShortChar arg, char (*acceptCharacter)(char), char (*acceptEscape)(char))
{
  if (SDF_isLexShortCharRegular(arg)) {
    return SDF_makeLexShortCharRegular(
        acceptCharacter ? acceptCharacter(SDF_getLexShortCharCharacter(arg)) : SDF_getLexShortCharCharacter(arg));
  }
  if (SDF_isLexShortCharEscaped(arg)) {
    return SDF_makeLexShortCharEscaped(
        acceptEscape ? acceptEscape(SDF_getLexShortCharEscape(arg)) : SDF_getLexShortCharEscape(arg));
  }
  ATabort("not a LexShortChar: %t\n", arg);
  return (SDF_LexShortChar)NULL;
}

/*}}}  */
/*{{{  SDF_ShortChar SDF_visitShortChar(SDF_ShortChar arg, SDF_LexShortChar (*acceptShortChar)(SDF_LexShortChar)) */

SDF_ShortChar SDF_visitShortChar(SDF_ShortChar arg, SDF_LexShortChar (*acceptShortChar)(SDF_LexShortChar))
{
  if (SDF_isShortCharLexToCf(arg)) {
    return SDF_makeShortCharLexToCf(
        acceptShortChar ? acceptShortChar(SDF_getShortCharShortChar(arg)) : SDF_getShortCharShortChar(arg));
  }
  ATabort("not a ShortChar: %t\n", arg);
  return (SDF_ShortChar)NULL;
}

/*}}}  */
/*{{{  SDF_Character SDF_visitCharacter(SDF_Character arg, SDF_NumChar (*acceptNumChar)(SDF_NumChar), SDF_ShortChar (*acceptShortChar)(SDF_ShortChar)) */

SDF_Character SDF_visitCharacter(SDF_Character arg, SDF_NumChar (*acceptNumChar)(SDF_NumChar), SDF_ShortChar (*acceptShortChar)(SDF_ShortChar))
{
  if (SDF_isCharacterNumeric(arg)) {
    return SDF_makeCharacterNumeric(
        acceptNumChar ? acceptNumChar(SDF_getCharacterNumChar(arg)) : SDF_getCharacterNumChar(arg));
  }
  if (SDF_isCharacterShort(arg)) {
    return SDF_makeCharacterShort(
        acceptShortChar ? acceptShortChar(SDF_getCharacterShortChar(arg)) : SDF_getCharacterShortChar(arg));
  }
  if (SDF_isCharacterTop(arg)) {
    return SDF_makeCharacterTop();
  }
  if (SDF_isCharacterEof(arg)) {
    return SDF_makeCharacterEof();
  }
  if (SDF_isCharacterBot(arg)) {
    return SDF_makeCharacterBot();
  }
  if (SDF_isCharacterLabelUnderscoreStart(arg)) {
    return SDF_makeCharacterLabelUnderscoreStart();
  }
  ATabort("not a Character: %t\n", arg);
  return (SDF_Character)NULL;
}

/*}}}  */
/*{{{  SDF_ATermAttribute SDF_visitATermAttribute(SDF_ATermAttribute arg, SDF_ATerm (*acceptAterm)(SDF_ATerm)) */

SDF_ATermAttribute SDF_visitATermAttribute(SDF_ATermAttribute arg, SDF_ATerm (*acceptAterm)(SDF_ATerm))
{
  if (SDF_isATermAttributeDefault(arg)) {
    return SDF_makeATermAttributeDefault(
        acceptAterm ? acceptAterm(SDF_getATermAttributeAterm(arg)) : SDF_getATermAttributeAterm(arg));
  }
  ATabort("not a ATermAttribute: %t\n", arg);
  return (SDF_ATermAttribute)NULL;
}

/*}}}  */
/*{{{  SDF_Attribute SDF_visitAttribute(SDF_Attribute arg, SDF_ATermAttribute (*acceptATermAttribute)(SDF_ATermAttribute), SDF_OptLayout (*acceptWsAfterId)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_ModuleName (*acceptModuleName)(SDF_ModuleName), SDF_OptLayout (*acceptWsAfterModuleName)(SDF_OptLayout), SDF_Associativity (*acceptAssociativity)(SDF_Associativity)) */

SDF_Attribute SDF_visitAttribute(SDF_Attribute arg, SDF_ATermAttribute (*acceptATermAttribute)(SDF_ATermAttribute), SDF_OptLayout (*acceptWsAfterId)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_ModuleName (*acceptModuleName)(SDF_ModuleName), SDF_OptLayout (*acceptWsAfterModuleName)(SDF_OptLayout), SDF_Associativity (*acceptAssociativity)(SDF_Associativity))
{
  if (SDF_isAttributeTerm(arg)) {
    return SDF_makeAttributeTerm(
        acceptATermAttribute ? acceptATermAttribute(SDF_getAttributeATermAttribute(arg)) : SDF_getAttributeATermAttribute(arg));
  }
  if (SDF_isAttributeId(arg)) {
    return SDF_makeAttributeId(
        acceptWsAfterId ? acceptWsAfterId(SDF_getAttributeWsAfterId(arg)) : SDF_getAttributeWsAfterId(arg),
        acceptWsAfterParenOpen ? acceptWsAfterParenOpen(SDF_getAttributeWsAfterParenOpen(arg)) : SDF_getAttributeWsAfterParenOpen(arg),
        acceptModuleName ? acceptModuleName(SDF_getAttributeModuleName(arg)) : SDF_getAttributeModuleName(arg),
        acceptWsAfterModuleName ? acceptWsAfterModuleName(SDF_getAttributeWsAfterModuleName(arg)) : SDF_getAttributeWsAfterModuleName(arg));
  }
  if (SDF_isAttributeBracket(arg)) {
    return SDF_makeAttributeBracket();
  }
  if (SDF_isAttributeAssoc(arg)) {
    return SDF_makeAttributeAssoc(
        acceptAssociativity ? acceptAssociativity(SDF_getAttributeAssociativity(arg)) : SDF_getAttributeAssociativity(arg));
  }
  if (SDF_isAttributeReject(arg)) {
    return SDF_makeAttributeReject();
  }
  if (SDF_isAttributePrefer(arg)) {
    return SDF_makeAttributePrefer();
  }
  if (SDF_isAttributeAvoid(arg)) {
    return SDF_makeAttributeAvoid();
  }
  ATabort("not a Attribute: %t\n", arg);
  return (SDF_Attribute)NULL;
}

/*}}}  */
/*{{{  SDF_Attributes SDF_visitAttributes(SDF_Attributes arg, SDF_OptLayout (*acceptWsAfterBraceOpen)(SDF_OptLayout), SDF_AttributeList (*acceptList)(SDF_AttributeList), SDF_OptLayout (*acceptWsAfterList)(SDF_OptLayout)) */

SDF_Attributes SDF_visitAttributes(SDF_Attributes arg, SDF_OptLayout (*acceptWsAfterBraceOpen)(SDF_OptLayout), SDF_AttributeList (*acceptList)(SDF_AttributeList), SDF_OptLayout (*acceptWsAfterList)(SDF_OptLayout))
{
  if (SDF_isAttributesAttrs(arg)) {
    return SDF_makeAttributesAttrs(
        acceptWsAfterBraceOpen ? acceptWsAfterBraceOpen(SDF_getAttributesWsAfterBraceOpen(arg)) : SDF_getAttributesWsAfterBraceOpen(arg),
        acceptList ? acceptList(SDF_getAttributesList(arg)) : SDF_getAttributesList(arg),
        acceptWsAfterList ? acceptWsAfterList(SDF_getAttributesWsAfterList(arg)) : SDF_getAttributesWsAfterList(arg));
  }
  if (SDF_isAttributesNoAttrs(arg)) {
    return SDF_makeAttributesNoAttrs();
  }
  ATabort("not a Attributes: %t\n", arg);
  return (SDF_Attributes)NULL;
}

/*}}}  */
/*{{{  SDF_Production SDF_visitProduction(SDF_Production arg, SDF_Symbols (*acceptSymbols)(SDF_Symbols), SDF_OptLayout (*acceptWsAfterSymbols)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterGreaterThan)(SDF_OptLayout), SDF_Symbol (*acceptResult)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterResult)(SDF_OptLayout), SDF_Attributes (*acceptAttributes)(SDF_Attributes), SDF_IdCon (*acceptFun)(SDF_IdCon), SDF_OptLayout (*acceptWsAfterFun)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_SymbolArguments (*acceptArguments)(SDF_SymbolArguments), SDF_OptLayout (*acceptWsAfterArguments)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenClose)(SDF_OptLayout), SDF_StrCon (*acceptStr)(SDF_StrCon), SDF_OptLayout (*acceptWsAfterStr)(SDF_OptLayout)) */

SDF_Production SDF_visitProduction(SDF_Production arg, SDF_Symbols (*acceptSymbols)(SDF_Symbols), SDF_OptLayout (*acceptWsAfterSymbols)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterGreaterThan)(SDF_OptLayout), SDF_Symbol (*acceptResult)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterResult)(SDF_OptLayout), SDF_Attributes (*acceptAttributes)(SDF_Attributes), SDF_IdCon (*acceptFun)(SDF_IdCon), SDF_OptLayout (*acceptWsAfterFun)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_SymbolArguments (*acceptArguments)(SDF_SymbolArguments), SDF_OptLayout (*acceptWsAfterArguments)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenClose)(SDF_OptLayout), SDF_StrCon (*acceptStr)(SDF_StrCon), SDF_OptLayout (*acceptWsAfterStr)(SDF_OptLayout))
{
  if (SDF_isProductionProd(arg)) {
    return SDF_makeProductionProd(
        acceptSymbols ? acceptSymbols(SDF_getProductionSymbols(arg)) : SDF_getProductionSymbols(arg),
        acceptWsAfterSymbols ? acceptWsAfterSymbols(SDF_getProductionWsAfterSymbols(arg)) : SDF_getProductionWsAfterSymbols(arg),
        acceptWsAfterGreaterThan ? acceptWsAfterGreaterThan(SDF_getProductionWsAfterGreaterThan(arg)) : SDF_getProductionWsAfterGreaterThan(arg),
        acceptResult ? acceptResult(SDF_getProductionResult(arg)) : SDF_getProductionResult(arg),
        acceptWsAfterResult ? acceptWsAfterResult(SDF_getProductionWsAfterResult(arg)) : SDF_getProductionWsAfterResult(arg),
        acceptAttributes ? acceptAttributes(SDF_getProductionAttributes(arg)) : SDF_getProductionAttributes(arg));
  }
  if (SDF_isProductionUnquotedFun(arg)) {
    return SDF_makeProductionUnquotedFun(
        acceptFun ? acceptFun(SDF_getProductionFun(arg)) : SDF_getProductionFun(arg),
        acceptWsAfterFun ? acceptWsAfterFun(SDF_getProductionWsAfterFun(arg)) : SDF_getProductionWsAfterFun(arg),
        acceptWsAfterParenOpen ? acceptWsAfterParenOpen(SDF_getProductionWsAfterParenOpen(arg)) : SDF_getProductionWsAfterParenOpen(arg),
        acceptArguments ? acceptArguments(SDF_getProductionArguments(arg)) : SDF_getProductionArguments(arg),
        acceptWsAfterArguments ? acceptWsAfterArguments(SDF_getProductionWsAfterArguments(arg)) : SDF_getProductionWsAfterArguments(arg),
        acceptWsAfterParenClose ? acceptWsAfterParenClose(SDF_getProductionWsAfterParenClose(arg)) : SDF_getProductionWsAfterParenClose(arg),
        acceptWsAfterGreaterThan ? acceptWsAfterGreaterThan(SDF_getProductionWsAfterGreaterThan(arg)) : SDF_getProductionWsAfterGreaterThan(arg),
        acceptResult ? acceptResult(SDF_getProductionResult(arg)) : SDF_getProductionResult(arg),
        acceptWsAfterResult ? acceptWsAfterResult(SDF_getProductionWsAfterResult(arg)) : SDF_getProductionWsAfterResult(arg),
        acceptAttributes ? acceptAttributes(SDF_getProductionAttributes(arg)) : SDF_getProductionAttributes(arg));
  }
  if (SDF_isProductionQuotedFun(arg)) {
    return SDF_makeProductionQuotedFun(
        acceptStr ? acceptStr(SDF_getProductionStr(arg)) : SDF_getProductionStr(arg),
        acceptWsAfterStr ? acceptWsAfterStr(SDF_getProductionWsAfterStr(arg)) : SDF_getProductionWsAfterStr(arg),
        acceptWsAfterParenOpen ? acceptWsAfterParenOpen(SDF_getProductionWsAfterParenOpen(arg)) : SDF_getProductionWsAfterParenOpen(arg),
        acceptArguments ? acceptArguments(SDF_getProductionArguments(arg)) : SDF_getProductionArguments(arg),
        acceptWsAfterArguments ? acceptWsAfterArguments(SDF_getProductionWsAfterArguments(arg)) : SDF_getProductionWsAfterArguments(arg),
        acceptWsAfterParenClose ? acceptWsAfterParenClose(SDF_getProductionWsAfterParenClose(arg)) : SDF_getProductionWsAfterParenClose(arg),
        acceptWsAfterGreaterThan ? acceptWsAfterGreaterThan(SDF_getProductionWsAfterGreaterThan(arg)) : SDF_getProductionWsAfterGreaterThan(arg),
        acceptResult ? acceptResult(SDF_getProductionResult(arg)) : SDF_getProductionResult(arg),
        acceptWsAfterResult ? acceptWsAfterResult(SDF_getProductionWsAfterResult(arg)) : SDF_getProductionWsAfterResult(arg),
        acceptAttributes ? acceptAttributes(SDF_getProductionAttributes(arg)) : SDF_getProductionAttributes(arg));
  }
  ATabort("not a Production: %t\n", arg);
  return (SDF_Production)NULL;
}

/*}}}  */
/*{{{  SDF_Productions SDF_visitProductions(SDF_Productions arg, SDF_ProductionList (*acceptList)(SDF_ProductionList)) */

SDF_Productions SDF_visitProductions(SDF_Productions arg, SDF_ProductionList (*acceptList)(SDF_ProductionList))
{
  if (SDF_isProductionsDefault(arg)) {
    return SDF_makeProductionsDefault(
        acceptList ? acceptList(SDF_getProductionsList(arg)) : SDF_getProductionsList(arg));
  }
  ATabort("not a Productions: %t\n", arg);
  return (SDF_Productions)NULL;
}

/*}}}  */
/*{{{  SDF_AttributeList SDF_visitAttributeList(SDF_AttributeList arg, SDF_Attribute (*acceptHead)(SDF_Attribute), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout)) */

SDF_AttributeList SDF_visitAttributeList(SDF_AttributeList arg, SDF_Attribute (*acceptHead)(SDF_Attribute), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout))
{
  if (SDF_isAttributeListEmpty(arg)) {
    return SDF_makeAttributeListEmpty();
  }
  if (SDF_isAttributeListSingle(arg)) {
    return SDF_makeAttributeListSingle(
        acceptHead ? acceptHead(SDF_getAttributeListHead(arg)) : SDF_getAttributeListHead(arg));
  }
  if (SDF_isAttributeListMany(arg)) {
    return SDF_makeAttributeListMany(
        acceptHead ? acceptHead(SDF_getAttributeListHead(arg)) : SDF_getAttributeListHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getAttributeListWsAfterHead(arg)) : SDF_getAttributeListWsAfterHead(arg),
        acceptWsAfterSep ? acceptWsAfterSep(SDF_getAttributeListWsAfterSep(arg)) : SDF_getAttributeListWsAfterSep(arg),
        SDF_visitAttributeList(SDF_getAttributeListTail(arg), acceptHead, acceptWsAfterHead, acceptWsAfterSep));
  }
  ATabort("not a AttributeList: %t\n", arg);
  return (SDF_AttributeList)NULL;
}

/*}}}  */
/*{{{  SDF_ProductionList SDF_visitProductionList(SDF_ProductionList arg, SDF_Production (*acceptHead)(SDF_Production), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout)) */

SDF_ProductionList SDF_visitProductionList(SDF_ProductionList arg, SDF_Production (*acceptHead)(SDF_Production), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout))
{
  if (SDF_isProductionListEmpty(arg)) {
    return SDF_makeProductionListEmpty();
  }
  if (SDF_isProductionListSingle(arg)) {
    return SDF_makeProductionListSingle(
        acceptHead ? acceptHead(SDF_getProductionListHead(arg)) : SDF_getProductionListHead(arg));
  }
  if (SDF_isProductionListMany(arg)) {
    return SDF_makeProductionListMany(
        acceptHead ? acceptHead(SDF_getProductionListHead(arg)) : SDF_getProductionListHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getProductionListWsAfterHead(arg)) : SDF_getProductionListWsAfterHead(arg),
        SDF_visitProductionList(SDF_getProductionListTail(arg), acceptHead, acceptWsAfterHead));
  }
  ATabort("not a ProductionList: %t\n", arg);
  return (SDF_ProductionList)NULL;
}

/*}}}  */
/*{{{  SDF_Label SDF_visitLabel(SDF_Label arg, SDF_StrCon (*acceptStrCon)(SDF_StrCon), SDF_IdCon (*acceptIdCon)(SDF_IdCon)) */

SDF_Label SDF_visitLabel(SDF_Label arg, SDF_StrCon (*acceptStrCon)(SDF_StrCon), SDF_IdCon (*acceptIdCon)(SDF_IdCon))
{
  if (SDF_isLabelQuoted(arg)) {
    return SDF_makeLabelQuoted(
        acceptStrCon ? acceptStrCon(SDF_getLabelStrCon(arg)) : SDF_getLabelStrCon(arg));
  }
  if (SDF_isLabelUnquoted(arg)) {
    return SDF_makeLabelUnquoted(
        acceptIdCon ? acceptIdCon(SDF_getLabelIdCon(arg)) : SDF_getLabelIdCon(arg));
  }
  ATabort("not a Label: %t\n", arg);
  return (SDF_Label)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolArguments SDF_visitSymbolArguments(SDF_SymbolArguments arg, SDF_Symbol (*acceptHead)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout)) */

SDF_SymbolArguments SDF_visitSymbolArguments(SDF_SymbolArguments arg, SDF_Symbol (*acceptHead)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout))
{
  if (SDF_isSymbolArgumentsEmpty(arg)) {
    return SDF_makeSymbolArgumentsEmpty();
  }
  if (SDF_isSymbolArgumentsSingle(arg)) {
    return SDF_makeSymbolArgumentsSingle(
        acceptHead ? acceptHead(SDF_getSymbolArgumentsHead(arg)) : SDF_getSymbolArgumentsHead(arg));
  }
  if (SDF_isSymbolArgumentsMany(arg)) {
    return SDF_makeSymbolArgumentsMany(
        acceptHead ? acceptHead(SDF_getSymbolArgumentsHead(arg)) : SDF_getSymbolArgumentsHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getSymbolArgumentsWsAfterHead(arg)) : SDF_getSymbolArgumentsWsAfterHead(arg),
        acceptWsAfterSep ? acceptWsAfterSep(SDF_getSymbolArgumentsWsAfterSep(arg)) : SDF_getSymbolArgumentsWsAfterSep(arg),
        SDF_visitSymbolArguments(SDF_getSymbolArgumentsTail(arg), acceptHead, acceptWsAfterHead, acceptWsAfterSep));
  }
  ATabort("not a SymbolArguments: %t\n", arg);
  return (SDF_SymbolArguments)NULL;
}

/*}}}  */
/*{{{  SDF_LexModuleWord SDF_visitLexModuleWord(SDF_LexModuleWord arg, char* (*acceptLetters)(char*)) */

SDF_LexModuleWord SDF_visitLexModuleWord(SDF_LexModuleWord arg, char* (*acceptLetters)(char*))
{
  if (SDF_isLexModuleWordWord(arg)) {
    return SDF_makeLexModuleWordWord(
        acceptLetters ? acceptLetters(SDF_getLexModuleWordLetters(arg)) : SDF_getLexModuleWordLetters(arg));
  }
  ATabort("not a LexModuleWord: %t\n", arg);
  return (SDF_LexModuleWord)NULL;
}

/*}}}  */
/*{{{  SDF_ModuleWord SDF_visitModuleWord(SDF_ModuleWord arg, SDF_LexModuleWord (*acceptModuleWord)(SDF_LexModuleWord)) */

SDF_ModuleWord SDF_visitModuleWord(SDF_ModuleWord arg, SDF_LexModuleWord (*acceptModuleWord)(SDF_LexModuleWord))
{
  if (SDF_isModuleWordLexToCf(arg)) {
    return SDF_makeModuleWordLexToCf(
        acceptModuleWord ? acceptModuleWord(SDF_getModuleWordModuleWord(arg)) : SDF_getModuleWordModuleWord(arg));
  }
  ATabort("not a ModuleWord: %t\n", arg);
  return (SDF_ModuleWord)NULL;
}

/*}}}  */
/*{{{  SDF_LexModuleId SDF_visitLexModuleId(SDF_LexModuleId arg, SDF_LexModuleWord (*acceptModuleWord)(SDF_LexModuleWord), SDF_LexModuleWord (*acceptDirname)(SDF_LexModuleWord)) */

SDF_LexModuleId SDF_visitLexModuleId(SDF_LexModuleId arg, SDF_LexModuleWord (*acceptModuleWord)(SDF_LexModuleWord), SDF_LexModuleWord (*acceptDirname)(SDF_LexModuleWord))
{
  if (SDF_isLexModuleIdLeaf(arg)) {
    return SDF_makeLexModuleIdLeaf(
        acceptModuleWord ? acceptModuleWord(SDF_getLexModuleIdModuleWord(arg)) : SDF_getLexModuleIdModuleWord(arg));
  }
  if (SDF_isLexModuleIdRoot(arg)) {
    return SDF_makeLexModuleIdRoot(
        SDF_visitLexModuleId(SDF_getLexModuleIdBasename(arg), acceptModuleWord, acceptDirname));
  }
  if (SDF_isLexModuleIdPath(arg)) {
    return SDF_makeLexModuleIdPath(
        acceptDirname ? acceptDirname(SDF_getLexModuleIdDirname(arg)) : SDF_getLexModuleIdDirname(arg),
        SDF_visitLexModuleId(SDF_getLexModuleIdBasename(arg), acceptModuleWord, acceptDirname));
  }
  ATabort("not a LexModuleId: %t\n", arg);
  return (SDF_LexModuleId)NULL;
}

/*}}}  */
/*{{{  SDF_ModuleId SDF_visitModuleId(SDF_ModuleId arg, SDF_LexModuleId (*acceptModuleId)(SDF_LexModuleId)) */

SDF_ModuleId SDF_visitModuleId(SDF_ModuleId arg, SDF_LexModuleId (*acceptModuleId)(SDF_LexModuleId))
{
  if (SDF_isModuleIdLexToCf(arg)) {
    return SDF_makeModuleIdLexToCf(
        acceptModuleId ? acceptModuleId(SDF_getModuleIdModuleId(arg)) : SDF_getModuleIdModuleId(arg));
  }
  ATabort("not a ModuleId: %t\n", arg);
  return (SDF_ModuleId)NULL;
}

/*}}}  */
/*{{{  SDF_Definition SDF_visitDefinition(SDF_Definition arg, SDF_ModuleList (*acceptList)(SDF_ModuleList)) */

SDF_Definition SDF_visitDefinition(SDF_Definition arg, SDF_ModuleList (*acceptList)(SDF_ModuleList))
{
  if (SDF_isDefinitionDefault(arg)) {
    return SDF_makeDefinitionDefault(
        acceptList ? acceptList(SDF_getDefinitionList(arg)) : SDF_getDefinitionList(arg));
  }
  ATabort("not a Definition: %t\n", arg);
  return (SDF_Definition)NULL;
}

/*}}}  */
/*{{{  SDF_Module SDF_visitModule(SDF_Module arg, SDF_OptLayout (*acceptWsAfterModule)(SDF_OptLayout), SDF_ModuleName (*acceptModuleName)(SDF_ModuleName), SDF_OptLayout (*acceptWsAfterModuleName)(SDF_OptLayout), SDF_ImpSectionList (*acceptList)(SDF_ImpSectionList), SDF_OptLayout (*acceptWsAfterList)(SDF_OptLayout), SDF_Sections (*acceptSections)(SDF_Sections)) */

SDF_Module SDF_visitModule(SDF_Module arg, SDF_OptLayout (*acceptWsAfterModule)(SDF_OptLayout), SDF_ModuleName (*acceptModuleName)(SDF_ModuleName), SDF_OptLayout (*acceptWsAfterModuleName)(SDF_OptLayout), SDF_ImpSectionList (*acceptList)(SDF_ImpSectionList), SDF_OptLayout (*acceptWsAfterList)(SDF_OptLayout), SDF_Sections (*acceptSections)(SDF_Sections))
{
  if (SDF_isModuleModule(arg)) {
    return SDF_makeModuleModule(
        acceptWsAfterModule ? acceptWsAfterModule(SDF_getModuleWsAfterModule(arg)) : SDF_getModuleWsAfterModule(arg),
        acceptModuleName ? acceptModuleName(SDF_getModuleModuleName(arg)) : SDF_getModuleModuleName(arg),
        acceptWsAfterModuleName ? acceptWsAfterModuleName(SDF_getModuleWsAfterModuleName(arg)) : SDF_getModuleWsAfterModuleName(arg),
        acceptList ? acceptList(SDF_getModuleList(arg)) : SDF_getModuleList(arg),
        acceptWsAfterList ? acceptWsAfterList(SDF_getModuleWsAfterList(arg)) : SDF_getModuleWsAfterList(arg),
        acceptSections ? acceptSections(SDF_getModuleSections(arg)) : SDF_getModuleSections(arg));
  }
  ATabort("not a Module: %t\n", arg);
  return (SDF_Module)NULL;
}

/*}}}  */
/*{{{  SDF_Section SDF_visitSection(SDF_Section arg, SDF_OptLayout (*acceptWsAfterExports)(SDF_OptLayout), SDF_Grammar (*acceptGrammar)(SDF_Grammar), SDF_OptLayout (*acceptWsAfterHiddens)(SDF_OptLayout)) */

SDF_Section SDF_visitSection(SDF_Section arg, SDF_OptLayout (*acceptWsAfterExports)(SDF_OptLayout), SDF_Grammar (*acceptGrammar)(SDF_Grammar), SDF_OptLayout (*acceptWsAfterHiddens)(SDF_OptLayout))
{
  if (SDF_isSectionExports(arg)) {
    return SDF_makeSectionExports(
        acceptWsAfterExports ? acceptWsAfterExports(SDF_getSectionWsAfterExports(arg)) : SDF_getSectionWsAfterExports(arg),
        acceptGrammar ? acceptGrammar(SDF_getSectionGrammar(arg)) : SDF_getSectionGrammar(arg));
  }
  if (SDF_isSectionHiddens(arg)) {
    return SDF_makeSectionHiddens(
        acceptWsAfterHiddens ? acceptWsAfterHiddens(SDF_getSectionWsAfterHiddens(arg)) : SDF_getSectionWsAfterHiddens(arg),
        acceptGrammar ? acceptGrammar(SDF_getSectionGrammar(arg)) : SDF_getSectionGrammar(arg));
  }
  ATabort("not a Section: %t\n", arg);
  return (SDF_Section)NULL;
}

/*}}}  */
/*{{{  SDF_Sections SDF_visitSections(SDF_Sections arg, SDF_SectionList (*acceptList)(SDF_SectionList)) */

SDF_Sections SDF_visitSections(SDF_Sections arg, SDF_SectionList (*acceptList)(SDF_SectionList))
{
  if (SDF_isSectionsDefault(arg)) {
    return SDF_makeSectionsDefault(
        acceptList ? acceptList(SDF_getSectionsList(arg)) : SDF_getSectionsList(arg));
  }
  ATabort("not a Sections: %t\n", arg);
  return (SDF_Sections)NULL;
}

/*}}}  */
/*{{{  SDF_ModuleName SDF_visitModuleName(SDF_ModuleName arg, SDF_ModuleId (*acceptModuleId)(SDF_ModuleId), SDF_OptLayout (*acceptWsAfterModuleId)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBracketOpen)(SDF_OptLayout), SDF_Symbols (*acceptParams)(SDF_Symbols), SDF_OptLayout (*acceptWsAfterParams)(SDF_OptLayout)) */

SDF_ModuleName SDF_visitModuleName(SDF_ModuleName arg, SDF_ModuleId (*acceptModuleId)(SDF_ModuleId), SDF_OptLayout (*acceptWsAfterModuleId)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBracketOpen)(SDF_OptLayout), SDF_Symbols (*acceptParams)(SDF_Symbols), SDF_OptLayout (*acceptWsAfterParams)(SDF_OptLayout))
{
  if (SDF_isModuleNameUnparameterized(arg)) {
    return SDF_makeModuleNameUnparameterized(
        acceptModuleId ? acceptModuleId(SDF_getModuleNameModuleId(arg)) : SDF_getModuleNameModuleId(arg));
  }
  if (SDF_isModuleNameParameterized(arg)) {
    return SDF_makeModuleNameParameterized(
        acceptModuleId ? acceptModuleId(SDF_getModuleNameModuleId(arg)) : SDF_getModuleNameModuleId(arg),
        acceptWsAfterModuleId ? acceptWsAfterModuleId(SDF_getModuleNameWsAfterModuleId(arg)) : SDF_getModuleNameWsAfterModuleId(arg),
        acceptWsAfterBracketOpen ? acceptWsAfterBracketOpen(SDF_getModuleNameWsAfterBracketOpen(arg)) : SDF_getModuleNameWsAfterBracketOpen(arg),
        acceptParams ? acceptParams(SDF_getModuleNameParams(arg)) : SDF_getModuleNameParams(arg),
        acceptWsAfterParams ? acceptWsAfterParams(SDF_getModuleNameWsAfterParams(arg)) : SDF_getModuleNameWsAfterParams(arg));
  }
  ATabort("not a ModuleName: %t\n", arg);
  return (SDF_ModuleName)NULL;
}

/*}}}  */
/*{{{  SDF_ImpSection SDF_visitImpSection(SDF_ImpSection arg, SDF_OptLayout (*acceptWsAfterImports)(SDF_OptLayout), SDF_Imports (*acceptList)(SDF_Imports)) */

SDF_ImpSection SDF_visitImpSection(SDF_ImpSection arg, SDF_OptLayout (*acceptWsAfterImports)(SDF_OptLayout), SDF_Imports (*acceptList)(SDF_Imports))
{
  if (SDF_isImpSectionImports(arg)) {
    return SDF_makeImpSectionImports(
        acceptWsAfterImports ? acceptWsAfterImports(SDF_getImpSectionWsAfterImports(arg)) : SDF_getImpSectionWsAfterImports(arg),
        acceptList ? acceptList(SDF_getImpSectionList(arg)) : SDF_getImpSectionList(arg));
  }
  ATabort("not a ImpSection: %t\n", arg);
  return (SDF_ImpSection)NULL;
}

/*}}}  */
/*{{{  SDF_Imports SDF_visitImports(SDF_Imports arg, SDF_ImportList (*acceptList)(SDF_ImportList)) */

SDF_Imports SDF_visitImports(SDF_Imports arg, SDF_ImportList (*acceptList)(SDF_ImportList))
{
  if (SDF_isImportsDefault(arg)) {
    return SDF_makeImportsDefault(
        acceptList ? acceptList(SDF_getImportsList(arg)) : SDF_getImportsList(arg));
  }
  ATabort("not a Imports: %t\n", arg);
  return (SDF_Imports)NULL;
}

/*}}}  */
/*{{{  SDF_Import SDF_visitImport(SDF_Import arg, SDF_ModuleName (*acceptModuleName)(SDF_ModuleName), SDF_OptLayout (*acceptWsAfterModuleName)(SDF_OptLayout), SDF_Renamings (*acceptRenamings)(SDF_Renamings), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterImport)(SDF_OptLayout)) */

SDF_Import SDF_visitImport(SDF_Import arg, SDF_ModuleName (*acceptModuleName)(SDF_ModuleName), SDF_OptLayout (*acceptWsAfterModuleName)(SDF_OptLayout), SDF_Renamings (*acceptRenamings)(SDF_Renamings), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterImport)(SDF_OptLayout))
{
  if (SDF_isImportModule(arg)) {
    return SDF_makeImportModule(
        acceptModuleName ? acceptModuleName(SDF_getImportModuleName(arg)) : SDF_getImportModuleName(arg));
  }
  if (SDF_isImportRenamedModule(arg)) {
    return SDF_makeImportRenamedModule(
        acceptModuleName ? acceptModuleName(SDF_getImportModuleName(arg)) : SDF_getImportModuleName(arg),
        acceptWsAfterModuleName ? acceptWsAfterModuleName(SDF_getImportWsAfterModuleName(arg)) : SDF_getImportWsAfterModuleName(arg),
        acceptRenamings ? acceptRenamings(SDF_getImportRenamings(arg)) : SDF_getImportRenamings(arg));
  }
  if (SDF_isImportBracket(arg)) {
    return SDF_makeImportBracket(
        acceptWsAfterParenOpen ? acceptWsAfterParenOpen(SDF_getImportWsAfterParenOpen(arg)) : SDF_getImportWsAfterParenOpen(arg),
        SDF_visitImport(SDF_getImportImport(arg), acceptModuleName, acceptWsAfterModuleName, acceptRenamings, acceptWsAfterParenOpen, acceptWsAfterImport),
        acceptWsAfterImport ? acceptWsAfterImport(SDF_getImportWsAfterImport(arg)) : SDF_getImportWsAfterImport(arg));
  }
  ATabort("not a Import: %t\n", arg);
  return (SDF_Import)NULL;
}

/*}}}  */
/*{{{  SDF_ModuleList SDF_visitModuleList(SDF_ModuleList arg, SDF_Module (*acceptHead)(SDF_Module), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout)) */

SDF_ModuleList SDF_visitModuleList(SDF_ModuleList arg, SDF_Module (*acceptHead)(SDF_Module), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout))
{
  if (SDF_isModuleListEmpty(arg)) {
    return SDF_makeModuleListEmpty();
  }
  if (SDF_isModuleListSingle(arg)) {
    return SDF_makeModuleListSingle(
        acceptHead ? acceptHead(SDF_getModuleListHead(arg)) : SDF_getModuleListHead(arg));
  }
  if (SDF_isModuleListMany(arg)) {
    return SDF_makeModuleListMany(
        acceptHead ? acceptHead(SDF_getModuleListHead(arg)) : SDF_getModuleListHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getModuleListWsAfterHead(arg)) : SDF_getModuleListWsAfterHead(arg),
        SDF_visitModuleList(SDF_getModuleListTail(arg), acceptHead, acceptWsAfterHead));
  }
  ATabort("not a ModuleList: %t\n", arg);
  return (SDF_ModuleList)NULL;
}

/*}}}  */
/*{{{  SDF_ImpSectionList SDF_visitImpSectionList(SDF_ImpSectionList arg, SDF_ImpSection (*acceptHead)(SDF_ImpSection), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout)) */

SDF_ImpSectionList SDF_visitImpSectionList(SDF_ImpSectionList arg, SDF_ImpSection (*acceptHead)(SDF_ImpSection), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout))
{
  if (SDF_isImpSectionListEmpty(arg)) {
    return SDF_makeImpSectionListEmpty();
  }
  if (SDF_isImpSectionListSingle(arg)) {
    return SDF_makeImpSectionListSingle(
        acceptHead ? acceptHead(SDF_getImpSectionListHead(arg)) : SDF_getImpSectionListHead(arg));
  }
  if (SDF_isImpSectionListMany(arg)) {
    return SDF_makeImpSectionListMany(
        acceptHead ? acceptHead(SDF_getImpSectionListHead(arg)) : SDF_getImpSectionListHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getImpSectionListWsAfterHead(arg)) : SDF_getImpSectionListWsAfterHead(arg),
        SDF_visitImpSectionList(SDF_getImpSectionListTail(arg), acceptHead, acceptWsAfterHead));
  }
  ATabort("not a ImpSectionList: %t\n", arg);
  return (SDF_ImpSectionList)NULL;
}

/*}}}  */
/*{{{  SDF_SectionList SDF_visitSectionList(SDF_SectionList arg, SDF_Section (*acceptHead)(SDF_Section), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout)) */

SDF_SectionList SDF_visitSectionList(SDF_SectionList arg, SDF_Section (*acceptHead)(SDF_Section), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout))
{
  if (SDF_isSectionListEmpty(arg)) {
    return SDF_makeSectionListEmpty();
  }
  if (SDF_isSectionListSingle(arg)) {
    return SDF_makeSectionListSingle(
        acceptHead ? acceptHead(SDF_getSectionListHead(arg)) : SDF_getSectionListHead(arg));
  }
  if (SDF_isSectionListMany(arg)) {
    return SDF_makeSectionListMany(
        acceptHead ? acceptHead(SDF_getSectionListHead(arg)) : SDF_getSectionListHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getSectionListWsAfterHead(arg)) : SDF_getSectionListWsAfterHead(arg),
        SDF_visitSectionList(SDF_getSectionListTail(arg), acceptHead, acceptWsAfterHead));
  }
  ATabort("not a SectionList: %t\n", arg);
  return (SDF_SectionList)NULL;
}

/*}}}  */
/*{{{  SDF_ImportList SDF_visitImportList(SDF_ImportList arg, SDF_Import (*acceptHead)(SDF_Import), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout)) */

SDF_ImportList SDF_visitImportList(SDF_ImportList arg, SDF_Import (*acceptHead)(SDF_Import), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout))
{
  if (SDF_isImportListEmpty(arg)) {
    return SDF_makeImportListEmpty();
  }
  if (SDF_isImportListSingle(arg)) {
    return SDF_makeImportListSingle(
        acceptHead ? acceptHead(SDF_getImportListHead(arg)) : SDF_getImportListHead(arg));
  }
  if (SDF_isImportListMany(arg)) {
    return SDF_makeImportListMany(
        acceptHead ? acceptHead(SDF_getImportListHead(arg)) : SDF_getImportListHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getImportListWsAfterHead(arg)) : SDF_getImportListWsAfterHead(arg),
        SDF_visitImportList(SDF_getImportListTail(arg), acceptHead, acceptWsAfterHead));
  }
  ATabort("not a ImportList: %t\n", arg);
  return (SDF_ImportList)NULL;
}

/*}}}  */
/*{{{  SDF_Associativity SDF_visitAssociativity(SDF_Associativity arg) */

SDF_Associativity SDF_visitAssociativity(SDF_Associativity arg)
{
  if (SDF_isAssociativityLeft(arg)) {
    return SDF_makeAssociativityLeft();
  }
  if (SDF_isAssociativityRight(arg)) {
    return SDF_makeAssociativityRight();
  }
  if (SDF_isAssociativityNonAssoc(arg)) {
    return SDF_makeAssociativityNonAssoc();
  }
  if (SDF_isAssociativityAssoc(arg)) {
    return SDF_makeAssociativityAssoc();
  }
  ATabort("not a Associativity: %t\n", arg);
  return (SDF_Associativity)NULL;
}

/*}}}  */
/*{{{  SDF_ArgumentIndicator SDF_visitArgumentIndicator(SDF_ArgumentIndicator arg, SDF_OptLayout (*acceptWsAfterLessThan)(SDF_OptLayout), SDF_NatConArguments (*acceptArguments)(SDF_NatConArguments), SDF_OptLayout (*acceptWsAfterArguments)(SDF_OptLayout)) */

SDF_ArgumentIndicator SDF_visitArgumentIndicator(SDF_ArgumentIndicator arg, SDF_OptLayout (*acceptWsAfterLessThan)(SDF_OptLayout), SDF_NatConArguments (*acceptArguments)(SDF_NatConArguments), SDF_OptLayout (*acceptWsAfterArguments)(SDF_OptLayout))
{
  if (SDF_isArgumentIndicatorDefault(arg)) {
    return SDF_makeArgumentIndicatorDefault(
        acceptWsAfterLessThan ? acceptWsAfterLessThan(SDF_getArgumentIndicatorWsAfterLessThan(arg)) : SDF_getArgumentIndicatorWsAfterLessThan(arg),
        acceptArguments ? acceptArguments(SDF_getArgumentIndicatorArguments(arg)) : SDF_getArgumentIndicatorArguments(arg),
        acceptWsAfterArguments ? acceptWsAfterArguments(SDF_getArgumentIndicatorWsAfterArguments(arg)) : SDF_getArgumentIndicatorWsAfterArguments(arg));
  }
  ATabort("not a ArgumentIndicator: %t\n", arg);
  return (SDF_ArgumentIndicator)NULL;
}

/*}}}  */
/*{{{  SDF_Group SDF_visitGroup(SDF_Group arg, SDF_OptLayout (*acceptWsAfterGroup)(SDF_OptLayout), SDF_ArgumentIndicator (*acceptArgumentIndicator)(SDF_ArgumentIndicator), SDF_Production (*acceptProduction)(SDF_Production), SDF_OptLayout (*acceptWsAfterBraceOpen)(SDF_OptLayout), SDF_Productions (*acceptProductions)(SDF_Productions), SDF_OptLayout (*acceptWsAfterProductions)(SDF_OptLayout), SDF_Associativity (*acceptAssociativity)(SDF_Associativity), SDF_OptLayout (*acceptWsAfterAssociativity)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterColon)(SDF_OptLayout)) */

SDF_Group SDF_visitGroup(SDF_Group arg, SDF_OptLayout (*acceptWsAfterGroup)(SDF_OptLayout), SDF_ArgumentIndicator (*acceptArgumentIndicator)(SDF_ArgumentIndicator), SDF_Production (*acceptProduction)(SDF_Production), SDF_OptLayout (*acceptWsAfterBraceOpen)(SDF_OptLayout), SDF_Productions (*acceptProductions)(SDF_Productions), SDF_OptLayout (*acceptWsAfterProductions)(SDF_OptLayout), SDF_Associativity (*acceptAssociativity)(SDF_Associativity), SDF_OptLayout (*acceptWsAfterAssociativity)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterColon)(SDF_OptLayout))
{
  if (SDF_isGroupWithArguments(arg)) {
    return SDF_makeGroupWithArguments(
        SDF_visitGroup(SDF_getGroupGroup(arg), acceptWsAfterGroup, acceptArgumentIndicator, acceptProduction, acceptWsAfterBraceOpen, acceptProductions, acceptWsAfterProductions, acceptAssociativity, acceptWsAfterAssociativity, acceptWsAfterColon),
        acceptWsAfterGroup ? acceptWsAfterGroup(SDF_getGroupWsAfterGroup(arg)) : SDF_getGroupWsAfterGroup(arg),
        acceptArgumentIndicator ? acceptArgumentIndicator(SDF_getGroupArgumentIndicator(arg)) : SDF_getGroupArgumentIndicator(arg));
  }
  if (SDF_isGroupSimpleGroup(arg)) {
    return SDF_makeGroupSimpleGroup(
        acceptProduction ? acceptProduction(SDF_getGroupProduction(arg)) : SDF_getGroupProduction(arg));
  }
  if (SDF_isGroupProdsGroup(arg)) {
    return SDF_makeGroupProdsGroup(
        acceptWsAfterBraceOpen ? acceptWsAfterBraceOpen(SDF_getGroupWsAfterBraceOpen(arg)) : SDF_getGroupWsAfterBraceOpen(arg),
        acceptProductions ? acceptProductions(SDF_getGroupProductions(arg)) : SDF_getGroupProductions(arg),
        acceptWsAfterProductions ? acceptWsAfterProductions(SDF_getGroupWsAfterProductions(arg)) : SDF_getGroupWsAfterProductions(arg));
  }
  if (SDF_isGroupAssocGroup(arg)) {
    return SDF_makeGroupAssocGroup(
        acceptWsAfterBraceOpen ? acceptWsAfterBraceOpen(SDF_getGroupWsAfterBraceOpen(arg)) : SDF_getGroupWsAfterBraceOpen(arg),
        acceptAssociativity ? acceptAssociativity(SDF_getGroupAssociativity(arg)) : SDF_getGroupAssociativity(arg),
        acceptWsAfterAssociativity ? acceptWsAfterAssociativity(SDF_getGroupWsAfterAssociativity(arg)) : SDF_getGroupWsAfterAssociativity(arg),
        acceptWsAfterColon ? acceptWsAfterColon(SDF_getGroupWsAfterColon(arg)) : SDF_getGroupWsAfterColon(arg),
        acceptProductions ? acceptProductions(SDF_getGroupProductions(arg)) : SDF_getGroupProductions(arg),
        acceptWsAfterProductions ? acceptWsAfterProductions(SDF_getGroupWsAfterProductions(arg)) : SDF_getGroupWsAfterProductions(arg));
  }
  ATabort("not a Group: %t\n", arg);
  return (SDF_Group)NULL;
}

/*}}}  */
/*{{{  SDF_Priority SDF_visitPriority(SDF_Priority arg, SDF_GroupList (*acceptList)(SDF_GroupList), SDF_Group (*acceptLeft)(SDF_Group), SDF_OptLayout (*acceptWsAfterLeft)(SDF_OptLayout), SDF_Associativity (*acceptAssociativity)(SDF_Associativity), SDF_OptLayout (*acceptWsAfterAssociativity)(SDF_OptLayout), SDF_Group (*acceptRight)(SDF_Group)) */

SDF_Priority SDF_visitPriority(SDF_Priority arg, SDF_GroupList (*acceptList)(SDF_GroupList), SDF_Group (*acceptLeft)(SDF_Group), SDF_OptLayout (*acceptWsAfterLeft)(SDF_OptLayout), SDF_Associativity (*acceptAssociativity)(SDF_Associativity), SDF_OptLayout (*acceptWsAfterAssociativity)(SDF_OptLayout), SDF_Group (*acceptRight)(SDF_Group))
{
  if (SDF_isPriorityChain(arg)) {
    return SDF_makePriorityChain(
        acceptList ? acceptList(SDF_getPriorityList(arg)) : SDF_getPriorityList(arg));
  }
  if (SDF_isPriorityAssoc(arg)) {
    return SDF_makePriorityAssoc(
        acceptLeft ? acceptLeft(SDF_getPriorityLeft(arg)) : SDF_getPriorityLeft(arg),
        acceptWsAfterLeft ? acceptWsAfterLeft(SDF_getPriorityWsAfterLeft(arg)) : SDF_getPriorityWsAfterLeft(arg),
        acceptAssociativity ? acceptAssociativity(SDF_getPriorityAssociativity(arg)) : SDF_getPriorityAssociativity(arg),
        acceptWsAfterAssociativity ? acceptWsAfterAssociativity(SDF_getPriorityWsAfterAssociativity(arg)) : SDF_getPriorityWsAfterAssociativity(arg),
        acceptRight ? acceptRight(SDF_getPriorityRight(arg)) : SDF_getPriorityRight(arg));
  }
  ATabort("not a Priority: %t\n", arg);
  return (SDF_Priority)NULL;
}

/*}}}  */
/*{{{  SDF_Priorities SDF_visitPriorities(SDF_Priorities arg, SDF_PriorityList (*acceptList)(SDF_PriorityList)) */

SDF_Priorities SDF_visitPriorities(SDF_Priorities arg, SDF_PriorityList (*acceptList)(SDF_PriorityList))
{
  if (SDF_isPrioritiesDefault(arg)) {
    return SDF_makePrioritiesDefault(
        acceptList ? acceptList(SDF_getPrioritiesList(arg)) : SDF_getPrioritiesList(arg));
  }
  ATabort("not a Priorities: %t\n", arg);
  return (SDF_Priorities)NULL;
}

/*}}}  */
/*{{{  SDF_NatConArguments SDF_visitNatConArguments(SDF_NatConArguments arg, SDF_NatCon (*acceptHead)(SDF_NatCon), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout)) */

SDF_NatConArguments SDF_visitNatConArguments(SDF_NatConArguments arg, SDF_NatCon (*acceptHead)(SDF_NatCon), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout))
{
  if (SDF_isNatConArgumentsEmpty(arg)) {
    return SDF_makeNatConArgumentsEmpty();
  }
  if (SDF_isNatConArgumentsSingle(arg)) {
    return SDF_makeNatConArgumentsSingle(
        acceptHead ? acceptHead(SDF_getNatConArgumentsHead(arg)) : SDF_getNatConArgumentsHead(arg));
  }
  if (SDF_isNatConArgumentsMany(arg)) {
    return SDF_makeNatConArgumentsMany(
        acceptHead ? acceptHead(SDF_getNatConArgumentsHead(arg)) : SDF_getNatConArgumentsHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getNatConArgumentsWsAfterHead(arg)) : SDF_getNatConArgumentsWsAfterHead(arg),
        acceptWsAfterSep ? acceptWsAfterSep(SDF_getNatConArgumentsWsAfterSep(arg)) : SDF_getNatConArgumentsWsAfterSep(arg),
        SDF_visitNatConArguments(SDF_getNatConArgumentsTail(arg), acceptHead, acceptWsAfterHead, acceptWsAfterSep));
  }
  ATabort("not a NatConArguments: %t\n", arg);
  return (SDF_NatConArguments)NULL;
}

/*}}}  */
/*{{{  SDF_GroupList SDF_visitGroupList(SDF_GroupList arg, SDF_Group (*acceptHead)(SDF_Group), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout)) */

SDF_GroupList SDF_visitGroupList(SDF_GroupList arg, SDF_Group (*acceptHead)(SDF_Group), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout))
{
  if (SDF_isGroupListEmpty(arg)) {
    return SDF_makeGroupListEmpty();
  }
  if (SDF_isGroupListSingle(arg)) {
    return SDF_makeGroupListSingle(
        acceptHead ? acceptHead(SDF_getGroupListHead(arg)) : SDF_getGroupListHead(arg));
  }
  if (SDF_isGroupListMany(arg)) {
    return SDF_makeGroupListMany(
        acceptHead ? acceptHead(SDF_getGroupListHead(arg)) : SDF_getGroupListHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getGroupListWsAfterHead(arg)) : SDF_getGroupListWsAfterHead(arg),
        acceptWsAfterSep ? acceptWsAfterSep(SDF_getGroupListWsAfterSep(arg)) : SDF_getGroupListWsAfterSep(arg),
        SDF_visitGroupList(SDF_getGroupListTail(arg), acceptHead, acceptWsAfterHead, acceptWsAfterSep));
  }
  ATabort("not a GroupList: %t\n", arg);
  return (SDF_GroupList)NULL;
}

/*}}}  */
/*{{{  SDF_PriorityList SDF_visitPriorityList(SDF_PriorityList arg, SDF_Priority (*acceptHead)(SDF_Priority), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout)) */

SDF_PriorityList SDF_visitPriorityList(SDF_PriorityList arg, SDF_Priority (*acceptHead)(SDF_Priority), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout))
{
  if (SDF_isPriorityListEmpty(arg)) {
    return SDF_makePriorityListEmpty();
  }
  if (SDF_isPriorityListSingle(arg)) {
    return SDF_makePriorityListSingle(
        acceptHead ? acceptHead(SDF_getPriorityListHead(arg)) : SDF_getPriorityListHead(arg));
  }
  if (SDF_isPriorityListMany(arg)) {
    return SDF_makePriorityListMany(
        acceptHead ? acceptHead(SDF_getPriorityListHead(arg)) : SDF_getPriorityListHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getPriorityListWsAfterHead(arg)) : SDF_getPriorityListWsAfterHead(arg),
        acceptWsAfterSep ? acceptWsAfterSep(SDF_getPriorityListWsAfterSep(arg)) : SDF_getPriorityListWsAfterSep(arg),
        SDF_visitPriorityList(SDF_getPriorityListTail(arg), acceptHead, acceptWsAfterHead, acceptWsAfterSep));
  }
  ATabort("not a PriorityList: %t\n", arg);
  return (SDF_PriorityList)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolTail SDF_visitSymbolTail(SDF_SymbolTail arg, SDF_Symbol (*acceptHead)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout)) */

SDF_SymbolTail SDF_visitSymbolTail(SDF_SymbolTail arg, SDF_Symbol (*acceptHead)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout))
{
  if (SDF_isSymbolTailEmpty(arg)) {
    return SDF_makeSymbolTailEmpty();
  }
  if (SDF_isSymbolTailSingle(arg)) {
    return SDF_makeSymbolTailSingle(
        acceptHead ? acceptHead(SDF_getSymbolTailHead(arg)) : SDF_getSymbolTailHead(arg));
  }
  if (SDF_isSymbolTailMany(arg)) {
    return SDF_makeSymbolTailMany(
        acceptHead ? acceptHead(SDF_getSymbolTailHead(arg)) : SDF_getSymbolTailHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getSymbolTailWsAfterHead(arg)) : SDF_getSymbolTailWsAfterHead(arg),
        SDF_visitSymbolTail(SDF_getSymbolTailTail(arg), acceptHead, acceptWsAfterHead));
  }
  ATabort("not a SymbolTail: %t\n", arg);
  return (SDF_SymbolTail)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolRest SDF_visitSymbolRest(SDF_SymbolRest arg, SDF_Symbol (*acceptHead)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout)) */

SDF_SymbolRest SDF_visitSymbolRest(SDF_SymbolRest arg, SDF_Symbol (*acceptHead)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout))
{
  if (SDF_isSymbolRestEmpty(arg)) {
    return SDF_makeSymbolRestEmpty();
  }
  if (SDF_isSymbolRestSingle(arg)) {
    return SDF_makeSymbolRestSingle(
        acceptHead ? acceptHead(SDF_getSymbolRestHead(arg)) : SDF_getSymbolRestHead(arg));
  }
  if (SDF_isSymbolRestMany(arg)) {
    return SDF_makeSymbolRestMany(
        acceptHead ? acceptHead(SDF_getSymbolRestHead(arg)) : SDF_getSymbolRestHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getSymbolRestWsAfterHead(arg)) : SDF_getSymbolRestWsAfterHead(arg),
        acceptWsAfterSep ? acceptWsAfterSep(SDF_getSymbolRestWsAfterSep(arg)) : SDF_getSymbolRestWsAfterSep(arg),
        SDF_visitSymbolRest(SDF_getSymbolRestTail(arg), acceptHead, acceptWsAfterHead, acceptWsAfterSep));
  }
  ATabort("not a SymbolRest: %t\n", arg);
  return (SDF_SymbolRest)NULL;
}

/*}}}  */
/*{{{  SDF_Renamings SDF_visitRenamings(SDF_Renamings arg, SDF_OptLayout (*acceptWsAfterBracketOpen)(SDF_OptLayout), SDF_RenamingList (*acceptList)(SDF_RenamingList), SDF_OptLayout (*acceptWsAfterList)(SDF_OptLayout)) */

SDF_Renamings SDF_visitRenamings(SDF_Renamings arg, SDF_OptLayout (*acceptWsAfterBracketOpen)(SDF_OptLayout), SDF_RenamingList (*acceptList)(SDF_RenamingList), SDF_OptLayout (*acceptWsAfterList)(SDF_OptLayout))
{
  if (SDF_isRenamingsRenamings(arg)) {
    return SDF_makeRenamingsRenamings(
        acceptWsAfterBracketOpen ? acceptWsAfterBracketOpen(SDF_getRenamingsWsAfterBracketOpen(arg)) : SDF_getRenamingsWsAfterBracketOpen(arg),
        acceptList ? acceptList(SDF_getRenamingsList(arg)) : SDF_getRenamingsList(arg),
        acceptWsAfterList ? acceptWsAfterList(SDF_getRenamingsWsAfterList(arg)) : SDF_getRenamingsWsAfterList(arg));
  }
  ATabort("not a Renamings: %t\n", arg);
  return (SDF_Renamings)NULL;
}

/*}}}  */
/*{{{  SDF_Renaming SDF_visitRenaming(SDF_Renaming arg, SDF_Symbol (*acceptFrom)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterFrom)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterEqualsGreaterThan)(SDF_OptLayout), SDF_Symbol (*acceptTo)(SDF_Symbol), SDF_Production (*acceptFromProd)(SDF_Production), SDF_OptLayout (*acceptWsAfterFromProd)(SDF_OptLayout), SDF_Production (*acceptToProd)(SDF_Production)) */

SDF_Renaming SDF_visitRenaming(SDF_Renaming arg, SDF_Symbol (*acceptFrom)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterFrom)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterEqualsGreaterThan)(SDF_OptLayout), SDF_Symbol (*acceptTo)(SDF_Symbol), SDF_Production (*acceptFromProd)(SDF_Production), SDF_OptLayout (*acceptWsAfterFromProd)(SDF_OptLayout), SDF_Production (*acceptToProd)(SDF_Production))
{
  if (SDF_isRenamingSymbol(arg)) {
    return SDF_makeRenamingSymbol(
        acceptFrom ? acceptFrom(SDF_getRenamingFrom(arg)) : SDF_getRenamingFrom(arg),
        acceptWsAfterFrom ? acceptWsAfterFrom(SDF_getRenamingWsAfterFrom(arg)) : SDF_getRenamingWsAfterFrom(arg),
        acceptWsAfterEqualsGreaterThan ? acceptWsAfterEqualsGreaterThan(SDF_getRenamingWsAfterEqualsGreaterThan(arg)) : SDF_getRenamingWsAfterEqualsGreaterThan(arg),
        acceptTo ? acceptTo(SDF_getRenamingTo(arg)) : SDF_getRenamingTo(arg));
  }
  if (SDF_isRenamingProduction(arg)) {
    return SDF_makeRenamingProduction(
        acceptFromProd ? acceptFromProd(SDF_getRenamingFromProd(arg)) : SDF_getRenamingFromProd(arg),
        acceptWsAfterFromProd ? acceptWsAfterFromProd(SDF_getRenamingWsAfterFromProd(arg)) : SDF_getRenamingWsAfterFromProd(arg),
        acceptWsAfterEqualsGreaterThan ? acceptWsAfterEqualsGreaterThan(SDF_getRenamingWsAfterEqualsGreaterThan(arg)) : SDF_getRenamingWsAfterEqualsGreaterThan(arg),
        acceptToProd ? acceptToProd(SDF_getRenamingToProd(arg)) : SDF_getRenamingToProd(arg));
  }
  ATabort("not a Renaming: %t\n", arg);
  return (SDF_Renaming)NULL;
}

/*}}}  */
/*{{{  SDF_RenamingList SDF_visitRenamingList(SDF_RenamingList arg, SDF_Renaming (*acceptHead)(SDF_Renaming), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout)) */

SDF_RenamingList SDF_visitRenamingList(SDF_RenamingList arg, SDF_Renaming (*acceptHead)(SDF_Renaming), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout))
{
  if (SDF_isRenamingListEmpty(arg)) {
    return SDF_makeRenamingListEmpty();
  }
  if (SDF_isRenamingListSingle(arg)) {
    return SDF_makeRenamingListSingle(
        acceptHead ? acceptHead(SDF_getRenamingListHead(arg)) : SDF_getRenamingListHead(arg));
  }
  if (SDF_isRenamingListMany(arg)) {
    return SDF_makeRenamingListMany(
        acceptHead ? acceptHead(SDF_getRenamingListHead(arg)) : SDF_getRenamingListHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getRenamingListWsAfterHead(arg)) : SDF_getRenamingListWsAfterHead(arg),
        SDF_visitRenamingList(SDF_getRenamingListTail(arg), acceptHead, acceptWsAfterHead));
  }
  ATabort("not a RenamingList: %t\n", arg);
  return (SDF_RenamingList)NULL;
}

/*}}}  */
/*{{{  SDF_Lookahead SDF_visitLookahead(SDF_Lookahead arg, SDF_CharClass (*acceptCharClass)(SDF_CharClass), SDF_CharClass (*acceptHead)(SDF_CharClass), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterPeriod)(SDF_OptLayout), SDF_Lookaheads (*acceptTail)(SDF_Lookaheads)) */

SDF_Lookahead SDF_visitLookahead(SDF_Lookahead arg, SDF_CharClass (*acceptCharClass)(SDF_CharClass), SDF_CharClass (*acceptHead)(SDF_CharClass), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterPeriod)(SDF_OptLayout), SDF_Lookaheads (*acceptTail)(SDF_Lookaheads))
{
  if (SDF_isLookaheadCharClass(arg)) {
    return SDF_makeLookaheadCharClass(
        acceptCharClass ? acceptCharClass(SDF_getLookaheadCharClass(arg)) : SDF_getLookaheadCharClass(arg));
  }
  if (SDF_isLookaheadSeq(arg)) {
    return SDF_makeLookaheadSeq(
        acceptHead ? acceptHead(SDF_getLookaheadHead(arg)) : SDF_getLookaheadHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getLookaheadWsAfterHead(arg)) : SDF_getLookaheadWsAfterHead(arg),
        acceptWsAfterPeriod ? acceptWsAfterPeriod(SDF_getLookaheadWsAfterPeriod(arg)) : SDF_getLookaheadWsAfterPeriod(arg),
        acceptTail ? acceptTail(SDF_getLookaheadTail(arg)) : SDF_getLookaheadTail(arg));
  }
  ATabort("not a Lookahead: %t\n", arg);
  return (SDF_Lookahead)NULL;
}

/*}}}  */
/*{{{  SDF_Lookaheads SDF_visitLookaheads(SDF_Lookaheads arg, SDF_Lookahead (*acceptLookahead)(SDF_Lookahead), SDF_OptLayout (*acceptWsAfterLeft)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBar)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterLookaheads)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBracketOpenBracketOpen)(SDF_OptLayout), SDF_LookaheadList (*acceptList)(SDF_LookaheadList), SDF_OptLayout (*acceptWsAfterList)(SDF_OptLayout)) */

SDF_Lookaheads SDF_visitLookaheads(SDF_Lookaheads arg, SDF_Lookahead (*acceptLookahead)(SDF_Lookahead), SDF_OptLayout (*acceptWsAfterLeft)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBar)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterParenOpen)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterLookaheads)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterBracketOpenBracketOpen)(SDF_OptLayout), SDF_LookaheadList (*acceptList)(SDF_LookaheadList), SDF_OptLayout (*acceptWsAfterList)(SDF_OptLayout))
{
  if (SDF_isLookaheadsSingle(arg)) {
    return SDF_makeLookaheadsSingle(
        acceptLookahead ? acceptLookahead(SDF_getLookaheadsLookahead(arg)) : SDF_getLookaheadsLookahead(arg));
  }
  if (SDF_isLookaheadsAlt(arg)) {
    return SDF_makeLookaheadsAlt(
        SDF_visitLookaheads(SDF_getLookaheadsLeft(arg), acceptLookahead, acceptWsAfterLeft, acceptWsAfterBar, acceptWsAfterParenOpen, acceptWsAfterLookaheads, acceptWsAfterBracketOpenBracketOpen, acceptList, acceptWsAfterList),
        acceptWsAfterLeft ? acceptWsAfterLeft(SDF_getLookaheadsWsAfterLeft(arg)) : SDF_getLookaheadsWsAfterLeft(arg),
        acceptWsAfterBar ? acceptWsAfterBar(SDF_getLookaheadsWsAfterBar(arg)) : SDF_getLookaheadsWsAfterBar(arg),
        SDF_visitLookaheads(SDF_getLookaheadsRight(arg), acceptLookahead, acceptWsAfterLeft, acceptWsAfterBar, acceptWsAfterParenOpen, acceptWsAfterLookaheads, acceptWsAfterBracketOpenBracketOpen, acceptList, acceptWsAfterList));
  }
  if (SDF_isLookaheadsBracket(arg)) {
    return SDF_makeLookaheadsBracket(
        acceptWsAfterParenOpen ? acceptWsAfterParenOpen(SDF_getLookaheadsWsAfterParenOpen(arg)) : SDF_getLookaheadsWsAfterParenOpen(arg),
        SDF_visitLookaheads(SDF_getLookaheadsLookaheads(arg), acceptLookahead, acceptWsAfterLeft, acceptWsAfterBar, acceptWsAfterParenOpen, acceptWsAfterLookaheads, acceptWsAfterBracketOpenBracketOpen, acceptList, acceptWsAfterList),
        acceptWsAfterLookaheads ? acceptWsAfterLookaheads(SDF_getLookaheadsWsAfterLookaheads(arg)) : SDF_getLookaheadsWsAfterLookaheads(arg));
  }
  if (SDF_isLookaheadsList(arg)) {
    return SDF_makeLookaheadsList(
        acceptWsAfterBracketOpenBracketOpen ? acceptWsAfterBracketOpenBracketOpen(SDF_getLookaheadsWsAfterBracketOpenBracketOpen(arg)) : SDF_getLookaheadsWsAfterBracketOpenBracketOpen(arg),
        acceptList ? acceptList(SDF_getLookaheadsList(arg)) : SDF_getLookaheadsList(arg),
        acceptWsAfterList ? acceptWsAfterList(SDF_getLookaheadsWsAfterList(arg)) : SDF_getLookaheadsWsAfterList(arg));
  }
  ATabort("not a Lookaheads: %t\n", arg);
  return (SDF_Lookaheads)NULL;
}

/*}}}  */
/*{{{  SDF_Restriction SDF_visitRestriction(SDF_Restriction arg, SDF_Symbols (*acceptSymbols)(SDF_Symbols), SDF_OptLayout (*acceptWsAfterSymbols)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSlash)(SDF_OptLayout), SDF_Lookaheads (*acceptLookaheads)(SDF_Lookaheads)) */

SDF_Restriction SDF_visitRestriction(SDF_Restriction arg, SDF_Symbols (*acceptSymbols)(SDF_Symbols), SDF_OptLayout (*acceptWsAfterSymbols)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSlash)(SDF_OptLayout), SDF_Lookaheads (*acceptLookaheads)(SDF_Lookaheads))
{
  if (SDF_isRestrictionFollow(arg)) {
    return SDF_makeRestrictionFollow(
        acceptSymbols ? acceptSymbols(SDF_getRestrictionSymbols(arg)) : SDF_getRestrictionSymbols(arg),
        acceptWsAfterSymbols ? acceptWsAfterSymbols(SDF_getRestrictionWsAfterSymbols(arg)) : SDF_getRestrictionWsAfterSymbols(arg),
        acceptWsAfterSlash ? acceptWsAfterSlash(SDF_getRestrictionWsAfterSlash(arg)) : SDF_getRestrictionWsAfterSlash(arg),
        acceptLookaheads ? acceptLookaheads(SDF_getRestrictionLookaheads(arg)) : SDF_getRestrictionLookaheads(arg));
  }
  ATabort("not a Restriction: %t\n", arg);
  return (SDF_Restriction)NULL;
}

/*}}}  */
/*{{{  SDF_Restrictions SDF_visitRestrictions(SDF_Restrictions arg, SDF_RestrictionList (*acceptList)(SDF_RestrictionList)) */

SDF_Restrictions SDF_visitRestrictions(SDF_Restrictions arg, SDF_RestrictionList (*acceptList)(SDF_RestrictionList))
{
  if (SDF_isRestrictionsDefault(arg)) {
    return SDF_makeRestrictionsDefault(
        acceptList ? acceptList(SDF_getRestrictionsList(arg)) : SDF_getRestrictionsList(arg));
  }
  ATabort("not a Restrictions: %t\n", arg);
  return (SDF_Restrictions)NULL;
}

/*}}}  */
/*{{{  SDF_LookaheadList SDF_visitLookaheadList(SDF_LookaheadList arg, SDF_Lookahead (*acceptHead)(SDF_Lookahead), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout)) */

SDF_LookaheadList SDF_visitLookaheadList(SDF_LookaheadList arg, SDF_Lookahead (*acceptHead)(SDF_Lookahead), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout))
{
  if (SDF_isLookaheadListEmpty(arg)) {
    return SDF_makeLookaheadListEmpty();
  }
  if (SDF_isLookaheadListSingle(arg)) {
    return SDF_makeLookaheadListSingle(
        acceptHead ? acceptHead(SDF_getLookaheadListHead(arg)) : SDF_getLookaheadListHead(arg));
  }
  if (SDF_isLookaheadListMany(arg)) {
    return SDF_makeLookaheadListMany(
        acceptHead ? acceptHead(SDF_getLookaheadListHead(arg)) : SDF_getLookaheadListHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getLookaheadListWsAfterHead(arg)) : SDF_getLookaheadListWsAfterHead(arg),
        acceptWsAfterSep ? acceptWsAfterSep(SDF_getLookaheadListWsAfterSep(arg)) : SDF_getLookaheadListWsAfterSep(arg),
        SDF_visitLookaheadList(SDF_getLookaheadListTail(arg), acceptHead, acceptWsAfterHead, acceptWsAfterSep));
  }
  ATabort("not a LookaheadList: %t\n", arg);
  return (SDF_LookaheadList)NULL;
}

/*}}}  */
/*{{{  SDF_RestrictionList SDF_visitRestrictionList(SDF_RestrictionList arg, SDF_Restriction (*acceptHead)(SDF_Restriction), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout)) */

SDF_RestrictionList SDF_visitRestrictionList(SDF_RestrictionList arg, SDF_Restriction (*acceptHead)(SDF_Restriction), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout))
{
  if (SDF_isRestrictionListEmpty(arg)) {
    return SDF_makeRestrictionListEmpty();
  }
  if (SDF_isRestrictionListSingle(arg)) {
    return SDF_makeRestrictionListSingle(
        acceptHead ? acceptHead(SDF_getRestrictionListHead(arg)) : SDF_getRestrictionListHead(arg));
  }
  if (SDF_isRestrictionListMany(arg)) {
    return SDF_makeRestrictionListMany(
        acceptHead ? acceptHead(SDF_getRestrictionListHead(arg)) : SDF_getRestrictionListHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getRestrictionListWsAfterHead(arg)) : SDF_getRestrictionListWsAfterHead(arg),
        SDF_visitRestrictionList(SDF_getRestrictionListTail(arg), acceptHead, acceptWsAfterHead));
  }
  ATabort("not a RestrictionList: %t\n", arg);
  return (SDF_RestrictionList)NULL;
}

/*}}}  */
/*{{{  SDF_SDF SDF_visitSDF(SDF_SDF arg, SDF_OptLayout (*acceptWsAfterDefinition)(SDF_OptLayout), SDF_Definition (*acceptDefinition)(SDF_Definition)) */

SDF_SDF SDF_visitSDF(SDF_SDF arg, SDF_OptLayout (*acceptWsAfterDefinition)(SDF_OptLayout), SDF_Definition (*acceptDefinition)(SDF_Definition))
{
  if (SDF_isSDFDefinition(arg)) {
    return SDF_makeSDFDefinition(
        acceptWsAfterDefinition ? acceptWsAfterDefinition(SDF_getSDFWsAfterDefinition(arg)) : SDF_getSDFWsAfterDefinition(arg),
        acceptDefinition ? acceptDefinition(SDF_getSDFDefinition(arg)) : SDF_getSDFDefinition(arg));
  }
  ATabort("not a SDF: %t\n", arg);
  return (SDF_SDF)NULL;
}

/*}}}  */
/*{{{  SDF_Sort SDF_visitSort(SDF_Sort arg, SDF_LexSort (*acceptSort)(SDF_LexSort)) */

SDF_Sort SDF_visitSort(SDF_Sort arg, SDF_LexSort (*acceptSort)(SDF_LexSort))
{
  if (SDF_isSortLexToCf(arg)) {
    return SDF_makeSortLexToCf(
        acceptSort ? acceptSort(SDF_getSortSort(arg)) : SDF_getSortSort(arg));
  }
  ATabort("not a Sort: %t\n", arg);
  return (SDF_Sort)NULL;
}

/*}}}  */
/*{{{  SDF_LexSort SDF_visitLexSort(SDF_LexSort arg, char (*acceptHead)(char), char* (*acceptMiddle)(char*), char (*acceptLast)(char)) */

SDF_LexSort SDF_visitLexSort(SDF_LexSort arg, char (*acceptHead)(char), char* (*acceptMiddle)(char*), char (*acceptLast)(char))
{
  if (SDF_isLexSortOneChar(arg)) {
    return SDF_makeLexSortOneChar(
        acceptHead ? acceptHead(SDF_getLexSortHead(arg)) : SDF_getLexSortHead(arg));
  }
  if (SDF_isLexSortMoreChars(arg)) {
    return SDF_makeLexSortMoreChars(
        acceptHead ? acceptHead(SDF_getLexSortHead(arg)) : SDF_getLexSortHead(arg),
        acceptMiddle ? acceptMiddle(SDF_getLexSortMiddle(arg)) : SDF_getLexSortMiddle(arg),
        acceptLast ? acceptLast(SDF_getLexSortLast(arg)) : SDF_getLexSortLast(arg));
  }
  ATabort("not a LexSort: %t\n", arg);
  return (SDF_LexSort)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolParameters SDF_visitSymbolParameters(SDF_SymbolParameters arg, SDF_Symbol (*acceptHead)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout)) */

SDF_SymbolParameters SDF_visitSymbolParameters(SDF_SymbolParameters arg, SDF_Symbol (*acceptHead)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout), SDF_OptLayout (*acceptWsAfterSep)(SDF_OptLayout))
{
  if (SDF_isSymbolParametersEmpty(arg)) {
    return SDF_makeSymbolParametersEmpty();
  }
  if (SDF_isSymbolParametersSingle(arg)) {
    return SDF_makeSymbolParametersSingle(
        acceptHead ? acceptHead(SDF_getSymbolParametersHead(arg)) : SDF_getSymbolParametersHead(arg));
  }
  if (SDF_isSymbolParametersMany(arg)) {
    return SDF_makeSymbolParametersMany(
        acceptHead ? acceptHead(SDF_getSymbolParametersHead(arg)) : SDF_getSymbolParametersHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getSymbolParametersWsAfterHead(arg)) : SDF_getSymbolParametersWsAfterHead(arg),
        acceptWsAfterSep ? acceptWsAfterSep(SDF_getSymbolParametersWsAfterSep(arg)) : SDF_getSymbolParametersWsAfterSep(arg),
        SDF_visitSymbolParameters(SDF_getSymbolParametersTail(arg), acceptHead, acceptWsAfterHead, acceptWsAfterSep));
  }
  ATabort("not a SymbolParameters: %t\n", arg);
  return (SDF_SymbolParameters)NULL;
}

/*}}}  */
/*{{{  SDF_Symbols SDF_visitSymbols(SDF_Symbols arg, SDF_SymbolList (*acceptList)(SDF_SymbolList)) */

SDF_Symbols SDF_visitSymbols(SDF_Symbols arg, SDF_SymbolList (*acceptList)(SDF_SymbolList))
{
  if (SDF_isSymbolsDefault(arg)) {
    return SDF_makeSymbolsDefault(
        acceptList ? acceptList(SDF_getSymbolsList(arg)) : SDF_getSymbolsList(arg));
  }
  ATabort("not a Symbols: %t\n", arg);
  return (SDF_Symbols)NULL;
}

/*}}}  */
/*{{{  SDF_SymbolList SDF_visitSymbolList(SDF_SymbolList arg, SDF_Symbol (*acceptHead)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout)) */

SDF_SymbolList SDF_visitSymbolList(SDF_SymbolList arg, SDF_Symbol (*acceptHead)(SDF_Symbol), SDF_OptLayout (*acceptWsAfterHead)(SDF_OptLayout))
{
  if (SDF_isSymbolListEmpty(arg)) {
    return SDF_makeSymbolListEmpty();
  }
  if (SDF_isSymbolListSingle(arg)) {
    return SDF_makeSymbolListSingle(
        acceptHead ? acceptHead(SDF_getSymbolListHead(arg)) : SDF_getSymbolListHead(arg));
  }
  if (SDF_isSymbolListMany(arg)) {
    return SDF_makeSymbolListMany(
        acceptHead ? acceptHead(SDF_getSymbolListHead(arg)) : SDF_getSymbolListHead(arg),
        acceptWsAfterHead ? acceptWsAfterHead(SDF_getSymbolListWsAfterHead(arg)) : SDF_getSymbolListWsAfterHead(arg),
        SDF_visitSymbolList(SDF_getSymbolListTail(arg), acceptHead, acceptWsAfterHead));
  }
  ATabort("not a SymbolList: %t\n", arg);
  return (SDF_SymbolList)NULL;
}

/*}}}  */
/*{{{  SDF_LexLayout SDF_visitLexLayout(SDF_LexLayout arg, char (*acceptCh)(char), char* (*acceptLine)(char*), char* (*acceptContent)(char*)) */

SDF_LexLayout SDF_visitLexLayout(SDF_LexLayout arg, char (*acceptCh)(char), char* (*acceptLine)(char*), char* (*acceptContent)(char*))
{
  if (SDF_isLexLayoutWhitespace(arg)) {
    return SDF_makeLexLayoutWhitespace(
        acceptCh ? acceptCh(SDF_getLexLayoutCh(arg)) : SDF_getLexLayoutCh(arg));
  }
  if (SDF_isLexLayoutLine(arg)) {
    return SDF_makeLexLayoutLine(
        acceptLine ? acceptLine(SDF_getLexLayoutLine(arg)) : SDF_getLexLayoutLine(arg));
  }
  if (SDF_isLexLayoutNested(arg)) {
    return SDF_makeLexLayoutNested(
        acceptContent ? acceptContent(SDF_getLexLayoutContent(arg)) : SDF_getLexLayoutContent(arg));
  }
  ATabort("not a LexLayout: %t\n", arg);
  return (SDF_LexLayout)NULL;
}

/*}}}  */
/*{{{  SDF_LexStrChar SDF_visitLexStrChar(SDF_LexStrChar arg, char (*acceptA)(char), char (*acceptB)(char), char (*acceptC)(char), char (*acceptCh)(char)) */

SDF_LexStrChar SDF_visitLexStrChar(SDF_LexStrChar arg, char (*acceptA)(char), char (*acceptB)(char), char (*acceptC)(char), char (*acceptCh)(char))
{
  if (SDF_isLexStrCharNewline(arg)) {
    return SDF_makeLexStrCharNewline();
  }
  if (SDF_isLexStrCharTab(arg)) {
    return SDF_makeLexStrCharTab();
  }
  if (SDF_isLexStrCharQuote(arg)) {
    return SDF_makeLexStrCharQuote();
  }
  if (SDF_isLexStrCharBackslash(arg)) {
    return SDF_makeLexStrCharBackslash();
  }
  if (SDF_isLexStrCharDecimal(arg)) {
    return SDF_makeLexStrCharDecimal(
        acceptA ? acceptA(SDF_getLexStrCharA(arg)) : SDF_getLexStrCharA(arg),
        acceptB ? acceptB(SDF_getLexStrCharB(arg)) : SDF_getLexStrCharB(arg),
        acceptC ? acceptC(SDF_getLexStrCharC(arg)) : SDF_getLexStrCharC(arg));
  }
  if (SDF_isLexStrCharNormal(arg)) {
    return SDF_makeLexStrCharNormal(
        acceptCh ? acceptCh(SDF_getLexStrCharCh(arg)) : SDF_getLexStrCharCh(arg));
  }
  ATabort("not a LexStrChar: %t\n", arg);
  return (SDF_LexStrChar)NULL;
}

/*}}}  */
/*{{{  SDF_StrChar SDF_visitStrChar(SDF_StrChar arg, SDF_LexStrChar (*acceptStrChar)(SDF_LexStrChar)) */

SDF_StrChar SDF_visitStrChar(SDF_StrChar arg, SDF_LexStrChar (*acceptStrChar)(SDF_LexStrChar))
{
  if (SDF_isStrCharLexToCf(arg)) {
    return SDF_makeStrCharLexToCf(
        acceptStrChar ? acceptStrChar(SDF_getStrCharStrChar(arg)) : SDF_getStrCharStrChar(arg));
  }
  ATabort("not a StrChar: %t\n", arg);
  return (SDF_StrChar)NULL;
}

/*}}}  */
/*{{{  SDF_LexStrCon SDF_visitLexStrCon(SDF_LexStrCon arg, SDF_LexStrCharChars (*acceptChars)(SDF_LexStrCharChars)) */

SDF_LexStrCon SDF_visitLexStrCon(SDF_LexStrCon arg, SDF_LexStrCharChars (*acceptChars)(SDF_LexStrCharChars))
{
  if (SDF_isLexStrConDefault(arg)) {
    return SDF_makeLexStrConDefault(
        acceptChars ? acceptChars(SDF_getLexStrConChars(arg)) : SDF_getLexStrConChars(arg));
  }
  ATabort("not a LexStrCon: %t\n", arg);
  return (SDF_LexStrCon)NULL;
}

/*}}}  */
/*{{{  SDF_StrCon SDF_visitStrCon(SDF_StrCon arg, SDF_LexStrCon (*acceptStrCon)(SDF_LexStrCon)) */

SDF_StrCon SDF_visitStrCon(SDF_StrCon arg, SDF_LexStrCon (*acceptStrCon)(SDF_LexStrCon))
{
  if (SDF_isStrConLexToCf(arg)) {
    return SDF_makeStrConLexToCf(
        acceptStrCon ? acceptStrCon(SDF_getStrConStrCon(arg)) : SDF_getStrConStrCon(arg));
  }
  ATabort("not a StrCon: %t\n", arg);
  return (SDF_StrCon)NULL;
}

/*}}}  */
/*{{{  SDF_LexStrCharChars SDF_visitLexStrCharChars(SDF_LexStrCharChars arg, SDF_LexStrChar (*acceptHead)(SDF_LexStrChar)) */

SDF_LexStrCharChars SDF_visitLexStrCharChars(SDF_LexStrCharChars arg, SDF_LexStrChar (*acceptHead)(SDF_LexStrChar))
{
  if (SDF_isLexStrCharCharsEmpty(arg)) {
    return SDF_makeLexStrCharCharsEmpty();
  }
  if (SDF_isLexStrCharCharsSingle(arg)) {
    return SDF_makeLexStrCharCharsSingle(
        acceptHead ? acceptHead(SDF_getLexStrCharCharsHead(arg)) : SDF_getLexStrCharCharsHead(arg));
  }
  if (SDF_isLexStrCharCharsMany(arg)) {
    return SDF_makeLexStrCharCharsMany(
        acceptHead ? acceptHead(SDF_getLexStrCharCharsHead(arg)) : SDF_getLexStrCharCharsHead(arg),
        SDF_visitLexStrCharChars(SDF_getLexStrCharCharsTail(arg), acceptHead));
  }
  ATabort("not a LexStrCharChars: %t\n", arg);
  return (SDF_LexStrCharChars)NULL;
}

/*}}}  */
/*{{{  SDF_LexNatCon SDF_visitLexNatCon(SDF_LexNatCon arg, char* (*acceptList)(char*)) */

SDF_LexNatCon SDF_visitLexNatCon(SDF_LexNatCon arg, char* (*acceptList)(char*))
{
  if (SDF_isLexNatConDigits(arg)) {
    return SDF_makeLexNatConDigits(
        acceptList ? acceptList(SDF_getLexNatConList(arg)) : SDF_getLexNatConList(arg));
  }
  ATabort("not a LexNatCon: %t\n", arg);
  return (SDF_LexNatCon)NULL;
}

/*}}}  */
/*{{{  SDF_NatCon SDF_visitNatCon(SDF_NatCon arg, SDF_LexNatCon (*acceptNatCon)(SDF_LexNatCon)) */

SDF_NatCon SDF_visitNatCon(SDF_NatCon arg, SDF_LexNatCon (*acceptNatCon)(SDF_LexNatCon))
{
  if (SDF_isNatConLexToCf(arg)) {
    return SDF_makeNatConLexToCf(
        acceptNatCon ? acceptNatCon(SDF_getNatConNatCon(arg)) : SDF_getNatConNatCon(arg));
  }
  ATabort("not a NatCon: %t\n", arg);
  return (SDF_NatCon)NULL;
}

/*}}}  */
/*{{{  SDF_LexIdCon SDF_visitLexIdCon(SDF_LexIdCon arg, char (*acceptHead)(char), char* (*acceptTail)(char*)) */

SDF_LexIdCon SDF_visitLexIdCon(SDF_LexIdCon arg, char (*acceptHead)(char), char* (*acceptTail)(char*))
{
  if (SDF_isLexIdConDefault(arg)) {
    return SDF_makeLexIdConDefault(
        acceptHead ? acceptHead(SDF_getLexIdConHead(arg)) : SDF_getLexIdConHead(arg),
        acceptTail ? acceptTail(SDF_getLexIdConTail(arg)) : SDF_getLexIdConTail(arg));
  }
  ATabort("not a LexIdCon: %t\n", arg);
  return (SDF_LexIdCon)NULL;
}

/*}}}  */
/*{{{  SDF_IdCon SDF_visitIdCon(SDF_IdCon arg, SDF_LexIdCon (*acceptIdCon)(SDF_LexIdCon)) */

SDF_IdCon SDF_visitIdCon(SDF_IdCon arg, SDF_LexIdCon (*acceptIdCon)(SDF_LexIdCon))
{
  if (SDF_isIdConLexToCf(arg)) {
    return SDF_makeIdConLexToCf(
        acceptIdCon ? acceptIdCon(SDF_getIdConIdCon(arg)) : SDF_getIdConIdCon(arg));
  }
  ATabort("not a IdCon: %t\n", arg);
  return (SDF_IdCon)NULL;
}

/*}}}  */

/*}}}  */
